/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.activityassignment.web;

import java.util.Date;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;

import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.google.common.collect.Lists;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.MyBeanUtils;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.modules.activityassignment.entity.ActivityAssignment;
import com.jeeplus.modules.activityassignment.service.ActivityAssignmentService;

/**
 * 工作流任务委托Controller
 * @author 杨帆
 * @version 2018-04-27
 */
@Controller
@RequestMapping(value = "${adminPath}/activityassignment/activityAssignment")
public class ActivityAssignmentController extends BaseController {

	@Autowired
	private ActivityAssignmentService activityAssignmentService;
	
	@ModelAttribute
	public ActivityAssignment get(@RequestParam(required=false) String id) {
		ActivityAssignment entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = activityAssignmentService.get(id);
		}
		if (entity == null){
			entity = new ActivityAssignment();
		}
		return entity;
	}
	
	/**
	 * 工作流任务委托列表页面
	 */
	@RequiresPermissions("activityassignment:activityAssignment:list")
	@RequestMapping(value = {"list", ""})
	public String list(ActivityAssignment activityAssignment, HttpServletRequest request, HttpServletResponse response, Model model) {
		activityAssignment.setStartDate(new Date());
		Page<ActivityAssignment> page = activityAssignmentService.findPage(new Page<ActivityAssignment>(request, response), activityAssignment);
		List<ActivityAssignment> lists = page.getList();
		for(int i=0;i< lists.size();i++)
		{
			ActivityAssignment e = lists.get(i);
			String s = e.getRemarks();
			s = s.replaceAll("\r\n"," ");
			s = s.replace('\n',' ');
			e.setRemarks(s);
			lists.set(i,e);
		}
		page.setList(lists);
		model.addAttribute("page", page);
		return "modules/activityassignment/activityAssignmentList";
	}

	/**
	 * 查看，增加，编辑工作流任务委托表单页面
	 */
	@RequiresPermissions(value={"activityassignment:activityAssignment:view","activityassignment:activityAssignment:add","activityassignment:activityAssignment:edit"},logical=Logical.OR)
	@RequestMapping(value = "form")
	public String form(ActivityAssignment activityAssignment, Model model) {
		model.addAttribute("activityAssignment", activityAssignment);
		if (activityAssignment!=null && "view".equals(activityAssignment.getView())){
			return "modules/activityassignment/activityAssignmentView";
		}
		return "modules/activityassignment/activityAssignmentForm";
	}

	/**
	 * 保存工作流任务委托
	 */
	@RequiresPermissions(value={"activityassignment:activityAssignment:add","activityassignment:activityAssignment:edit"},logical=Logical.OR)
	@RequestMapping(value = "save")
	public String save(ActivityAssignment activityAssignment, Model model, RedirectAttributes redirectAttributes) throws Exception{
		if (!beanValidator(model, activityAssignment)){
			return form(activityAssignment, model);
		}
		if(!activityAssignment.getIsNewRecord()){//编辑表单保存
			ActivityAssignment t = activityAssignmentService.get(activityAssignment.getId());//从数据库取出记录的值
			MyBeanUtils.copyBeanNotNull2Bean(activityAssignment, t);//将编辑表单中的非NULL值覆盖数据库记录中的值
			activityAssignmentService.save(t);//保存
		}else{//新增表单保存
			activityAssignmentService.save(activityAssignment);//保存
		}
		addMessage(redirectAttributes, "保存工作流任务委托成功");
		return "redirect:"+Global.getAdminPath()+"/activityassignment/activityAssignment/?repage";
	}
	
	/**
	 * 删除工作流任务委托
	 */
	@RequiresPermissions("activityassignment:activityAssignment:del")
	@RequestMapping(value = "delete")
	public String delete(ActivityAssignment activityAssignment, RedirectAttributes redirectAttributes) {
		activityAssignmentService.delete(activityAssignment);
		addMessage(redirectAttributes, "删除工作流任务委托成功");
		return "redirect:"+Global.getAdminPath()+"/activityassignment/activityAssignment/?repage";
	}
	
	/**
	 * 批量删除工作流任务委托
	 */
	@RequiresPermissions("activityassignment:activityAssignment:del")
	@RequestMapping(value = "deleteAll")
	public String deleteAll(String ids, RedirectAttributes redirectAttributes) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			activityAssignmentService.delete(activityAssignmentService.get(id));
		}
		addMessage(redirectAttributes, "删除工作流任务委托成功");
		return "redirect:"+Global.getAdminPath()+"/activityassignment/activityAssignment/?repage";
	}
	
	/**
	 * 导出excel文件
	 */
	@RequiresPermissions("activityassignment:activityAssignment:export")
    @RequestMapping(value = "export", method=RequestMethod.POST)
    public String exportFile(ActivityAssignment activityAssignment, HttpServletRequest request, HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "工作流任务委托"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<ActivityAssignment> page = activityAssignmentService.findPage(new Page<ActivityAssignment>(request, response, -1), activityAssignment);
    		new ExportExcel("工作流任务委托", ActivityAssignment.class).setDataList(page.getList()).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导出工作流任务委托记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/activityassignment/activityAssignment/?repage";
    }

	/**
	 * 导入Excel数据

	 */
	@RequiresPermissions("activityassignment:activityAssignment:import")
    @RequestMapping(value = "import", method=RequestMethod.POST)
    public String importFile(MultipartFile file, RedirectAttributes redirectAttributes) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<ActivityAssignment> list = ei.getDataList(ActivityAssignment.class);
			for (ActivityAssignment activityAssignment : list){
				try{
					activityAssignmentService.save(activityAssignment);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条工作流任务委托记录。");
			}
			addMessage(redirectAttributes, "已成功导入 "+successNum+" 条工作流任务委托记录"+failureMsg);
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入工作流任务委托失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/activityassignment/activityAssignment/?repage";
    }
	
	/**
	 * 下载导入工作流任务委托数据模板
	 */
	@RequiresPermissions("activityassignment:activityAssignment:import")
    @RequestMapping(value = "import/template")
    public String importFileTemplate(HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "工作流任务委托数据导入模板.xlsx";
    		List<ActivityAssignment> list = Lists.newArrayList(); 
    		new ExportExcel("工作流任务委托数据", ActivityAssignment.class, 1).setDataList(list).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模板下载失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/activityassignment/activityAssignment/?repage";
    }
	
	
	

}