/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.contractclient.web;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;

import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.google.common.collect.Lists;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.MyBeanUtils;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.modules.contractclient.entity.WorkContractClient;
import com.jeeplus.modules.contractclient.service.WorkContractClientService;

/**
 * 合同涉及客户列表Controller
 * @author lw
 * @version 2018-06-05
 */
@Controller
@RequestMapping(value = "${adminPath}/contractclient/workContractClient")
public class WorkContractClientController extends BaseController {

	@Autowired
	private WorkContractClientService workContractClientService;
	
	@ModelAttribute
	public WorkContractClient get(@RequestParam(required=false) String id) {
		WorkContractClient entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = workContractClientService.get(id);
		}
		if (entity == null){
			entity = new WorkContractClient();
		}
		return entity;
	}
	
	/**
	 * 合同涉及客户列表列表页面
	 */
	@RequiresPermissions("contractclient:workContractClient:list")
	@RequestMapping(value = {"list", ""})
	public String list(WorkContractClient workContractClient, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<WorkContractClient> page = workContractClientService.findPage(new Page<WorkContractClient>(request, response), workContractClient); 
		model.addAttribute("page", page);
		return "modules/contractclient/workContractClientList";
	}

	/**
	 * 查看，增加，编辑合同涉及客户列表表单页面
	 */
	@RequiresPermissions(value={"contractclient:workContractClient:view","contractclient:workContractClient:add","contractclient:workContractClient:edit"},logical=Logical.OR)
	@RequestMapping(value = "form")
	public String form(WorkContractClient workContractClient, Model model) {
		model.addAttribute("workContractClient", workContractClient);
		return "modules/contractclient/workContractClientForm";
	}

	/**
	 * 保存合同涉及客户列表
	 */
	@RequiresPermissions(value={"contractclient:workContractClient:add","contractclient:workContractClient:edit"},logical=Logical.OR)
	@RequestMapping(value = "save")
	public String save(WorkContractClient workContractClient, Model model, RedirectAttributes redirectAttributes) throws Exception{
		if (!beanValidator(model, workContractClient)){
			return form(workContractClient, model);
		}
		if(!workContractClient.getIsNewRecord()){//编辑表单保存
			WorkContractClient t = workContractClientService.get(workContractClient.getId());//从数据库取出记录的值
			MyBeanUtils.copyBeanNotNull2Bean(workContractClient, t);//将编辑表单中的非NULL值覆盖数据库记录中的值
			workContractClientService.save(t);//保存
		}else{//新增表单保存
			workContractClientService.save(workContractClient);//保存
		}
		addMessage(redirectAttributes, "保存合同涉及客户列表成功");
		return "redirect:"+Global.getAdminPath()+"/contractclient/workContractClient/?repage";
	}
	
	/**
	 * 删除合同涉及客户列表
	 */
	@RequiresPermissions("contractclient:workContractClient:del")
	@RequestMapping(value = "delete")
	public String delete(WorkContractClient workContractClient, RedirectAttributes redirectAttributes) {
		workContractClientService.delete(workContractClient);
		addMessage(redirectAttributes, "删除合同涉及客户列表成功");
		return "redirect:"+Global.getAdminPath()+"/contractclient/workContractClient/?repage";
	}
	
	/**
	 * 批量删除合同涉及客户列表
	 */
	@RequiresPermissions("contractclient:workContractClient:del")
	@RequestMapping(value = "deleteAll")
	public String deleteAll(String ids, RedirectAttributes redirectAttributes) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			workContractClientService.delete(workContractClientService.get(id));
		}
		addMessage(redirectAttributes, "删除合同涉及客户列表成功");
		return "redirect:"+Global.getAdminPath()+"/contractclient/workContractClient/?repage";
	}
	
	/**
	 * 导出excel文件
	 */
	@RequiresPermissions("contractclient:workContractClient:export")
    @RequestMapping(value = "export", method=RequestMethod.POST)
    public String exportFile(WorkContractClient workContractClient, HttpServletRequest request, HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "合同涉及客户列表"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<WorkContractClient> page = workContractClientService.findPage(new Page<WorkContractClient>(request, response, -1), workContractClient);
    		new ExportExcel("合同涉及客户列表", WorkContractClient.class).setDataList(page.getList()).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导出合同涉及客户列表记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/contractclient/workContractClient/?repage";
    }

	/**
	 * 导入Excel数据

	 */
	@RequiresPermissions("contractclient:workContractClient:import")
    @RequestMapping(value = "import", method=RequestMethod.POST)
    public String importFile(MultipartFile file, RedirectAttributes redirectAttributes) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<WorkContractClient> list = ei.getDataList(WorkContractClient.class);
			for (WorkContractClient workContractClient : list){
				try{
					workContractClientService.save(workContractClient);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条合同涉及客户列表记录。");
			}
			addMessage(redirectAttributes, "已成功导入 "+successNum+" 条合同涉及客户列表记录"+failureMsg);
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入合同涉及客户列表失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/contractclient/workContractClient/?repage";
    }
	
	/**
	 * 下载导入合同涉及客户列表数据模板
	 */
	@RequiresPermissions("contractclient:workContractClient:import")
    @RequestMapping(value = "import/template")
    public String importFileTemplate(HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "合同涉及客户列表数据导入模板.xlsx";
    		List<WorkContractClient> list = Lists.newArrayList(); 
    		new ExportExcel("合同涉及客户列表数据", WorkContractClient.class, 1).setDataList(list).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模板下载失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/contractclient/workContractClient/?repage";
    }
	
	
	

}