/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.officehonor.web;

import java.io.*;
import java.net.URLDecoder;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;

import com.jeeplus.common.oss.OSSClientUtil;
import com.jeeplus.modules.sys.utils.UserUtils;
import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.jeeplus.modules.sys.entity.Office;
import com.google.common.collect.Lists;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.MyBeanUtils;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.modules.officehonor.entity.Officehonor;
import com.jeeplus.modules.officehonor.service.OfficehonorService;

/**
 * 企业荣誉Controller
 * @author liuw
 * @version 2017-12-05
 */
@Controller
@RequestMapping(value = "${adminPath}/officehonor/officehonor")
public class OfficehonorController extends BaseController {

	@Autowired
	private OfficehonorService officehonorService;
	
	@ModelAttribute
	public Officehonor get(@RequestParam(required=false) String id) {
		Officehonor entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = officehonorService.get(id);
		}
		if (entity == null){
			entity = new Officehonor();
		}
		return entity;
	}
	
	/**
	 * 企业荣誉列表页面
	 */
	@RequiresPermissions("officehonor:officehonor:list")
	@RequestMapping(value = {"list", ""})
	public String list(Officehonor officehonor, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<Officehonor> page = officehonorService.findPage(new Page<Officehonor>(request, response), officehonor);
		model.addAttribute("page", page);
		return "modules/officehonor/officehonorList";
	}

	/**
	 * 查看，增加，编辑企业荣誉表单页面
	 */
	@RequiresPermissions(value={"officehonor:officehonor:view","officehonor:officehonor:add","officehonor:officehonor:edit"},logical=Logical.OR)
	@RequestMapping(value = "form")
	public String form(Officehonor officehonor, Model model,HttpServletRequest request) {
		if(StringUtils.isBlank(officehonor.getId())){
			Office office = UserUtils.getSelectOffice();
			officehonor.setCompanyName(office.getParent().getName());
			officehonor.setUploadDate(new Date());
		}
		model.addAttribute("officehonor", officehonor);
		String view=request.getParameter("tabId");
		if(StringUtils.isNotBlank(view) && view.equals("view")){
			return "modules/officehonor/officehonorView";
		}
		return "modules/officehonor/officehonorForm";
	}

	/**
	 * 保存企业荣誉
	 */
	@RequiresPermissions(value={"officehonor:officehonor:add","officehonor:officehonor:edit"},logical=Logical.OR)
	@RequestMapping(value = "save")
	public String save(Officehonor officehonor, Model model,HttpServletRequest request, RedirectAttributes redirectAttributes) throws Exception{
		if (!beanValidator(model, officehonor)){
			return form(officehonor, model,request);
		}
		if(!officehonor.getIsNewRecord()){//编辑表单保存
			Officehonor t = officehonorService.get(officehonor.getId());//从数据库取出记录的值
			MyBeanUtils.copyBeanNotNull2Bean(officehonor, t);//将编辑表单中的非NULL值覆盖数据库记录中的值
			officehonorService.save(t);//保存
		}else{//新增表单保存
			/*Office office = UserUtils.getSelectCompany();
			officehonor.setOffice(office);//当前用户所在企业*/
			officehonor.setCompanyId(UserUtils.getSelectCompany().getId());
			officehonorService.save(officehonor);//保存
		}
		addMessage(redirectAttributes, "保存企业荣誉成功");
		return "redirect:"+Global.getAdminPath()+"/officehonor/officehonor/?repage";
	}
	
	/**
	 * 删除企业荣誉
	 */
	@RequiresPermissions("officehonor:officehonor:del")
	@RequestMapping(value = "delete")
	public String delete(Officehonor officehonor, RedirectAttributes redirectAttributes) {
		officehonorService.delete(officehonor);
		addMessage(redirectAttributes, "删除企业荣誉成功");
		return "redirect:"+Global.getAdminPath()+"/officehonor/officehonor/?repage";
	}
	
	/**
	 * 批量删除企业荣誉
	 */
	@RequiresPermissions("officehonor:officehonor:del")
	@RequestMapping(value = "deleteAll")
	public String deleteAll(String ids, RedirectAttributes redirectAttributes) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			officehonorService.delete(officehonorService.get(id));
		}
		addMessage(redirectAttributes, "删除企业荣誉成功");
		return "redirect:"+Global.getAdminPath()+"/officehonor/officehonor/?repage";
	}
	
	/**
	 * 导出excel文件
	 */
	@RequiresPermissions("officehonor:officehonor:export")
    @RequestMapping(value = "export", method=RequestMethod.POST)
    public String exportFile(Officehonor officehonor, HttpServletRequest request, HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "企业荣誉"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<Officehonor> page = officehonorService.findPage(new Page<Officehonor>(request, response, -1), officehonor);
    		new ExportExcel("企业荣誉", Officehonor.class).setDataList(page.getList()).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导出企业荣誉记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/officehonor/officehonor/?repage";
    }

	/**
	 * 导入Excel数据

	 */
	@RequiresPermissions("officehonor:officehonor:import")
    @RequestMapping(value = "import", method=RequestMethod.POST)
    public String importFile(MultipartFile file, RedirectAttributes redirectAttributes) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<Officehonor> list = ei.getDataList(Officehonor.class);
			for (Officehonor officehonor : list){
				try{
					officehonorService.save(officehonor);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条企业荣誉记录。");
			}
			addMessage(redirectAttributes, "已成功导入 "+successNum+" 条企业荣誉记录"+failureMsg);
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入企业荣誉失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/officehonor/officehonor/?repage";
    }
	
	/**
	 * 下载导入企业荣誉数据模板
	 */
	@RequiresPermissions("officehonor:officehonor:import")
    @RequestMapping(value = "import/template")
    public String importFileTemplate(HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "企业荣誉数据导入模板.xlsx";
    		List<Officehonor> list = Lists.newArrayList(); 
    		new ExportExcel("企业荣誉数据", Officehonor.class, 1).setDataList(list).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模板下载失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/officehonor/officehonor/?repage";
    }
	
	
	/**
	 * 选择企业外键
	 */
	@RequestMapping(value = "selectoffice")
	public String selectoffice(Office office, String url, String fieldLabels, String fieldKeys, String searchLabel, String searchKey, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<Office> page = officehonorService.findPageByoffice(new Page<Office>(request, response),  office);
		try {
			fieldLabels = URLDecoder.decode(fieldLabels, "UTF-8");
			fieldKeys = URLDecoder.decode(fieldKeys, "UTF-8");
			searchLabel = URLDecoder.decode(searchLabel, "UTF-8");
			searchKey = URLDecoder.decode(searchKey, "UTF-8");
		} catch (UnsupportedEncodingException e) {
			e.printStackTrace();
		}
		model.addAttribute("labelNames", fieldLabels.split("\\|"));
		model.addAttribute("labelValues", fieldKeys.split("\\|"));
		model.addAttribute("fieldLabels", fieldLabels);
		model.addAttribute("fieldKeys", fieldKeys);
		model.addAttribute("url", url);
		model.addAttribute("searchLabel", searchLabel);
		model.addAttribute("searchKey", searchKey);
		model.addAttribute("obj", office);
		model.addAttribute("page", page);
		return "modules/sys/gridselect";
	}
	//--- 上传图片
	private void upload_image(MultipartFile[] next_upload_files, HttpServletRequest request, StringBuilder office_himage){
		String  next_uploadFolder = request.getSession().getServletContext().getRealPath("local_upload/office_himage/"+System.nanoTime());
		new File(next_uploadFolder).mkdirs();
		List<String> next_imgList = new ArrayList<>();
		if(next_upload_files!=null && next_upload_files.length>0){
			for(int i=0;i<next_upload_files.length;i++){
				if(next_upload_files[i]==null || next_upload_files[i].getSize()==0)
					continue;
				String imageName = System.nanoTime()+next_upload_files[i].getOriginalFilename();
				String imageNewName = next_uploadFolder+"/"+imageName;
				next_imgList.add(imageNewName);
				try{
					next_upload_files[i].transferTo(new File(imageNewName));
				}catch (IllegalStateException | IOException e) {
					e.printStackTrace();
				}
			}
		}

		InputStream next_inputStream = null;
		for(int i=0;i<next_imgList.size();i++) {
			String filePath = next_imgList.get(i);
			try {
				next_inputStream = new FileInputStream(new File(filePath));
				OSSClientUtil ossUtil = new OSSClientUtil();
				String suffix = filePath.substring(filePath.lastIndexOf("."));
				String name = System.nanoTime() + suffix;
				ossUtil.uploadFile2OSS(next_inputStream, Global.getLogo(), name);
				office_himage.append(Global.getAliyunUrl() + "/" + Global.getLogo() + name + ",");
			} catch (IOException e) {
				e.printStackTrace();
			} finally {
				try {
					if (next_inputStream != null)
						next_inputStream.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		}
	}
}