/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.officeintroduce.web;

import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.Date;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;

import com.jeeplus.modules.sys.utils.UserUtils;
import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.jeeplus.modules.sys.entity.Office;
import com.google.common.collect.Lists;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.MyBeanUtils;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.modules.officeintroduce.entity.Officeintroduce;
import com.jeeplus.modules.officeintroduce.service.OfficeintroduceService;

/**
 * 企业介绍Controller
 * @author liuw
 * @version 2017-12-05
 */
@Controller
@RequestMapping(value = "${adminPath}/officeintroduce/officeintroduce")
public class OfficeintroduceController extends BaseController {

	@Autowired
	private OfficeintroduceService officeintroduceService;
	
	@ModelAttribute
	public Officeintroduce get(@RequestParam(required=false) String id) {
		Officeintroduce entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = officeintroduceService.get(id);
		}
		if (entity == null){
			entity = new Officeintroduce();
		}
		return entity;
	}
	
	/**
	 * 企业介绍列表页面
	 */
	@RequiresPermissions("officeintroduce:officeintroduce:list")
	@RequestMapping(value = {"list", ""})
	public String list(Officeintroduce officeintroduce, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<Officeintroduce> page = officeintroduceService.findPage(new Page<Officeintroduce>(request, response), officeintroduce);
		model.addAttribute("page", page);
		return "modules/officeintroduce/officeintroduceList";
	}

	/**
	 * 查看，增加，编辑企业介绍表单页面
	 */
	@RequiresPermissions(value={"officeintroduce:officeintroduce:view","officeintroduce:officeintroduce:add","officeintroduce:officeintroduce:edit"},logical=Logical.OR)
	@RequestMapping(value = "form")
	public String form(Officeintroduce officeintroduce, Model model,HttpServletRequest request) {

		if(StringUtils.isBlank(officeintroduce.getId())){
			Office office = UserUtils.getSelectOffice();
			officeintroduce.setCompanyName(office.getParent().getName());
			officeintroduce.setUploadDate(new Date());
		}
		model.addAttribute("officeintroduce", officeintroduce);
		String view=request.getParameter("tabId");
		if(StringUtils.isNotBlank(view) && view.equals("view")){
			return "modules/officeintroduce/officeintroduceView";
		}
		return "modules/officeintroduce/officeintroduceForm";
	}

	/**
	 * 保存企业介绍
	 */
	@RequiresPermissions(value={"officeintroduce:officeintroduce:add","officeintroduce:officeintroduce:edit"},logical=Logical.OR)
	@RequestMapping(value = "save")
	public String save(Officeintroduce officeintroduce, Model model, RedirectAttributes redirectAttributes,HttpServletRequest request) throws Exception{
		if (!beanValidator(model, officeintroduce)){
			return form(officeintroduce, model,request);
		}
		if(!officeintroduce.getIsNewRecord()){//编辑表单保存
			Officeintroduce t = officeintroduceService.get(officeintroduce.getId());//从数据库取出记录的值
			MyBeanUtils.copyBeanNotNull2Bean(officeintroduce, t);//将编辑表单中的非NULL值覆盖数据库记录中的值
			officeintroduceService.save(t);//保存
		}else{//新增表单保存
			officeintroduce.setCompanyId(UserUtils.getSelectCompany().getId());
			officeintroduceService.save(officeintroduce);//保存
		}
		addMessage(redirectAttributes, "保存企业介绍成功");
		return "redirect:"+Global.getAdminPath()+"/officeintroduce/officeintroduce/?repage";
	}
	
	/**
	 * 删除企业介绍
	 */
	@RequiresPermissions("officeintroduce:officeintroduce:del")
	@RequestMapping(value = "delete")
	public String delete(Officeintroduce officeintroduce, RedirectAttributes redirectAttributes) {
		officeintroduceService.delete(officeintroduce);
		addMessage(redirectAttributes, "删除企业介绍成功");
		return "redirect:"+Global.getAdminPath()+"/officeintroduce/officeintroduce/?repage";
	}
	
	/**
	 * 批量删除企业介绍
	 */
	@RequiresPermissions("officeintroduce:officeintroduce:del")
	@RequestMapping(value = "deleteAll")
	public String deleteAll(String ids, RedirectAttributes redirectAttributes) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			officeintroduceService.delete(officeintroduceService.get(id));
		}
		addMessage(redirectAttributes, "删除企业介绍成功");
		return "redirect:"+Global.getAdminPath()+"/officeintroduce/officeintroduce/?repage";
	}
	
	/**
	 * 导出excel文件
	 */
	@RequiresPermissions("officeintroduce:officeintroduce:export")
    @RequestMapping(value = "export", method=RequestMethod.POST)
    public String exportFile(Officeintroduce officeintroduce, HttpServletRequest request, HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "企业介绍"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<Officeintroduce> page = officeintroduceService.findPage(new Page<Officeintroduce>(request, response, -1), officeintroduce);
    		new ExportExcel("企业介绍", Officeintroduce.class).setDataList(page.getList()).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导出企业介绍记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/officeintroduce/officeintroduce/?repage";
    }

	/**
	 * 导入Excel数据

	 */
	@RequiresPermissions("officeintroduce:officeintroduce:import")
    @RequestMapping(value = "import", method=RequestMethod.POST)
    public String importFile(MultipartFile file, RedirectAttributes redirectAttributes) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<Officeintroduce> list = ei.getDataList(Officeintroduce.class);
			for (Officeintroduce officeintroduce : list){
				try{
					officeintroduceService.save(officeintroduce);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条企业介绍记录。");
			}
			addMessage(redirectAttributes, "已成功导入 "+successNum+" 条企业介绍记录"+failureMsg);
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入企业介绍失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/officeintroduce/officeintroduce/?repage";
    }
	
	/**
	 * 下载导入企业介绍数据模板
	 */
	@RequiresPermissions("officeintroduce:officeintroduce:import")
    @RequestMapping(value = "import/template")
    public String importFileTemplate(HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "企业介绍数据导入模板.xlsx";
    		List<Officeintroduce> list = Lists.newArrayList(); 
    		new ExportExcel("企业介绍数据", Officeintroduce.class, 1).setDataList(list).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模板下载失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/officeintroduce/officeintroduce/?repage";
    }
	
	
	/**
	 * 选择企业外键
	 */
	@RequestMapping(value = "selectoffice")
	public String selectoffice(Office office, String url, String fieldLabels, String fieldKeys, String searchLabel, String searchKey, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<Office> page = officeintroduceService.findPageByoffice(new Page<Office>(request, response),  office);
		try {
			fieldLabels = URLDecoder.decode(fieldLabels, "UTF-8");
			fieldKeys = URLDecoder.decode(fieldKeys, "UTF-8");
			searchLabel = URLDecoder.decode(searchLabel, "UTF-8");
			searchKey = URLDecoder.decode(searchKey, "UTF-8");
		} catch (UnsupportedEncodingException e) {
			e.printStackTrace();
		}
		model.addAttribute("labelNames", fieldLabels.split("\\|"));
		model.addAttribute("labelValues", fieldKeys.split("\\|"));
		model.addAttribute("fieldLabels", fieldLabels);
		model.addAttribute("fieldKeys", fieldKeys);
		model.addAttribute("url", url);
		model.addAttribute("searchLabel", searchLabel);
		model.addAttribute("searchKey", searchKey);
		model.addAttribute("obj", office);
		model.addAttribute("page", page);
		return "modules/sys/gridselect";
	}
	

}