/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.project.web;

import com.google.common.collect.Lists;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.MyBeanUtils;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.modules.project.entity.ProjectDepositManagement;
import com.jeeplus.modules.project.service.ProjectDepositManagementService;
import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;
import java.util.List;

/**
 * 保证金管理Controller
 * @author yangfan
 * @version 2017-10-11
 */
@Controller
@RequestMapping(value = "${adminPath}/project/projectDepositReturn")
public class ProjectDepositReturnController extends BaseController {

	@Autowired
	private ProjectDepositManagementService projectDepositManagementService;
	
	@ModelAttribute
	public ProjectDepositManagement get(@RequestParam(required=false) String id) {
		ProjectDepositManagement entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = projectDepositManagementService.get(id);
		}
		if (entity == null){
			entity = new ProjectDepositManagement();
		}
		return entity;
	}

	/**
	 * 保证金管理列表页面
	 */
	@RequiresPermissions("project:projectDepositReturn:list")
	@RequestMapping(value = {"list", ""})
	public String list(ProjectDepositManagement projectDepositManagement, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<ProjectDepositManagement> page = projectDepositManagementService.findPage(new Page<ProjectDepositManagement>(request, response), projectDepositManagement); 
		model.addAttribute("page", page);
		return "modules/project/projectDepositReturnList";
	}

	/**
	 * 查看，增加，编辑保证金管理表单页面
	 */
	@RequiresPermissions(value={"project:projectDepositReturn:view","project:projectDepositReturn:add","project:projectDepositReturn:edit"},logical=Logical.OR)
	@RequestMapping(value = "form")
	public String form(ProjectDepositManagement projectDepositManagement, Model model) {
		model.addAttribute("projectDepositManagement", projectDepositManagement);
		return "modules/project/projectDepositReturnForm";
	}

	/**
	 * 保存保证金管理
	 */
	@RequiresPermissions(value={"project:projectDepositReturn:add","project:projectDepositReturn:edit"},logical=Logical.OR)
	@RequestMapping(value = "save")
	public String save(ProjectDepositManagement projectDepositManagement, Model model, RedirectAttributes redirectAttributes) throws Exception{
		if (!beanValidator(model, projectDepositManagement)){
			return form(projectDepositManagement, model);
		}
		if(!projectDepositManagement.getIsNewRecord()){//编辑表单保存
			ProjectDepositManagement t = projectDepositManagementService.get(projectDepositManagement.getId());//从数据库取出记录的值
			MyBeanUtils.copyBeanNotNull2Bean(projectDepositManagement, t);//将编辑表单中的非NULL值覆盖数据库记录中的值
			projectDepositManagementService.save(t);//保存
		}else{//新增表单保存
			projectDepositManagementService.save(projectDepositManagement);//保存
		}
		addMessage(redirectAttributes, "保存保证金管理成功");
		return "redirect:"+Global.getAdminPath()+"/project/projectDepositReturn/?repage";
	}
	
	/**
	 * 删除保证金管理
	 */
	@RequiresPermissions("project:projectDepositReturn:del")
	@RequestMapping(value = "delete")
	public String delete(ProjectDepositManagement projectDepositManagement, RedirectAttributes redirectAttributes) {
		projectDepositManagementService.delete(projectDepositManagement);
		addMessage(redirectAttributes, "删除保证金管理成功");
		return "redirect:"+Global.getAdminPath()+"/project/projectDepositReturn/?repage";
	}
	
	/**
	 * 批量删除保证金管理
	 */
	@RequiresPermissions("project:projectDepositReturn:del")
	@RequestMapping(value = "deleteAll")
	public String deleteAll(String ids, RedirectAttributes redirectAttributes) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			projectDepositManagementService.delete(projectDepositManagementService.get(id));
		}
		addMessage(redirectAttributes, "删除保证金管理成功");
		return "redirect:"+Global.getAdminPath()+"/project/projectDepositReturn/?repage";
	}
	
	/**
	 * 导出excel文件
	 */
	@RequiresPermissions("project:projectDepositReturn:export")
    @RequestMapping(value = "export", method=RequestMethod.POST)
    public String exportFile(ProjectDepositManagement projectDepositManagement, HttpServletRequest request, HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "保证金管理"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<ProjectDepositManagement> page = projectDepositManagementService.findPage(new Page<ProjectDepositManagement>(request, response, -1), projectDepositManagement);
    		new ExportExcel("保证金管理", ProjectDepositManagement.class).setDataList(page.getList()).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导出保证金管理记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/project/projectDepositReturn/?repage";
    }

	/**
	 * 导入Excel数据

	 */
	@RequiresPermissions("project:projectDepositReturn:import")
    @RequestMapping(value = "import", method=RequestMethod.POST)
    public String importFile(MultipartFile file, RedirectAttributes redirectAttributes) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<ProjectDepositManagement> list = ei.getDataList(ProjectDepositManagement.class);
			for (ProjectDepositManagement projectDepositManagement : list){
				try{
					projectDepositManagementService.save(projectDepositManagement);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条保证金管理记录。");
			}
			addMessage(redirectAttributes, "已成功导入 "+successNum+" 条保证金管理记录"+failureMsg);
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入保证金管理失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/project/projectDepositReturn/?repage";
    }
	
	/**
	 * 下载导入保证金管理数据模板
	 */
	@RequiresPermissions("project:projectDepositReturn:import")
    @RequestMapping(value = "import/template")
    public String importFileTemplate(HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "保证金管理数据导入模板.xlsx";
    		List<ProjectDepositManagement> list = Lists.newArrayList(); 
    		new ExportExcel("保证金管理数据", ProjectDepositManagement.class, 1).setDataList(list).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模板下载失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/project/projectDepositReturn/?repage";
    }
	
	
	

}