/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.projectcontentinfo.web;

import com.google.common.collect.Lists;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.json.AjaxJson;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.MyBeanUtils;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.modules.projectcontentinfo.entity.ProjectContentData;
import com.jeeplus.modules.projectcontentinfo.service.ProjectContentDataService;
import com.jeeplus.modules.workreview.entity.WorkReviewAudit;
import com.jeeplus.modules.workreview.entity.WorkReviewStandard;
import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.List;

/**
 * 工作内容详情Controller
 * @author yangfan
 * @version 2018-06-05
 */
@Controller
@RequestMapping(value = "${adminPath}/projectcontentinfo/projectContentData")
public class ProjectContentDataController extends BaseController {

	@Autowired
	private ProjectContentDataService projectContentDataService;
	
	@ModelAttribute
	public ProjectContentData get(@RequestParam(required=false) String id) {
		ProjectContentData entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = projectContentDataService.get(id);
		}
		if (entity == null){
			entity = new ProjectContentData();
		}
		return entity;
	}
	
	/**
	 * 工作内容详情列表页面
	 */
	@RequiresPermissions("projectcontentinfo:projectContentData:list")
	@RequestMapping(value = {"list", ""})
	public String list(ProjectContentData projectContentData, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<ProjectContentData> page = projectContentDataService.findPage(new Page<ProjectContentData>(request, response), projectContentData); 
		model.addAttribute("page", page);
		return "modules/projectcontentinfo/projectContentDataList";
	}

	/**
	 * 查看，增加，编辑工作内容详情表单页面
	 */
	@RequiresPermissions(value={"projectcontentinfo:projectContentData:view","projectcontentinfo:projectContentData:add","projectcontentinfo:projectContentData:edit"},logical=Logical.OR)
	@RequestMapping(value = "form")
	public String form(ProjectContentData projectContentData, HttpServletRequest request,Model model) {
		String view = request.getParameter("view");
		if("control".equals(view)){
			view = "modules/projectcontroltable/projectControlTableForm";
		}else{
			view = "modules/projectcontentinfo/projectContentDataForm";
		}
		model.addAttribute("projectContentData", projectContentData);
		return "modules/projectcontentinfo/projectContentDataForm";
	}

	/**
	 * 保存工作内容详情
	 */
	@RequiresPermissions(value={"projectcontentinfo:projectContentData:add","projectcontentinfo:projectContentData:edit"},logical=Logical.OR)
	@RequestMapping(value = "save")
	public String save(ProjectContentData projectContentData, HttpServletRequest request,Model model, RedirectAttributes redirectAttributes) throws Exception{
		if (!beanValidator(model, projectContentData)){
			return form(projectContentData, request,model);
		}
		if(!projectContentData.getIsNewRecord()){//编辑表单保存
			ProjectContentData t = projectContentDataService.get(projectContentData.getId());//从数据库取出记录的值
			MyBeanUtils.copyBeanNotNull2Bean(projectContentData, t);//将编辑表单中的非NULL值覆盖数据库记录中的值
			projectContentDataService.save(t);//保存
		}else{//新增表单保存
			projectContentDataService.save(projectContentData);//保存
		}
		addMessage(redirectAttributes, "保存工作内容详情成功");
		return "redirect:"+Global.getAdminPath()+"/projectcontentinfo/projectContentData/?repage";
	}
	
	/**
	 * 删除工作内容详情
	 */
	@RequiresPermissions("projectcontentinfo:projectContentData:del")
	@RequestMapping(value = "delete")
	public String delete(ProjectContentData projectContentData, RedirectAttributes redirectAttributes) {
		projectContentDataService.delete(projectContentData);
		addMessage(redirectAttributes, "删除工作内容详情成功");
		return "redirect:"+Global.getAdminPath()+"/projectcontentinfo/projectContentData/?repage";
	}
	
	/**
	 * 批量删除工作内容详情
	 */
	@RequiresPermissions("projectcontentinfo:projectContentData:del")
	@RequestMapping(value = "deleteAll")
	public String deleteAll(String ids, RedirectAttributes redirectAttributes) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			projectContentDataService.delete(projectContentDataService.get(id));
		}
		addMessage(redirectAttributes, "删除工作内容详情成功");
		return "redirect:"+Global.getAdminPath()+"/projectcontentinfo/projectContentData/?repage";
	}
	
	/**
	 * 导出excel文件
	 */
	@RequiresPermissions("projectcontentinfo:projectContentData:export")
    @RequestMapping(value = "export", method=RequestMethod.POST)
    public String exportFile(ProjectContentData projectContentData, HttpServletRequest request, HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "工作内容详情"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<ProjectContentData> page = projectContentDataService.findPage(new Page<ProjectContentData>(request, response, -1), projectContentData);
    		new ExportExcel("工作内容详情", ProjectContentData.class).setDataList(page.getList()).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导出工作内容详情记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/projectcontentinfo/projectContentData/?repage";
    }

	/**
	 * 导入Excel数据

	 */
	@RequiresPermissions("projectcontentinfo:projectContentData:import")
    @RequestMapping(value = "import", method=RequestMethod.POST)
    public String importFile(MultipartFile file, RedirectAttributes redirectAttributes) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<ProjectContentData> list = ei.getDataList(ProjectContentData.class);
			for (ProjectContentData projectContentData : list){
				try{
					projectContentDataService.save(projectContentData);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条工作内容详情记录。");
			}
			addMessage(redirectAttributes, "已成功导入 "+successNum+" 条工作内容详情记录"+failureMsg);
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入工作内容详情失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/projectcontentinfo/projectContentData/?repage";
    }
	
	/**
	 * 下载导入工作内容详情数据模板
	 */
	@RequiresPermissions("projectcontentinfo:projectContentData:import")
    @RequestMapping(value = "import/template")
    public String importFileTemplate(HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "工作内容详情数据导入模板.xlsx";
    		List<ProjectContentData> list = Lists.newArrayList(); 
    		new ExportExcel("工作内容详情数据", ProjectContentData.class, 1).setDataList(list).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模板下载失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/projectcontentinfo/projectContentData/?repage";
    }

    @RequestMapping("selectcontent")
	public String gridSelect(ProjectContentData projectContentData, String searchLabel ,HttpServletRequest request, HttpServletResponse response, Model model) throws UnsupportedEncodingException {
        Page<ProjectContentData> page = projectContentDataService.findPage(new Page<ProjectContentData>(request, response), projectContentData);
        model.addAttribute("obj", projectContentData);
        model.addAttribute("page", page);
		searchLabel = URLDecoder.decode(searchLabel, "UTF-8");
        model.addAttribute("searchLabel",searchLabel);
	    return "modules/sys/gridselectProjcontent";
    }
    @RequestMapping("selectcontentData")
	public String gridSelectData(ProjectContentData projectContentData, String searchLabel ,HttpServletRequest request, HttpServletResponse response, Model model) throws UnsupportedEncodingException {
        Page<ProjectContentData> page = projectContentDataService.findPage(new Page<ProjectContentData>(request, response), projectContentData);
        model.addAttribute("obj", projectContentData);
        model.addAttribute("page", page);
		searchLabel = URLDecoder.decode(searchLabel, "UTF-8");
        model.addAttribute("searchLabel",searchLabel);
	    return "modules/sys/gridselectProjcontentData";
    }


    @RequestMapping("ajaxdelete")
    @ResponseBody
    public AjaxJson deleteBased(String contentId,String basedId){
        AjaxJson ajaxJson = new AjaxJson();
        try {
            if (StringUtils.isNotBlank(contentId)) {
                projectContentDataService.deleteBased(contentId, basedId);
            }
            Integer count = projectContentDataService.countBased(basedId);
            ajaxJson.getBody().put("inuse", count == null ? true : count > 0);
        }catch (Exception e){
            logger.error("删除依据资料异常!",e);
            ajaxJson.setSuccess(false);
            ajaxJson.setMsg("删除依据资料失败");
        }
        return  ajaxJson;
    }
    @RequestMapping("addReview")
    @ResponseBody
    public AjaxJson addReview(String reviewId,String name ,String infoId){
        AjaxJson ajaxJson = new AjaxJson();
        try {
			List<WorkReviewAudit> workReviewAudits = projectContentDataService.findListReviewAudit(reviewId,name,infoId);
			ajaxJson.getBody().put("list",workReviewAudits);
			ajaxJson.setMsg("获取数据成功");
        }catch (Exception e){
            logger.error("获取数据异常!",e);
            ajaxJson.setSuccess(false);
            ajaxJson.setMsg("获取数据异常");
        }
        return  ajaxJson;
    }
}