/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.pushinfo.web;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;

import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.google.common.collect.Lists;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.MyBeanUtils;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.modules.pushinfo.entity.Pushinfo;
import com.jeeplus.modules.pushinfo.service.PushinfoService;

/**
 * 测试Controller
 * @author yangfan
 * @version 2017-07-28
 */
@Controller
@RequestMapping(value = "${adminPath}/pushinfo/pushinfo")
public class PushinfoController extends BaseController {

	@Autowired
	private PushinfoService pushinfoService;
	
	@ModelAttribute
	public Pushinfo get(@RequestParam(required=false) String id) {
		Pushinfo entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = pushinfoService.get(id);
		}
		if (entity == null){
			entity = new Pushinfo();
		}
		return entity;
	}
	
	/**
	 * 推送信息列表页面
	 */
	@RequiresPermissions("pushinfo:pushinfo:list")
	@RequestMapping(value = {"list", ""})
	public String list(Pushinfo pushinfo, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<Pushinfo> page = pushinfoService.findPage(new Page<Pushinfo>(request, response), pushinfo); 
		model.addAttribute("page", page);
		return "modules/pushinfo/pushinfoList";
	}

	/**
	 * 查看，增加，编辑推送信息表单页面
	 */
	@RequiresPermissions(value={"pushinfo:pushinfo:view","pushinfo:pushinfo:add","pushinfo:pushinfo:edit"},logical=Logical.OR)
	@RequestMapping(value = "form")
	public String form(Pushinfo pushinfo, Model model) {
		model.addAttribute("pushinfo", pushinfo);
		return "modules/pushinfo/pushinfoForm";
	}

	/**
	 * 保存推送信息
	 */
	@RequiresPermissions(value={"pushinfo:pushinfo:add","pushinfo:pushinfo:edit"},logical=Logical.OR)
	@RequestMapping(value = "save")
	public String save(Pushinfo pushinfo, Model model, RedirectAttributes redirectAttributes) throws Exception{
		if (!beanValidator(model, pushinfo)){
			return form(pushinfo, model);
		}
		if(!pushinfo.getIsNewRecord()){//编辑表单保存
			Pushinfo t = pushinfoService.get(pushinfo.getId());//从数据库取出记录的值
			MyBeanUtils.copyBeanNotNull2Bean(pushinfo, t);//将编辑表单中的非NULL值覆盖数据库记录中的值
			pushinfoService.save(t);//保存
		}else{//新增表单保存
			pushinfoService.save(pushinfo);//保存
		}
		addMessage(redirectAttributes, "保存推送信息成功");
		return "redirect:"+Global.getAdminPath()+"/pushinfo/pushinfo/?repage";
	}
	
	/**
	 * 删除推送信息
	 */
	@RequiresPermissions("pushinfo:pushinfo:del")
	@RequestMapping(value = "delete")
	public String delete(Pushinfo pushinfo, RedirectAttributes redirectAttributes) {
		pushinfoService.delete(pushinfo);
		addMessage(redirectAttributes, "删除推送信息成功");
		return "redirect:"+Global.getAdminPath()+"/pushinfo/pushinfo/?repage";
	}
	
	/**
	 * 批量删除推送信息
	 */
	@RequiresPermissions("pushinfo:pushinfo:del")
	@RequestMapping(value = "deleteAll")
	public String deleteAll(String ids, RedirectAttributes redirectAttributes) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			pushinfoService.delete(pushinfoService.get(id));
		}
		addMessage(redirectAttributes, "删除推送信息成功");
		return "redirect:"+Global.getAdminPath()+"/pushinfo/pushinfo/?repage";
	}
	
	/**
	 * 导出excel文件
	 */
	@RequiresPermissions("pushinfo:pushinfo:export")
    @RequestMapping(value = "export", method=RequestMethod.POST)
    public String exportFile(Pushinfo pushinfo, HttpServletRequest request, HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "推送信息"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<Pushinfo> page = pushinfoService.findPage(new Page<Pushinfo>(request, response, -1), pushinfo);
    		new ExportExcel("推送信息", Pushinfo.class).setDataList(page.getList()).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导出推送信息记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/pushinfo/pushinfo/?repage";
    }

	/**
	 * 导入Excel数据

	 */
	@RequiresPermissions("pushinfo:pushinfo:import")
    @RequestMapping(value = "import", method=RequestMethod.POST)
    public String importFile(MultipartFile file, RedirectAttributes redirectAttributes) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<Pushinfo> list = ei.getDataList(Pushinfo.class);
			for (Pushinfo pushinfo : list){
				try{
					pushinfoService.save(pushinfo);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条推送信息记录。");
			}
			addMessage(redirectAttributes, "已成功导入 "+successNum+" 条推送信息记录"+failureMsg);
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入推送信息失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/pushinfo/pushinfo/?repage";
    }
	
	/**
	 * 下载导入推送信息数据模板
	 */
	@RequiresPermissions("pushinfo:pushinfo:import")
    @RequestMapping(value = "import/template")
    public String importFileTemplate(HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "推送信息数据导入模板.xlsx";
    		List<Pushinfo> list = Lists.newArrayList(); 
    		new ExportExcel("推送信息数据", Pushinfo.class, 1).setDataList(list).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模板下载失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/pushinfo/pushinfo/?repage";
    }
	
	
	

}