/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.sysimportinfo.web;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;

import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.google.common.collect.Lists;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.MyBeanUtils;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.modules.sysimportinfo.entity.SysImportInfo;
import com.jeeplus.modules.sysimportinfo.service.SysImportInfoService;

/**
 * 导入日志信息Controller
 * @author 杨帆
 * @version 2018-04-02
 */
@Controller
@RequestMapping(value = "${adminPath}/sysimportinfo/sysimportinfo")
public class SysImportInfoController extends BaseController {

	@Autowired
	private SysImportInfoService sysImportInfoService;
	
	@ModelAttribute
	public SysImportInfo get(@RequestParam(required=false) String id) {
		SysImportInfo entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = sysImportInfoService.get(id);
		}
		if (entity == null){
			entity = new SysImportInfo();
		}
		return entity;
	}
	
	/**
	 * 导入日志信息列表页面
	 */
	//@RequiresPermissions("sysimportinfo:sysimportinfo:list")
	@RequestMapping(value = {"list", ""})
	public String list(SysImportInfo sysImportInfo, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<SysImportInfo> page = sysImportInfoService.findPage(new Page<SysImportInfo>(request, response), sysImportInfo);
		model.addAttribute("page", page);
		model.addAttribute("modular", sysImportInfo.getModular());
		return "modules/sysimportinfo/sysimportinfoList";
	}

	/**
	 * 查看，增加，编辑导入日志信息表单页面
	 */
	@RequiresPermissions(value={"sysimportinfo:sysimportinfo:view","sysimportinfo:sysimportinfo:add","sysimportinfo:sysimportinfo:edit"},logical=Logical.OR)
	@RequestMapping(value = "form")
	public String form(SysImportInfo sysImportInfo, Model model) {
		model.addAttribute("sysimportinfo", sysImportInfo);
		return "modules/sysimportinfo/sysimportinfoForm";
	}

	/**
	 * 保存导入日志信息
	 */
	@RequiresPermissions(value={"sysimportinfo:sysimportinfo:add","sysimportinfo:sysimportinfo:edit"},logical=Logical.OR)
	@RequestMapping(value = "save")
	public String save(SysImportInfo sysImportInfo, Model model, RedirectAttributes redirectAttributes) throws Exception{
		if (!beanValidator(model, sysImportInfo)){
			return form(sysImportInfo, model);
		}
		if(!sysImportInfo.getIsNewRecord()){//编辑表单保存
			SysImportInfo t = sysImportInfoService.get(sysImportInfo.getId());//从数据库取出记录的值
			MyBeanUtils.copyBeanNotNull2Bean(sysImportInfo, t);//将编辑表单中的非NULL值覆盖数据库记录中的值
			sysImportInfoService.save(t);//保存
		}else{//新增表单保存
			sysImportInfoService.save(sysImportInfo);//保存
		}
		addMessage(redirectAttributes, "保存导入日志信息成功");
		return "redirect:"+Global.getAdminPath()+"/sysimportinfo/sysimportinfo/?repage";
	}
	
	/**
	 * 删除导入日志信息
	 */
	@RequiresPermissions("sysimportinfo:sysimportinfo:del")
	@RequestMapping(value = "delete")
	public String delete(SysImportInfo sysImportInfo, RedirectAttributes redirectAttributes) {
		sysImportInfoService.delete(sysImportInfo);
		addMessage(redirectAttributes, "删除导入日志信息成功");
		return "redirect:"+Global.getAdminPath()+"/sysimportinfo/sysimportinfo/?repage";
	}
	
	/**
	 * 批量删除导入日志信息
	 */
	@RequiresPermissions("sysimportinfo:sysimportinfo:del")
	@RequestMapping(value = "deleteAll")
	public String deleteAll(String ids, RedirectAttributes redirectAttributes) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			sysImportInfoService.delete(sysImportInfoService.get(id));
		}
		addMessage(redirectAttributes, "删除导入日志信息成功");
		return "redirect:"+Global.getAdminPath()+"/sysimportinfo/sysimportinfo/?repage";
	}
	
	/**
	 * 导出excel文件
	 */
	@RequiresPermissions("sysimportinfo:sysimportinfo:export")
    @RequestMapping(value = "export", method=RequestMethod.POST)
    public String exportFile(SysImportInfo sysImportInfo, HttpServletRequest request, HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "导入日志信息"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<SysImportInfo> page = sysImportInfoService.findPage(new Page<SysImportInfo>(request, response, -1), sysImportInfo);
    		new ExportExcel("导入日志信息", SysImportInfo.class).setDataList(page.getList()).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导出导入日志信息记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/sysimportinfo/sysimportinfo/?repage";
    }

	/**
	 * 导入Excel数据

	 */
	@RequiresPermissions("sysimportinfo:sysimportinfo:import")
    @RequestMapping(value = "import", method=RequestMethod.POST)
    public String importFile(MultipartFile file, RedirectAttributes redirectAttributes) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<SysImportInfo> list = ei.getDataList(SysImportInfo.class);
			for (SysImportInfo sysImportInfo : list){
				try{
					sysImportInfoService.save(sysImportInfo);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条导入日志信息记录。");
			}
			addMessage(redirectAttributes, "已成功导入 "+successNum+" 条导入日志信息记录"+failureMsg);
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入导入日志信息失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/sysimportinfo/sysimportinfo/?repage";
    }
	
	/**
	 * 下载导入导入日志信息数据模板
	 */
	@RequiresPermissions("sysimportinfo:sysimportinfo:import")
    @RequestMapping(value = "import/template")
    public String importFileTemplate(HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "导入日志信息数据导入模板.xlsx";
    		List<SysImportInfo> list = Lists.newArrayList();
    		new ExportExcel("导入日志信息数据", SysImportInfo.class, 1).setDataList(list).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模板下载失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/sysimportinfo/sysimportinfo/?repage";
    }
	
	
	

}