/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.sysmtree.web;

import com.google.common.collect.Lists;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.MyBeanUtils;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.modules.sysmtree.entity.SysMtree;
import com.jeeplus.modules.sysmtree.service.SysMtreeService;
import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;
import java.util.List;

/**
 * 树Controller
 * @author 马鹏波
 * @version 2017-10-22
 */
@Controller
@RequestMapping(value = "${adminPath}/sysmtree/sysMtree")
public class SysMtreeController extends BaseController {

	@Autowired
	private SysMtreeService sysMtreeService;
	
	@ModelAttribute
	public SysMtree get(@RequestParam(required=false) String id) {
		SysMtree entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = sysMtreeService.get(id);
		}
		if (entity == null){
			entity = new SysMtree();
		}
		return entity;
	}
	
	/**
	 * 树形菜单列表页面
	 */
	@RequiresPermissions("sysmtree:sysMtree:list")
	@RequestMapping(value = {"list", ""})
	public String list(SysMtree sysMtree, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<SysMtree> page = sysMtreeService.findPage(new Page<SysMtree>(request, response), sysMtree); 
		model.addAttribute("page", page);
		return "modules/sysmtree/sysMtreeList";
	}

	/**
	 * 查看，增加，编辑树形菜单表单页面
	 */
	@RequiresPermissions(value={"sysmtree:sysMtree:view","sysmtree:sysMtree:add","sysmtree:sysMtree:edit"},logical=Logical.OR)
	@RequestMapping(value = "form")
	public String form(SysMtree sysMtree, Model model) {
		model.addAttribute("sysMtree", sysMtree);
		return "modules/sysmtree/sysMtreeForm";
	}

	/**
	 * 保存树形菜单
	 */
	@RequiresPermissions(value={"sysmtree:sysMtree:add","sysmtree:sysMtree:edit"},logical=Logical.OR)
	@RequestMapping(value = "save")
	public String save(SysMtree sysMtree, Model model, RedirectAttributes redirectAttributes) throws Exception{
		if (!beanValidator(model, sysMtree)){
			return form(sysMtree, model);
		}
		if(!sysMtree.getIsNewRecord()){//编辑表单保存
			SysMtree t = sysMtreeService.get(sysMtree.getId());//从数据库取出记录的值
			MyBeanUtils.copyBeanNotNull2Bean(sysMtree, t);//将编辑表单中的非NULL值覆盖数据库记录中的值
			sysMtreeService.save(t);//保存
		}else{//新增表单保存
			sysMtreeService.save(sysMtree);//保存
		}
		addMessage(redirectAttributes, "保存树形菜单成功");
		return "redirect:"+Global.getAdminPath()+"/sysmtree/sysMtree/?repage";
	}
	
	/**
	 * 删除树形菜单
	 */
	@RequiresPermissions("sysmtree:sysMtree:del")
	@RequestMapping(value = "delete")
	public String delete(SysMtree sysMtree, RedirectAttributes redirectAttributes) {
		sysMtreeService.delete(sysMtree);
		addMessage(redirectAttributes, "删除树形菜单成功");
		return "redirect:"+Global.getAdminPath()+"/sysmtree/sysMtree/?repage";
	}
	
	/**
	 * 批量删除树形菜单
	 */
	@RequiresPermissions("sysmtree:sysMtree:del")
	@RequestMapping(value = "deleteAll")
	public String deleteAll(String ids, RedirectAttributes redirectAttributes) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			sysMtreeService.delete(sysMtreeService.get(id));
		}
		addMessage(redirectAttributes, "删除树形菜单成功");
		return "redirect:"+Global.getAdminPath()+"/sysmtree/sysMtree/?repage";
	}
	
	/**
	 * 导出excel文件
	 */
	@RequiresPermissions("sysmtree:sysMtree:export")
    @RequestMapping(value = "export", method=RequestMethod.POST)
    public String exportFile(SysMtree sysMtree, HttpServletRequest request, HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "树形菜单"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<SysMtree> page = sysMtreeService.findPage(new Page<SysMtree>(request, response, -1), sysMtree);
    		new ExportExcel("树形菜单", SysMtree.class).setDataList(page.getList()).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导出树形菜单记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/sysmtree/sysMtree/?repage";
    }

	/**
	 * 导入Excel数据

	 */
	@RequiresPermissions("sysmtree:sysMtree:import")
    @RequestMapping(value = "import", method=RequestMethod.POST)
    public String importFile(MultipartFile file, RedirectAttributes redirectAttributes) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<SysMtree> list = ei.getDataList(SysMtree.class);
			for (SysMtree sysMtree : list){
				try{
					sysMtreeService.save(sysMtree);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条树形菜单记录。");
			}
			addMessage(redirectAttributes, "已成功导入 "+successNum+" 条树形菜单记录"+failureMsg);
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入树形菜单失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/sysmtree/sysMtree/?repage";
    }
	
	/**
	 * 下载导入树形菜单数据模板
	 */
	@RequiresPermissions("sysmtree:sysMtree:import")
    @RequestMapping(value = "import/template")
    public String importFileTemplate(HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "树形菜单数据导入模板.xlsx";
    		List<SysMtree> list = Lists.newArrayList(); 
    		new ExportExcel("树形菜单数据", SysMtree.class, 1).setDataList(list).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模板下载失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/sysmtree/sysMtree/?repage";
    }
	
	
	

}