/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.workcompanyinfo.web;

import com.google.common.collect.Lists;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.MyBeanUtils;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.modules.sysuseroffice.entity.Useroffice;
import com.jeeplus.modules.workcompanyinfo.service.CompanyinfoService;
import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;
import java.util.List;

/**
 * 公司管理Controller
 * @author yangfan
 * @version 2017-07-25
 */
@Controller
@RequestMapping(value = "${adminPath}/workcompanyinfo/companyinfo")
public class CompanyinfoController extends BaseController {

	@Autowired
	private CompanyinfoService companyinfoService;
	
	@ModelAttribute
	public Useroffice get(@RequestParam(required=false) String id) {
		Useroffice entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = companyinfoService.get(id);
		}
		if (entity == null){
			entity = new Useroffice();
		}
		return entity;
	}
	
	/**
	 * 公司管理列表页面
	 */
	@RequiresPermissions("workcompanyinfo:companyinfo:list")
	@RequestMapping(value = {"list", ""})
	public String list(Useroffice useroffice, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<Useroffice> page = companyinfoService.findPage(new Page<Useroffice>(request, response), useroffice);
		model.addAttribute("page", page);
		return "modules/workcompanyinfo/companyinfoList";
	}

	/**
	 * 查看，增加，编辑公司管理表单页面
	 */
	@RequiresPermissions(value={"workcompanyinfo:companyinfo:view","workcompanyinfo:companyinfo:add","workcompanyinfo:companyinfo:edit"},logical=Logical.OR)
	@RequestMapping(value = "form")
	public String form(Useroffice useroffice, Model model) {
		model.addAttribute("companyinfo", useroffice);
		return "modules/workcompanyinfo/companyinfoForm";
	}

	/**
	 * 保存公司管理
	 */
	@RequiresPermissions(value={"workcompanyinfo:companyinfo:add","workcompanyinfo:companyinfo:edit"},logical=Logical.OR)
	@RequestMapping(value = "save")
	public String save(Useroffice useroffice, Model model, RedirectAttributes redirectAttributes) throws Exception{
		if (!beanValidator(model, useroffice)){
			return form(useroffice, model);
		}
		if(!useroffice.getIsNewRecord()){//编辑表单保存
			Useroffice t = companyinfoService.get(useroffice.getId());//从数据库取出记录的值
			MyBeanUtils.copyBeanNotNull2Bean(useroffice, t);//将编辑表单中的非NULL值覆盖数据库记录中的值
			companyinfoService.save(t);//保存
		}else{//新增表单保存
			companyinfoService.save(useroffice);//保存
		}
		addMessage(redirectAttributes, "保存公司管理成功");
		return "redirect:"+Global.getAdminPath()+"/workcompanyinfo/companyinfo/?repage";
	}
	
	/**
	 * 删除公司管理
	 */
	@RequiresPermissions("workcompanyinfo:companyinfo:del")
	@RequestMapping(value = "delete")
	public String delete(Useroffice useroffice, RedirectAttributes redirectAttributes) {
		companyinfoService.delete(useroffice);
		addMessage(redirectAttributes, "删除公司管理成功");
		return "redirect:"+Global.getAdminPath()+"/workcompanyinfo/companyinfo/?repage";
	}
	
	/**
	 * 批量删除公司管理
	 */
	@RequiresPermissions("workcompanyinfo:companyinfo:del")
	@RequestMapping(value = "deleteAll")
	public String deleteAll(String ids, RedirectAttributes redirectAttributes) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			companyinfoService.delete(companyinfoService.get(id));
		}
		addMessage(redirectAttributes, "删除公司管理成功");
		return "redirect:"+Global.getAdminPath()+"/workcompanyinfo/companyinfo/?repage";
	}
	
	/**
	 * 导出excel文件
	 */
	@RequiresPermissions("workcompanyinfo:companyinfo:export")
    @RequestMapping(value = "export", method=RequestMethod.POST)
    public String exportFile(Useroffice useroffice, HttpServletRequest request, HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "公司管理"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<Useroffice> page = companyinfoService.findPage(new Page<Useroffice>(request, response, -1), useroffice);
    		new ExportExcel("公司管理", Useroffice.class).setDataList(page.getList()).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导出公司管理记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workcompanyinfo/companyinfo/?repage";
    }

	/**
	 * 导入Excel数据

	 */
	@RequiresPermissions("workcompanyinfo:companyinfo:import")
    @RequestMapping(value = "import", method=RequestMethod.POST)
    public String importFile(MultipartFile file, RedirectAttributes redirectAttributes) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<Useroffice> list = ei.getDataList(Useroffice.class);
			for (Useroffice useroffice : list){
				try{
					companyinfoService.save(useroffice);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条公司管理记录。");
			}
			addMessage(redirectAttributes, "已成功导入 "+successNum+" 条公司管理记录"+failureMsg);
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入公司管理失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workcompanyinfo/companyinfo/?repage";
    }
	
	/**
	 * 下载导入公司管理数据模板
	 */
	@RequiresPermissions("workcompanyinfo:companyinfo:import")
    @RequestMapping(value = "import/template")
    public String importFileTemplate(HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "公司管理数据导入模板.xlsx";
    		List<Useroffice> list = Lists.newArrayList();
    		new ExportExcel("公司管理数据", Useroffice.class, 1).setDataList(list).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模板下载失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workcompanyinfo/companyinfo/?repage";
    }
	
	
	

}