/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.workcontent.web;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;

import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.google.common.collect.Lists;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.MyBeanUtils;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.modules.workcontent.entity.WorkContentDocument;
import com.jeeplus.modules.workcontent.service.WorkContentDocumentService;

/**
 * 资料管理Controller
 * @author ssrh
 * @version 2018-06-06
 */
@Controller
@RequestMapping(value = "${adminPath}/workcontent/workContentDocument")
public class WorkContentDocumentController extends BaseController {

	@Autowired
	private WorkContentDocumentService workContentDocumentService;
	
	@ModelAttribute
	public WorkContentDocument get(@RequestParam(required=false) String id) {
		WorkContentDocument entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = workContentDocumentService.get(id);
		}
		if (entity == null){
			entity = new WorkContentDocument();
		}
		return entity;
	}
	
	/**
	 * 资料管理列表页面
	 */
	@RequiresPermissions("workcontent:workContentDocument:list")
	@RequestMapping(value = {"list", ""})
	public String list(WorkContentDocument workContentDocument, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<WorkContentDocument> page = workContentDocumentService.findPage(new Page<WorkContentDocument>(request, response), workContentDocument); 
		model.addAttribute("page", page);
		return "modules/workcontent/workContentDocumentList";
	}

	/**
	 * 查看，增加，编辑资料管理表单页面
	 */
	@RequestMapping(value = "form")
	public String form(WorkContentDocument workContentDocument,String view, Model model) {
        WorkContentDocument oldRecord = null;
        if(StringUtils.isNotBlank(workContentDocument.getContentId())){
            List<WorkContentDocument> list = workContentDocumentService.findList(workContentDocument);
            if(list!=null&&!list.isEmpty()){
                oldRecord = list.get(0);
            }
        }
        if(oldRecord!=null){
            model.addAttribute("workContentDocument", oldRecord);
        }else {
            model.addAttribute("workContentDocument", workContentDocument);
        }
        String viewPath = "modules/workcontent/workContentDocument";
        if("view".equals(view)){
            viewPath+="View";
        }
        return viewPath;
	}

	/**
	 * 保存资料管理
	 */
	@RequiresPermissions(value={"workcontent:workContentDocument:add","workcontent:workContentDocument:edit"},logical=Logical.OR)
	@RequestMapping(value = "save")
	public String save(WorkContentDocument workContentDocument, Model model, RedirectAttributes redirectAttributes) throws Exception{
		if (!beanValidator(model, workContentDocument)){
			return form(workContentDocument,null, model);
		}
		if(!workContentDocument.getIsNewRecord()){//编辑表单保存
			WorkContentDocument t = workContentDocumentService.get(workContentDocument.getId());//从数据库取出记录的值
			MyBeanUtils.copyBeanNotNull2Bean(workContentDocument, t);//将编辑表单中的非NULL值覆盖数据库记录中的值
			workContentDocumentService.save(t);//保存
		}else{//新增表单保存
			workContentDocumentService.save(workContentDocument);//保存
		}
		addMessage(redirectAttributes, "保存资料管理成功");
		return "redirect:"+Global.getAdminPath()+"/workcontent/workContentDocument/?repage";
	}
	
	/**
	 * 删除资料管理
	 */
	@RequiresPermissions("workcontent:workContentDocument:del")
	@RequestMapping(value = "delete")
	public String delete(WorkContentDocument workContentDocument, RedirectAttributes redirectAttributes) {
		workContentDocumentService.delete(workContentDocument);
		addMessage(redirectAttributes, "删除资料管理成功");
		return "redirect:"+Global.getAdminPath()+"/workcontent/workContentDocument/?repage";
	}
	
	/**
	 * 批量删除资料管理
	 */
	@RequiresPermissions("workcontent:workContentDocument:del")
	@RequestMapping(value = "deleteAll")
	public String deleteAll(String ids, RedirectAttributes redirectAttributes) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			workContentDocumentService.delete(workContentDocumentService.get(id));
		}
		addMessage(redirectAttributes, "删除资料管理成功");
		return "redirect:"+Global.getAdminPath()+"/workcontent/workContentDocument/?repage";
	}
	
	/**
	 * 导出excel文件
	 */
	@RequiresPermissions("workcontent:workContentDocument:export")
    @RequestMapping(value = "export", method=RequestMethod.POST)
    public String exportFile(WorkContentDocument workContentDocument, HttpServletRequest request, HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "资料管理"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<WorkContentDocument> page = workContentDocumentService.findPage(new Page<WorkContentDocument>(request, response, -1), workContentDocument);
    		new ExportExcel("资料管理", WorkContentDocument.class).setDataList(page.getList()).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导出资料管理记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workcontent/workContentDocument/?repage";
    }

	/**
	 * 导入Excel数据

	 */
	@RequiresPermissions("workcontent:workContentDocument:import")
    @RequestMapping(value = "import", method=RequestMethod.POST)
    public String importFile(MultipartFile file, RedirectAttributes redirectAttributes) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<WorkContentDocument> list = ei.getDataList(WorkContentDocument.class);
			for (WorkContentDocument workContentDocument : list){
				try{
					workContentDocumentService.save(workContentDocument);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条资料管理记录。");
			}
			addMessage(redirectAttributes, "已成功导入 "+successNum+" 条资料管理记录"+failureMsg);
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入资料管理失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workcontent/workContentDocument/?repage";
    }
	
	/**
	 * 下载导入资料管理数据模板
	 */
	@RequiresPermissions("workcontent:workContentDocument:import")
    @RequestMapping(value = "import/template")
    public String importFileTemplate(HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "资料管理数据导入模板.xlsx";
    		List<WorkContentDocument> list = Lists.newArrayList(); 
    		new ExportExcel("资料管理数据", WorkContentDocument.class, 1).setDataList(list).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模板下载失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workcontent/workContentDocument/?repage";
    }
	
	
	

}