/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.workengineeringprice.web;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;

import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.google.common.collect.Lists;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.MyBeanUtils;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.modules.workengineeringprice.entity.WorkEngineeringPrice;
import com.jeeplus.modules.workengineeringprice.service.WorkEngineeringPriceService;

/**
 * 工程咨询项目三级校审项目信息Controller
 * @author 杨帆
 * @version 2018-01-10
 */
@Controller
@RequestMapping(value = "${adminPath}/workengineeringprice/workEngineeringPrice")
public class WorkEngineeringPriceController extends BaseController {

	@Autowired
	private WorkEngineeringPriceService workEngineeringPriceService;
	
	@ModelAttribute
	public WorkEngineeringPrice get(@RequestParam(required=false) String id) {
		WorkEngineeringPrice entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = workEngineeringPriceService.get(id);
		}
		if (entity == null){
			entity = new WorkEngineeringPrice();
		}
		return entity;
	}
	
	/**
	 * 工程咨询项目三级校审项目信息列表页面
	 */
	@RequiresPermissions("workengineeringprice:workEngineeringPrice:list")
	@RequestMapping(value = {"list", ""})
	public String list(WorkEngineeringPrice workEngineeringPrice, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<WorkEngineeringPrice> page = workEngineeringPriceService.findPage(new Page<WorkEngineeringPrice>(request, response), workEngineeringPrice); 
		model.addAttribute("page", page);
		return "modules/workengineeringprice/workEngineeringPriceList";
	}

	/**
	 * 查看，增加，编辑工程咨询项目三级校审项目信息表单页面
	 */
	@RequiresPermissions(value={"workengineeringprice:workEngineeringPrice:view","workengineeringprice:workEngineeringPrice:add","workengineeringprice:workEngineeringPrice:edit"},logical=Logical.OR)
	@RequestMapping(value = "form")
	public String form(WorkEngineeringPrice workEngineeringPrice, Model model) {
		model.addAttribute("workEngineeringPrice", workEngineeringPrice);
		return "modules/workengineeringprice/workEngineeringPriceForm";
	}

	/**
	 * 保存工程咨询项目三级校审项目信息
	 */
	@RequiresPermissions(value={"workengineeringprice:workEngineeringPrice:add","workengineeringprice:workEngineeringPrice:edit"},logical=Logical.OR)
	@RequestMapping(value = "save")
	public String save(WorkEngineeringPrice workEngineeringPrice, Model model, RedirectAttributes redirectAttributes) throws Exception{
		if (!beanValidator(model, workEngineeringPrice)){
			return form(workEngineeringPrice, model);
		}
		if(!workEngineeringPrice.getIsNewRecord()){//编辑表单保存
			WorkEngineeringPrice t = workEngineeringPriceService.get(workEngineeringPrice.getId());//从数据库取出记录的值
			MyBeanUtils.copyBeanNotNull2Bean(workEngineeringPrice, t);//将编辑表单中的非NULL值覆盖数据库记录中的值
			workEngineeringPriceService.save(t);//保存
		}else{//新增表单保存
			workEngineeringPriceService.save(workEngineeringPrice);//保存
		}
		addMessage(redirectAttributes, "保存工程咨询项目三级校审项目信息成功");
		return "redirect:"+Global.getAdminPath()+"/workengineeringprice/workEngineeringPrice/?repage";
	}
	
	/**
	 * 删除工程咨询项目三级校审项目信息
	 */
	@RequiresPermissions("workengineeringprice:workEngineeringPrice:del")
	@RequestMapping(value = "delete")
	public String delete(WorkEngineeringPrice workEngineeringPrice, RedirectAttributes redirectAttributes) {
		workEngineeringPriceService.delete(workEngineeringPrice);
		addMessage(redirectAttributes, "删除工程咨询项目三级校审项目信息成功");
		return "redirect:"+Global.getAdminPath()+"/workengineeringprice/workEngineeringPrice/?repage";
	}
	
	/**
	 * 批量删除工程咨询项目三级校审项目信息
	 */
	@RequiresPermissions("workengineeringprice:workEngineeringPrice:del")
	@RequestMapping(value = "deleteAll")
	public String deleteAll(String ids, RedirectAttributes redirectAttributes) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			workEngineeringPriceService.delete(workEngineeringPriceService.get(id));
		}
		addMessage(redirectAttributes, "删除工程咨询项目三级校审项目信息成功");
		return "redirect:"+Global.getAdminPath()+"/workengineeringprice/workEngineeringPrice/?repage";
	}
	
	/**
	 * 导出excel文件
	 */
	@RequiresPermissions("workengineeringprice:workEngineeringPrice:export")
    @RequestMapping(value = "export", method=RequestMethod.POST)
    public String exportFile(WorkEngineeringPrice workEngineeringPrice, HttpServletRequest request, HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "工程咨询项目三级校审项目信息"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<WorkEngineeringPrice> page = workEngineeringPriceService.findPage(new Page<WorkEngineeringPrice>(request, response, -1), workEngineeringPrice);
    		new ExportExcel("工程咨询项目三级校审项目信息", WorkEngineeringPrice.class).setDataList(page.getList()).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导出工程咨询项目三级校审项目信息记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workengineeringprice/workEngineeringPrice/?repage";
    }

	/**
	 * 导入Excel数据

	 */
	@RequiresPermissions("workengineeringprice:workEngineeringPrice:import")
    @RequestMapping(value = "import", method=RequestMethod.POST)
    public String importFile(MultipartFile file, RedirectAttributes redirectAttributes) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<WorkEngineeringPrice> list = ei.getDataList(WorkEngineeringPrice.class);
			for (WorkEngineeringPrice workEngineeringPrice : list){
				try{
					workEngineeringPriceService.save(workEngineeringPrice);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条工程咨询项目三级校审项目信息记录。");
			}
			addMessage(redirectAttributes, "已成功导入 "+successNum+" 条工程咨询项目三级校审项目信息记录"+failureMsg);
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入工程咨询项目三级校审项目信息失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workengineeringprice/workEngineeringPrice/?repage";
    }
	
	/**
	 * 下载导入工程咨询项目三级校审项目信息数据模板
	 */
	@RequiresPermissions("workengineeringprice:workEngineeringPrice:import")
    @RequestMapping(value = "import/template")
    public String importFileTemplate(HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "工程咨询项目三级校审项目信息数据导入模板.xlsx";
    		List<WorkEngineeringPrice> list = Lists.newArrayList(); 
    		new ExportExcel("工程咨询项目三级校审项目信息数据", WorkEngineeringPrice.class, 1).setDataList(list).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模板下载失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workengineeringprice/workEngineeringPrice/?repage";
    }
	
	
	

}