/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.workhandbill.web;

import com.google.common.base.Strings;
import com.google.common.collect.Lists;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.MyBeanUtils;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.modules.workhandbill.entity.WorkHandBill;
import com.jeeplus.modules.workhandbill.entity.WorkHandBillDetail;
import com.jeeplus.modules.workhandbill.service.WorkHandBillDetailService;
import com.jeeplus.modules.workhandbill.service.WorkHandBillService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;
import java.util.List;

/**
 * 交接单Controller
 * @author 马鹏波
 * @version 2018-06-05
 */
@Controller
@RequestMapping(value = "${adminPath}/workhandbill/workHandBill")
public class WorkHandBillController extends BaseController {

	@Autowired
	private WorkHandBillService workHandBillService;
	@Autowired
	private WorkHandBillDetailService workHandBillDetailService;
	
	@ModelAttribute
	public WorkHandBill get(@RequestParam(required=false) String id) {
		WorkHandBill entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = workHandBillService.get(id);
		}
		if (entity == null){
			entity = new WorkHandBill();
		}
		return entity;
	}
	
	/**
	 * 交接单列表页面
	 */
//	@RequiresPermissions("workhandbill:workHandBill:list")
	@RequestMapping(value = {"list", ""})
	public String list(WorkHandBill workHandBill, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<WorkHandBill> page = workHandBillService.findPage(new Page<WorkHandBill>(request, response), workHandBill); 
		model.addAttribute("page", page);
		return "modules/workhandbill/workHandBillList";
	}

	/**
	 * 查看，增加，编辑交接单表单页面
	 */
//	@RequiresPermissions(value={"workhandbill:workHandBill:view","workhandbill:workHandBill:add","workhandbill:workHandBill:edit"},logical=Logical.OR)
	@RequestMapping(value = "form")
	public String form(WorkHandBill workHandBill, HttpServletRequest request, Model model) {
		String view = request.getParameter("view");
		if("view".equals(view)){
			view = "modules/workhandbill/workHandBillFormView";
		}else{
			view = "modules/workhandbill/workHandBillForm";
		}
		if(!Strings.isNullOrEmpty(workHandBill.getProjectId())){
			workHandBill = workHandBillService.findListByProjectId(workHandBill);
			if(!workHandBill.getIsNewRecord()){
				WorkHandBillDetail workHandBillDetail = new WorkHandBillDetail();
				workHandBillDetail.setBillId(workHandBill.getId());
				List<WorkHandBillDetail> listByBillId = workHandBillDetailService.findListByBillId(workHandBillDetail);
				workHandBill.setWorkHandBillDetailList(listByBillId);
			}
		}
		model.addAttribute("workHandBill", workHandBill);
		return view;
	}

	/**
	 * 保存交接单
	 */
//	@RequiresPermissions(value={"workhandbill:workHandBill:add","workhandbill:workHandBill:edit"},logical=Logical.OR)
	@RequestMapping(value = "save")
	public String save(WorkHandBill workHandBill, HttpServletRequest request,Model model, RedirectAttributes redirectAttributes) throws Exception{
		if (!beanValidator(model, workHandBill)){
			return form(workHandBill, request, model);
		}

		if(!workHandBill.getIsNewRecord()){//编辑表单保存
			WorkHandBill t = workHandBillService.get(workHandBill.getId());//从数据库取出记录的值
			MyBeanUtils.copyBeanNotNull2Bean(workHandBill, t);//将编辑表单中的非NULL值覆盖数据库记录中的值
			workHandBillService.save(t);//保存
		}else{//新增表单保存
			// 公司编号
			workHandBillService.save(workHandBill);//保存
		}
		addMessage(redirectAttributes, "保存交接单成功");
		return "redirect:"+Global.getAdminPath()+"/workhandbill/workHandBill/?repage";
	}
	
	/**
	 * 删除交接单
	 */
//	@RequiresPermissions("workhandbill:workHandBill:del")
	@RequestMapping(value = "delete")
	public String delete(WorkHandBill workHandBill, RedirectAttributes redirectAttributes) {
		workHandBillService.delete(workHandBill);
		addMessage(redirectAttributes, "删除交接单成功");
		return "redirect:"+Global.getAdminPath()+"/workhandbill/workHandBill/?repage";
	}
	
	/**
	 * 批量删除交接单
	 */
//	@RequiresPermissions("workhandbill:workHandBill:del")
	@RequestMapping(value = "deleteAll")
	public String deleteAll(String ids, RedirectAttributes redirectAttributes) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			workHandBillService.delete(workHandBillService.get(id));
		}
		addMessage(redirectAttributes, "删除交接单成功");
		return "redirect:"+Global.getAdminPath()+"/workhandbill/workHandBill/?repage";
	}
	
	/**
	 * 导出excel文件
	 */
//	@RequiresPermissions("workhandbill:workHandBill:export")
    @RequestMapping(value = "export", method=RequestMethod.POST)
    public String exportFile(WorkHandBill workHandBill, HttpServletRequest request, HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "交接单"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<WorkHandBill> page = workHandBillService.findPage(new Page<WorkHandBill>(request, response, -1), workHandBill);
    		new ExportExcel("交接单", WorkHandBill.class).setDataList(page.getList()).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导出交接单记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workhandbill/workHandBill/?repage";
    }

	/**
	 * 导入Excel数据

	 */
//	@RequiresPermissions("workhandbill:workHandBill:import")
    @RequestMapping(value = "import", method=RequestMethod.POST)
    public String importFile(MultipartFile file, RedirectAttributes redirectAttributes) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<WorkHandBill> list = ei.getDataList(WorkHandBill.class);
			for (WorkHandBill workHandBill : list){
				try{
					workHandBillService.save(workHandBill);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条交接单记录。");
			}
			addMessage(redirectAttributes, "已成功导入 "+successNum+" 条交接单记录"+failureMsg);
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入交接单失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workhandbill/workHandBill/?repage";
    }
	
	/**
	 * 下载导入交接单数据模板
	 */
//	@RequiresPermissions("workhandbill:workHandBill:import")
    @RequestMapping(value = "import/template")
    public String importFileTemplate(HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "交接单数据导入模板.xlsx";
    		List<WorkHandBill> list = Lists.newArrayList(); 
    		new ExportExcel("交接单数据", WorkHandBill.class, 1).setDataList(list).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模板下载失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workhandbill/workHandBill/?repage";
    }
	
	
	

}