/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.workitempre.web;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;

import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.google.common.collect.Lists;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.MyBeanUtils;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.modules.workitempre.entity.WorkItemPre;
import com.jeeplus.modules.workitempre.service.WorkItemPreService;

/**
 * 项目预审Controller
 * @author 丁旭
 * @version 2017-03-20
 */
@Controller
@RequestMapping(value = "${adminPath}/workitempre/workItemPre")
public class WorkItemPreController extends BaseController {

	@Autowired
	private WorkItemPreService workItemPreService;
	
	@ModelAttribute
	public WorkItemPre get(@RequestParam(required=false) String id) {
		WorkItemPre entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = workItemPreService.get(id);
		}
		if (entity == null){
			entity = new WorkItemPre();
		}
		return entity;
	}
	
	/**
	 * 项目预审列表页面
	 */
	@RequiresPermissions("workitempre:workItemPre:list")
	@RequestMapping(value = {"list", ""})
	public String list(WorkItemPre workItemPre, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<WorkItemPre> page = workItemPreService.findPage(new Page<WorkItemPre>(request, response), workItemPre); 
		model.addAttribute("page", page);
		return "modules/workitempre/workItemPreList";
	}

	/**
	 * 查看，增加，编辑项目预审表单页面
	 */
	@RequiresPermissions(value={"workitempre:workItemPre:view","workitempre:workItemPre:add","workitempre:workItemPre:edit"},logical=Logical.OR)
	@RequestMapping(value = "form")
	public String form(WorkItemPre workItemPre, Model model) {
		model.addAttribute("workItemPre", workItemPre);
		return "modules/workitempre/workItemPreForm";
	}

	/**
	 * 保存项目预审
	 */
	@RequiresPermissions(value={"workitempre:workItemPre:add","workitempre:workItemPre:edit"},logical=Logical.OR)
	@RequestMapping(value = "save")
	public String save(WorkItemPre workItemPre, Model model, RedirectAttributes redirectAttributes) throws Exception{
		if (!beanValidator(model, workItemPre)){
			return form(workItemPre, model);
		}
		if(!workItemPre.getIsNewRecord()){//编辑表单保存
			WorkItemPre t = workItemPreService.get(workItemPre.getId());//从数据库取出记录的值
			MyBeanUtils.copyBeanNotNull2Bean(workItemPre, t);//将编辑表单中的非NULL值覆盖数据库记录中的值
			workItemPreService.save(t);//保存
		}else{//新增表单保存
			workItemPreService.save(workItemPre);//保存
		}
		addMessage(redirectAttributes, "保存项目预审成功");
		return "redirect:"+Global.getAdminPath()+"/workitempre/workItemPre/?repage";
	}
	
	/**
	 * 删除项目预审
	 */
	@RequiresPermissions("workitempre:workItemPre:del")
	@RequestMapping(value = "delete")
	public String delete(WorkItemPre workItemPre, RedirectAttributes redirectAttributes) {
		workItemPreService.delete(workItemPre);
		addMessage(redirectAttributes, "删除项目预审成功");
		return "redirect:"+Global.getAdminPath()+"/workitempre/workItemPre/?repage";
	}
	
	/**
	 * 批量删除项目预审
	 */
	@RequiresPermissions("workitempre:workItemPre:del")
	@RequestMapping(value = "deleteAll")
	public String deleteAll(String ids, RedirectAttributes redirectAttributes) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			workItemPreService.delete(workItemPreService.get(id));
		}
		addMessage(redirectAttributes, "删除项目预审成功");
		return "redirect:"+Global.getAdminPath()+"/workitempre/workItemPre/?repage";
	}
	
	/**
	 * 导出excel文件
	 */
	@RequiresPermissions("workitempre:workItemPre:export")
    @RequestMapping(value = "export", method=RequestMethod.POST)
    public String exportFile(WorkItemPre workItemPre, HttpServletRequest request, HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "项目预审"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<WorkItemPre> page = workItemPreService.findPage(new Page<WorkItemPre>(request, response, -1), workItemPre);
    		new ExportExcel("项目预审", WorkItemPre.class).setDataList(page.getList()).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导出项目预审记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workitempre/workItemPre/?repage";
    }

	/**
	 * 导入Excel数据

	 */
	@RequiresPermissions("workitempre:workItemPre:import")
    @RequestMapping(value = "import", method=RequestMethod.POST)
    public String importFile(MultipartFile file, RedirectAttributes redirectAttributes) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<WorkItemPre> list = ei.getDataList(WorkItemPre.class);
			for (WorkItemPre workItemPre : list){
				try{
					workItemPreService.save(workItemPre);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条项目预审记录。");
			}
			addMessage(redirectAttributes, "已成功导入 "+successNum+" 条项目预审记录"+failureMsg);
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入项目预审失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workitempre/workItemPre/?repage";
    }
	
	/**
	 * 下载导入项目预审数据模板
	 */
	@RequiresPermissions("workitempre:workItemPre:import")
    @RequestMapping(value = "import/template")
    public String importFileTemplate(HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "项目预审数据导入模板.xlsx";
    		List<WorkItemPre> list = Lists.newArrayList(); 
    		new ExportExcel("项目预审数据", WorkItemPre.class, 1).setDataList(list).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模板下载失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workitempre/workItemPre/?repage";
    }
	
	
	

}