/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.workmeeting.web;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;

import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.google.common.collect.Lists;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.MyBeanUtils;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.modules.workmeeting.entity.Workmeetinguser;
import com.jeeplus.modules.workmeeting.service.WorkmeetinguserService;

/**
 * 会议用户信息Controller
 * @author 杨帆
 * @version 2018-03-05
 */
@Controller
@RequestMapping(value = "${adminPath}/workmeeting/workmeetinguser")
public class WorkmeetinguserController extends BaseController {

	@Autowired
	private WorkmeetinguserService workmeetinguserService;
	
	@ModelAttribute
	public Workmeetinguser get(@RequestParam(required=false) String id) {
		Workmeetinguser entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = workmeetinguserService.get(id);
		}
		if (entity == null){
			entity = new Workmeetinguser();
		}
		return entity;
	}
	
	/**
	 * 会议用户信息列表页面
	 */
	@RequiresPermissions("workmeeting:workmeetinguser:list")
	@RequestMapping(value = {"list", ""})
	public String list(Workmeetinguser workmeetinguser, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<Workmeetinguser> page = workmeetinguserService.findPage(new Page<Workmeetinguser>(request, response), workmeetinguser); 
		model.addAttribute("page", page);
		return "modules/workmeeting/workmeetinguserList";
	}

	/**
	 * 查看，增加，编辑会议用户信息表单页面
	 */
	@RequiresPermissions(value={"workmeeting:workmeetinguser:view","workmeeting:workmeetinguser:add","workmeeting:workmeetinguser:edit"},logical=Logical.OR)
	@RequestMapping(value = "form")
	public String form(Workmeetinguser workmeetinguser, Model model) {
		model.addAttribute("workmeetinguser", workmeetinguser);
		return "modules/workmeeting/workmeetinguserForm";
	}

	/**
	 * 保存会议用户信息
	 */
	@RequiresPermissions(value={"workmeeting:workmeetinguser:add","workmeeting:workmeetinguser:edit"},logical=Logical.OR)
	@RequestMapping(value = "save")
	public String save(Workmeetinguser workmeetinguser, Model model, RedirectAttributes redirectAttributes) throws Exception{
		if (!beanValidator(model, workmeetinguser)){
			return form(workmeetinguser, model);
		}
		if(!workmeetinguser.getIsNewRecord()){//编辑表单保存
			Workmeetinguser t = workmeetinguserService.get(workmeetinguser.getId());//从数据库取出记录的值
			MyBeanUtils.copyBeanNotNull2Bean(workmeetinguser, t);//将编辑表单中的非NULL值覆盖数据库记录中的值
			workmeetinguserService.save(t);//保存
		}else{//新增表单保存
			workmeetinguserService.save(workmeetinguser);//保存
		}
		addMessage(redirectAttributes, "保存会议用户信息成功");
		return "redirect:"+Global.getAdminPath()+"/workmeeting/workmeetinguser/?repage";
	}
	
	/**
	 * 删除会议用户信息
	 */
	@RequiresPermissions("workmeeting:workmeetinguser:del")
	@RequestMapping(value = "delete")
	public String delete(Workmeetinguser workmeetinguser, RedirectAttributes redirectAttributes) {
		workmeetinguserService.delete(workmeetinguser);
		addMessage(redirectAttributes, "删除会议用户信息成功");
		return "redirect:"+Global.getAdminPath()+"/workmeeting/workmeetinguser/?repage";
	}
	
	/**
	 * 批量删除会议用户信息
	 */
	@RequiresPermissions("workmeeting:workmeetinguser:del")
	@RequestMapping(value = "deleteAll")
	public String deleteAll(String ids, RedirectAttributes redirectAttributes) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			workmeetinguserService.delete(workmeetinguserService.get(id));
		}
		addMessage(redirectAttributes, "删除会议用户信息成功");
		return "redirect:"+Global.getAdminPath()+"/workmeeting/workmeetinguser/?repage";
	}
	
	/**
	 * 导出excel文件
	 */
	@RequiresPermissions("workmeeting:workmeetinguser:export")
    @RequestMapping(value = "export", method=RequestMethod.POST)
    public String exportFile(Workmeetinguser workmeetinguser, HttpServletRequest request, HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "会议用户信息"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<Workmeetinguser> page = workmeetinguserService.findPage(new Page<Workmeetinguser>(request, response, -1), workmeetinguser);
    		new ExportExcel("会议用户信息", Workmeetinguser.class).setDataList(page.getList()).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导出会议用户信息记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workmeeting/workmeetinguser/?repage";
    }

	/**
	 * 导入Excel数据

	 */
	@RequiresPermissions("workmeeting:workmeetinguser:import")
    @RequestMapping(value = "import", method=RequestMethod.POST)
    public String importFile(MultipartFile file, RedirectAttributes redirectAttributes) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<Workmeetinguser> list = ei.getDataList(Workmeetinguser.class);
			for (Workmeetinguser workmeetinguser : list){
				try{
					workmeetinguserService.save(workmeetinguser);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条会议用户信息记录。");
			}
			addMessage(redirectAttributes, "已成功导入 "+successNum+" 条会议用户信息记录"+failureMsg);
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入会议用户信息失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workmeeting/workmeetinguser/?repage";
    }
	
	/**
	 * 下载导入会议用户信息数据模板
	 */
	@RequiresPermissions("workmeeting:workmeetinguser:import")
    @RequestMapping(value = "import/template")
    public String importFileTemplate(HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "会议用户信息数据导入模板.xlsx";
    		List<Workmeetinguser> list = Lists.newArrayList(); 
    		new ExportExcel("会议用户信息数据", Workmeetinguser.class, 1).setDataList(list).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模板下载失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workmeeting/workmeetinguser/?repage";
    }
	
	
	

}