/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.workprojectcalc.web;

import com.google.common.base.Strings;
import com.google.common.collect.Lists;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.modules.workprojectcalc.entity.WorkProjectCalcBuild;
import com.jeeplus.modules.workprojectcalc.entity.WorkProjectCalcDecor;
import com.jeeplus.modules.workprojectcalc.service.WorkProjectCalcBuildService;
import com.jeeplus.modules.workprojectcalc.service.WorkProjectCalcDecorService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;
import java.util.List;

/**
 * 工程计算量底稿-装饰Controller
 * @author 马鹏波
 * @version 2018-06-06
 */
@Controller
@RequestMapping(value = "${adminPath}/workprojectcalc/workProjectCalcDecor")
public class WorkProjectCalcDecorController extends BaseController {

	@Autowired
	private WorkProjectCalcDecorService workProjectCalcDecorService;
	@Autowired
	private WorkProjectCalcBuildService workProjectCalcBuildService;
	
	@ModelAttribute
	public WorkProjectCalcDecor get(@RequestParam(required=false) String id) {
		WorkProjectCalcDecor entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = workProjectCalcDecorService.get(id);
		}
		if (entity == null){
			entity = new WorkProjectCalcDecor();
		}
		return entity;
	}
	
	/**
	 * 工程计算量底稿列表页面
	 */
//	@RequiresPermissions("workprojectcalc:workProjectCalcDecor:list")
	@RequestMapping(value = {"list", ""})
	public String list(WorkProjectCalcDecor workProjectCalcDecor, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<WorkProjectCalcDecor> page = workProjectCalcDecorService.findPage(new Page<WorkProjectCalcDecor>(request, response), workProjectCalcDecor); 
		model.addAttribute("page", page);
		return "modules/workprojectcalc/workProjectCalcDecorList";
	}

	/**
	 * 查看，增加，编辑工程计算量底稿表单页面
	 */
//	@RequiresPermissions(value={"workprojectcalc:workProjectCalcDecor:view","workprojectcalc:workProjectCalcDecor:add","workprojectcalc:workProjectCalcDecor:edit"},logical=Logical.OR)
	@RequestMapping(value = "form")
	public String form(WorkProjectCalcDecor workProjectCalcDecor, Model model,HttpServletRequest request) 	{
		String view = request.getParameter("view");
		if("view".equals(view)){
			view = "modules/workprojectcalc/workProjectCalcDecorFormView";
		}else{
			view = "modules/workprojectcalc/workProjectCalcDecorForm";
		}
		if(!Strings.isNullOrEmpty(workProjectCalcDecor.getProjectId())){
			WorkProjectCalcBuild workProjectCalcBuild = new WorkProjectCalcBuild();
			workProjectCalcBuild.setProjectId(workProjectCalcDecor.getProjectId());
			List<WorkProjectCalcDecor> listdecor = workProjectCalcDecorService.findList(workProjectCalcDecor);
			List<WorkProjectCalcBuild> listbuild = workProjectCalcBuildService.findList(workProjectCalcBuild);
			workProjectCalcDecor.setWorkProjectCalcDecorList(listdecor);
			workProjectCalcDecor.setWorkProjectCalcBuildList(listbuild);
		}
		model.addAttribute("workProjectCalcDecor", workProjectCalcDecor);
		return view;
	}

	/**
	 * 保存工程计算量底稿
	 */
//	@RequiresPermissions(value={"workprojectcalc:workProjectCalcDecor:add","workprojectcalc:workProjectCalcDecor:edit"},logical=Logical.OR)
	@RequestMapping(value = "save")
	public String save(WorkProjectCalcDecor workProjectCalcDecor, Model model,HttpServletRequest request, RedirectAttributes redirectAttributes) throws Exception{
		if (!beanValidator(model, workProjectCalcDecor)){
			return form(workProjectCalcDecor, model,request);
		}
		workProjectCalcDecorService.save(workProjectCalcDecor);//保存
		addMessage(redirectAttributes, "保存工程计算量底稿成功");
		return "redirect:"+Global.getAdminPath()+"/workprojectcalc/workProjectCalcDecor/?repage";
	}
	
	/**
	 * 删除工程计算量底稿
	 */
//	@RequiresPermissions("workprojectcalc:workProjectCalcDecor:del")
	@RequestMapping(value = "delete")
	public String delete(WorkProjectCalcDecor workProjectCalcDecor, RedirectAttributes redirectAttributes) {
		workProjectCalcDecorService.delete(workProjectCalcDecor);
		addMessage(redirectAttributes, "删除工程计算量底稿成功");
		return "redirect:"+Global.getAdminPath()+"/workprojectcalc/workProjectCalcDecor/?repage";
	}
	
	/**
	 * 批量删除工程计算量底稿
	 */
//	@RequiresPermissions("workprojectcalc:workProjectCalcDecor:del")
	@RequestMapping(value = "deleteAll")
	public String deleteAll(String ids, RedirectAttributes redirectAttributes) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			workProjectCalcDecorService.delete(workProjectCalcDecorService.get(id));
		}
		addMessage(redirectAttributes, "删除工程计算量底稿成功");
		return "redirect:"+Global.getAdminPath()+"/workprojectcalc/workProjectCalcDecor/?repage";
	}
	
	/**
	 * 导出excel文件
	 */
//	@RequiresPermissions("workprojectcalc:workProjectCalcDecor:export")
    @RequestMapping(value = "export", method=RequestMethod.POST)
    public String exportFile(WorkProjectCalcDecor workProjectCalcDecor, HttpServletRequest request, HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "工程计算量底稿"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<WorkProjectCalcDecor> page = workProjectCalcDecorService.findPage(new Page<WorkProjectCalcDecor>(request, response, -1), workProjectCalcDecor);
    		new ExportExcel("工程计算量底稿", WorkProjectCalcDecor.class).setDataList(page.getList()).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导出工程计算量底稿记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workprojectcalc/workProjectCalcDecor/?repage";
    }

	/**
	 * 导入Excel数据

	 */
//	@RequiresPermissions("workprojectcalc:workProjectCalcDecor:import")
    @RequestMapping(value = "import", method=RequestMethod.POST)
    public String importFile(MultipartFile file, RedirectAttributes redirectAttributes) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<WorkProjectCalcDecor> list = ei.getDataList(WorkProjectCalcDecor.class);
			for (WorkProjectCalcDecor workProjectCalcDecor : list){
				try{
					workProjectCalcDecorService.save(workProjectCalcDecor);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条工程计算量底稿记录。");
			}
			addMessage(redirectAttributes, "已成功导入 "+successNum+" 条工程计算量底稿记录"+failureMsg);
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入工程计算量底稿失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workprojectcalc/workProjectCalcDecor/?repage";
    }
	
	/**
	 * 下载导入工程计算量底稿数据模板
	 */
//	@RequiresPermissions("workprojectcalc:workProjectCalcDecor:import")
    @RequestMapping(value = "import/template")
    public String importFileTemplate(HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "工程计算量底稿数据导入模板.xlsx";
    		List<WorkProjectCalcDecor> list = Lists.newArrayList(); 
    		new ExportExcel("工程计算量底稿数据", WorkProjectCalcDecor.class, 1).setDataList(list).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模板下载失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workprojectcalc/workProjectCalcDecor/?repage";
    }
	
	
	

}