/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.workregularapply.web;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;

import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.google.common.collect.Lists;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.MyBeanUtils;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.modules.workregularapply.entity.WorkRegularScore;
import com.jeeplus.modules.workregularapply.service.WorkRegularScoreService;

/**
 * 转正申请评分表Controller
 * @author chenyuping
 * @version 2018-07-30
 */
@Controller
@RequestMapping(value = "${adminPath}/workregularapply/workRegularScore")
public class WorkRegularScoreController extends BaseController {

	@Autowired
	private WorkRegularScoreService workRegularScoreService;
	
	@ModelAttribute
	public WorkRegularScore get(@RequestParam(required=false) String id) {
		WorkRegularScore entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = workRegularScoreService.get(id);
		}
		if (entity == null){
			entity = new WorkRegularScore();
		}
		return entity;
	}
	
	/**
	 * 转正申请评分表列表页面
	 */
	@RequiresPermissions("workregularapply:workRegularScore:list")
	@RequestMapping(value = {"list", ""})
	public String list(WorkRegularScore workRegularScore, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<WorkRegularScore> page = workRegularScoreService.findPage(new Page<WorkRegularScore>(request, response), workRegularScore); 
		model.addAttribute("page", page);
		return "modules/workregularapply/workRegularScoreList";
	}

	/**
	 * 查看，增加，编辑转正申请评分表表单页面
	 */
	@RequiresPermissions(value={"workregularapply:workRegularScore:view","workregularapply:workRegularScore:add","workregularapply:workRegularScore:edit"},logical=Logical.OR)
	@RequestMapping(value = "form")
	public String form(WorkRegularScore workRegularScore, Model model) {
		model.addAttribute("workRegularScore", workRegularScore);
		return "modules/workregularapply/workRegularScoreForm";
	}

	/**
	 * 保存转正申请评分表
	 */
	@RequiresPermissions(value={"workregularapply:workRegularScore:add","workregularapply:workRegularScore:edit"},logical=Logical.OR)
	@RequestMapping(value = "save")
	public String save(WorkRegularScore workRegularScore, Model model, RedirectAttributes redirectAttributes) throws Exception{
		if (!beanValidator(model, workRegularScore)){
			return form(workRegularScore, model);
		}
		if(!workRegularScore.getIsNewRecord()){//编辑表单保存
			WorkRegularScore t = workRegularScoreService.get(workRegularScore.getId());//从数据库取出记录的值
			MyBeanUtils.copyBeanNotNull2Bean(workRegularScore, t);//将编辑表单中的非NULL值覆盖数据库记录中的值
			workRegularScoreService.save(t);//保存
		}else{//新增表单保存
			workRegularScoreService.save(workRegularScore);//保存
		}
		addMessage(redirectAttributes, "保存转正申请评分表成功");
		return "redirect:"+Global.getAdminPath()+"/workregularapply/workRegularScore/?repage";
	}
	
	/**
	 * 删除转正申请评分表
	 */
	@RequiresPermissions("workregularapply:workRegularScore:del")
	@RequestMapping(value = "delete")
	public String delete(WorkRegularScore workRegularScore, RedirectAttributes redirectAttributes) {
		workRegularScoreService.delete(workRegularScore);
		addMessage(redirectAttributes, "删除转正申请评分表成功");
		return "redirect:"+Global.getAdminPath()+"/workregularapply/workRegularScore/?repage";
	}
	
	/**
	 * 批量删除转正申请评分表
	 */
	@RequiresPermissions("workregularapply:workRegularScore:del")
	@RequestMapping(value = "deleteAll")
	public String deleteAll(String ids, RedirectAttributes redirectAttributes) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			workRegularScoreService.delete(workRegularScoreService.get(id));
		}
		addMessage(redirectAttributes, "删除转正申请评分表成功");
		return "redirect:"+Global.getAdminPath()+"/workregularapply/workRegularScore/?repage";
	}
	
	/**
	 * 导出excel文件
	 */
	@RequiresPermissions("workregularapply:workRegularScore:export")
    @RequestMapping(value = "export", method=RequestMethod.POST)
    public String exportFile(WorkRegularScore workRegularScore, HttpServletRequest request, HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "转正申请评分表"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<WorkRegularScore> page = workRegularScoreService.findPage(new Page<WorkRegularScore>(request, response, -1), workRegularScore);
    		new ExportExcel("转正申请评分表", WorkRegularScore.class).setDataList(page.getList()).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导出转正申请评分表记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workregularapply/workRegularScore/?repage";
    }

	/**
	 * 导入Excel数据

	 */
	@RequiresPermissions("workregularapply:workRegularScore:import")
    @RequestMapping(value = "import", method=RequestMethod.POST)
    public String importFile(MultipartFile file, RedirectAttributes redirectAttributes) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<WorkRegularScore> list = ei.getDataList(WorkRegularScore.class);
			for (WorkRegularScore workRegularScore : list){
				try{
					workRegularScoreService.save(workRegularScore);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条转正申请评分表记录。");
			}
			addMessage(redirectAttributes, "已成功导入 "+successNum+" 条转正申请评分表记录"+failureMsg);
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入转正申请评分表失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workregularapply/workRegularScore/?repage";
    }
	
	/**
	 * 下载导入转正申请评分表数据模板
	 */
	@RequiresPermissions("workregularapply:workRegularScore:import")
    @RequestMapping(value = "import/template")
    public String importFileTemplate(HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "转正申请评分表数据导入模板.xlsx";
    		List<WorkRegularScore> list = Lists.newArrayList(); 
    		new ExportExcel("转正申请评分表数据", WorkRegularScore.class, 1).setDataList(list).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模板下载失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/workregularapply/workRegularScore/?repage";
    }
	
	
	

}