/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.workstaff.service;

import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.Map;

import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.oss.OSSClientUtil;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.IdGen;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.modules.sys.dao.UserDao;
import com.jeeplus.modules.sys.entity.User;
import com.jeeplus.modules.sys.utils.DictUtils;
import com.jeeplus.modules.workstaff.entity.WorkStaffBasicInfo;
import com.jeeplus.modules.workstaff.entity.WorkStaffLanguages;
import com.jeeplus.modules.workstaffachiveslog.entity.WorkStaffAchivesLog;
import com.jeeplus.modules.workstaffachiveslog.service.WorkStaffAchivesLogService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.service.CrudService;
import com.jeeplus.modules.workstaff.entity.WorkStaffRecord;
import com.jeeplus.modules.workstaff.dao.WorkStaffRecordDao;
import org.springframework.web.multipart.MultipartFile;

/**
 * 电子档案Service
 * @author ssrh
 * @version 2018-07-27
 */
@Service
@Transactional(readOnly = true)
public class WorkStaffRecordService extends CrudService<WorkStaffRecordDao, WorkStaffRecord> {
    @Autowired
    private WorkStaffAchivesLogService workStaffAchivesLogService;
    @Autowired
    private UserDao userDao;
	public WorkStaffRecord get(String id) {
		return super.get(id);
	}
	
	public List<WorkStaffRecord> findList(WorkStaffRecord workStaffRecord) {
		return super.findList(workStaffRecord);
	}
	
	public Page<WorkStaffRecord> findPage(Page<WorkStaffRecord> page, WorkStaffRecord workStaffRecord) {
		return super.findPage(page, workStaffRecord);
	}
	
	@Transactional(readOnly = false)
	public void save(WorkStaffRecord workStaffRecord) {
		super.save(workStaffRecord);
	}
	
	@Transactional(readOnly = false)
	public void delete(WorkStaffRecord workStaffRecord) {
		super.delete(workStaffRecord);
	}


    public boolean save(WorkStaffBasicInfo workStaffBasicInfo,boolean first) {
        if(workStaffBasicInfo==null|| StringUtils.isBlank(workStaffBasicInfo.getId())){
            return false;
        }
        List<WorkStaffRecord> list = workStaffBasicInfo.getRecordList();
        if(list==null||list.size()<=0){
            return false;
        }
        boolean flag=false;
        for (WorkStaffRecord entity : list) {
            if(entity!=null&&"1".equals(entity.getDelFlag())&&StringUtils.isBlank(entity.getId())){
                continue;
            }
            if(entity!=null&&"1".equals(entity.getDelFlag())){
                dao.deleteByLogic(entity);
                continue;
            }
            if (entity!=null){
                if(entity.getFile()!=null&&!entity.getFile().isEmpty()&&entity.getFile().getSize()>0){
                    MultipartFile file = entity.getFile();
                    entity.setFileName(file.getOriginalFilename());
                    entity.setFilePath(this.uploadFile(file,entity.getFilePath()));
                }
                entity.setStaff(workStaffBasicInfo.getId());
                this.save(entity);
                if(first){
                    entity.setId(null);
                    entity.setFilePath("");
                }
                flag =true;
            }
        }
        return flag;
    }

    public String uploadFile(MultipartFile file, String filePath) {
        if (file != null && !file.isEmpty() &&file.getSize()>0) {
            String key = "";
            if (StringUtils.isNotBlank(filePath)) {
                String aliyunUrl = Global.getAliyunUrl();
                String aliDownloadUrl = Global.getAliDownloadUrl();
                String cons = "";
                if (filePath.contains(aliyunUrl)) {
                    cons = aliyunUrl;
                } else if (filePath.contains("http://gangwan-app.oss-cn-hangzhou.aliyuncs.com")) {
                    cons = "http://gangwan-app.oss-cn-hangzhou.aliyuncs.com";
                } else {
                    cons = aliDownloadUrl;
                }
                String[] arr = filePath.split(cons + "/");
                key = arr[1];
            }
            try {
                OSSClientUtil ossUtil1 = new OSSClientUtil();
                if (StringUtils.isNotBlank(key)) {
                    ossUtil1.deleteSingleObject(key);
                }
            }catch (Exception e){
                logger.error("oss删除文件失败！");

            }
            OSSClientUtil ossUtil = new OSSClientUtil();
            try {
                String path = Global.getStaffBasicFilePath()+ DateUtils.formatDate(new Date(),null)+"/";
                String newName = System.currentTimeMillis()+"-"+file.getOriginalFilename();
                ossUtil.uploadFile2OSS(file.getInputStream(),path,newName);
                return  Global.getAliDownloadUrl() + "/" + path + newName;
            } catch (Exception e) {
                logger.error("oss上传文件失败！");
            }
        }
        return filePath;
    }

    public void query(WorkStaffBasicInfo workStaffBasicInfo) {
        if(workStaffBasicInfo==null|| StringUtils.isBlank(workStaffBasicInfo.getId())){
            return;
        }
        WorkStaffRecord entity = new WorkStaffRecord();
        entity.setStaff(workStaffBasicInfo.getId());
        workStaffBasicInfo.setRecordList(this.findList(entity));
    }
    private static Map<String,String> EDU_MAP = Maps.newHashMap();//教育经历

    static{//教育经历
        EDU_MAP.put("name", "档案名称");
        EDU_MAP.put("num", "编号");
        EDU_MAP.put("filePath", "文件");
        EDU_MAP.put("status", "验证");
        EDU_MAP.put("fileName", "文件名");
    }
    public boolean saveApply(WorkStaffBasicInfo workStaffBasicInfo,boolean first) {
        if(workStaffBasicInfo==null|| StringUtils.isBlank(workStaffBasicInfo.getId())){
            return false;
        }
        List<WorkStaffRecord> list = workStaffBasicInfo.getRecordList();
        if(list==null||list.size()<=0){
            return false;
        }
        boolean flag = false;
        User u=userDao.get(workStaffBasicInfo.getUserId());
        for (WorkStaffRecord entity : list) {
            entity.setStaff(workStaffBasicInfo.getId());
            WorkStaffRecord workStaffRecord=dao.get(entity.getId());
            if(workStaffRecord==null){
                entity.setCreateBy(u);
                entity.setCreateDate(new Date());
                entity.setUpdateBy(u);
                entity.setUpdateDate(new Date());
                dao.insert(entity);
            }else{
                this.save(entity);
            }
            if(entity!=null&&"1".equals(entity.getDelFlag())){
                dao.deleteByLogic(entity);
                continue;
            }
            flag =true;
        }
        dao.updateFalt(workStaffBasicInfo.getId());
        return flag;
    }
    public void querys(WorkStaffBasicInfo workStaffBasicInfo) {
        SimpleDateFormat dateFormat1 = new SimpleDateFormat("yyyy-MM-dd");
        if(workStaffBasicInfo==null|| StringUtils.isBlank(workStaffBasicInfo.getId())){
            return;
        }
        WorkStaffRecord entity = new WorkStaffRecord();
        entity.setStaff(workStaffBasicInfo.getId());
        List<WorkStaffRecord> list=this.findList(entity);
        List<WorkStaffRecord> newList=Lists.newArrayList();
        if(list!=null&&list.size()>0){//修改
            for(WorkStaffRecord workStaffRecord:list){
                Class<? extends WorkStaffRecord> newClass = workStaffRecord.getClass();
                Field[] declaredFields = newClass.getDeclaredFields();
                for (Field field : declaredFields) {
                    String name = field.getName();
                    if("name".equals(name)||"num".equals(name)||"filePath".equals(name)||"status".equals(name)||"fileName".equals(name)){
                        String setMethod = "set"+name.substring(0,1).toUpperCase()+name.substring(1);
                        String getMethod = "get"+name.substring(0,1).toUpperCase()+name.substring(1);
                        WorkStaffAchivesLog workStaffAchivesLog=new WorkStaffAchivesLog();
                        workStaffAchivesLog.setStaffId(workStaffBasicInfo.getId());
                        workStaffAchivesLog.setModule("电子档案");
                        workStaffAchivesLog.setSonId(workStaffRecord.getId());
                        workStaffAchivesLog.setFields(name);
                        workStaffAchivesLog.setType("修改");
                        workStaffAchivesLog.setState("1");
                        List<WorkStaffAchivesLog> logs=workStaffAchivesLogService.findList(workStaffAchivesLog);
                        try {
                            Object newInvoke = newClass.getMethod(getMethod).invoke(workStaffRecord);//修改后
                            if(newInvoke instanceof String){
                                Method setReadOnly = workStaffRecord.getClass().getMethod(setMethod, String.class);
                                if(logs!=null&&logs.size()>0){
                                    setReadOnly.invoke(workStaffRecord,logs.get(0).getNewKey());
                                }
                            }
                            if(newInvoke instanceof Date){//日期
                                Method setReadOnly = workStaffRecord.getClass().getMethod(setMethod, Date.class);
                                if(logs!=null&&logs.size()>0){
                                    Date myDate = dateFormat1.parse(logs.get(0).getNewKey());
                                    setReadOnly.invoke(workStaffRecord,myDate);
                                }
                            }
                        } catch (NoSuchMethodException e) {
                            e.printStackTrace();
                        } catch (IllegalAccessException e) {
                            e.printStackTrace();
                        } catch (InvocationTargetException e) {
                            e.printStackTrace();
                        } catch (ParseException e) {
                            e.printStackTrace();
                        }
                    }
                }
                newList.add(workStaffRecord);
            }
        }
        //新增
        WorkStaffAchivesLog workStaffAchivesLog=new WorkStaffAchivesLog();
        workStaffAchivesLog.setStaffId(workStaffBasicInfo.getId());
        workStaffAchivesLog.setModule("电子档案");
        workStaffAchivesLog.setType("新增");
        workStaffAchivesLog.setState("1");
        List<WorkStaffAchivesLog> sonIds=workStaffAchivesLogService.findSonId(workStaffAchivesLog);
        if(sonIds!=null&&sonIds.size()>0){
            for(WorkStaffAchivesLog w:sonIds){
                workStaffAchivesLog.setSonId(w.getSonId());
                WorkStaffRecord workStaffRecord = new WorkStaffRecord();
                workStaffRecord.setId(w.getSonId());
                workStaffRecord.setDelFlag("0");
                workStaffRecord.setStaff(workStaffBasicInfo.getId());
                Class<? extends WorkStaffRecord> newClass = workStaffRecord.getClass();
                Field[] declaredFields = newClass.getDeclaredFields();
                for (Field field : declaredFields) {
                    String name = field.getName();
                    if("name".equals(name)||"num".equals(name)||"filePath".equals(name)||"status".equals(name)||"fileName".equals(name)){
                        String setMethod = "set"+name.substring(0,1).toUpperCase()+name.substring(1);
                        String getMethod = "get"+name.substring(0,1).toUpperCase()+name.substring(1);
                        workStaffAchivesLog.setFields(name);
                        List<WorkStaffAchivesLog> logs=workStaffAchivesLogService.findList(workStaffAchivesLog);
                        try {
                                Method setReadOnly = workStaffRecord.getClass().getMethod(setMethod, String.class);
                                if(logs!=null&&logs.size()>0){
                                    setReadOnly.invoke(workStaffRecord,logs.get(0).getNewKey());
                                }
                        } catch (NoSuchMethodException e) {
                            e.printStackTrace();
                        } catch (IllegalAccessException e) {
                            e.printStackTrace();
                        } catch (InvocationTargetException e) {
                            e.printStackTrace();
                        }
                    }
                }
                newList.add(workStaffRecord);
            }
        }
        //删除
        WorkStaffAchivesLog delLog=new WorkStaffAchivesLog();
        delLog.setStaffId(workStaffBasicInfo.getId());
        delLog.setModule("电子档案");
        delLog.setType("删除");
        workStaffAchivesLog.setState("1");
        List<WorkStaffAchivesLog> dellogs=workStaffAchivesLogService.findList(delLog);
        if(dellogs!=null&&dellogs.size()>0){
            for(WorkStaffAchivesLog w:dellogs){
                newList.remove(dao.get(w.getSonId()));
            }
        }
        workStaffBasicInfo.setRecordList(newList);
    }
    @Transactional(readOnly = false)
    public void saveEdu(WorkStaffBasicInfo workStaffBasicInfo) {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        List<WorkStaffRecord> recordList =workStaffBasicInfo.getRecordList();
        if(recordList!=null&&recordList.size()>0){
            for(WorkStaffRecord newRecord:recordList){
                Class<? extends WorkStaffRecord> newClass = newRecord.getClass();
                Field[] declaredFields = newClass.getDeclaredFields();
                if(newRecord.getId()!=null&&!"1".equals(newRecord.getDelFlag())&&!"".equals(newRecord.getId())){
                    if(newRecord.getFile()!=null&&!newRecord.getFile().isEmpty()&&newRecord.getFile().getSize()>0){
                        MultipartFile file = newRecord.getFile();
                        newRecord.setFilePath(this.uploadFile(file,newRecord.getFilePath()));
                    }
                    WorkStaffRecord oldRecord=this.get(newRecord.getId());
                    if(oldRecord!=null){
                        Class<? extends WorkStaffRecord> oldClass = oldRecord.getClass();
                        for (Field field : declaredFields) {
                            String name = field.getName();
                            if("name".equals(name)||"num".equals(name)||"filePath".equals(name)||"status".equals(name)||"fileName".equals(name)){
                                String methodName = "get"+name.substring(0,1).toUpperCase()+name.substring(1);
                                WorkStaffAchivesLog workStaffAchivesLog=new WorkStaffAchivesLog();
                                workStaffAchivesLog.setStaffId(workStaffBasicInfo.getId());
                                workStaffAchivesLog.setModule("电子档案");
                                workStaffAchivesLog.setSonId(newRecord.getId());
                                workStaffAchivesLog.setFields(name);
                                workStaffAchivesLog.setState("1");
                                try {
                                    Object newInvoke = newClass.getMethod(methodName).invoke(newRecord);//修改后
                                    Object oldInvoke = oldClass.getMethod(methodName).invoke(oldRecord);//修改前
                                    String describes=EDU_MAP.get(name);
                                    if(newInvoke instanceof String){
                                        String newValue=newInvoke.toString();
                                        String oldValue=oldInvoke.toString();
                                        if(!newValue.equals(oldValue)){
                                            if("name".equals(name)){
                                                workStaffAchivesLogService.saveStaffLog(workStaffAchivesLog,workStaffBasicInfo.getId(),
                                                        "电子档案",describes,name,
                                                        oldValue,
                                                        newValue,
                                                        oldValue,newValue,newRecord.getId(),"修改");
                                            }else{
                                                workStaffAchivesLogService.saveStaffLog(workStaffAchivesLog,workStaffBasicInfo.getId(),
                                                        "电子档案",describes,name,
                                                        oldInvoke.toString(),newInvoke.toString(),
                                                        oldInvoke.toString(),newInvoke.toString(),newRecord.getId(),"修改");
                                            }
                                        }
                                    }
                                    if(newInvoke instanceof Date){//日期
                                        String newDate="";
                                        String oldDate="";
                                        if(newInvoke!=null){
                                            newDate=sdf.format(newInvoke);
                                        }
                                        if (oldInvoke!=null){
                                            oldDate=sdf.format(oldInvoke);
                                        }
                                        if(!newDate.equals(oldDate)){
                                            workStaffAchivesLogService.saveStaffLog(workStaffAchivesLog,workStaffBasicInfo.getId(),
                                                    "电子档案",describes,name,
                                                    oldDate,newDate,oldDate,newDate,newRecord.getId(),"修改");
                                        }
                                    }
                                } catch (NoSuchMethodException e) {
                                    e.printStackTrace();
                                } catch (IllegalAccessException e) {
                                    e.printStackTrace();
                                } catch (InvocationTargetException e) {
                                    e.printStackTrace();
                                }
                            }
                        }

                    }
                }else if((newRecord.getId()==null||"".equals(newRecord.getId()))&&"0".equals(newRecord.getDelFlag())){//新增
                    if(newRecord.getFile()!=null&&!newRecord.getFile().isEmpty()&&newRecord.getFile().getSize()>0){
                        MultipartFile file = newRecord.getFile();
                        newRecord.setFilePath(this.uploadFile(file,newRecord.getFilePath()));
                    }
                    String newId=IdGen.uuid();
                    for (Field field : declaredFields) {
                        String name = field.getName();
                        if("name".equals(name)||"num".equals(name)||"filePath".equals(name)||"status".equals(name)||"fileName".equals(name)){
                            String methodName = "get"+name.substring(0,1).toUpperCase()+name.substring(1);
                            WorkStaffAchivesLog workStaffAchivesLog=new WorkStaffAchivesLog();
                            workStaffAchivesLog.setStaffId(workStaffBasicInfo.getId());
                            workStaffAchivesLog.setModule("电子档案");
                            workStaffAchivesLog.setSonId(newId);
                            workStaffAchivesLog.setFields(name);
                            workStaffAchivesLog.setState("1");
                            try {
                                Object newInvoke = newClass.getMethod(methodName).invoke(newRecord);//修改后
                                String describes=EDU_MAP.get(name);
                                if(newInvoke instanceof String){
                                    String newValue=newInvoke.toString();
                                    if("name".equals(name)){
                                        workStaffAchivesLogService.saveStaffLog(workStaffAchivesLog,workStaffBasicInfo.getId(),
                                                "电子档案",describes,name,
                                                "",
                                                newValue,
                                                "",newValue,newId,"新增");
                                    }else{
                                        workStaffAchivesLogService.saveStaffLog(workStaffAchivesLog,workStaffBasicInfo.getId(),
                                                "电子档案",describes,name,
                                                "",newInvoke.toString(),
                                                "",newInvoke.toString(),newId,"新增");
                                    }
                                }
                                if(newInvoke instanceof Date){//日期
                                    String newDate=sdf.format(newInvoke);
                                    workStaffAchivesLogService.saveStaffLog(workStaffAchivesLog,workStaffBasicInfo.getId(),
                                            "电子档案",describes,name,
                                            "",newDate,"",newDate,newId,"新增");
                                };
                            } catch (NoSuchMethodException e) {
                                e.printStackTrace();
                            } catch (IllegalAccessException e) {
                                e.printStackTrace();
                            } catch (InvocationTargetException e) {
                                e.printStackTrace();
                            }
                        }
                    }
                }else if(newRecord.getId()!=null&&"1".equals(newRecord.getDelFlag())&&!"".equals(newRecord.getId())){//删除
                    WorkStaffAchivesLog workStaffAchivesLog=new WorkStaffAchivesLog();
                    workStaffAchivesLog.setStaffId(workStaffBasicInfo.getId());
                    workStaffAchivesLog.setModule("电子档案");
                    workStaffAchivesLog.setSonId(newRecord.getId());
                    workStaffAchivesLog.setType("删除");
                    workStaffAchivesLog.setState("1");
                    workStaffAchivesLogService.saveStaffLog(workStaffAchivesLog,workStaffBasicInfo.getId(),
                            "电子档案","","",
                            "","",
                            "","",newRecord.getId(),"删除");
                }
            }
        }
    }
}