/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.sys.service;

import com.google.common.collect.Maps;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.json.AjaxJson;
import com.jeeplus.common.oss.OSSClientUtil;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.service.CrudService;
import com.jeeplus.common.utils.FileUtils;
import com.jeeplus.common.utils.IdGen;
import com.jeeplus.common.utils.JPushClientUtil;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.modules.iim.entity.LayGroup;
import com.jeeplus.modules.iim.service.LayGroupService;
import com.jeeplus.modules.pushinfo.entity.Pushinfo;
import com.jeeplus.modules.pushinfo.service.PushinfoService;
import com.jeeplus.modules.serialnum.dao.SerialNumTplDao;
import com.jeeplus.modules.serialnum.entity.SerialNumTpl;
import com.jeeplus.modules.sys.dao.CompanyDao;
import com.jeeplus.modules.sys.dao.RoleDao;
import com.jeeplus.modules.sys.dao.UserDao;
import com.jeeplus.modules.sys.entity.*;
import com.jeeplus.modules.sys.utils.UserUtils;
import com.jeeplus.modules.sysuseroffice.entity.Useroffice;
import com.jeeplus.modules.sysuseroffice.service.UserofficeService;
import com.jeeplus.modules.tools.utils.TwoDimensionCode;
import com.jeeplus.modules.utils.ErrorCode;
import com.jeeplus.modules.workcompanyinfo.service.CompanyinfoService;
import com.jeeplus.modules.workprojectnotify.entity.WorkProjectNotify;
import com.jeeplus.modules.workprojectnotify.service.WorkProjectNotifyService;
import com.jeeplus.modules.workprojectnotify.util.UtilNotify;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletRequest;
import java.io.InputStream;
import java.math.BigDecimal;
import java.net.HttpURLConnection;
import java.net.URL;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;

/**
 * 企业管理Service
 *
 * @author meng
 * @version 2017-03-15
 */
@Service
@Transactional(readOnly = true)
public class CompanyService extends CrudService<CompanyDao, Company> {

    @Autowired
    private SystemService systemService;
    @Autowired
    private OfficeService officeService;
    @Autowired
    private CompanyDao companyDao;
    @Autowired
    private WorkProjectNotifyService workProjectNotifyService;
    @Autowired
    private PushinfoService pushinfoService;
    @Autowired
    private SysRoleActivityService sysRoleActivityService;
    @Autowired
    private RoleDao roleDao;
    @Autowired
    private UserDao userDao;
    @Autowired
    private LayGroupService layGroupService;
    @Autowired
    private SerialNumTplDao serialNumTplDao;
    @Autowired
    private CompanyinfoService companyinfoService;

    private Lock lock = new ReentrantLock();

    public Company get(String id) {
        return super.get(id);
    }

    public List<Company> getByCompanyId(Company company) {
        return dao.getByCompanyId(company);
    }

    public List<Company> getByCompany(String companyId) {
        return dao.getByCompany(companyId);
    }

    public List<Company> findList(Company company) {
        return super.findList(company);
    }

    public Page<Company> findPage(Page<Company> page, Company company) {
        if (company == null){
            company = new Company();
            company.setPage(page);
            page.setList(new ArrayList<Company>());
            return page;
        }else {
            company.setPage(page);
            page.setList(dao.findList(company));
            return page;
        }
    }

    public Page<Company> findPages(Page<Company> page, Company company) {
        if (company == null){
            company = new Company();
            company.setPage(page);
            page.setList(new ArrayList<Company>());
            return page;
        }else {
            company.setPage(page);
            page.setList(dao.findLists(company));
            return page;
        }
    }

    @Autowired
    private UserofficeService userofficeService;

    @Transactional(readOnly = false)
    public void save(Company company) {
        if (StringUtils.isBlank(company.getUseableStatus())){
            company.setUseableStatus("1");
        }
        super.save(company);
    }

    @Transactional(readOnly = false)
    public void addCompany(Company company) {
        super.save(company);

        //给admin发送通知
        WorkProjectNotify notify = new WorkProjectNotify();
        notify.setTitle("企业:"+company.getName()+"的申请待审批");
        notify.setContent("申请企业");
        notify.setRemarks("待审批");
        notify.setCompanyId("1");
        notify.setType("18");
        notify.setUser(new User("1"));
        notify.setNotifyId(company.getId());
        notify.setStatus("0");
        workProjectNotifyService.save(notify);
    }
    @Transactional(readOnly = false)
    public AjaxJson userAddCompany(Company company) {
        AjaxJson ajaxJson = new AjaxJson();
        try {
            lock.lock();
            Map<String, Object> variables = Maps.newHashMap();
            Useroffice companyinfo = new Useroffice();
            String companyId = company.getCompanyId();
            if(UserUtils.getSelectCompany().getId().equals(companyId)){
                logger.error("加入公司流程失败：您已经是该企业员工");
                ajaxJson.setSuccess(false);
                ajaxJson.setErrorCode(ErrorCode.code_1009);
                ajaxJson.setMsg("您已经是该企业员工,不能重复加入");
                return ajaxJson;
            }
            //该公司下所有的岗位
            List<Role> roleList = systemService.getRoleList("",companyId);
            Role role = new Role();
            if (roleList!=null && roleList.size()!=0){
                for (Role r :roleList){
                    //要的是管理员
                    if (org.apache.commons.lang3.StringUtils.isNotBlank(r.getEnname()) && r.getEnname().equals("gly")){
                        role = r;
                        break;
                    }
                }
            }
            //该公司下拥有管理员岗位的所有员工
            List<User> userList = userDao.findUserByRole(role);
            //该公司下拥有管理员岗位的所有员工
            List<String> assigneeList = new ArrayList<String>();
            if (userList!=null && userList.size()!=0) {
                for (User user : userList) {
                    assigneeList.add(user.getId());
                }
            }

            variables.put("glyList", assigneeList);
            String delFlag="0";
            variables.put("glycount",assigneeList.size());
            companyinfo.setUserId(UserUtils.getUser().getId());//申请人
            companyinfo.setCompanyId(companyId);//这一步必须
            List<Useroffice> companyinfoList = companyinfoService.findListByCompany(companyinfo);
            boolean state = false;
            if (companyinfoList!=null){
                if (companyinfoList.size()==0) {
                    state = true;
                }/*else{
                    for (Useroffice companyinfo1:companyinfoList){
                        if (companyinfo1.getStatus().equals("4")){
                            state = true;
                        }else {
                            state = false;
                            break;
                        }
                    }
                }*/
            }
            if (state){
                companyinfo.setStatus("1");
                companyinfo.setDelFlag(delFlag);
                companyinfo.setRemarks("");
                companyinfo.setMaster("");//
                companyinfoService.save(companyinfo, variables);
                ajaxJson.setSuccess(true);
                ajaxJson.setErrorCode(ErrorCode.code_1004);
                ajaxJson.setMsg("加入公司申请已经提交");
                ajaxJson.put("id", companyinfo.getId());
                String title = "团队申请";
                String content="您有新的团队申请";
                Map extras = new HashMap();
                Pushinfo pushinfo = new Pushinfo();
                pushinfo.setCurrentUser(UserUtils.getUser());
                Office office = officeService.get(companyId);
                String reason = UserUtils.getUser().getName()+ "\n"
                        +"申请加入："+office==null || office.getName()==null || office.getName().equals("")?"":office.getName();
                pushinfo.setRemarks(reason);
                pushinfo.setUserId(UserUtils.getUser().getId());
                pushinfo.setType("2002");
                pushinfo.setPushId(companyinfo.getId());
                pushinfo.setTitle(title);
                pushinfo.setContent(content);
                pushinfo.setParentType("singleCompanyNews");
                pushinfo.setCompanyId(companyId);
                pushinfo.setAddcontent("joincompany");
                pushinfo.setMobile("ios,android");
                pushinfo.setStatus("0");
                extras.put("type", "2002");
                extras.put("id", companyinfo.getId());
                extras.put("userId",UserUtils.getUser().getId());
                extras.put("userName",UserUtils.getUser().getName());
                //向所有管理员发送推送
                int num = 0 ;
                SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                String str = "申请人:"+UserUtils.getUser().getName()+",申请时间:"+sdf.format(new Date());
                WorkProjectNotify workProjectNotify = UtilNotify
                        .saveNotify(companyinfo.getId(),
                                null,
                                companyId,
                                title,
                                str,
                                "75",
                                "0",
                                "待审批",
                                ""
                        );
                if(assigneeList!=null && assigneeList.size()>0){
                    for(String alias : assigneeList){
                        if(StringUtils.isNotBlank(alias)){
                            pushinfo.setPushUserId(alias);
                            pushinfo.setId("");
                            pushinfoService.save(pushinfo);
                            boolean b = JPushClientUtil.sendNotificationToAlias(title, content, extras, alias);
                            if(b) num++;
                            workProjectNotify.setUser(new User(alias));
                            workProjectNotify.setId("");
                            workProjectNotify.setNotifyRole("公司管理员审批");
                            workProjectNotifyService
                                    .save(workProjectNotify);
                            UserUtils.pushIm(alias,str);
                        }
                    }
                }

                if (num>0){
                    ajaxJson.setMsg("申请加入公司流程成功!");
                }else {
                    ajaxJson.setMsg("申请加入公司流程成功");
                    ajaxJson.setErrorCode(ErrorCode.code_2006);
                    ajaxJson.setSuccess(false);
                }
            }else{
                logger.error("申请加入公司流程失败：您已申请加入企业");
                ajaxJson.setSuccess(false);
                ajaxJson.setErrorCode(ErrorCode.code_1009);
                ajaxJson.setMsg("您已申请加入企业,不能重复申请加入");
                return ajaxJson;
            }

        } catch (Exception e) {
            logger.error("申请加入公司流程失败：", e);
            ajaxJson.setSuccess(false);
            ajaxJson.setErrorCode(ErrorCode.code_2004);
            ajaxJson.setMsg("申请加入公司流程失败");
        }finally {
            lock.unlock();
        }
        return ajaxJson;
    }

    /**
     * @param company 修改原因：企业退出后删除了用户的信息，改为保留用户信息和企业信息，物理删除企业与用户管理表
     *                修改部分：super.delete(company);删除用户信息改为删除关联关系
     *                super.delete(company);删除用户信息之前代码
     */
    @Transactional(readOnly = false)
    public String delete(Company company, int type) {
        if (type == 1) {
            //根据公司id查询公司下所有的用户
            List<Useroffice> userOfficeList = companyDao.updateUserCompanyId(company);
            if (userOfficeList != null && userOfficeList.size() > 0) {
                for (int i = 0; i < userOfficeList.size(); i++) {
                    //判断公司id和用户的公司id相同且用户有多个公司就删除公司
                    //查询用户所在的公司
                    String userId = userOfficeList.get(i).getUserId();
                    //systemService.removeSession(new User(userId));
                    //UserUtils.clearCache(UserUtils.get(userId));
                    List<Useroffice> userCompanys = companyDao.selectUserCompanyIds(userId);
                    //如果用户在多个公司就物理删除当前用户和公司岗位的关系和公司的关系
                    if (userCompanys.size() > 1) {
                        for (int j = 0; j < userCompanys.size(); j++) {
                            if (userCompanys.get(j).getCompanyId().equals(company.getCompanyId())) {
                                companyDao.deleteRoleUserId(company.getCompanyId(), userId);
                                //查询用户表对应的公司和部门信息
                                User user = companyDao.selectUser(userOfficeList.get(i).getUserId());
                                if (user.getComId().equals(company.getCompanyId())) {
                                    String comId = null;
                                    String offId = null;
                                    if (j == 0) {
                                        comId = userCompanys.get(j + 1).getCompanyId();
                                        offId = userCompanys.get(j + 1).getOfficeId();
                                    } else if (j > 0) {
                                        comId = userCompanys.get(0).getCompanyId();
                                        offId = userCompanys.get(0).getOfficeId();
                                    }
                                    //更新用户信息中对应的公司和部门ID
                                    companyDao.updateUserComOffId(comId, offId, userId);
                                }
                                //对用户和公司关联表进行逻辑删除
                                companyDao.updateUserOfficeId(company.getCompanyId(), userId);
                                deletePush(company.getCompanyId(),userId);
                                //对目前公司记录数据进行逻辑删除
                                companyDao.updateCompanyId(company);
                                //未审核或者为审核通过的企业
                                companyDao.deleteCompanyInfo(company);
                                //对岗位表进行逻辑删除
                                companyDao.updateRoleId(company);
                                //对sys_office公司 逻辑删除
                                companyDao.updateOfficeId(company);
                                //删除work_app_show表（）工作协同办公
                                companyDao.deleteWorkAppShow(company.getCompanyId(), userId);
                            }
                        }
                    } else if (userCompanys.size() == 1) {//如果一个用户对应一个公司
                        //用户表信息的公司为1和部门为5
                        companyDao.updateUserComOffId("1", "5", userId);
                        //修改用户和公司的关系
                        Company coms = new Company();
                        coms.setCompanyId(userCompanys.get(0).getCompanyId());
                        coms.setUserId(userOfficeList.get(i).getUserId());
                        List<String> samelist = companyDao.selectUserSameRole(company);
                        if (samelist != null && samelist.size() > 0) {
                            companyDao.deleteRoleUserId(company.getCompanyId(), company.getUserId());
                        } else {
                            //逻辑删除用户所对应的岗位表信息，岗位id变为1
                            companyDao.updateNewCompany(userOfficeList.get(i).getUserId());
                            companyDao.deleteRoleUsers(coms);
                        }
                        //对用户和公司关联表进行逻辑删除
                        companyDao.updateUserOfficeId(company.getCompanyId(), userId);
                        deletePush(company.getCompanyId(),userId);
                        //对目前公司记录数据进行逻辑删除
                        companyDao.updateCompanyId(company);
                        companyDao.deleteCompanyInfo(company);
                        //对岗位表进行逻辑删除sys_role

                        companyDao.updateRoleId(company);
                        //对sys_office公司 逻辑删除
                        companyDao.updateOfficeId(company);
                        companyDao.deleteWorkAppShow(company.getCompanyId(), userId);
                    }
                }
            } else {
                companyDao.deleteCompanyId(company);
            }
        } else {
            /**
             * 	查询当事人有几个公司
             */
            List<Useroffice> list = companyDao.selectUserCompanyIds(company.getUserId());
            if (list != null && list.size() > 1) {//多家公司
                for (int i = 0; i < list.size(); i++) {
                    //如果当前用户的公司和退出的公司相同
                    if (list.get(i).getCompanyId().equals(company.getCompanyId())) {
                        //物理删除岗位和用户表数据
                        companyDao.deleteRoleUserId(company.getCompanyId(), company.getUserId());
                        //根据公司id和用户id逻辑删除公司和用户基本信息sys_user_office
                        companyDao.deleteUserOfficeId(company);
                        deletePush(company.getCompanyId(),company.getUserId());
                        company.getUserId();
                        //逻辑删除公司基本信息sys_company
                        companyDao.updateCompanyUserId(company);
                        //逻辑删除sys_company_info信息
                        companyDao.deleteCompanyInfo(company);
                        companyDao.deleteWorkAppShow(company.getCompanyId(), company.getUserId());
                        //查询用户表对应的公司和部门信息
                        User user = companyDao.selectUser(list.get(i).getUserId());
                        if (user.getComId().equals(company.getCompanyId())) {
                            String comId = null;
                            String offId = null;
                            if (i == 0) {
                                comId = list.get(i + 1).getCompanyId();
                                offId = list.get(i + 1).getOfficeId();
                            } else if (i > 0) {
                                comId = list.get(0).getCompanyId();
                                offId = list.get(0).getOfficeId();
                            }
                            //更新用户信息中对应的公司和部门ID
                            companyDao.updateUserComOffId(comId, offId, company.getUserId());
                        }
                        break;
                    }
                }
            } else if (list.size() == 1) {
                //逻辑删除用户表信息，企业id变为1，部门id变为5
                companyDao.updateUserComOffId("1", "5", company.getUserId());
                //查询一个用户的岗位已经有了则删除现在的岗位信息
                List<String> samelist = companyDao.selectUserSameRole(company);
                if (samelist != null && samelist.size() > 0) {
                    companyDao.deleteRoleUserId(company.getCompanyId(), company.getUserId());
                } else {
                    //逻辑删除用户所对应的岗位表信息，岗位id变为1
                    companyDao.deleteRoleUsers(company);
                }
                List<Role> roleList = new ArrayList<>();
                User user = new User(company.getUserId());
                Role role1 = new Role();
                role1.setId("1");
                role1.setOffice(new Office("5"));
                roleList.add(role1);
                user.setRoleList(roleList);
                userDao.insertUserRole(user);
                companyDao.deleteWorkAppShow(company.getCompanyId(), company.getUserId());
                //根据公司id和用户id逻辑删除公司和用户基本信息sys_user_office
                companyDao.deleteUserOfficeId(company);
                deletePush(company.getCompanyId(),company.getUserId());
                //逻辑删除公司基本信息sys_company
                companyDao.updateCompanyUserId(company);
                //逻辑删除sys_company_info信息
                companyDao.deleteCompanyInfo(company);
            } else {
                return "Invalid";
            }
            Office office = officeService.get(company.getCompanyId());
               /* String groupId = office.getGroupId();
                String userId = company.getUserId();
                easemobChatGroup.removeSingleUserFromChatGroup(groupId, userId);
                EasemobInfo e = new EasemobInfo();
                e.setUserId(userId);
                e.setJoinStatus("2");
                e.setGroupId(groupId);
                e.setDelFlag("1");
                easemobInfoService.deleteByUserIdAndGroupId(e);*/
        }
        return "Success";
    }

    @Transactional(readOnly = false)
    public void deletePush(String companyId,String userId){
        Useroffice useroffice = new Useroffice();
        useroffice.setCompanyId(companyId);
        useroffice.setUserId(userId);
        List<Useroffice> useroffices = userofficeService.findList(useroffice);
        if (useroffices!=null && useroffices.size()!=0){
            for (Useroffice info:useroffices){
                Pushinfo pushinfo = new Pushinfo();
                pushinfo.setPushId(info.getId());
                List<Pushinfo> pushinfos = pushinfoService.findList(pushinfo);
                if (pushinfos!=null && pushinfos.size()!=0){
                    pushinfoService.updateDelflagByPushId(info.getId(),"1");
                }
            }
        }
    }

    @Transactional(readOnly = false)
    public void deleteCompanyId(Company company) {
        companyDao.deleteCompanyId(company);
    }

    @Transactional(readOnly = false)
    public Office createOffice(Company company) {
        Office office = new Office();
        //父级编号
        office.setParent(new Office("0"));
        // 归属区域
        office.setArea(new Area("1"));
        // 自动获取排序号  获取所有的用户岗位
        List<Office> list = officeService.findAll();
        int max = 10;
        for (int i = 0; i < list.size(); i++) {
            Office e = list.get(i);

            if ("1".equals(e.getType())) {
                String code = e.getCode();
                int num = Integer.parseInt(code.substring(0, 2));
                if (max < num) {
                    max = num;
                }
            }
        }
        max = max + 1;
        office.setCode(max + "0000");
        office.setType("1");
        office.setGrade("1");
        office.setName(company.getName());
        office.setEmail(company.getEmail());
        office.setPhone(company.getPhone());
        office.setMaster(company.getTuser().getId());
        office.setEmail(company.getEmail());
        office.setUseable("1");
        office.setTopCompany(company.getName());
        String logo = "";
        if (company.getLogo() != null) {
            logo = company.getLogo();
        }
        office.setLogo(logo);
        officeService.save(office);
        return office;
    }

    @Transactional(readOnly = false)
    public Office createOfficeDept(Office office,int type) {
        Office dept = new Office();
        dept.setParent(office);
        dept.setArea(office.getArea());
        // 自动获取排序号
        Double code = Double.valueOf(office.getCode()) + 1;
        BigDecimal bd1 = new BigDecimal(code);
        dept.setCode(bd1.toPlainString());
        dept.setType("2");
        dept.setGrade("1");
        dept.setState("1");
        dept.setName("人力资源部");
        dept.setEmail(office.getEmail());
        dept.setPhone(office.getPhone());
        dept.setMaster(office.getMaster());
        dept.setEmail(office.getEmail());
        dept.setUseable("1");
        dept.setTopCompany(office.getName()+"-"+"人力资源部");
        dept.setIsSon(office.getIsSon());
        officeService.save(dept);
        return dept;
    }

    @Transactional(readOnly = false)
    public Office createOfficeDeptAND(Office office,int type) {
        Office dept = new Office();
        dept.setParent(office);
        dept.setArea(office.getArea());
        // 自动获取排序号
        Double code = Double.valueOf(office.getCode()) + 1;
        BigDecimal bd1 = new BigDecimal(code);
        dept.setCode(bd1.toPlainString());
        dept.setType("2");
        dept.setGrade("1");
        dept.setState("0");
        dept.setName("人力资源部");
        dept.setEmail(office.getEmail());
        dept.setPhone(office.getPhone());
        dept.setMaster(office.getMaster());
        dept.setEmail(office.getEmail());
        dept.setBranchOffice(office.getId());
        dept.setUseable("1");
        dept.setTopCompany(office.getName()+"-"+"人力资源部");
        dept.setIsSon(office.getIsSon());
        officeService.save(dept);
        return dept;
    }


    /*@Transactional(readOnly = false)
    public Office createOfficeDeptZB(Office office) {
        Office dept = new Office();
        dept.setParent(office);
        dept.setArea(office.getArea());
        // 自动获取排序号
        Double code = Double.valueOf(office.getCode()) + 1;
        BigDecimal bd1 = new BigDecimal(code);
        dept.setCode(bd1.toPlainString());
        dept.setType("2");
        dept.setGrade("1");
        dept.setName("招标部");
        dept.setEmail(office.getEmail());
        dept.setPhone(office.getPhone());
        dept.setMaster(office.getMaster());
        dept.setEmail(office.getEmail());
        dept.setUseable("1");
        officeService.save(dept);
        return dept;
    }*/

    @Transactional(readOnly = false)
    public Office createOfficeSCB(Office office,int type) {
        Office dept = new Office();
        dept.setParent(office);
        dept.setArea(office.getArea());
        // 自动获取排序号
        Double code = Double.valueOf(office.getCode()) + 1;
        BigDecimal bd1 = new BigDecimal(code);
        dept.setCode(bd1.toPlainString());
        dept.setType("2");
        dept.setGrade("1");
        dept.setName("市场部");
        dept.setEmail(office.getEmail());
        dept.setPhone(office.getPhone());
        dept.setMaster(office.getMaster());
        dept.setEmail(office.getEmail());
        dept.setUseable("1");
        if(type == 2 ) {
            dept.setBranchOffice(office.getId());
        }
        dept.setTopCompany(office.getName()+"-"+"市场部");
        dept.setIsSon(office.getIsSon());
        officeService.save(dept);
        return dept;
    }
    @Transactional(readOnly = false)
    public Office createOfficeBGS(Office office,int type) {
        Office dept = new Office();
        dept.setParent(office);
        dept.setArea(office.getArea());
        // 自动获取排序号
        Double code = Double.valueOf(office.getCode()) + 1;
        BigDecimal bd1 = new BigDecimal(code);
        dept.setCode(bd1.toPlainString());
        dept.setType("2");
        dept.setGrade("1");
        dept.setName("办公室");
        dept.setEmail(office.getEmail());
        dept.setPhone(office.getPhone());
        dept.setMaster(office.getMaster());
        dept.setEmail(office.getEmail());
        dept.setUseable("1");
        if(type == 2 ) {
            dept.setBranchOffice(office.getId());
        }
        dept.setTopCompany(office.getName()+"-"+"办公室");
        dept.setIsSon(office.getIsSon());
        officeService.save(dept);
        return dept;
    }
    @Transactional(readOnly = false)
    public Office createOfficeZLJG(Office office,int type) {
        Office dept = new Office();
        dept.setParent(office);
        dept.setArea(office.getArea());
        // 自动获取排序号
        Double code = Double.valueOf(office.getCode()) + 1;
        BigDecimal bd1 = new BigDecimal(code);
        dept.setCode(bd1.toPlainString());
        dept.setType("2");
        dept.setGrade("1");
        dept.setName("质量监管部");
        dept.setEmail(office.getEmail());
        dept.setPhone(office.getPhone());
        dept.setMaster(office.getMaster());
        dept.setEmail(office.getEmail());
        dept.setUseable("1");
        if(type == 2 ) {
            dept.setBranchOffice(office.getId());
        }
        dept.setTopCompany(office.getName()+"-"+"质量监管部");
        dept.setIsSon(office.getIsSon());
        officeService.save(dept);
        return dept;
    }
    @Transactional(readOnly = false)
    public Office createOfficeYWB(Office office,int type) {
        Office dept = new Office();
        dept.setParent(office);
        dept.setArea(office.getArea());
        // 自动获取排序号
        Double code = Double.valueOf(office.getCode()) + 1;
        BigDecimal bd1 = new BigDecimal(code);
        dept.setCode(bd1.toPlainString());
        dept.setType("2");
        dept.setGrade("1");
        dept.setName("业务部");
        dept.setEmail(office.getEmail());
        dept.setPhone(office.getPhone());
        dept.setMaster(office.getMaster());
        dept.setEmail(office.getEmail());
        dept.setUseable("1");
        if(type == 2 ) {
            dept.setBranchOffice(office.getId());
        }
        dept.setTopCompany(office.getName()+"-"+"业务部");
        dept.setIsSon(office.getIsSon());
        officeService.save(dept);
        return dept;
    }

    @Transactional(readOnly = false)
    public Office createOfficeFinance(Office office,int type) {
        Office dept = new Office();
        dept.setParent(office);
        dept.setArea(office.getArea());
        // 自动获取排序号
        Double code = Double.valueOf(office.getCode()) + 1;
        BigDecimal bd1 = new BigDecimal(code);
        dept.setCode(bd1.toPlainString());
        dept.setType("2");
        dept.setGrade("1");
        dept.setName("财务部");
        dept.setEmail(office.getEmail());
        dept.setPhone(office.getPhone());
        dept.setMaster(office.getMaster());
        dept.setEmail(office.getEmail());
        dept.setUseable("1");
        dept.setIsSon(office.getIsSon());
        if(type == 2 ) {
            dept.setBranchOffice(office.getId());
        }
        dept.setTopCompany(office.getName()+"-"+"财务部");
        officeService.save(dept);
        return dept;
    }


    @Transactional(readOnly = false)
    public Role createCompanyGDManagerRole( Office com,Office comOffice,Office office,String type) {
        Role role1 = new Role();
        role1.setName("归档管理员");
        role1.setSysData(Global.YES);
        role1.setDataScope("4");
        role1.setIsNewRecord(false);
        role1.setCompany(com);
        role1.setOffice(office);
        role1.setRoleType("assignment");
        role1.setSysData("1");
        role1.setUseable("1");
        role1.setNumber("5");
        if ("2".equals(type)) {
            role1.setBranchCompany(comOffice);
        }
        role1.setEnname("glygd");
        String defalutMenu1 = "1,27,29,e3b80fae59ab4b4099d8349ecaf8c4dc,61dd3aa279684f92bb1c5d4019a32503,b0c5737cdc4f452684ce93589427c97d,af6b0e5d61fc44ecbebb449703a3df72,18c14562978e4b129fe3afb7dca8b868,6123fcb3d810492789e82a63b0d4310a,0aa3712414d049a6a24e8bcddeae509a,95a6a82dc5fc4d07b46e5df57a0606a3,62,b6fa4abb624349918fe531274fc15d23,89,76bdb735faa34667a6cbf13ebe937533,90,f5b2028cfad9470085c7c846de33193a,28abe4b0cb3b4c8480f84c07a5dfaf06,c7fa36c7142f481397c2cc12e2bc828a,e2efcf80fc3747329fe02d489dbfda95,e3b5b7451e164089aba0b9bf844b0398,438c28c3cbe44ed7bff951b2a3dc77cd,59e50eebe56743a78eecbc6eff1ef94c,cc1027689aca41a89302217127253034,200fe6cc80174065b382c961a4379d79,4c13ca96376341dfa25980e86964c1a9,638be8b3b5284e5088003917685bc19b,811600aadcb54b098b7dda4bf95ffc51,ce8dc7c97d93481dbd76cabb6e905b55,aa25b3904abb4bd7b779e5be37b4382e,aae6f4c7fd4c4a25afa5285210edfded,84ff9d5c165c4528af9361b6127cf5af,31b349a3624b47f28eb86d614f9f95b5,040b42e395de41d6805a958eb1897863,34874260e6fc4200a76b33c3cfa4d80d,5b6c6ea5ed2d45478e8144351d853e21,7c032c32512847f882a87416359ef0a3,6d30ea8c710a42b89da2f2506059b0ef,2ca7bf4ce6804a1c9e5b331d2df27c72,7717cbb6d71f405d890e9b648cbd2088,bddb0400764c42b08e427059c7752057,23d6c56239804053910e5c5aec481330,7f8b2018113545bcbdbfc87e7a236bec,547e121ffb714d16ae142b6935abb78f,af24101b962444f29d166b638b2fea30,6ea6519ec4d548128aa42f3ce004def3,4379bf54c2a543869abf5031b97b8efe,5e6da3f5368b4f06b7268b31a1157e9f,2bbf0e75f21d42938954391d0776fbde,545b39dacf564a5eb9d1658f80a0b8b0,b4ddedf9b451494a97a1ff9476e3897d,1b145ca23d3e4d81994a84d30a262d18,10a8c7c1d34e46dd8ef461cdad928afc,5ae4911d14bb48b39885b1fe522e999d,7f09def2533b45fdb3c733f1e72ca5fb,9ef29dca19d342b795efcbfe32793602,be72d72ae7034c509c164467b2abddda,ab879705f2354813acf8724404520e28,82487b87b97b475b98892b80cf83c96a,0f4a461b97be48eeab43bd052f630057,72e1a28b7251438a89b0a4f6bd522b89,e1f9a246057347d78a1966d6dc0168e3,5d3645d53ce14c9ab498fd9f050ba264,d361e6890cc84b049fc8268c2a8c4952,5cc0002cbcc14d3485ab8ae6d57368f2,fa7bb243643c44b18e989e3419eeb171,62768b6c65134f1e927b1807bc72115a,7b69b0e5d2d344e6aaa2b8726c23561a,434bbb1f9f8140b1beeb57d0c4db660d,7920c4c8da294d87bcb94e7756bf35ae,0752b85660694033bbdb013f30cde17f,bd6258cfbea64896b0401517e440d6a2,9aa3539682514546866d2998d741ac6c,eb3ad93ce6a44921b0490c842dd47d8c,9ce19042d497473c92d199a5e5a7daf0,1b3eb6ba64ea4589a8e2d42efa5d3f89,6b2dd5ff2a754a53a80798341a7d35d8,e53d754160f74d28b6d091829fc289f8,92d762bca040407bb4ce4b06a2cfa195,aa8a2b00622147259c85290f4d95b46d,0be3e487d874487fbfb5c49b52440a48,ce60b25f7da04af8b7cefccf2dbc7d36,6464ac9cc3fc47d88598f6c4e74caa19,04b312df403d4855b013743206ddb1de,17356c44ae894782a1015ec2ddaeaeda,a06eaefba7084e219f0c6ce4010b82e7,95a54e4db7e74f0586cd85cb4e2bd6f7,1a93047d6b0f4e3ca70fcf836b2517c4,93263a29941648979bb8de774978a880,8eba834a28864375aa3137590d26f37e,3c52b0a890524c6fb66cefc32114c60e,8b20519b794c42d28fdecbd0ccbba5c1,77189ceba655488681964a609c28256d,39e776aaf3644d24bb78ecf43704b71b,0dd5fa1192e34d5fac5e78ca38969126,1a6f82a80a054bc7ba4b516187231a54,5993ce4ec80a41fb9d4ea0713b74f4a4,3b42071536124a9c9dbf2ec319004b29,a966beefe58e4f95b24e89f4172cb7ce,2e1e75dbe28841529576dd1cd48d2437,cdd8829e5976438aaa66ce608f0175f5,71015695cf1445ecacce90669c67d733,31e9dd26b7314d8a9bf97dcf6dbdb22d,e5ecaa091bb0464f8553d4e7aa60647d,39055be3a23945868dbf4b5930043a88,07ba7c9aed104657aac3be87aa346d3b,5a7a9bfff0f44a2089a7d277f2c04557,be53cc152339400ba705c1d1dceff9ab,ed591ccba0564abd9979899049dc7b36,9a4425b817e94dbe844d7fd3e96a9033,bf4e54bfe9984441a8efa4dd8a4a23a6,5a1bcf2a2e0943f88bf655ca011fb288,a12c8904ff0147a99010eaa5aa0ea704,e7c6f2581c6e41bf8fe88a3d7fadc986,89709767beea48d0b0fb047316eb37d1,04d6a49b35514f91be43e9576f8d32fc,bd50e783ec714e5d86aafcca54e87c33,4acb0c7d002b47e29be7ebe9f40490f6,67b5ddef866b4982aa5afef7aad69924,7e090632f713440bba11ed6fc09eb291,f308051e11ca47eebb12709725b52eb4,dbea976ef1d248cabf13b88d311c4533,e0132620d4044e38a92f75b8335005ed,1b598563135e42d4bfe729cf32649f14,c4de9219f4f14684ae21d0904b7b78df,520983ab7dd64104a47e273941236b2d,133ba69bd04046b89eadaf793b9063db,11f24ca662044292a8ca95930ccc3581,0f30b4d621294d4fab359611cf82ab6f,b3c93fcd54b544e5b2c8e70ee7eb93ce,79ebe6595ce44208853ba553186b8348,8d639afccbec43f2be8d3ed88702893b,4e3483f3bbbb48f59b0e92a7d0ccc1c0,b160fccc15fa4ba09c74018d82d16eb7,52f248eadea74cf2b7780bf991d95e4e,d4cb9017aa1f4f8ca34ae8d31c986210,eff19819b1284424a9ad7879991ee080,79551897f86b4338bd09b41c5953a7db,f11bd4ef5ddd4c82865fad3f1f9d80a9,76ee95093a8748b29d55df1cb8cdebd0,081af742f6384a20b492ca99704f51ca,c26c844bbb2f4746904948f3ecfeb1ae,ba9bbc5581a043d8b9c204b8ddb6aa01,cde1afb048a743bb9caedb5770aabca1,0b2b37a28bb0457bbc4bf33c11b4c6e1,9f94d338d6c94b95814b2c50f9e463d1,7d1745ce2c444db9bf4b4e100c476fe8,e711e93aadd54eef8feeb0fa07f054d2,0e914774210646f784d02fb22f8df7db,ca0ce5e9eb09477c8c99b7b94e7869ce,760d9072f9bc480fa36fdb876930467c,5742a628f13045cda0317c3d4d839bc5,b9309305d94f4ee8b66eebbcae9f2db4,a42216d3eedd48f0ba3d3bef2470605f,9cdcda30da704c21b19ea0616de3ea86,f05ecbf7e4b04b268b38cb6e5c8124bf,788d2c1496874eeeae3b7aabbab95f76,8b112af90e054671adc30373c6dafd21,3bdb29667807493292a0ddc9bc0ccdb7,ffd6ebecc869410dab6410a07f3362ae,196868a4610b45588062aed357fc7c58,4cdd2d11be4c4c098b27c10d7cb3fa9e,fda40125207d426dbf86270002ff0c4a,7af028c6048c44cc91415a58d25d286e,ff7d2c8e96e840e1a5a690e4e0f96bad,9f806cf868dc48d398836ebd60bd2ec6,50ff5cdc6c8645b3bb61b8327d204f01,bafb57ea8d84430cbc0c11e5c67643f1,e2c91860b96c46619a74215627560235,f5b68a43bbae44c8a1fe821dfcaca26e,70b771772a0b4f1cb1e592f9a01b08a5,7992a39d64b2483789ae274e2df3c01d,f8d711312da14871a77b58f41a6f6242,673b27b0ab9148b7ae03933c18433971,624886a950bd4d18900a877738696432,685d8d9267f745f39bcc6fc2cc553d31,7d40ad05a1a54165a28ec897ddbcef01,4bcdb5199e8442078999bb8611ea6398,3b0c0ffea0cf45db816993baf64b3fb3,ad2d6d9581034e04addcb5b034a88fb3,8440c4fc5056426996e3e488b2261d28,e367c4223d794572a05fd256759d6a53,1b2dc5784c3c41988d5634e566b4a404,11be1538d7f54d7786004503b2e684a7,b462cb8c2ec7485d938416831a9771cc,f747656a36204e9588aba97064332436,78d0b2337b484c6298c9b3afc01fdb85,8a63d1f8d4bc4eb4a4db437491c9983a,2247f64a15e54053abb2e13152003c6b,d78ef9c5997d4e94804b8126cc387ea1,6ce3cd5a86ef43219dbda2b78e974c93,414b5fd5ebee45d98eca809a524dbe93,4b9ea9db5c834b698a78c7052c87b432,8590fa7adc144f49ae3d2575b7280c08,6148e4b152b9494fb9e7035e9b58e624,be1cff2a29f64800a5c4f91009711988,49b85b35bca5461993eac91ffd0b1fe4,5abae35db6de475e904425af8dc8ecbb,e3bc44b82c3543758d5a6c02b41dd526,89323021918e444bae3537521046d19f,141dd3257f85455d86f3828cc0217b5a,7bc2ad90009a4de2a0fa2947ccc09886,e51874f4b6ac455db76e30582659368b,519dee074e4c48ae9edd58d7c1e330da,060ab186c22f4f42a83f1622bf47ca52,8873bbcf4b42445f8117108769358a5c,46fa5286441a444095b917165c31304c,fa9fdd56222a4ce7ba2b37922afe0eaa,6885932aa4b443babc2b69fdd495889c,be71b947b33a48d8aee36ad0c60d10b8,b52457719429466687a2f228d20571ac,0bf937d745494d998cb8bd2cf3a63f4e,c17feb3860184dd59931f35814ad68b0,4cc68cbd880c48948a1f18fe18ec938c,c37b0691a05c4d659dcd01571f1907da,c1334fe3e8484079bfd62e479307b5b9,ebb58027b6c647b7965eeb31b63ec0c0,1821c5f9392e4859a829c78ebda44df6,dca21ab79bd14004b94c55e36604c30d,8d96cc3457644378b9b643243464f735,631437054079401fbe2c3c9a3f3bc375,c2f78ab4372243da8775f4d71a16fef5,03792664a45d4f63ac3efb899ce1b61e,fec04c46683a41e5a344f4d057342829,320fa70de6bc4890a0135bbfe8735b49,5a03493dce9f4323a0da7bf4fd5e78ab,b9bd46a1da1846d099566fd5c6db00cb,647021c7f798472889fd82f6c4a27ca2,313e905b78d746cb9e574b9634c3a113,623fd328c4cd4ee8b21eb03425b1f100,4676b4d1130146629fd98e2d2eb9a5b4,34d380813a36481ea2534c46d18b5927,894d87e55ac640b99e966a8786a2392b,477ba74d182a4ec0b86926dc67a0f419,362ca5fdca174fa3bdd6325a947320db,1ac3ad8dfa184fe4b6815950d3d23fdd,fdebee18c71f4a9db60096e75ad34d83,91432be4178a4781917fba0abd51df61,480c71baa65145f4bd037332d68e5e4f,536e3a3702b0453d92939846dc1f33b7,0333aedf1be640c78d2ee02473b7fd97,cbb3a6f1772b4f72a8af1375ef16e914,367ad9c43a6246dfac364ca59874883b,bac18430f6db466195a3a7116cdfb958,c4ec39be11be4b0eae321ee0559be8e6,8f7ffae7e47d462aa4142db7f06633d5,82e3b2423d86425195929fc3810ce7a4,72973e0415004dfea220babc479be082,41d0f568bb1e4fed8ad965a79d9bccef,5854ef1d24784ca0a046514b19286113,04afc3dd1eec4d13bc06cd01e0a5e838,db76e3333da846d5ae55b3b9dd735f26,7aa0ca0bf1514fa6a3a14dcde44cf0b9,9c668726c9b6449a9045d3f78408a2a1,7caf148d68a6431bbe56bcc6472d8720,babd36d1d3484fb99fe4e7ca78560ccf,7507865343e24cff80d55a29ec57f0e1,63f8b7d9b1f34851802e0408bf997ea5,a3369d5dd6b74af7b35edf1f10350881,9b8b8c3f86854416b82d57583896c1ff,881815c772564a8dbd5eb6de65ecbe7a,4b1bcf46753a48258d1d514440140b7f,1ca4c6a49e794954a235a82ee8981ecd,ab381d9129bb4b979fe3774e262524f8,a07d2c98831a4d79bc6c5c4288a24720,d8debd28039d494aa7fe18bc8e81af27,d0402b1f00f94acca96276c6ea8c8397,ebcdebd9639d4ec2b7072ad32bf48f50,636b3211100245919c09177ccea562e0,280c4440e18a487396a81b92cd3efa13,5163cf6740334a7d83375385c3ae2b67,a6083e4fab7a43498bfb3257658f9278,3b5c8b5cfb124ebd8f4a798d2a47d832,9fda332474534f96955b5bce8548947f,b5f5e62aba9842d8b48a28111ca6a372,c0cecff1207c4692a634d7de449d8d6d,e4c6428f1d264544b0eec52214e7b6ae,cbc8c5490a7f46b996fd4c3a9711f2d7,cfbdd82f4ce741a88109353c5fc898e4,6bfb965bc1b240a9ae2bd7a1e884ea51,eb36d63a993148289950af9098727759,f7357c3954d2408facaa6771a758ebcc,c05853b94ab8405eb26e0a8bd481b662,da1e6d3ebfa140b3a645342fd3bb5d95,988f3f6b607d442b8df006c980ddc2cd,b6af3c01968141baa3e99ca4b8d83847,75f669faee5d4023a425f2461e5fc5e9,165eb05677aa45fc9a53dfd8bfb7a45d,409891afd7cc4e12ba6b002581fef4f3,02ee0a87956f4bfab91b88282e502a3d,48601418028145ff84a450c877ea35aa,8de73f8d59eb45aa8c61138ab93c4f86,5dd82a46f48945b4bbb61f58b9d1c856,97993db997bd4ff88acfe1340ca86845,282aee84915b43179a09101b3ee6ad0e,8750e994985f43f383321bf0d6ca7932";

        role1.setMenuIds(defalutMenu1);
        String[] defalutMenus1 = defalutMenu1.split(",");
        ArrayList<String> menuIdList1 = new ArrayList<String>();
        for (int i = 0; i < defalutMenus1.length; i++) {
            menuIdList1.add(defalutMenus1[i]);
        }
        role1.setMenuIdList(menuIdList1);
        if (Global.isDemoMode()) {
            return null;
        }
        systemService.saveRole(role1, true,1);
        SysRoleActivity sysRoleActivity = new SysRoleActivity();
        SysRoleActivity roleActivity = new SysRoleActivity();
        if ("1".equals(type)) {
            String enname = UserUtils.getRoleActivity(com.getId()) + StringUtils.getPinYinHeadChar("合同归档管理员", 1);
            sysRoleActivity.setName("合同归档管理员");
            sysRoleActivity.setEnname(enname);
            sysRoleActivity.setSort("4000");
            sysRoleActivity.setCompanyId(com.getId());
            sysRoleActivity.setType("3");
            sysRoleActivity.setDelType("1");
            sysRoleActivityService.save(sysRoleActivity);

            roleActivity.setEnname(UserUtils.getRoleActivity(com.getId()) + StringUtils.getPinYinHeadChar("报告归档管理员", 1));
            roleActivity.setName("报告归档管理员");
            roleActivity.setSort("4500");
            roleActivity.setCompanyId(com.getId());
            roleActivity.setType("3");
            roleActivity.setDelType("1");
            sysRoleActivityService.save(roleActivity);

        }else {
            String enname = UserUtils.getRoleActivityEnname(com.getId(),UserUtils.getUser());
            sysRoleActivity = sysRoleActivityService.getByEnname(enname+"htgdgly");
            roleActivity = sysRoleActivityService.getByEnname(enname+"bggdgly");
        }
        List<SysRoleActivity> roleActivities = new ArrayList<>();
        roleActivities.add(sysRoleActivity);
        roleActivities.add(roleActivity);
        role1.setRoleActivityList(roleActivities);
        roleDao.insertRoleActivity(role1);
        return role1;
    }

    @Transactional(readOnly = false)
    public Role createCompanyManagerRole( Office com, Office office,String type) {
        Role role1 = new Role();
        role1.setName("总经理");
        role1.setNumber("1");
        role1.setSysData(Global.YES);
        if (type.equals("1")){
            role1.setDataScope("2");
        }else {
            role1.setDataScope("5");
            role1.setBranchCompany(office);
        }
        role1.setIsNewRecord(false);
        role1.setCompany(com);
        role1.setOffice(office);
        role1.setRoleType("assignment");
        role1.setSysData("1");
        role1.setUseable("1");
        role1.setEnname("zjl");
        String defalutMenu1 = "1,27,29,e3b80fae59ab4b4099d8349ecaf8c4dc,61dd3aa279684f92bb1c5d4019a32503,b0c5737cdc4f452684ce93589427c97d,af6b0e5d61fc44ecbebb449703a3df72,18c14562978e4b129fe3afb7dca8b868,6123fcb3d810492789e82a63b0d4310a,0aa3712414d049a6a24e8bcddeae509a,95a6a82dc5fc4d07b46e5df57a0606a3,62,b6fa4abb624349918fe531274fc15d23,89,76bdb735faa34667a6cbf13ebe937533,90,f5b2028cfad9470085c7c846de33193a,28abe4b0cb3b4c8480f84c07a5dfaf06,c7fa36c7142f481397c2cc12e2bc828a,e2efcf80fc3747329fe02d489dbfda95,e3b5b7451e164089aba0b9bf844b0398,438c28c3cbe44ed7bff951b2a3dc77cd,59e50eebe56743a78eecbc6eff1ef94c,cc1027689aca41a89302217127253034,200fe6cc80174065b382c961a4379d79,4c13ca96376341dfa25980e86964c1a9,638be8b3b5284e5088003917685bc19b,811600aadcb54b098b7dda4bf95ffc51,ce8dc7c97d93481dbd76cabb6e905b55,aa25b3904abb4bd7b779e5be37b4382e,aae6f4c7fd4c4a25afa5285210edfded,84ff9d5c165c4528af9361b6127cf5af,31b349a3624b47f28eb86d614f9f95b5,040b42e395de41d6805a958eb1897863,34874260e6fc4200a76b33c3cfa4d80d,5b6c6ea5ed2d45478e8144351d853e21,7c032c32512847f882a87416359ef0a3,6d30ea8c710a42b89da2f2506059b0ef,2ca7bf4ce6804a1c9e5b331d2df27c72,7717cbb6d71f405d890e9b648cbd2088,bddb0400764c42b08e427059c7752057,23d6c56239804053910e5c5aec481330,7f8b2018113545bcbdbfc87e7a236bec,547e121ffb714d16ae142b6935abb78f,af24101b962444f29d166b638b2fea30,6ea6519ec4d548128aa42f3ce004def3,4379bf54c2a543869abf5031b97b8efe,5e6da3f5368b4f06b7268b31a1157e9f,2bbf0e75f21d42938954391d0776fbde,545b39dacf564a5eb9d1658f80a0b8b0,b4ddedf9b451494a97a1ff9476e3897d,1b145ca23d3e4d81994a84d30a262d18,10a8c7c1d34e46dd8ef461cdad928afc,5ae4911d14bb48b39885b1fe522e999d,7f09def2533b45fdb3c733f1e72ca5fb,9ef29dca19d342b795efcbfe32793602,be72d72ae7034c509c164467b2abddda,ab879705f2354813acf8724404520e28,82487b87b97b475b98892b80cf83c96a,0f4a461b97be48eeab43bd052f630057,72e1a28b7251438a89b0a4f6bd522b89,e1f9a246057347d78a1966d6dc0168e3,5d3645d53ce14c9ab498fd9f050ba264,d361e6890cc84b049fc8268c2a8c4952,5cc0002cbcc14d3485ab8ae6d57368f2,fa7bb243643c44b18e989e3419eeb171,62768b6c65134f1e927b1807bc72115a,7b69b0e5d2d344e6aaa2b8726c23561a,434bbb1f9f8140b1beeb57d0c4db660d,7920c4c8da294d87bcb94e7756bf35ae,0752b85660694033bbdb013f30cde17f,bd6258cfbea64896b0401517e440d6a2,9aa3539682514546866d2998d741ac6c,eb3ad93ce6a44921b0490c842dd47d8c,9ce19042d497473c92d199a5e5a7daf0,1b3eb6ba64ea4589a8e2d42efa5d3f89,6b2dd5ff2a754a53a80798341a7d35d8,e53d754160f74d28b6d091829fc289f8,92d762bca040407bb4ce4b06a2cfa195,aa8a2b00622147259c85290f4d95b46d,0be3e487d874487fbfb5c49b52440a48,ce60b25f7da04af8b7cefccf2dbc7d36,6464ac9cc3fc47d88598f6c4e74caa19,04b312df403d4855b013743206ddb1de,17356c44ae894782a1015ec2ddaeaeda,a06eaefba7084e219f0c6ce4010b82e7,95a54e4db7e74f0586cd85cb4e2bd6f7,1a93047d6b0f4e3ca70fcf836b2517c4,93263a29941648979bb8de774978a880,8eba834a28864375aa3137590d26f37e,3c52b0a890524c6fb66cefc32114c60e,8b20519b794c42d28fdecbd0ccbba5c1,77189ceba655488681964a609c28256d,39e776aaf3644d24bb78ecf43704b71b,0dd5fa1192e34d5fac5e78ca38969126,1a6f82a80a054bc7ba4b516187231a54,5993ce4ec80a41fb9d4ea0713b74f4a4,3b42071536124a9c9dbf2ec319004b29,a966beefe58e4f95b24e89f4172cb7ce,2e1e75dbe28841529576dd1cd48d2437,cdd8829e5976438aaa66ce608f0175f5,71015695cf1445ecacce90669c67d733,31e9dd26b7314d8a9bf97dcf6dbdb22d,e5ecaa091bb0464f8553d4e7aa60647d,39055be3a23945868dbf4b5930043a88,07ba7c9aed104657aac3be87aa346d3b,5a7a9bfff0f44a2089a7d277f2c04557,be53cc152339400ba705c1d1dceff9ab,ed591ccba0564abd9979899049dc7b36,9a4425b817e94dbe844d7fd3e96a9033,bf4e54bfe9984441a8efa4dd8a4a23a6,5a1bcf2a2e0943f88bf655ca011fb288,a12c8904ff0147a99010eaa5aa0ea704,e7c6f2581c6e41bf8fe88a3d7fadc986,89709767beea48d0b0fb047316eb37d1,04d6a49b35514f91be43e9576f8d32fc,bd50e783ec714e5d86aafcca54e87c33,4acb0c7d002b47e29be7ebe9f40490f6,67b5ddef866b4982aa5afef7aad69924,7e090632f713440bba11ed6fc09eb291,f308051e11ca47eebb12709725b52eb4,dbea976ef1d248cabf13b88d311c4533,e0132620d4044e38a92f75b8335005ed,1b598563135e42d4bfe729cf32649f14,c4de9219f4f14684ae21d0904b7b78df,520983ab7dd64104a47e273941236b2d,133ba69bd04046b89eadaf793b9063db,11f24ca662044292a8ca95930ccc3581,0f30b4d621294d4fab359611cf82ab6f,b3c93fcd54b544e5b2c8e70ee7eb93ce,79ebe6595ce44208853ba553186b8348,8d639afccbec43f2be8d3ed88702893b,4e3483f3bbbb48f59b0e92a7d0ccc1c0,b160fccc15fa4ba09c74018d82d16eb7,52f248eadea74cf2b7780bf991d95e4e,d4cb9017aa1f4f8ca34ae8d31c986210,eff19819b1284424a9ad7879991ee080,79551897f86b4338bd09b41c5953a7db,f11bd4ef5ddd4c82865fad3f1f9d80a9,76ee95093a8748b29d55df1cb8cdebd0,081af742f6384a20b492ca99704f51ca,c26c844bbb2f4746904948f3ecfeb1ae,ba9bbc5581a043d8b9c204b8ddb6aa01,cde1afb048a743bb9caedb5770aabca1,0b2b37a28bb0457bbc4bf33c11b4c6e1,9f94d338d6c94b95814b2c50f9e463d1,7d1745ce2c444db9bf4b4e100c476fe8,e711e93aadd54eef8feeb0fa07f054d2,0e914774210646f784d02fb22f8df7db,ca0ce5e9eb09477c8c99b7b94e7869ce,760d9072f9bc480fa36fdb876930467c,5742a628f13045cda0317c3d4d839bc5,b9309305d94f4ee8b66eebbcae9f2db4,a42216d3eedd48f0ba3d3bef2470605f,9cdcda30da704c21b19ea0616de3ea86,f05ecbf7e4b04b268b38cb6e5c8124bf,788d2c1496874eeeae3b7aabbab95f76,8b112af90e054671adc30373c6dafd21,3bdb29667807493292a0ddc9bc0ccdb7,ffd6ebecc869410dab6410a07f3362ae,196868a4610b45588062aed357fc7c58,4cdd2d11be4c4c098b27c10d7cb3fa9e,fda40125207d426dbf86270002ff0c4a,7af028c6048c44cc91415a58d25d286e,ff7d2c8e96e840e1a5a690e4e0f96bad,9f806cf868dc48d398836ebd60bd2ec6,50ff5cdc6c8645b3bb61b8327d204f01,bafb57ea8d84430cbc0c11e5c67643f1,e2c91860b96c46619a74215627560235,f5b68a43bbae44c8a1fe821dfcaca26e,70b771772a0b4f1cb1e592f9a01b08a5,7992a39d64b2483789ae274e2df3c01d,f8d711312da14871a77b58f41a6f6242,673b27b0ab9148b7ae03933c18433971,624886a950bd4d18900a877738696432,685d8d9267f745f39bcc6fc2cc553d31,7d40ad05a1a54165a28ec897ddbcef01,4bcdb5199e8442078999bb8611ea6398,3b0c0ffea0cf45db816993baf64b3fb3,ad2d6d9581034e04addcb5b034a88fb3,8440c4fc5056426996e3e488b2261d28,e367c4223d794572a05fd256759d6a53,1b2dc5784c3c41988d5634e566b4a404,11be1538d7f54d7786004503b2e684a7,b462cb8c2ec7485d938416831a9771cc,f747656a36204e9588aba97064332436,78d0b2337b484c6298c9b3afc01fdb85,8a63d1f8d4bc4eb4a4db437491c9983a,2247f64a15e54053abb2e13152003c6b,d78ef9c5997d4e94804b8126cc387ea1,6ce3cd5a86ef43219dbda2b78e974c93,414b5fd5ebee45d98eca809a524dbe93,4b9ea9db5c834b698a78c7052c87b432,8590fa7adc144f49ae3d2575b7280c08,6148e4b152b9494fb9e7035e9b58e624,be1cff2a29f64800a5c4f91009711988,49b85b35bca5461993eac91ffd0b1fe4,5abae35db6de475e904425af8dc8ecbb,e3bc44b82c3543758d5a6c02b41dd526,89323021918e444bae3537521046d19f,141dd3257f85455d86f3828cc0217b5a,7bc2ad90009a4de2a0fa2947ccc09886,e51874f4b6ac455db76e30582659368b,519dee074e4c48ae9edd58d7c1e330da,060ab186c22f4f42a83f1622bf47ca52,8873bbcf4b42445f8117108769358a5c,46fa5286441a444095b917165c31304c,fa9fdd56222a4ce7ba2b37922afe0eaa,6885932aa4b443babc2b69fdd495889c,be71b947b33a48d8aee36ad0c60d10b8,b52457719429466687a2f228d20571ac,0bf937d745494d998cb8bd2cf3a63f4e,c17feb3860184dd59931f35814ad68b0,4cc68cbd880c48948a1f18fe18ec938c,c37b0691a05c4d659dcd01571f1907da,c1334fe3e8484079bfd62e479307b5b9,ebb58027b6c647b7965eeb31b63ec0c0,1821c5f9392e4859a829c78ebda44df6,dca21ab79bd14004b94c55e36604c30d,8d96cc3457644378b9b643243464f735,631437054079401fbe2c3c9a3f3bc375,c2f78ab4372243da8775f4d71a16fef5,03792664a45d4f63ac3efb899ce1b61e,fec04c46683a41e5a344f4d057342829,320fa70de6bc4890a0135bbfe8735b49,5a03493dce9f4323a0da7bf4fd5e78ab,b9bd46a1da1846d099566fd5c6db00cb,647021c7f798472889fd82f6c4a27ca2,313e905b78d746cb9e574b9634c3a113,623fd328c4cd4ee8b21eb03425b1f100,4676b4d1130146629fd98e2d2eb9a5b4,34d380813a36481ea2534c46d18b5927,894d87e55ac640b99e966a8786a2392b,477ba74d182a4ec0b86926dc67a0f419,362ca5fdca174fa3bdd6325a947320db,1ac3ad8dfa184fe4b6815950d3d23fdd,fdebee18c71f4a9db60096e75ad34d83,91432be4178a4781917fba0abd51df61,480c71baa65145f4bd037332d68e5e4f,536e3a3702b0453d92939846dc1f33b7,0333aedf1be640c78d2ee02473b7fd97,cbb3a6f1772b4f72a8af1375ef16e914,367ad9c43a6246dfac364ca59874883b,bac18430f6db466195a3a7116cdfb958,c4ec39be11be4b0eae321ee0559be8e6,8f7ffae7e47d462aa4142db7f06633d5,82e3b2423d86425195929fc3810ce7a4,72973e0415004dfea220babc479be082,41d0f568bb1e4fed8ad965a79d9bccef,5854ef1d24784ca0a046514b19286113,04afc3dd1eec4d13bc06cd01e0a5e838,db76e3333da846d5ae55b3b9dd735f26,7aa0ca0bf1514fa6a3a14dcde44cf0b9,9c668726c9b6449a9045d3f78408a2a1,7caf148d68a6431bbe56bcc6472d8720,babd36d1d3484fb99fe4e7ca78560ccf,7507865343e24cff80d55a29ec57f0e1,63f8b7d9b1f34851802e0408bf997ea5,a3369d5dd6b74af7b35edf1f10350881,9b8b8c3f86854416b82d57583896c1ff,881815c772564a8dbd5eb6de65ecbe7a,4b1bcf46753a48258d1d514440140b7f,1ca4c6a49e794954a235a82ee8981ecd,ab381d9129bb4b979fe3774e262524f8,a07d2c98831a4d79bc6c5c4288a24720,d8debd28039d494aa7fe18bc8e81af27,d0402b1f00f94acca96276c6ea8c8397,ebcdebd9639d4ec2b7072ad32bf48f50,636b3211100245919c09177ccea562e0,280c4440e18a487396a81b92cd3efa13,5163cf6740334a7d83375385c3ae2b67,a6083e4fab7a43498bfb3257658f9278,3b5c8b5cfb124ebd8f4a798d2a47d832,9fda332474534f96955b5bce8548947f,b5f5e62aba9842d8b48a28111ca6a372,c0cecff1207c4692a634d7de449d8d6d,e4c6428f1d264544b0eec52214e7b6ae,cbc8c5490a7f46b996fd4c3a9711f2d7,cfbdd82f4ce741a88109353c5fc898e4,6bfb965bc1b240a9ae2bd7a1e884ea51,eb36d63a993148289950af9098727759,f7357c3954d2408facaa6771a758ebcc,c05853b94ab8405eb26e0a8bd481b662,da1e6d3ebfa140b3a645342fd3bb5d95,988f3f6b607d442b8df006c980ddc2cd,b6af3c01968141baa3e99ca4b8d83847,75f669faee5d4023a425f2461e5fc5e9,165eb05677aa45fc9a53dfd8bfb7a45d,409891afd7cc4e12ba6b002581fef4f3,02ee0a87956f4bfab91b88282e502a3d,48601418028145ff84a450c877ea35aa,8de73f8d59eb45aa8c61138ab93c4f86,5dd82a46f48945b4bbb61f58b9d1c856,97993db997bd4ff88acfe1340ca86845,282aee84915b43179a09101b3ee6ad0e,8750e994985f43f383321bf0d6ca7932";

        role1.setMenuIds(defalutMenu1);
        String[] defalutMenus1 = defalutMenu1.split(",");
        ArrayList<String> menuIdList1 = new ArrayList<String>();
        for (int i = 0; i < defalutMenus1.length; i++) {
            menuIdList1.add(defalutMenus1[i]);
        }
        role1.setMenuIdList(menuIdList1);
        if (Global.isDemoMode()) {
            return null;
        }
        systemService.saveRole(role1,true,1);
        SysRoleActivity sysRoleActivity = new SysRoleActivity();
        SysRoleActivity xzroleActivity = new SysRoleActivity();
        SysRoleActivity zlroleActivity = new SysRoleActivity();
        SysRoleActivity xzroleActivity2 = new SysRoleActivity();
        SysRoleActivity cwroleActivity = new SysRoleActivity();
        if ("1".equals(type)) {
            String enname = UserUtils.getRoleActivity(com.getId()) + StringUtils.getPinYinHeadChar("总经理", 1);

            sysRoleActivity.setName("总经理");
            sysRoleActivity.setEnname(enname);
            sysRoleActivity.setSort("3000");
            sysRoleActivity.setCompanyId(com.getId());
            sysRoleActivity.setType("3");
            sysRoleActivity.setDelType("1");
            sysRoleActivityService.save(sysRoleActivity);

            cwroleActivity.setName("财务分管领导");
            cwroleActivity.setEnname(UserUtils.getRoleActivity(com.getId()) + StringUtils.getPinYinHeadChar("财务分管领导", 1));
            cwroleActivity.setSort("7700");
            cwroleActivity.setType("3");
            cwroleActivity.setCompanyId(com.getId());
            cwroleActivity.setDelType("1");
            sysRoleActivityService.save(cwroleActivity);

            xzroleActivity.setName("行政分管领导");
            xzroleActivity.setEnname(UserUtils.getRoleActivity(com.getId()) + StringUtils.getPinYinHeadChar("行政分管领导", 1));
            xzroleActivity.setSort("7800");
            xzroleActivity.setType("3");
            xzroleActivity.setCompanyId(com.getId());
            xzroleActivity.setDelType("1");
            sysRoleActivityService.save(xzroleActivity);
            zlroleActivity.setName("质量分管领导");
            zlroleActivity.setEnname(UserUtils.getRoleActivity(com.getId()) + StringUtils.getPinYinHeadChar("质量分管领导", 1));
            zlroleActivity.setSort("7900");
            zlroleActivity.setType("3");
            zlroleActivity.setCompanyId(com.getId());
            zlroleActivity.setDelType("1");
            sysRoleActivityService.save(zlroleActivity);
            xzroleActivity2.setName("市场分管领导");
            xzroleActivity2.setEnname(UserUtils.getRoleActivity(com.getId()) + StringUtils.getPinYinHeadChar("市场分管领导", 1));
            xzroleActivity2.setSort("7600");
            xzroleActivity2.setType("3");
            xzroleActivity2.setCompanyId(com.getId());
            xzroleActivity2.setDelType("1");
            sysRoleActivityService.save(xzroleActivity2);
        }else {
            String enname = UserUtils.getRoleActivityEnname(com.getId(),UserUtils.getUser());
            sysRoleActivity = sysRoleActivityService.getByEnname(enname+"cwfgld");
            cwroleActivity = sysRoleActivityService.getByEnname(enname+"xzfgld");
            xzroleActivity = sysRoleActivityService.getByEnname(enname+"zlfgld");
            zlroleActivity = sysRoleActivityService.getByEnname(enname+"scfgld");
            xzroleActivity2 = sysRoleActivityService.getByEnname(enname+"zjl");
        }
        List<SysRoleActivity> roleActivities = new ArrayList<>();
        roleActivities.add(sysRoleActivity);
        roleActivities.add(cwroleActivity);
        roleActivities.add(xzroleActivity);
        roleActivities.add(zlroleActivity);
        roleActivities.add(xzroleActivity2);
        role1.setRoleActivityList(roleActivities);
        roleDao.insertRoleActivity(role1);
        return role1;
    }

    @Transactional(readOnly = false)
    public Role createCompanyChairmanRole(Office com, Office office,String type) {
        Role role1 = new Role();
        role1.setName("董事长");
        role1.setNumber("2");
        role1.setSysData(Global.YES);
        if (type.equals("1")){
            role1.setDataScope("2");
        }else {
            role1.setDataScope("5");
            role1.setBranchCompany(office);
        }
        role1.setIsNewRecord(false);
        role1.setCompany(com);
        role1.setOffice(office);
        role1.setRoleType("assignment");
        role1.setSysData("1");
        role1.setUseable("1");
        role1.setEnname("dsz");
        String defalutMenu1 = "1,27,29,e3b80fae59ab4b4099d8349ecaf8c4dc,61dd3aa279684f92bb1c5d4019a32503,b0c5737cdc4f452684ce93589427c97d,af6b0e5d61fc44ecbebb449703a3df72,18c14562978e4b129fe3afb7dca8b868,6123fcb3d810492789e82a63b0d4310a,0aa3712414d049a6a24e8bcddeae509a,95a6a82dc5fc4d07b46e5df57a0606a3,62,b6fa4abb624349918fe531274fc15d23,89,76bdb735faa34667a6cbf13ebe937533,90,f5b2028cfad9470085c7c846de33193a,28abe4b0cb3b4c8480f84c07a5dfaf06,c7fa36c7142f481397c2cc12e2bc828a,e2efcf80fc3747329fe02d489dbfda95,e3b5b7451e164089aba0b9bf844b0398,438c28c3cbe44ed7bff951b2a3dc77cd,59e50eebe56743a78eecbc6eff1ef94c,cc1027689aca41a89302217127253034,200fe6cc80174065b382c961a4379d79,4c13ca96376341dfa25980e86964c1a9,638be8b3b5284e5088003917685bc19b,811600aadcb54b098b7dda4bf95ffc51,ce8dc7c97d93481dbd76cabb6e905b55,aa25b3904abb4bd7b779e5be37b4382e,aae6f4c7fd4c4a25afa5285210edfded,84ff9d5c165c4528af9361b6127cf5af,31b349a3624b47f28eb86d614f9f95b5,040b42e395de41d6805a958eb1897863,34874260e6fc4200a76b33c3cfa4d80d,5b6c6ea5ed2d45478e8144351d853e21,7c032c32512847f882a87416359ef0a3,6d30ea8c710a42b89da2f2506059b0ef,2ca7bf4ce6804a1c9e5b331d2df27c72,7717cbb6d71f405d890e9b648cbd2088,bddb0400764c42b08e427059c7752057,23d6c56239804053910e5c5aec481330,7f8b2018113545bcbdbfc87e7a236bec,547e121ffb714d16ae142b6935abb78f,af24101b962444f29d166b638b2fea30,6ea6519ec4d548128aa42f3ce004def3,4379bf54c2a543869abf5031b97b8efe,5e6da3f5368b4f06b7268b31a1157e9f,2bbf0e75f21d42938954391d0776fbde,545b39dacf564a5eb9d1658f80a0b8b0,b4ddedf9b451494a97a1ff9476e3897d,1b145ca23d3e4d81994a84d30a262d18,10a8c7c1d34e46dd8ef461cdad928afc,5ae4911d14bb48b39885b1fe522e999d,7f09def2533b45fdb3c733f1e72ca5fb,9ef29dca19d342b795efcbfe32793602,be72d72ae7034c509c164467b2abddda,ab879705f2354813acf8724404520e28,82487b87b97b475b98892b80cf83c96a,0f4a461b97be48eeab43bd052f630057,72e1a28b7251438a89b0a4f6bd522b89,e1f9a246057347d78a1966d6dc0168e3,5d3645d53ce14c9ab498fd9f050ba264,d361e6890cc84b049fc8268c2a8c4952,5cc0002cbcc14d3485ab8ae6d57368f2,fa7bb243643c44b18e989e3419eeb171,62768b6c65134f1e927b1807bc72115a,7b69b0e5d2d344e6aaa2b8726c23561a,434bbb1f9f8140b1beeb57d0c4db660d,7920c4c8da294d87bcb94e7756bf35ae,0752b85660694033bbdb013f30cde17f,bd6258cfbea64896b0401517e440d6a2,9aa3539682514546866d2998d741ac6c,eb3ad93ce6a44921b0490c842dd47d8c,9ce19042d497473c92d199a5e5a7daf0,1b3eb6ba64ea4589a8e2d42efa5d3f89,6b2dd5ff2a754a53a80798341a7d35d8,e53d754160f74d28b6d091829fc289f8,92d762bca040407bb4ce4b06a2cfa195,aa8a2b00622147259c85290f4d95b46d,0be3e487d874487fbfb5c49b52440a48,ce60b25f7da04af8b7cefccf2dbc7d36,6464ac9cc3fc47d88598f6c4e74caa19,04b312df403d4855b013743206ddb1de,17356c44ae894782a1015ec2ddaeaeda,a06eaefba7084e219f0c6ce4010b82e7,95a54e4db7e74f0586cd85cb4e2bd6f7,1a93047d6b0f4e3ca70fcf836b2517c4,93263a29941648979bb8de774978a880,8eba834a28864375aa3137590d26f37e,3c52b0a890524c6fb66cefc32114c60e,8b20519b794c42d28fdecbd0ccbba5c1,77189ceba655488681964a609c28256d,39e776aaf3644d24bb78ecf43704b71b,0dd5fa1192e34d5fac5e78ca38969126,1a6f82a80a054bc7ba4b516187231a54,5993ce4ec80a41fb9d4ea0713b74f4a4,3b42071536124a9c9dbf2ec319004b29,a966beefe58e4f95b24e89f4172cb7ce,2e1e75dbe28841529576dd1cd48d2437,cdd8829e5976438aaa66ce608f0175f5,71015695cf1445ecacce90669c67d733,31e9dd26b7314d8a9bf97dcf6dbdb22d,e5ecaa091bb0464f8553d4e7aa60647d,39055be3a23945868dbf4b5930043a88,07ba7c9aed104657aac3be87aa346d3b,5a7a9bfff0f44a2089a7d277f2c04557,be53cc152339400ba705c1d1dceff9ab,ed591ccba0564abd9979899049dc7b36,9a4425b817e94dbe844d7fd3e96a9033,bf4e54bfe9984441a8efa4dd8a4a23a6,5a1bcf2a2e0943f88bf655ca011fb288,a12c8904ff0147a99010eaa5aa0ea704,e7c6f2581c6e41bf8fe88a3d7fadc986,89709767beea48d0b0fb047316eb37d1,04d6a49b35514f91be43e9576f8d32fc,bd50e783ec714e5d86aafcca54e87c33,4acb0c7d002b47e29be7ebe9f40490f6,67b5ddef866b4982aa5afef7aad69924,7e090632f713440bba11ed6fc09eb291,f308051e11ca47eebb12709725b52eb4,dbea976ef1d248cabf13b88d311c4533,e0132620d4044e38a92f75b8335005ed,1b598563135e42d4bfe729cf32649f14,c4de9219f4f14684ae21d0904b7b78df,520983ab7dd64104a47e273941236b2d,133ba69bd04046b89eadaf793b9063db,11f24ca662044292a8ca95930ccc3581,0f30b4d621294d4fab359611cf82ab6f,b3c93fcd54b544e5b2c8e70ee7eb93ce,79ebe6595ce44208853ba553186b8348,8d639afccbec43f2be8d3ed88702893b,4e3483f3bbbb48f59b0e92a7d0ccc1c0,b160fccc15fa4ba09c74018d82d16eb7,52f248eadea74cf2b7780bf991d95e4e,d4cb9017aa1f4f8ca34ae8d31c986210,eff19819b1284424a9ad7879991ee080,79551897f86b4338bd09b41c5953a7db,f11bd4ef5ddd4c82865fad3f1f9d80a9,76ee95093a8748b29d55df1cb8cdebd0,081af742f6384a20b492ca99704f51ca,c26c844bbb2f4746904948f3ecfeb1ae,ba9bbc5581a043d8b9c204b8ddb6aa01,cde1afb048a743bb9caedb5770aabca1,0b2b37a28bb0457bbc4bf33c11b4c6e1,9f94d338d6c94b95814b2c50f9e463d1,7d1745ce2c444db9bf4b4e100c476fe8,e711e93aadd54eef8feeb0fa07f054d2,0e914774210646f784d02fb22f8df7db,ca0ce5e9eb09477c8c99b7b94e7869ce,760d9072f9bc480fa36fdb876930467c,5742a628f13045cda0317c3d4d839bc5,b9309305d94f4ee8b66eebbcae9f2db4,a42216d3eedd48f0ba3d3bef2470605f,9cdcda30da704c21b19ea0616de3ea86,f05ecbf7e4b04b268b38cb6e5c8124bf,788d2c1496874eeeae3b7aabbab95f76,8b112af90e054671adc30373c6dafd21,3bdb29667807493292a0ddc9bc0ccdb7,ffd6ebecc869410dab6410a07f3362ae,196868a4610b45588062aed357fc7c58,4cdd2d11be4c4c098b27c10d7cb3fa9e,fda40125207d426dbf86270002ff0c4a,7af028c6048c44cc91415a58d25d286e,ff7d2c8e96e840e1a5a690e4e0f96bad,9f806cf868dc48d398836ebd60bd2ec6,50ff5cdc6c8645b3bb61b8327d204f01,bafb57ea8d84430cbc0c11e5c67643f1,e2c91860b96c46619a74215627560235,f5b68a43bbae44c8a1fe821dfcaca26e,70b771772a0b4f1cb1e592f9a01b08a5,7992a39d64b2483789ae274e2df3c01d,f8d711312da14871a77b58f41a6f6242,673b27b0ab9148b7ae03933c18433971,624886a950bd4d18900a877738696432,685d8d9267f745f39bcc6fc2cc553d31,7d40ad05a1a54165a28ec897ddbcef01,4bcdb5199e8442078999bb8611ea6398,3b0c0ffea0cf45db816993baf64b3fb3,ad2d6d9581034e04addcb5b034a88fb3,8440c4fc5056426996e3e488b2261d28,e367c4223d794572a05fd256759d6a53,1b2dc5784c3c41988d5634e566b4a404,11be1538d7f54d7786004503b2e684a7,b462cb8c2ec7485d938416831a9771cc,f747656a36204e9588aba97064332436,78d0b2337b484c6298c9b3afc01fdb85,8a63d1f8d4bc4eb4a4db437491c9983a,2247f64a15e54053abb2e13152003c6b,d78ef9c5997d4e94804b8126cc387ea1,6ce3cd5a86ef43219dbda2b78e974c93,414b5fd5ebee45d98eca809a524dbe93,4b9ea9db5c834b698a78c7052c87b432,8590fa7adc144f49ae3d2575b7280c08,6148e4b152b9494fb9e7035e9b58e624,be1cff2a29f64800a5c4f91009711988,49b85b35bca5461993eac91ffd0b1fe4,5abae35db6de475e904425af8dc8ecbb,e3bc44b82c3543758d5a6c02b41dd526,89323021918e444bae3537521046d19f,141dd3257f85455d86f3828cc0217b5a,7bc2ad90009a4de2a0fa2947ccc09886,e51874f4b6ac455db76e30582659368b,519dee074e4c48ae9edd58d7c1e330da,060ab186c22f4f42a83f1622bf47ca52,8873bbcf4b42445f8117108769358a5c,46fa5286441a444095b917165c31304c,fa9fdd56222a4ce7ba2b37922afe0eaa,6885932aa4b443babc2b69fdd495889c,be71b947b33a48d8aee36ad0c60d10b8,b52457719429466687a2f228d20571ac,0bf937d745494d998cb8bd2cf3a63f4e,c17feb3860184dd59931f35814ad68b0,4cc68cbd880c48948a1f18fe18ec938c,c37b0691a05c4d659dcd01571f1907da,c1334fe3e8484079bfd62e479307b5b9,ebb58027b6c647b7965eeb31b63ec0c0,1821c5f9392e4859a829c78ebda44df6,dca21ab79bd14004b94c55e36604c30d,8d96cc3457644378b9b643243464f735,631437054079401fbe2c3c9a3f3bc375,c2f78ab4372243da8775f4d71a16fef5,03792664a45d4f63ac3efb899ce1b61e,fec04c46683a41e5a344f4d057342829,320fa70de6bc4890a0135bbfe8735b49,5a03493dce9f4323a0da7bf4fd5e78ab,b9bd46a1da1846d099566fd5c6db00cb,647021c7f798472889fd82f6c4a27ca2,313e905b78d746cb9e574b9634c3a113,623fd328c4cd4ee8b21eb03425b1f100,4676b4d1130146629fd98e2d2eb9a5b4,34d380813a36481ea2534c46d18b5927,894d87e55ac640b99e966a8786a2392b,477ba74d182a4ec0b86926dc67a0f419,362ca5fdca174fa3bdd6325a947320db,1ac3ad8dfa184fe4b6815950d3d23fdd,fdebee18c71f4a9db60096e75ad34d83,91432be4178a4781917fba0abd51df61,480c71baa65145f4bd037332d68e5e4f,536e3a3702b0453d92939846dc1f33b7,0333aedf1be640c78d2ee02473b7fd97,cbb3a6f1772b4f72a8af1375ef16e914,367ad9c43a6246dfac364ca59874883b,bac18430f6db466195a3a7116cdfb958,c4ec39be11be4b0eae321ee0559be8e6,8f7ffae7e47d462aa4142db7f06633d5,82e3b2423d86425195929fc3810ce7a4,72973e0415004dfea220babc479be082,41d0f568bb1e4fed8ad965a79d9bccef,5854ef1d24784ca0a046514b19286113,04afc3dd1eec4d13bc06cd01e0a5e838,db76e3333da846d5ae55b3b9dd735f26,7aa0ca0bf1514fa6a3a14dcde44cf0b9,9c668726c9b6449a9045d3f78408a2a1,7caf148d68a6431bbe56bcc6472d8720,babd36d1d3484fb99fe4e7ca78560ccf,7507865343e24cff80d55a29ec57f0e1,63f8b7d9b1f34851802e0408bf997ea5,a3369d5dd6b74af7b35edf1f10350881,9b8b8c3f86854416b82d57583896c1ff,881815c772564a8dbd5eb6de65ecbe7a,4b1bcf46753a48258d1d514440140b7f,1ca4c6a49e794954a235a82ee8981ecd,ab381d9129bb4b979fe3774e262524f8,a07d2c98831a4d79bc6c5c4288a24720,d8debd28039d494aa7fe18bc8e81af27,d0402b1f00f94acca96276c6ea8c8397,ebcdebd9639d4ec2b7072ad32bf48f50,636b3211100245919c09177ccea562e0,280c4440e18a487396a81b92cd3efa13,5163cf6740334a7d83375385c3ae2b67,a6083e4fab7a43498bfb3257658f9278,3b5c8b5cfb124ebd8f4a798d2a47d832,9fda332474534f96955b5bce8548947f,b5f5e62aba9842d8b48a28111ca6a372,c0cecff1207c4692a634d7de449d8d6d,e4c6428f1d264544b0eec52214e7b6ae,cbc8c5490a7f46b996fd4c3a9711f2d7,cfbdd82f4ce741a88109353c5fc898e4,6bfb965bc1b240a9ae2bd7a1e884ea51,eb36d63a993148289950af9098727759,f7357c3954d2408facaa6771a758ebcc,c05853b94ab8405eb26e0a8bd481b662,da1e6d3ebfa140b3a645342fd3bb5d95,988f3f6b607d442b8df006c980ddc2cd,b6af3c01968141baa3e99ca4b8d83847,75f669faee5d4023a425f2461e5fc5e9,165eb05677aa45fc9a53dfd8bfb7a45d,409891afd7cc4e12ba6b002581fef4f3,02ee0a87956f4bfab91b88282e502a3d,48601418028145ff84a450c877ea35aa,8de73f8d59eb45aa8c61138ab93c4f86,5dd82a46f48945b4bbb61f58b9d1c856,97993db997bd4ff88acfe1340ca86845,282aee84915b43179a09101b3ee6ad0e,8750e994985f43f383321bf0d6ca7932";
        role1.setMenuIds(defalutMenu1);
        String[] defalutMenus1 = defalutMenu1.split(",");
        ArrayList<String> menuIdList1 = new ArrayList<String>();
        for (int i = 0; i < defalutMenus1.length; i++) {
            menuIdList1.add(defalutMenus1[i]);
        }
        role1.setMenuIdList(menuIdList1);
        if (Global.isDemoMode()) {
            return null;
        }
        systemService.saveRole(role1,true,1);
        /*if ("1".equals(type)) {
            String enname = UserUtils.getRoleActivity(com.getId()) + StringUtils.getPinYinHeadChar("董事长", 1);
            SysRoleActivity sysRoleActivity = new SysRoleActivity();
            sysRoleActivity.setName("董事长");
            sysRoleActivity.setEnname(enname);
            sysRoleActivity.setSort("3500");
            sysRoleActivity.setType("3");
            sysRoleActivity.setDelType("1");
            sysRoleActivity.setCompanyId(com.getId());
            sysRoleActivityService.save(sysRoleActivity);
            List<SysRoleActivity> roleActivities = new ArrayList<>();
            roleActivities.add(sysRoleActivity);
            role1.setRoleActivityList(roleActivities);
            roleDao.insertRoleActivity(role1);
        }*/
        return role1;
    }

    @Transactional(readOnly = false)
    public Role createCompanyZongGongManagerRole( Office com,Office office,Office deptZLJG,String type) {
        Role role1 = new Role();
        role1.setName("总工");
        role1.setSysData(Global.YES);
        if (type.equals("1")){
            role1.setDataScope("2");
        }else {
            role1.setDataScope("5");
            role1.setBranchCompany(office);
        }
        role1.setIsNewRecord(false);
        role1.setCompany(com);
        role1.setOffice(deptZLJG);
        role1.setRoleType("assignment");
        role1.setSysData("1");
        role1.setUseable("1");
        role1.setEnname("zg");
        role1.setNumber("4");
        String defalutMenu1 = "1,27,29,e3b80fae59ab4b4099d8349ecaf8c4dc,61dd3aa279684f92bb1c5d4019a32503,b0c5737cdc4f452684ce93589427c97d,af6b0e5d61fc44ecbebb449703a3df72,18c14562978e4b129fe3afb7dca8b868,6123fcb3d810492789e82a63b0d4310a,0aa3712414d049a6a24e8bcddeae509a,95a6a82dc5fc4d07b46e5df57a0606a3,62,b6fa4abb624349918fe531274fc15d23,89,76bdb735faa34667a6cbf13ebe937533,90,f5b2028cfad9470085c7c846de33193a,28abe4b0cb3b4c8480f84c07a5dfaf06,c7fa36c7142f481397c2cc12e2bc828a,e2efcf80fc3747329fe02d489dbfda95,e3b5b7451e164089aba0b9bf844b0398,438c28c3cbe44ed7bff951b2a3dc77cd,59e50eebe56743a78eecbc6eff1ef94c,cc1027689aca41a89302217127253034,200fe6cc80174065b382c961a4379d79,4c13ca96376341dfa25980e86964c1a9,638be8b3b5284e5088003917685bc19b,811600aadcb54b098b7dda4bf95ffc51,ce8dc7c97d93481dbd76cabb6e905b55,aa25b3904abb4bd7b779e5be37b4382e,aae6f4c7fd4c4a25afa5285210edfded,84ff9d5c165c4528af9361b6127cf5af,31b349a3624b47f28eb86d614f9f95b5,040b42e395de41d6805a958eb1897863,34874260e6fc4200a76b33c3cfa4d80d,5b6c6ea5ed2d45478e8144351d853e21,7c032c32512847f882a87416359ef0a3,6d30ea8c710a42b89da2f2506059b0ef,2ca7bf4ce6804a1c9e5b331d2df27c72,7717cbb6d71f405d890e9b648cbd2088,bddb0400764c42b08e427059c7752057,23d6c56239804053910e5c5aec481330,7f8b2018113545bcbdbfc87e7a236bec,547e121ffb714d16ae142b6935abb78f,af24101b962444f29d166b638b2fea30,6ea6519ec4d548128aa42f3ce004def3,4379bf54c2a543869abf5031b97b8efe,5e6da3f5368b4f06b7268b31a1157e9f,2bbf0e75f21d42938954391d0776fbde,545b39dacf564a5eb9d1658f80a0b8b0,b4ddedf9b451494a97a1ff9476e3897d,1b145ca23d3e4d81994a84d30a262d18,10a8c7c1d34e46dd8ef461cdad928afc,5ae4911d14bb48b39885b1fe522e999d,7f09def2533b45fdb3c733f1e72ca5fb,9ef29dca19d342b795efcbfe32793602,be72d72ae7034c509c164467b2abddda,ab879705f2354813acf8724404520e28,82487b87b97b475b98892b80cf83c96a,0f4a461b97be48eeab43bd052f630057,72e1a28b7251438a89b0a4f6bd522b89,e1f9a246057347d78a1966d6dc0168e3,5d3645d53ce14c9ab498fd9f050ba264,d361e6890cc84b049fc8268c2a8c4952,5cc0002cbcc14d3485ab8ae6d57368f2,fa7bb243643c44b18e989e3419eeb171,62768b6c65134f1e927b1807bc72115a,7b69b0e5d2d344e6aaa2b8726c23561a,434bbb1f9f8140b1beeb57d0c4db660d,7920c4c8da294d87bcb94e7756bf35ae,0752b85660694033bbdb013f30cde17f,bd6258cfbea64896b0401517e440d6a2,9aa3539682514546866d2998d741ac6c,eb3ad93ce6a44921b0490c842dd47d8c,9ce19042d497473c92d199a5e5a7daf0,1b3eb6ba64ea4589a8e2d42efa5d3f89,6b2dd5ff2a754a53a80798341a7d35d8,e53d754160f74d28b6d091829fc289f8,92d762bca040407bb4ce4b06a2cfa195,aa8a2b00622147259c85290f4d95b46d,0be3e487d874487fbfb5c49b52440a48,ce60b25f7da04af8b7cefccf2dbc7d36,6464ac9cc3fc47d88598f6c4e74caa19,04b312df403d4855b013743206ddb1de,17356c44ae894782a1015ec2ddaeaeda,a06eaefba7084e219f0c6ce4010b82e7,95a54e4db7e74f0586cd85cb4e2bd6f7,1a93047d6b0f4e3ca70fcf836b2517c4,93263a29941648979bb8de774978a880,8eba834a28864375aa3137590d26f37e,3c52b0a890524c6fb66cefc32114c60e,8b20519b794c42d28fdecbd0ccbba5c1,77189ceba655488681964a609c28256d,39e776aaf3644d24bb78ecf43704b71b,0dd5fa1192e34d5fac5e78ca38969126,1a6f82a80a054bc7ba4b516187231a54,5993ce4ec80a41fb9d4ea0713b74f4a4,3b42071536124a9c9dbf2ec319004b29,a966beefe58e4f95b24e89f4172cb7ce,2e1e75dbe28841529576dd1cd48d2437,cdd8829e5976438aaa66ce608f0175f5,71015695cf1445ecacce90669c67d733,31e9dd26b7314d8a9bf97dcf6dbdb22d,e5ecaa091bb0464f8553d4e7aa60647d,39055be3a23945868dbf4b5930043a88,07ba7c9aed104657aac3be87aa346d3b,5a7a9bfff0f44a2089a7d277f2c04557,be53cc152339400ba705c1d1dceff9ab,ed591ccba0564abd9979899049dc7b36,9a4425b817e94dbe844d7fd3e96a9033,bf4e54bfe9984441a8efa4dd8a4a23a6,5a1bcf2a2e0943f88bf655ca011fb288,a12c8904ff0147a99010eaa5aa0ea704,e7c6f2581c6e41bf8fe88a3d7fadc986,89709767beea48d0b0fb047316eb37d1,04d6a49b35514f91be43e9576f8d32fc,bd50e783ec714e5d86aafcca54e87c33,4acb0c7d002b47e29be7ebe9f40490f6,67b5ddef866b4982aa5afef7aad69924,7e090632f713440bba11ed6fc09eb291,f308051e11ca47eebb12709725b52eb4,dbea976ef1d248cabf13b88d311c4533,e0132620d4044e38a92f75b8335005ed,1b598563135e42d4bfe729cf32649f14,c4de9219f4f14684ae21d0904b7b78df,520983ab7dd64104a47e273941236b2d,133ba69bd04046b89eadaf793b9063db,11f24ca662044292a8ca95930ccc3581,0f30b4d621294d4fab359611cf82ab6f,b3c93fcd54b544e5b2c8e70ee7eb93ce,79ebe6595ce44208853ba553186b8348,8d639afccbec43f2be8d3ed88702893b,4e3483f3bbbb48f59b0e92a7d0ccc1c0,b160fccc15fa4ba09c74018d82d16eb7,52f248eadea74cf2b7780bf991d95e4e,d4cb9017aa1f4f8ca34ae8d31c986210,eff19819b1284424a9ad7879991ee080,79551897f86b4338bd09b41c5953a7db,f11bd4ef5ddd4c82865fad3f1f9d80a9,76ee95093a8748b29d55df1cb8cdebd0,081af742f6384a20b492ca99704f51ca,c26c844bbb2f4746904948f3ecfeb1ae,ba9bbc5581a043d8b9c204b8ddb6aa01,cde1afb048a743bb9caedb5770aabca1,0b2b37a28bb0457bbc4bf33c11b4c6e1,9f94d338d6c94b95814b2c50f9e463d1,7d1745ce2c444db9bf4b4e100c476fe8,e711e93aadd54eef8feeb0fa07f054d2,0e914774210646f784d02fb22f8df7db,ca0ce5e9eb09477c8c99b7b94e7869ce,760d9072f9bc480fa36fdb876930467c,5742a628f13045cda0317c3d4d839bc5,b9309305d94f4ee8b66eebbcae9f2db4,a42216d3eedd48f0ba3d3bef2470605f,9cdcda30da704c21b19ea0616de3ea86,f05ecbf7e4b04b268b38cb6e5c8124bf,788d2c1496874eeeae3b7aabbab95f76,8b112af90e054671adc30373c6dafd21,3bdb29667807493292a0ddc9bc0ccdb7,ffd6ebecc869410dab6410a07f3362ae,196868a4610b45588062aed357fc7c58,4cdd2d11be4c4c098b27c10d7cb3fa9e,fda40125207d426dbf86270002ff0c4a,7af028c6048c44cc91415a58d25d286e,ff7d2c8e96e840e1a5a690e4e0f96bad,9f806cf868dc48d398836ebd60bd2ec6,50ff5cdc6c8645b3bb61b8327d204f01,bafb57ea8d84430cbc0c11e5c67643f1,e2c91860b96c46619a74215627560235,f5b68a43bbae44c8a1fe821dfcaca26e,70b771772a0b4f1cb1e592f9a01b08a5,7992a39d64b2483789ae274e2df3c01d,f8d711312da14871a77b58f41a6f6242,673b27b0ab9148b7ae03933c18433971,624886a950bd4d18900a877738696432,685d8d9267f745f39bcc6fc2cc553d31,7d40ad05a1a54165a28ec897ddbcef01,4bcdb5199e8442078999bb8611ea6398,3b0c0ffea0cf45db816993baf64b3fb3,ad2d6d9581034e04addcb5b034a88fb3,8440c4fc5056426996e3e488b2261d28,e367c4223d794572a05fd256759d6a53,1b2dc5784c3c41988d5634e566b4a404,11be1538d7f54d7786004503b2e684a7,b462cb8c2ec7485d938416831a9771cc,f747656a36204e9588aba97064332436,78d0b2337b484c6298c9b3afc01fdb85,8a63d1f8d4bc4eb4a4db437491c9983a,2247f64a15e54053abb2e13152003c6b,d78ef9c5997d4e94804b8126cc387ea1,6ce3cd5a86ef43219dbda2b78e974c93,414b5fd5ebee45d98eca809a524dbe93,4b9ea9db5c834b698a78c7052c87b432,8590fa7adc144f49ae3d2575b7280c08,6148e4b152b9494fb9e7035e9b58e624,be1cff2a29f64800a5c4f91009711988,49b85b35bca5461993eac91ffd0b1fe4,5abae35db6de475e904425af8dc8ecbb,e3bc44b82c3543758d5a6c02b41dd526,89323021918e444bae3537521046d19f,141dd3257f85455d86f3828cc0217b5a,7bc2ad90009a4de2a0fa2947ccc09886,e51874f4b6ac455db76e30582659368b,519dee074e4c48ae9edd58d7c1e330da,060ab186c22f4f42a83f1622bf47ca52,8873bbcf4b42445f8117108769358a5c,46fa5286441a444095b917165c31304c,fa9fdd56222a4ce7ba2b37922afe0eaa,6885932aa4b443babc2b69fdd495889c,be71b947b33a48d8aee36ad0c60d10b8,b52457719429466687a2f228d20571ac,0bf937d745494d998cb8bd2cf3a63f4e,c17feb3860184dd59931f35814ad68b0,4cc68cbd880c48948a1f18fe18ec938c,c37b0691a05c4d659dcd01571f1907da,c1334fe3e8484079bfd62e479307b5b9,ebb58027b6c647b7965eeb31b63ec0c0,1821c5f9392e4859a829c78ebda44df6,dca21ab79bd14004b94c55e36604c30d,8d96cc3457644378b9b643243464f735,631437054079401fbe2c3c9a3f3bc375,c2f78ab4372243da8775f4d71a16fef5,03792664a45d4f63ac3efb899ce1b61e,fec04c46683a41e5a344f4d057342829,320fa70de6bc4890a0135bbfe8735b49,5a03493dce9f4323a0da7bf4fd5e78ab,b9bd46a1da1846d099566fd5c6db00cb,647021c7f798472889fd82f6c4a27ca2,313e905b78d746cb9e574b9634c3a113,623fd328c4cd4ee8b21eb03425b1f100,4676b4d1130146629fd98e2d2eb9a5b4,34d380813a36481ea2534c46d18b5927,894d87e55ac640b99e966a8786a2392b,477ba74d182a4ec0b86926dc67a0f419,362ca5fdca174fa3bdd6325a947320db,1ac3ad8dfa184fe4b6815950d3d23fdd,fdebee18c71f4a9db60096e75ad34d83,91432be4178a4781917fba0abd51df61,480c71baa65145f4bd037332d68e5e4f,536e3a3702b0453d92939846dc1f33b7,0333aedf1be640c78d2ee02473b7fd97,cbb3a6f1772b4f72a8af1375ef16e914,367ad9c43a6246dfac364ca59874883b,bac18430f6db466195a3a7116cdfb958,c4ec39be11be4b0eae321ee0559be8e6,8f7ffae7e47d462aa4142db7f06633d5,82e3b2423d86425195929fc3810ce7a4,72973e0415004dfea220babc479be082,41d0f568bb1e4fed8ad965a79d9bccef,5854ef1d24784ca0a046514b19286113,04afc3dd1eec4d13bc06cd01e0a5e838,db76e3333da846d5ae55b3b9dd735f26,7aa0ca0bf1514fa6a3a14dcde44cf0b9,9c668726c9b6449a9045d3f78408a2a1,7caf148d68a6431bbe56bcc6472d8720,babd36d1d3484fb99fe4e7ca78560ccf,7507865343e24cff80d55a29ec57f0e1,63f8b7d9b1f34851802e0408bf997ea5,a3369d5dd6b74af7b35edf1f10350881,9b8b8c3f86854416b82d57583896c1ff,881815c772564a8dbd5eb6de65ecbe7a,4b1bcf46753a48258d1d514440140b7f,1ca4c6a49e794954a235a82ee8981ecd,ab381d9129bb4b979fe3774e262524f8,a07d2c98831a4d79bc6c5c4288a24720,d8debd28039d494aa7fe18bc8e81af27,d0402b1f00f94acca96276c6ea8c8397,ebcdebd9639d4ec2b7072ad32bf48f50,636b3211100245919c09177ccea562e0,280c4440e18a487396a81b92cd3efa13,5163cf6740334a7d83375385c3ae2b67,a6083e4fab7a43498bfb3257658f9278,3b5c8b5cfb124ebd8f4a798d2a47d832,9fda332474534f96955b5bce8548947f,b5f5e62aba9842d8b48a28111ca6a372,c0cecff1207c4692a634d7de449d8d6d,e4c6428f1d264544b0eec52214e7b6ae,cbc8c5490a7f46b996fd4c3a9711f2d7,cfbdd82f4ce741a88109353c5fc898e4,6bfb965bc1b240a9ae2bd7a1e884ea51,eb36d63a993148289950af9098727759,f7357c3954d2408facaa6771a758ebcc,c05853b94ab8405eb26e0a8bd481b662,da1e6d3ebfa140b3a645342fd3bb5d95,988f3f6b607d442b8df006c980ddc2cd,b6af3c01968141baa3e99ca4b8d83847,75f669faee5d4023a425f2461e5fc5e9,165eb05677aa45fc9a53dfd8bfb7a45d,409891afd7cc4e12ba6b002581fef4f3,02ee0a87956f4bfab91b88282e502a3d,48601418028145ff84a450c877ea35aa,8de73f8d59eb45aa8c61138ab93c4f86,5dd82a46f48945b4bbb61f58b9d1c856,97993db997bd4ff88acfe1340ca86845,282aee84915b43179a09101b3ee6ad0e,8750e994985f43f383321bf0d6ca7932";

        role1.setMenuIds(defalutMenu1);
        String[] defalutMenus1 = defalutMenu1.split(",");
        ArrayList<String> menuIdList1 = new ArrayList<String>();
        for (int i = 0; i < defalutMenus1.length; i++) {
            menuIdList1.add(defalutMenus1[i]);
        }
        role1.setMenuIdList(menuIdList1);
        if (Global.isDemoMode()) {
            return null;
        }
        systemService.saveRole(role1,true,1);
        SysRoleActivity sysRoleActivity = new SysRoleActivity();
        if ("1".equals(type)) {
            String enname = UserUtils.getRoleActivity(com.getId()) + StringUtils.getPinYinHeadChar("标准审核部", 1);
            sysRoleActivity.setName("总工");
            sysRoleActivity.setEnname(enname);
            sysRoleActivity.setSort("2000");
            sysRoleActivity.setCompanyId(com.getId());
            sysRoleActivity.setType("3");
            sysRoleActivity.setDelType("1");
            sysRoleActivityService.save(sysRoleActivity);
        }else {
            String enname = UserUtils.getRoleActivityEnname(com.getId(),UserUtils.getUser());
            sysRoleActivity = sysRoleActivityService.getByEnname(enname+"bzshb");
        }
        List<SysRoleActivity> roleActivities = new ArrayList<>();
        roleActivities.add(sysRoleActivity);
        role1.setRoleActivityList(roleActivities);
        roleDao.insertRoleActivity(role1);
        return role1;
    }

    @Transactional(readOnly = false)
    public Role createCompanyBZSHBManagerRole( Office com,Office office,Office deptZLJG,String type) {
        Role role1 = new Role();
        role1.setName("标准审核部");
        role1.setSysData(Global.YES);
        if (type.equals("1")){
            role1.setDataScope("2");
        }else {
            role1.setDataScope("5");
            role1.setBranchCompany(office);
        }
        role1.setIsNewRecord(false);
        role1.setCompany(com);
        role1.setOffice(deptZLJG);
        role1.setRoleType("assignment");
        role1.setSysData("1");
        role1.setUseable("1");
        role1.setEnname("bzshb");
        role1.setNumber("4");
        String defalutMenu1 = "1,27,29,e3b80fae59ab4b4099d8349ecaf8c4dc,61dd3aa279684f92bb1c5d4019a32503,b0c5737cdc4f452684ce93589427c97d,af6b0e5d61fc44ecbebb449703a3df72,18c14562978e4b129fe3afb7dca8b868,6123fcb3d810492789e82a63b0d4310a,0aa3712414d049a6a24e8bcddeae509a,95a6a82dc5fc4d07b46e5df57a0606a3,62,b6fa4abb624349918fe531274fc15d23,89,76bdb735faa34667a6cbf13ebe937533,90,f5b2028cfad9470085c7c846de33193a,28abe4b0cb3b4c8480f84c07a5dfaf06,c7fa36c7142f481397c2cc12e2bc828a,e2efcf80fc3747329fe02d489dbfda95,e3b5b7451e164089aba0b9bf844b0398,438c28c3cbe44ed7bff951b2a3dc77cd,59e50eebe56743a78eecbc6eff1ef94c,cc1027689aca41a89302217127253034,200fe6cc80174065b382c961a4379d79,4c13ca96376341dfa25980e86964c1a9,638be8b3b5284e5088003917685bc19b,811600aadcb54b098b7dda4bf95ffc51,ce8dc7c97d93481dbd76cabb6e905b55,aa25b3904abb4bd7b779e5be37b4382e,aae6f4c7fd4c4a25afa5285210edfded,84ff9d5c165c4528af9361b6127cf5af,31b349a3624b47f28eb86d614f9f95b5,040b42e395de41d6805a958eb1897863,34874260e6fc4200a76b33c3cfa4d80d,5b6c6ea5ed2d45478e8144351d853e21,7c032c32512847f882a87416359ef0a3,6d30ea8c710a42b89da2f2506059b0ef,2ca7bf4ce6804a1c9e5b331d2df27c72,7717cbb6d71f405d890e9b648cbd2088,bddb0400764c42b08e427059c7752057,23d6c56239804053910e5c5aec481330,7f8b2018113545bcbdbfc87e7a236bec,547e121ffb714d16ae142b6935abb78f,af24101b962444f29d166b638b2fea30,6ea6519ec4d548128aa42f3ce004def3,4379bf54c2a543869abf5031b97b8efe,5e6da3f5368b4f06b7268b31a1157e9f,2bbf0e75f21d42938954391d0776fbde,545b39dacf564a5eb9d1658f80a0b8b0,b4ddedf9b451494a97a1ff9476e3897d,1b145ca23d3e4d81994a84d30a262d18,10a8c7c1d34e46dd8ef461cdad928afc,5ae4911d14bb48b39885b1fe522e999d,7f09def2533b45fdb3c733f1e72ca5fb,9ef29dca19d342b795efcbfe32793602,be72d72ae7034c509c164467b2abddda,ab879705f2354813acf8724404520e28,82487b87b97b475b98892b80cf83c96a,0f4a461b97be48eeab43bd052f630057,72e1a28b7251438a89b0a4f6bd522b89,e1f9a246057347d78a1966d6dc0168e3,5d3645d53ce14c9ab498fd9f050ba264,d361e6890cc84b049fc8268c2a8c4952,5cc0002cbcc14d3485ab8ae6d57368f2,fa7bb243643c44b18e989e3419eeb171,62768b6c65134f1e927b1807bc72115a,7b69b0e5d2d344e6aaa2b8726c23561a,434bbb1f9f8140b1beeb57d0c4db660d,7920c4c8da294d87bcb94e7756bf35ae,0752b85660694033bbdb013f30cde17f,bd6258cfbea64896b0401517e440d6a2,9aa3539682514546866d2998d741ac6c,eb3ad93ce6a44921b0490c842dd47d8c,9ce19042d497473c92d199a5e5a7daf0,1b3eb6ba64ea4589a8e2d42efa5d3f89,6b2dd5ff2a754a53a80798341a7d35d8,e53d754160f74d28b6d091829fc289f8,92d762bca040407bb4ce4b06a2cfa195,aa8a2b00622147259c85290f4d95b46d,0be3e487d874487fbfb5c49b52440a48,ce60b25f7da04af8b7cefccf2dbc7d36,6464ac9cc3fc47d88598f6c4e74caa19,04b312df403d4855b013743206ddb1de,17356c44ae894782a1015ec2ddaeaeda,a06eaefba7084e219f0c6ce4010b82e7,95a54e4db7e74f0586cd85cb4e2bd6f7,1a93047d6b0f4e3ca70fcf836b2517c4,93263a29941648979bb8de774978a880,8eba834a28864375aa3137590d26f37e,3c52b0a890524c6fb66cefc32114c60e,8b20519b794c42d28fdecbd0ccbba5c1,77189ceba655488681964a609c28256d,39e776aaf3644d24bb78ecf43704b71b,0dd5fa1192e34d5fac5e78ca38969126,1a6f82a80a054bc7ba4b516187231a54,5993ce4ec80a41fb9d4ea0713b74f4a4,3b42071536124a9c9dbf2ec319004b29,a966beefe58e4f95b24e89f4172cb7ce,2e1e75dbe28841529576dd1cd48d2437,cdd8829e5976438aaa66ce608f0175f5,71015695cf1445ecacce90669c67d733,31e9dd26b7314d8a9bf97dcf6dbdb22d,e5ecaa091bb0464f8553d4e7aa60647d,39055be3a23945868dbf4b5930043a88,07ba7c9aed104657aac3be87aa346d3b,5a7a9bfff0f44a2089a7d277f2c04557,be53cc152339400ba705c1d1dceff9ab,ed591ccba0564abd9979899049dc7b36,9a4425b817e94dbe844d7fd3e96a9033,bf4e54bfe9984441a8efa4dd8a4a23a6,5a1bcf2a2e0943f88bf655ca011fb288,a12c8904ff0147a99010eaa5aa0ea704,e7c6f2581c6e41bf8fe88a3d7fadc986,89709767beea48d0b0fb047316eb37d1,04d6a49b35514f91be43e9576f8d32fc,bd50e783ec714e5d86aafcca54e87c33,4acb0c7d002b47e29be7ebe9f40490f6,67b5ddef866b4982aa5afef7aad69924,7e090632f713440bba11ed6fc09eb291,f308051e11ca47eebb12709725b52eb4,dbea976ef1d248cabf13b88d311c4533,e0132620d4044e38a92f75b8335005ed,1b598563135e42d4bfe729cf32649f14,c4de9219f4f14684ae21d0904b7b78df,520983ab7dd64104a47e273941236b2d,133ba69bd04046b89eadaf793b9063db,11f24ca662044292a8ca95930ccc3581,0f30b4d621294d4fab359611cf82ab6f,b3c93fcd54b544e5b2c8e70ee7eb93ce,79ebe6595ce44208853ba553186b8348,8d639afccbec43f2be8d3ed88702893b,4e3483f3bbbb48f59b0e92a7d0ccc1c0,b160fccc15fa4ba09c74018d82d16eb7,52f248eadea74cf2b7780bf991d95e4e,d4cb9017aa1f4f8ca34ae8d31c986210,eff19819b1284424a9ad7879991ee080,79551897f86b4338bd09b41c5953a7db,f11bd4ef5ddd4c82865fad3f1f9d80a9,76ee95093a8748b29d55df1cb8cdebd0,081af742f6384a20b492ca99704f51ca,c26c844bbb2f4746904948f3ecfeb1ae,ba9bbc5581a043d8b9c204b8ddb6aa01,cde1afb048a743bb9caedb5770aabca1,0b2b37a28bb0457bbc4bf33c11b4c6e1,9f94d338d6c94b95814b2c50f9e463d1,7d1745ce2c444db9bf4b4e100c476fe8,e711e93aadd54eef8feeb0fa07f054d2,0e914774210646f784d02fb22f8df7db,ca0ce5e9eb09477c8c99b7b94e7869ce,760d9072f9bc480fa36fdb876930467c,5742a628f13045cda0317c3d4d839bc5,b9309305d94f4ee8b66eebbcae9f2db4,a42216d3eedd48f0ba3d3bef2470605f,9cdcda30da704c21b19ea0616de3ea86,f05ecbf7e4b04b268b38cb6e5c8124bf,788d2c1496874eeeae3b7aabbab95f76,8b112af90e054671adc30373c6dafd21,3bdb29667807493292a0ddc9bc0ccdb7,ffd6ebecc869410dab6410a07f3362ae,196868a4610b45588062aed357fc7c58,4cdd2d11be4c4c098b27c10d7cb3fa9e,fda40125207d426dbf86270002ff0c4a,7af028c6048c44cc91415a58d25d286e,ff7d2c8e96e840e1a5a690e4e0f96bad,9f806cf868dc48d398836ebd60bd2ec6,50ff5cdc6c8645b3bb61b8327d204f01,bafb57ea8d84430cbc0c11e5c67643f1,e2c91860b96c46619a74215627560235,f5b68a43bbae44c8a1fe821dfcaca26e,70b771772a0b4f1cb1e592f9a01b08a5,7992a39d64b2483789ae274e2df3c01d,f8d711312da14871a77b58f41a6f6242,673b27b0ab9148b7ae03933c18433971,624886a950bd4d18900a877738696432,685d8d9267f745f39bcc6fc2cc553d31,7d40ad05a1a54165a28ec897ddbcef01,4bcdb5199e8442078999bb8611ea6398,3b0c0ffea0cf45db816993baf64b3fb3,ad2d6d9581034e04addcb5b034a88fb3,8440c4fc5056426996e3e488b2261d28,e367c4223d794572a05fd256759d6a53,1b2dc5784c3c41988d5634e566b4a404,11be1538d7f54d7786004503b2e684a7,b462cb8c2ec7485d938416831a9771cc,f747656a36204e9588aba97064332436,78d0b2337b484c6298c9b3afc01fdb85,8a63d1f8d4bc4eb4a4db437491c9983a,2247f64a15e54053abb2e13152003c6b,d78ef9c5997d4e94804b8126cc387ea1,6ce3cd5a86ef43219dbda2b78e974c93,414b5fd5ebee45d98eca809a524dbe93,4b9ea9db5c834b698a78c7052c87b432,8590fa7adc144f49ae3d2575b7280c08,6148e4b152b9494fb9e7035e9b58e624,be1cff2a29f64800a5c4f91009711988,49b85b35bca5461993eac91ffd0b1fe4,5abae35db6de475e904425af8dc8ecbb,e3bc44b82c3543758d5a6c02b41dd526,89323021918e444bae3537521046d19f,141dd3257f85455d86f3828cc0217b5a,7bc2ad90009a4de2a0fa2947ccc09886,e51874f4b6ac455db76e30582659368b,519dee074e4c48ae9edd58d7c1e330da,060ab186c22f4f42a83f1622bf47ca52,8873bbcf4b42445f8117108769358a5c,46fa5286441a444095b917165c31304c,fa9fdd56222a4ce7ba2b37922afe0eaa,6885932aa4b443babc2b69fdd495889c,be71b947b33a48d8aee36ad0c60d10b8,b52457719429466687a2f228d20571ac,0bf937d745494d998cb8bd2cf3a63f4e,c17feb3860184dd59931f35814ad68b0,4cc68cbd880c48948a1f18fe18ec938c,c37b0691a05c4d659dcd01571f1907da,c1334fe3e8484079bfd62e479307b5b9,ebb58027b6c647b7965eeb31b63ec0c0,1821c5f9392e4859a829c78ebda44df6,dca21ab79bd14004b94c55e36604c30d,8d96cc3457644378b9b643243464f735,631437054079401fbe2c3c9a3f3bc375,c2f78ab4372243da8775f4d71a16fef5,03792664a45d4f63ac3efb899ce1b61e,fec04c46683a41e5a344f4d057342829,320fa70de6bc4890a0135bbfe8735b49,5a03493dce9f4323a0da7bf4fd5e78ab,b9bd46a1da1846d099566fd5c6db00cb,647021c7f798472889fd82f6c4a27ca2,313e905b78d746cb9e574b9634c3a113,623fd328c4cd4ee8b21eb03425b1f100,4676b4d1130146629fd98e2d2eb9a5b4,34d380813a36481ea2534c46d18b5927,894d87e55ac640b99e966a8786a2392b,477ba74d182a4ec0b86926dc67a0f419,362ca5fdca174fa3bdd6325a947320db,1ac3ad8dfa184fe4b6815950d3d23fdd,fdebee18c71f4a9db60096e75ad34d83,91432be4178a4781917fba0abd51df61,480c71baa65145f4bd037332d68e5e4f,536e3a3702b0453d92939846dc1f33b7,0333aedf1be640c78d2ee02473b7fd97,cbb3a6f1772b4f72a8af1375ef16e914,367ad9c43a6246dfac364ca59874883b,bac18430f6db466195a3a7116cdfb958,c4ec39be11be4b0eae321ee0559be8e6,8f7ffae7e47d462aa4142db7f06633d5,82e3b2423d86425195929fc3810ce7a4,72973e0415004dfea220babc479be082,41d0f568bb1e4fed8ad965a79d9bccef,5854ef1d24784ca0a046514b19286113,04afc3dd1eec4d13bc06cd01e0a5e838,db76e3333da846d5ae55b3b9dd735f26,7aa0ca0bf1514fa6a3a14dcde44cf0b9,9c668726c9b6449a9045d3f78408a2a1,7caf148d68a6431bbe56bcc6472d8720,babd36d1d3484fb99fe4e7ca78560ccf,7507865343e24cff80d55a29ec57f0e1,63f8b7d9b1f34851802e0408bf997ea5,a3369d5dd6b74af7b35edf1f10350881,9b8b8c3f86854416b82d57583896c1ff,881815c772564a8dbd5eb6de65ecbe7a,4b1bcf46753a48258d1d514440140b7f,1ca4c6a49e794954a235a82ee8981ecd,ab381d9129bb4b979fe3774e262524f8,a07d2c98831a4d79bc6c5c4288a24720,d8debd28039d494aa7fe18bc8e81af27,d0402b1f00f94acca96276c6ea8c8397,ebcdebd9639d4ec2b7072ad32bf48f50,636b3211100245919c09177ccea562e0,280c4440e18a487396a81b92cd3efa13,5163cf6740334a7d83375385c3ae2b67,a6083e4fab7a43498bfb3257658f9278,3b5c8b5cfb124ebd8f4a798d2a47d832,9fda332474534f96955b5bce8548947f,b5f5e62aba9842d8b48a28111ca6a372,c0cecff1207c4692a634d7de449d8d6d,e4c6428f1d264544b0eec52214e7b6ae,cbc8c5490a7f46b996fd4c3a9711f2d7,cfbdd82f4ce741a88109353c5fc898e4,6bfb965bc1b240a9ae2bd7a1e884ea51,eb36d63a993148289950af9098727759,f7357c3954d2408facaa6771a758ebcc,c05853b94ab8405eb26e0a8bd481b662,da1e6d3ebfa140b3a645342fd3bb5d95,988f3f6b607d442b8df006c980ddc2cd,b6af3c01968141baa3e99ca4b8d83847,75f669faee5d4023a425f2461e5fc5e9,165eb05677aa45fc9a53dfd8bfb7a45d,409891afd7cc4e12ba6b002581fef4f3,02ee0a87956f4bfab91b88282e502a3d,48601418028145ff84a450c877ea35aa,8de73f8d59eb45aa8c61138ab93c4f86,5dd82a46f48945b4bbb61f58b9d1c856,97993db997bd4ff88acfe1340ca86845,282aee84915b43179a09101b3ee6ad0e,8750e994985f43f383321bf0d6ca7932";

        role1.setMenuIds(defalutMenu1);
        String[] defalutMenus1 = defalutMenu1.split(",");
        ArrayList<String> menuIdList1 = new ArrayList<String>();
        for (int i = 0; i < defalutMenus1.length; i++) {
            menuIdList1.add(defalutMenus1[i]);
        }
        role1.setMenuIdList(menuIdList1);
        if (Global.isDemoMode()) {
            return null;
        }
        systemService.saveRole(role1,true,1);
        SysRoleActivity sysRoleActivity = new SysRoleActivity();
        if ("1".equals(type)) {
            String enname = UserUtils.getRoleActivity(com.getId()) + StringUtils.getPinYinHeadChar("标准审核部员工", 1);
            sysRoleActivity.setName("标准审核部");
            sysRoleActivity.setEnname(enname);
            sysRoleActivity.setSort("2000");
            sysRoleActivity.setCompanyId(com.getId());
            sysRoleActivity.setType("4");
            sysRoleActivity.setDelType("1");
            sysRoleActivityService.save(sysRoleActivity);
        }else {
            String enname = UserUtils.getRoleActivityEnname(com.getId(),UserUtils.getUser());
            sysRoleActivity = sysRoleActivityService.getByEnname(enname+"bzshbyg");
        }
        if (sysRoleActivity!=null) {
            List<SysRoleActivity> roleActivities = new ArrayList<>();
            roleActivities.add(sysRoleActivity);
            role1.setRoleActivityList(roleActivities);
            roleDao.insertRoleActivity(role1);
        }
        return role1;
    }
    @Transactional(readOnly = false)
    public Role createCompanyHZZGRole( Office com,Office office,Office deptBGS,String type) {
        Role role1 = new Role();
        role1.setName("行政主管");
        role1.setNumber("4");
        role1.setSysData(Global.YES);
        if (type.equals("1")){
            role1.setDataScope("2");
        }else {
            role1.setDataScope("5");
            role1.setBranchCompany(office);
        }
        role1.setIsNewRecord(false);
        role1.setCompany(com);
        role1.setOffice(deptBGS);
        role1.setRoleType("assignment");
        role1.setSysData("1");
        role1.setUseable("1");
        role1.setEnname("xzzg");
        String defalutMenu1 = "1,27,29,e3b80fae59ab4b4099d8349ecaf8c4dc,61dd3aa279684f92bb1c5d4019a32503,b0c5737cdc4f452684ce93589427c97d,af6b0e5d61fc44ecbebb449703a3df72,18c14562978e4b129fe3afb7dca8b868,6123fcb3d810492789e82a63b0d4310a,0aa3712414d049a6a24e8bcddeae509a,95a6a82dc5fc4d07b46e5df57a0606a3,62,b6fa4abb624349918fe531274fc15d23,89,76bdb735faa34667a6cbf13ebe937533,90,f5b2028cfad9470085c7c846de33193a,28abe4b0cb3b4c8480f84c07a5dfaf06,c7fa36c7142f481397c2cc12e2bc828a,e2efcf80fc3747329fe02d489dbfda95,e3b5b7451e164089aba0b9bf844b0398,438c28c3cbe44ed7bff951b2a3dc77cd,59e50eebe56743a78eecbc6eff1ef94c,cc1027689aca41a89302217127253034,200fe6cc80174065b382c961a4379d79,4c13ca96376341dfa25980e86964c1a9,638be8b3b5284e5088003917685bc19b,811600aadcb54b098b7dda4bf95ffc51,ce8dc7c97d93481dbd76cabb6e905b55,aa25b3904abb4bd7b779e5be37b4382e,aae6f4c7fd4c4a25afa5285210edfded,84ff9d5c165c4528af9361b6127cf5af,31b349a3624b47f28eb86d614f9f95b5,040b42e395de41d6805a958eb1897863,34874260e6fc4200a76b33c3cfa4d80d,5b6c6ea5ed2d45478e8144351d853e21,7c032c32512847f882a87416359ef0a3,6d30ea8c710a42b89da2f2506059b0ef,2ca7bf4ce6804a1c9e5b331d2df27c72,7717cbb6d71f405d890e9b648cbd2088,bddb0400764c42b08e427059c7752057,23d6c56239804053910e5c5aec481330,7f8b2018113545bcbdbfc87e7a236bec,547e121ffb714d16ae142b6935abb78f,af24101b962444f29d166b638b2fea30,6ea6519ec4d548128aa42f3ce004def3,4379bf54c2a543869abf5031b97b8efe,5e6da3f5368b4f06b7268b31a1157e9f,2bbf0e75f21d42938954391d0776fbde,545b39dacf564a5eb9d1658f80a0b8b0,b4ddedf9b451494a97a1ff9476e3897d,1b145ca23d3e4d81994a84d30a262d18,10a8c7c1d34e46dd8ef461cdad928afc,5ae4911d14bb48b39885b1fe522e999d,7f09def2533b45fdb3c733f1e72ca5fb,9ef29dca19d342b795efcbfe32793602,be72d72ae7034c509c164467b2abddda,ab879705f2354813acf8724404520e28,82487b87b97b475b98892b80cf83c96a,0f4a461b97be48eeab43bd052f630057,72e1a28b7251438a89b0a4f6bd522b89,e1f9a246057347d78a1966d6dc0168e3,5d3645d53ce14c9ab498fd9f050ba264,d361e6890cc84b049fc8268c2a8c4952,5cc0002cbcc14d3485ab8ae6d57368f2,fa7bb243643c44b18e989e3419eeb171,62768b6c65134f1e927b1807bc72115a,7b69b0e5d2d344e6aaa2b8726c23561a,434bbb1f9f8140b1beeb57d0c4db660d,7920c4c8da294d87bcb94e7756bf35ae,0752b85660694033bbdb013f30cde17f,bd6258cfbea64896b0401517e440d6a2,9aa3539682514546866d2998d741ac6c,eb3ad93ce6a44921b0490c842dd47d8c,9ce19042d497473c92d199a5e5a7daf0,1b3eb6ba64ea4589a8e2d42efa5d3f89,6b2dd5ff2a754a53a80798341a7d35d8,e53d754160f74d28b6d091829fc289f8,92d762bca040407bb4ce4b06a2cfa195,aa8a2b00622147259c85290f4d95b46d,0be3e487d874487fbfb5c49b52440a48,ce60b25f7da04af8b7cefccf2dbc7d36,6464ac9cc3fc47d88598f6c4e74caa19,04b312df403d4855b013743206ddb1de,17356c44ae894782a1015ec2ddaeaeda,a06eaefba7084e219f0c6ce4010b82e7,95a54e4db7e74f0586cd85cb4e2bd6f7,1a93047d6b0f4e3ca70fcf836b2517c4,93263a29941648979bb8de774978a880,8eba834a28864375aa3137590d26f37e,3c52b0a890524c6fb66cefc32114c60e,8b20519b794c42d28fdecbd0ccbba5c1,77189ceba655488681964a609c28256d,39e776aaf3644d24bb78ecf43704b71b,0dd5fa1192e34d5fac5e78ca38969126,1a6f82a80a054bc7ba4b516187231a54,5993ce4ec80a41fb9d4ea0713b74f4a4,3b42071536124a9c9dbf2ec319004b29,a966beefe58e4f95b24e89f4172cb7ce,2e1e75dbe28841529576dd1cd48d2437,cdd8829e5976438aaa66ce608f0175f5,71015695cf1445ecacce90669c67d733,31e9dd26b7314d8a9bf97dcf6dbdb22d,e5ecaa091bb0464f8553d4e7aa60647d,39055be3a23945868dbf4b5930043a88,07ba7c9aed104657aac3be87aa346d3b,5a7a9bfff0f44a2089a7d277f2c04557,be53cc152339400ba705c1d1dceff9ab,ed591ccba0564abd9979899049dc7b36,9a4425b817e94dbe844d7fd3e96a9033,bf4e54bfe9984441a8efa4dd8a4a23a6,5a1bcf2a2e0943f88bf655ca011fb288,a12c8904ff0147a99010eaa5aa0ea704,e7c6f2581c6e41bf8fe88a3d7fadc986,89709767beea48d0b0fb047316eb37d1,04d6a49b35514f91be43e9576f8d32fc,bd50e783ec714e5d86aafcca54e87c33,4acb0c7d002b47e29be7ebe9f40490f6,67b5ddef866b4982aa5afef7aad69924,7e090632f713440bba11ed6fc09eb291,f308051e11ca47eebb12709725b52eb4,dbea976ef1d248cabf13b88d311c4533,e0132620d4044e38a92f75b8335005ed,1b598563135e42d4bfe729cf32649f14,c4de9219f4f14684ae21d0904b7b78df,520983ab7dd64104a47e273941236b2d,133ba69bd04046b89eadaf793b9063db,11f24ca662044292a8ca95930ccc3581,0f30b4d621294d4fab359611cf82ab6f,b3c93fcd54b544e5b2c8e70ee7eb93ce,79ebe6595ce44208853ba553186b8348,8d639afccbec43f2be8d3ed88702893b,4e3483f3bbbb48f59b0e92a7d0ccc1c0,b160fccc15fa4ba09c74018d82d16eb7,52f248eadea74cf2b7780bf991d95e4e,d4cb9017aa1f4f8ca34ae8d31c986210,eff19819b1284424a9ad7879991ee080,79551897f86b4338bd09b41c5953a7db,f11bd4ef5ddd4c82865fad3f1f9d80a9,76ee95093a8748b29d55df1cb8cdebd0,081af742f6384a20b492ca99704f51ca,c26c844bbb2f4746904948f3ecfeb1ae,ba9bbc5581a043d8b9c204b8ddb6aa01,cde1afb048a743bb9caedb5770aabca1,0b2b37a28bb0457bbc4bf33c11b4c6e1,9f94d338d6c94b95814b2c50f9e463d1,7d1745ce2c444db9bf4b4e100c476fe8,e711e93aadd54eef8feeb0fa07f054d2,0e914774210646f784d02fb22f8df7db,ca0ce5e9eb09477c8c99b7b94e7869ce,760d9072f9bc480fa36fdb876930467c,5742a628f13045cda0317c3d4d839bc5,b9309305d94f4ee8b66eebbcae9f2db4,a42216d3eedd48f0ba3d3bef2470605f,9cdcda30da704c21b19ea0616de3ea86,f05ecbf7e4b04b268b38cb6e5c8124bf,788d2c1496874eeeae3b7aabbab95f76,8b112af90e054671adc30373c6dafd21,3bdb29667807493292a0ddc9bc0ccdb7,ffd6ebecc869410dab6410a07f3362ae,196868a4610b45588062aed357fc7c58,4cdd2d11be4c4c098b27c10d7cb3fa9e,fda40125207d426dbf86270002ff0c4a,7af028c6048c44cc91415a58d25d286e,ff7d2c8e96e840e1a5a690e4e0f96bad,9f806cf868dc48d398836ebd60bd2ec6,50ff5cdc6c8645b3bb61b8327d204f01,bafb57ea8d84430cbc0c11e5c67643f1,e2c91860b96c46619a74215627560235,f5b68a43bbae44c8a1fe821dfcaca26e,70b771772a0b4f1cb1e592f9a01b08a5,7992a39d64b2483789ae274e2df3c01d,f8d711312da14871a77b58f41a6f6242,673b27b0ab9148b7ae03933c18433971,624886a950bd4d18900a877738696432,685d8d9267f745f39bcc6fc2cc553d31,7d40ad05a1a54165a28ec897ddbcef01,4bcdb5199e8442078999bb8611ea6398,3b0c0ffea0cf45db816993baf64b3fb3,ad2d6d9581034e04addcb5b034a88fb3,8440c4fc5056426996e3e488b2261d28,e367c4223d794572a05fd256759d6a53,1b2dc5784c3c41988d5634e566b4a404,11be1538d7f54d7786004503b2e684a7,b462cb8c2ec7485d938416831a9771cc,f747656a36204e9588aba97064332436,78d0b2337b484c6298c9b3afc01fdb85,8a63d1f8d4bc4eb4a4db437491c9983a,2247f64a15e54053abb2e13152003c6b,d78ef9c5997d4e94804b8126cc387ea1,6ce3cd5a86ef43219dbda2b78e974c93,414b5fd5ebee45d98eca809a524dbe93,4b9ea9db5c834b698a78c7052c87b432,8590fa7adc144f49ae3d2575b7280c08,6148e4b152b9494fb9e7035e9b58e624,be1cff2a29f64800a5c4f91009711988,49b85b35bca5461993eac91ffd0b1fe4,5abae35db6de475e904425af8dc8ecbb,e3bc44b82c3543758d5a6c02b41dd526,89323021918e444bae3537521046d19f,141dd3257f85455d86f3828cc0217b5a,7bc2ad90009a4de2a0fa2947ccc09886,e51874f4b6ac455db76e30582659368b,519dee074e4c48ae9edd58d7c1e330da,060ab186c22f4f42a83f1622bf47ca52,8873bbcf4b42445f8117108769358a5c,46fa5286441a444095b917165c31304c,fa9fdd56222a4ce7ba2b37922afe0eaa,6885932aa4b443babc2b69fdd495889c,be71b947b33a48d8aee36ad0c60d10b8,b52457719429466687a2f228d20571ac,0bf937d745494d998cb8bd2cf3a63f4e,c17feb3860184dd59931f35814ad68b0,4cc68cbd880c48948a1f18fe18ec938c,c37b0691a05c4d659dcd01571f1907da,c1334fe3e8484079bfd62e479307b5b9,ebb58027b6c647b7965eeb31b63ec0c0,1821c5f9392e4859a829c78ebda44df6,dca21ab79bd14004b94c55e36604c30d,8d96cc3457644378b9b643243464f735,631437054079401fbe2c3c9a3f3bc375,c2f78ab4372243da8775f4d71a16fef5,03792664a45d4f63ac3efb899ce1b61e,fec04c46683a41e5a344f4d057342829,320fa70de6bc4890a0135bbfe8735b49,5a03493dce9f4323a0da7bf4fd5e78ab,b9bd46a1da1846d099566fd5c6db00cb,647021c7f798472889fd82f6c4a27ca2,313e905b78d746cb9e574b9634c3a113,623fd328c4cd4ee8b21eb03425b1f100,4676b4d1130146629fd98e2d2eb9a5b4,34d380813a36481ea2534c46d18b5927,894d87e55ac640b99e966a8786a2392b,477ba74d182a4ec0b86926dc67a0f419,362ca5fdca174fa3bdd6325a947320db,1ac3ad8dfa184fe4b6815950d3d23fdd,fdebee18c71f4a9db60096e75ad34d83,91432be4178a4781917fba0abd51df61,480c71baa65145f4bd037332d68e5e4f,536e3a3702b0453d92939846dc1f33b7,0333aedf1be640c78d2ee02473b7fd97,cbb3a6f1772b4f72a8af1375ef16e914,367ad9c43a6246dfac364ca59874883b,bac18430f6db466195a3a7116cdfb958,c4ec39be11be4b0eae321ee0559be8e6,8f7ffae7e47d462aa4142db7f06633d5,82e3b2423d86425195929fc3810ce7a4,72973e0415004dfea220babc479be082,41d0f568bb1e4fed8ad965a79d9bccef,5854ef1d24784ca0a046514b19286113,04afc3dd1eec4d13bc06cd01e0a5e838,db76e3333da846d5ae55b3b9dd735f26,7aa0ca0bf1514fa6a3a14dcde44cf0b9,9c668726c9b6449a9045d3f78408a2a1,7caf148d68a6431bbe56bcc6472d8720,babd36d1d3484fb99fe4e7ca78560ccf,7507865343e24cff80d55a29ec57f0e1,63f8b7d9b1f34851802e0408bf997ea5,a3369d5dd6b74af7b35edf1f10350881,9b8b8c3f86854416b82d57583896c1ff,881815c772564a8dbd5eb6de65ecbe7a,4b1bcf46753a48258d1d514440140b7f,1ca4c6a49e794954a235a82ee8981ecd,ab381d9129bb4b979fe3774e262524f8,a07d2c98831a4d79bc6c5c4288a24720,d8debd28039d494aa7fe18bc8e81af27,d0402b1f00f94acca96276c6ea8c8397,ebcdebd9639d4ec2b7072ad32bf48f50,636b3211100245919c09177ccea562e0,280c4440e18a487396a81b92cd3efa13,5163cf6740334a7d83375385c3ae2b67,a6083e4fab7a43498bfb3257658f9278,3b5c8b5cfb124ebd8f4a798d2a47d832,9fda332474534f96955b5bce8548947f,b5f5e62aba9842d8b48a28111ca6a372,c0cecff1207c4692a634d7de449d8d6d,e4c6428f1d264544b0eec52214e7b6ae,cbc8c5490a7f46b996fd4c3a9711f2d7,cfbdd82f4ce741a88109353c5fc898e4,6bfb965bc1b240a9ae2bd7a1e884ea51,eb36d63a993148289950af9098727759,f7357c3954d2408facaa6771a758ebcc,c05853b94ab8405eb26e0a8bd481b662,da1e6d3ebfa140b3a645342fd3bb5d95,988f3f6b607d442b8df006c980ddc2cd,b6af3c01968141baa3e99ca4b8d83847,75f669faee5d4023a425f2461e5fc5e9,165eb05677aa45fc9a53dfd8bfb7a45d,409891afd7cc4e12ba6b002581fef4f3,02ee0a87956f4bfab91b88282e502a3d,48601418028145ff84a450c877ea35aa,8de73f8d59eb45aa8c61138ab93c4f86,5dd82a46f48945b4bbb61f58b9d1c856,97993db997bd4ff88acfe1340ca86845,282aee84915b43179a09101b3ee6ad0e,8750e994985f43f383321bf0d6ca7932";

        role1.setMenuIds(defalutMenu1);
        String[] defalutMenus1 = defalutMenu1.split(",");
        ArrayList<String> menuIdList1 = new ArrayList<String>();
        for (int i = 0; i < defalutMenus1.length; i++) {
            menuIdList1.add(defalutMenus1[i]);
        }
        role1.setMenuIdList(menuIdList1);
        if (Global.isDemoMode()) {
            return null;
        }
        systemService.saveRole(role1,true,1);
        SysRoleActivity sysRoleActivity = new SysRoleActivity();
        if ("1".equals(type)) {
            String enname = UserUtils.getRoleActivity(com.getId()) + StringUtils.getPinYinHeadChar("行政主管", 1);
            sysRoleActivity.setName("行政主管");
            sysRoleActivity.setEnname(enname);
            sysRoleActivity.setSort("6000");
            sysRoleActivity.setCompanyId(com.getId());
            sysRoleActivity.setType("3");
            sysRoleActivity.setDelType("1");
            sysRoleActivityService.save(sysRoleActivity);
        }else {
            String enname = UserUtils.getRoleActivityEnname(com.getId(),UserUtils.getUser());
            sysRoleActivity = sysRoleActivityService.getByEnname(enname+"xzzg");
        }
        List<SysRoleActivity> roleActivities = new ArrayList<>();
        roleActivities.add(sysRoleActivity);
        role1.setRoleActivityList(roleActivities);
        roleDao.insertRoleActivity(role1);
        return role1;
    }

    @Transactional(readOnly = false)
    public List<Role> createOfficeManagerRole(Office com,Office comOffice,List<Office> offices,String type) {
        String ename = UserUtils.getRoleActivityEnname(com.getId(),UserUtils.getUser());
        List<Role> roleList = new ArrayList<>();
        for (Office office:offices) {
            //人力部门负责人
            Role role1 = new Role();
            role1.setName("部门负责人");
            role1.setSysData(Global.YES);
            role1.setDataScope("4");
            role1.setIsNewRecord(false);
            role1.setCompany(com);
            role1.setNumber("2");
            role1.setOffice(office);
            if ("2".equals(type)) {
                role1.setBranchCompany(comOffice);
            }
            role1.setRoleType("security-role");
            role1.setSysData("1");
            role1.setUseable("1");
            role1.setEnname("bmzr");
            String defalutMenu1 = "1,27,29,e3b80fae59ab4b4099d8349ecaf8c4dc,61dd3aa279684f92bb1c5d4019a32503,b0c5737cdc4f452684ce93589427c97d,af6b0e5d61fc44ecbebb449703a3df72,18c14562978e4b129fe3afb7dca8b868,6123fcb3d810492789e82a63b0d4310a,0aa3712414d049a6a24e8bcddeae509a,95a6a82dc5fc4d07b46e5df57a0606a3,62,b6fa4abb624349918fe531274fc15d23,89,76bdb735faa34667a6cbf13ebe937533,90,f5b2028cfad9470085c7c846de33193a,28abe4b0cb3b4c8480f84c07a5dfaf06,c7fa36c7142f481397c2cc12e2bc828a,e2efcf80fc3747329fe02d489dbfda95,e3b5b7451e164089aba0b9bf844b0398,438c28c3cbe44ed7bff951b2a3dc77cd,59e50eebe56743a78eecbc6eff1ef94c,cc1027689aca41a89302217127253034,200fe6cc80174065b382c961a4379d79,4c13ca96376341dfa25980e86964c1a9,638be8b3b5284e5088003917685bc19b,811600aadcb54b098b7dda4bf95ffc51,ce8dc7c97d93481dbd76cabb6e905b55,aa25b3904abb4bd7b779e5be37b4382e,aae6f4c7fd4c4a25afa5285210edfded,84ff9d5c165c4528af9361b6127cf5af,31b349a3624b47f28eb86d614f9f95b5,040b42e395de41d6805a958eb1897863,34874260e6fc4200a76b33c3cfa4d80d,5b6c6ea5ed2d45478e8144351d853e21,7c032c32512847f882a87416359ef0a3,6d30ea8c710a42b89da2f2506059b0ef,2ca7bf4ce6804a1c9e5b331d2df27c72,7717cbb6d71f405d890e9b648cbd2088,bddb0400764c42b08e427059c7752057,23d6c56239804053910e5c5aec481330,7f8b2018113545bcbdbfc87e7a236bec,547e121ffb714d16ae142b6935abb78f,af24101b962444f29d166b638b2fea30,6ea6519ec4d548128aa42f3ce004def3,4379bf54c2a543869abf5031b97b8efe,5e6da3f5368b4f06b7268b31a1157e9f,2bbf0e75f21d42938954391d0776fbde,545b39dacf564a5eb9d1658f80a0b8b0,b4ddedf9b451494a97a1ff9476e3897d,1b145ca23d3e4d81994a84d30a262d18,10a8c7c1d34e46dd8ef461cdad928afc,5ae4911d14bb48b39885b1fe522e999d,7f09def2533b45fdb3c733f1e72ca5fb,9ef29dca19d342b795efcbfe32793602,be72d72ae7034c509c164467b2abddda,ab879705f2354813acf8724404520e28,82487b87b97b475b98892b80cf83c96a,0f4a461b97be48eeab43bd052f630057,72e1a28b7251438a89b0a4f6bd522b89,e1f9a246057347d78a1966d6dc0168e3,5d3645d53ce14c9ab498fd9f050ba264,d361e6890cc84b049fc8268c2a8c4952,5cc0002cbcc14d3485ab8ae6d57368f2,fa7bb243643c44b18e989e3419eeb171,62768b6c65134f1e927b1807bc72115a,7b69b0e5d2d344e6aaa2b8726c23561a,434bbb1f9f8140b1beeb57d0c4db660d,7920c4c8da294d87bcb94e7756bf35ae,0752b85660694033bbdb013f30cde17f,bd6258cfbea64896b0401517e440d6a2,9aa3539682514546866d2998d741ac6c,eb3ad93ce6a44921b0490c842dd47d8c,9ce19042d497473c92d199a5e5a7daf0,1b3eb6ba64ea4589a8e2d42efa5d3f89,6b2dd5ff2a754a53a80798341a7d35d8,e53d754160f74d28b6d091829fc289f8,92d762bca040407bb4ce4b06a2cfa195,aa8a2b00622147259c85290f4d95b46d,0be3e487d874487fbfb5c49b52440a48,ce60b25f7da04af8b7cefccf2dbc7d36,6464ac9cc3fc47d88598f6c4e74caa19,04b312df403d4855b013743206ddb1de,17356c44ae894782a1015ec2ddaeaeda,a06eaefba7084e219f0c6ce4010b82e7,95a54e4db7e74f0586cd85cb4e2bd6f7,1a93047d6b0f4e3ca70fcf836b2517c4,93263a29941648979bb8de774978a880,8eba834a28864375aa3137590d26f37e,3c52b0a890524c6fb66cefc32114c60e,8b20519b794c42d28fdecbd0ccbba5c1,77189ceba655488681964a609c28256d,39e776aaf3644d24bb78ecf43704b71b,0dd5fa1192e34d5fac5e78ca38969126,1a6f82a80a054bc7ba4b516187231a54,5993ce4ec80a41fb9d4ea0713b74f4a4,3b42071536124a9c9dbf2ec319004b29,a966beefe58e4f95b24e89f4172cb7ce,2e1e75dbe28841529576dd1cd48d2437,cdd8829e5976438aaa66ce608f0175f5,71015695cf1445ecacce90669c67d733,31e9dd26b7314d8a9bf97dcf6dbdb22d,e5ecaa091bb0464f8553d4e7aa60647d,39055be3a23945868dbf4b5930043a88,07ba7c9aed104657aac3be87aa346d3b,5a7a9bfff0f44a2089a7d277f2c04557,be53cc152339400ba705c1d1dceff9ab,ed591ccba0564abd9979899049dc7b36,9a4425b817e94dbe844d7fd3e96a9033,bf4e54bfe9984441a8efa4dd8a4a23a6,5a1bcf2a2e0943f88bf655ca011fb288,a12c8904ff0147a99010eaa5aa0ea704,e7c6f2581c6e41bf8fe88a3d7fadc986,89709767beea48d0b0fb047316eb37d1,04d6a49b35514f91be43e9576f8d32fc,bd50e783ec714e5d86aafcca54e87c33,4acb0c7d002b47e29be7ebe9f40490f6,67b5ddef866b4982aa5afef7aad69924,7e090632f713440bba11ed6fc09eb291,f308051e11ca47eebb12709725b52eb4,dbea976ef1d248cabf13b88d311c4533,e0132620d4044e38a92f75b8335005ed,1b598563135e42d4bfe729cf32649f14,c4de9219f4f14684ae21d0904b7b78df,520983ab7dd64104a47e273941236b2d,133ba69bd04046b89eadaf793b9063db,11f24ca662044292a8ca95930ccc3581,0f30b4d621294d4fab359611cf82ab6f,b3c93fcd54b544e5b2c8e70ee7eb93ce,79ebe6595ce44208853ba553186b8348,8d639afccbec43f2be8d3ed88702893b,4e3483f3bbbb48f59b0e92a7d0ccc1c0,b160fccc15fa4ba09c74018d82d16eb7,52f248eadea74cf2b7780bf991d95e4e,d4cb9017aa1f4f8ca34ae8d31c986210,eff19819b1284424a9ad7879991ee080,79551897f86b4338bd09b41c5953a7db,f11bd4ef5ddd4c82865fad3f1f9d80a9,76ee95093a8748b29d55df1cb8cdebd0,081af742f6384a20b492ca99704f51ca,c26c844bbb2f4746904948f3ecfeb1ae,ba9bbc5581a043d8b9c204b8ddb6aa01,cde1afb048a743bb9caedb5770aabca1,0b2b37a28bb0457bbc4bf33c11b4c6e1,9f94d338d6c94b95814b2c50f9e463d1,7d1745ce2c444db9bf4b4e100c476fe8,e711e93aadd54eef8feeb0fa07f054d2,0e914774210646f784d02fb22f8df7db,ca0ce5e9eb09477c8c99b7b94e7869ce,760d9072f9bc480fa36fdb876930467c,5742a628f13045cda0317c3d4d839bc5,b9309305d94f4ee8b66eebbcae9f2db4,a42216d3eedd48f0ba3d3bef2470605f,9cdcda30da704c21b19ea0616de3ea86,f05ecbf7e4b04b268b38cb6e5c8124bf,788d2c1496874eeeae3b7aabbab95f76,8b112af90e054671adc30373c6dafd21,3bdb29667807493292a0ddc9bc0ccdb7,ffd6ebecc869410dab6410a07f3362ae,196868a4610b45588062aed357fc7c58,4cdd2d11be4c4c098b27c10d7cb3fa9e,fda40125207d426dbf86270002ff0c4a,7af028c6048c44cc91415a58d25d286e,ff7d2c8e96e840e1a5a690e4e0f96bad,9f806cf868dc48d398836ebd60bd2ec6,50ff5cdc6c8645b3bb61b8327d204f01,bafb57ea8d84430cbc0c11e5c67643f1,e2c91860b96c46619a74215627560235,f5b68a43bbae44c8a1fe821dfcaca26e,70b771772a0b4f1cb1e592f9a01b08a5,7992a39d64b2483789ae274e2df3c01d,f8d711312da14871a77b58f41a6f6242,673b27b0ab9148b7ae03933c18433971,624886a950bd4d18900a877738696432,685d8d9267f745f39bcc6fc2cc553d31,7d40ad05a1a54165a28ec897ddbcef01,4bcdb5199e8442078999bb8611ea6398,3b0c0ffea0cf45db816993baf64b3fb3,ad2d6d9581034e04addcb5b034a88fb3,8440c4fc5056426996e3e488b2261d28,e367c4223d794572a05fd256759d6a53,1b2dc5784c3c41988d5634e566b4a404,11be1538d7f54d7786004503b2e684a7,b462cb8c2ec7485d938416831a9771cc,f747656a36204e9588aba97064332436,78d0b2337b484c6298c9b3afc01fdb85,8a63d1f8d4bc4eb4a4db437491c9983a,2247f64a15e54053abb2e13152003c6b,d78ef9c5997d4e94804b8126cc387ea1,6ce3cd5a86ef43219dbda2b78e974c93,414b5fd5ebee45d98eca809a524dbe93,4b9ea9db5c834b698a78c7052c87b432,8590fa7adc144f49ae3d2575b7280c08,6148e4b152b9494fb9e7035e9b58e624,be1cff2a29f64800a5c4f91009711988,49b85b35bca5461993eac91ffd0b1fe4,5abae35db6de475e904425af8dc8ecbb,e3bc44b82c3543758d5a6c02b41dd526,89323021918e444bae3537521046d19f,141dd3257f85455d86f3828cc0217b5a,7bc2ad90009a4de2a0fa2947ccc09886,e51874f4b6ac455db76e30582659368b,519dee074e4c48ae9edd58d7c1e330da,060ab186c22f4f42a83f1622bf47ca52,8873bbcf4b42445f8117108769358a5c,46fa5286441a444095b917165c31304c,fa9fdd56222a4ce7ba2b37922afe0eaa,6885932aa4b443babc2b69fdd495889c,be71b947b33a48d8aee36ad0c60d10b8,b52457719429466687a2f228d20571ac,0bf937d745494d998cb8bd2cf3a63f4e,c17feb3860184dd59931f35814ad68b0,4cc68cbd880c48948a1f18fe18ec938c,c37b0691a05c4d659dcd01571f1907da,c1334fe3e8484079bfd62e479307b5b9,ebb58027b6c647b7965eeb31b63ec0c0,1821c5f9392e4859a829c78ebda44df6,dca21ab79bd14004b94c55e36604c30d,8d96cc3457644378b9b643243464f735,631437054079401fbe2c3c9a3f3bc375,c2f78ab4372243da8775f4d71a16fef5,03792664a45d4f63ac3efb899ce1b61e,fec04c46683a41e5a344f4d057342829,320fa70de6bc4890a0135bbfe8735b49,5a03493dce9f4323a0da7bf4fd5e78ab,b9bd46a1da1846d099566fd5c6db00cb,647021c7f798472889fd82f6c4a27ca2,313e905b78d746cb9e574b9634c3a113,623fd328c4cd4ee8b21eb03425b1f100,4676b4d1130146629fd98e2d2eb9a5b4,34d380813a36481ea2534c46d18b5927,894d87e55ac640b99e966a8786a2392b,477ba74d182a4ec0b86926dc67a0f419,362ca5fdca174fa3bdd6325a947320db,1ac3ad8dfa184fe4b6815950d3d23fdd,fdebee18c71f4a9db60096e75ad34d83,91432be4178a4781917fba0abd51df61,480c71baa65145f4bd037332d68e5e4f,536e3a3702b0453d92939846dc1f33b7,0333aedf1be640c78d2ee02473b7fd97,cbb3a6f1772b4f72a8af1375ef16e914,367ad9c43a6246dfac364ca59874883b,bac18430f6db466195a3a7116cdfb958,c4ec39be11be4b0eae321ee0559be8e6,8f7ffae7e47d462aa4142db7f06633d5,82e3b2423d86425195929fc3810ce7a4,72973e0415004dfea220babc479be082,41d0f568bb1e4fed8ad965a79d9bccef,5854ef1d24784ca0a046514b19286113,04afc3dd1eec4d13bc06cd01e0a5e838,db76e3333da846d5ae55b3b9dd735f26,7aa0ca0bf1514fa6a3a14dcde44cf0b9,9c668726c9b6449a9045d3f78408a2a1,7caf148d68a6431bbe56bcc6472d8720,babd36d1d3484fb99fe4e7ca78560ccf,7507865343e24cff80d55a29ec57f0e1,63f8b7d9b1f34851802e0408bf997ea5,a3369d5dd6b74af7b35edf1f10350881,9b8b8c3f86854416b82d57583896c1ff,881815c772564a8dbd5eb6de65ecbe7a,4b1bcf46753a48258d1d514440140b7f,1ca4c6a49e794954a235a82ee8981ecd,ab381d9129bb4b979fe3774e262524f8,a07d2c98831a4d79bc6c5c4288a24720,d8debd28039d494aa7fe18bc8e81af27,d0402b1f00f94acca96276c6ea8c8397,ebcdebd9639d4ec2b7072ad32bf48f50,636b3211100245919c09177ccea562e0,280c4440e18a487396a81b92cd3efa13,5163cf6740334a7d83375385c3ae2b67,a6083e4fab7a43498bfb3257658f9278,3b5c8b5cfb124ebd8f4a798d2a47d832,9fda332474534f96955b5bce8548947f,b5f5e62aba9842d8b48a28111ca6a372,c0cecff1207c4692a634d7de449d8d6d,e4c6428f1d264544b0eec52214e7b6ae,cbc8c5490a7f46b996fd4c3a9711f2d7,cfbdd82f4ce741a88109353c5fc898e4,6bfb965bc1b240a9ae2bd7a1e884ea51,eb36d63a993148289950af9098727759,f7357c3954d2408facaa6771a758ebcc,c05853b94ab8405eb26e0a8bd481b662,da1e6d3ebfa140b3a645342fd3bb5d95,988f3f6b607d442b8df006c980ddc2cd,b6af3c01968141baa3e99ca4b8d83847,75f669faee5d4023a425f2461e5fc5e9,165eb05677aa45fc9a53dfd8bfb7a45d,409891afd7cc4e12ba6b002581fef4f3,02ee0a87956f4bfab91b88282e502a3d,48601418028145ff84a450c877ea35aa,8de73f8d59eb45aa8c61138ab93c4f86,5dd82a46f48945b4bbb61f58b9d1c856,97993db997bd4ff88acfe1340ca86845,282aee84915b43179a09101b3ee6ad0e,8750e994985f43f383321bf0d6ca7932";

            role1.setMenuIds(defalutMenu1);
            String[] defalutMenus1 = defalutMenu1.split(",");
            ArrayList<String> menuIdList1 = new ArrayList<String>();
            for (int i = 0; i < defalutMenus1.length; i++) {
                menuIdList1.add(defalutMenus1[i]);
            }
            role1.setMenuIdList(menuIdList1);
            if (Global.isDemoMode()) {
                return null;
            }
            systemService.saveRole(role1, true,1);
            if (office.getName().equals("财务部")) {
                String enname = UserUtils.getRoleActivity(com.getId()) + StringUtils.getPinYinHeadChar( "财务部主任", 1);
                SysRoleActivity sysRoleActivity = new SysRoleActivity();
                if ("1".equals(type)){
                    sysRoleActivity.setName("财务部负责人");
                    sysRoleActivity.setEnname(enname);
                    sysRoleActivity.setSort("6000");
                    sysRoleActivity.setType("3");
                    sysRoleActivity.setDelType("1");
                    sysRoleActivity.setCompanyId(com.getId());
                    sysRoleActivityService.save(sysRoleActivity);
                }else {
                    sysRoleActivity = sysRoleActivityService.getByEnname(ename+"cwbzr");
                }
                List<SysRoleActivity> roleActivities = new ArrayList<>();
                roleActivities.add(sysRoleActivity);
                role1.setRoleActivityList(roleActivities);
                roleDao.insertRoleActivity(role1);
            }else if (office.getName().equals("办公室")){
                String enname = UserUtils.getRoleActivity(com.getId()) + StringUtils.getPinYinHeadChar( "行政部主任", 1);
                SysRoleActivity sysRoleActivity = new SysRoleActivity();
                if ("1".equals(type)){
                    sysRoleActivity.setName("行政部负责人");
                    sysRoleActivity.setEnname(enname);
                    sysRoleActivity.setSort("6100");
                    sysRoleActivity.setType("3");
                    sysRoleActivity.setDelType("1");
                    sysRoleActivity.setCompanyId(com.getId());
                    sysRoleActivityService.save(sysRoleActivity);
                }else {
                    sysRoleActivity = sysRoleActivityService.getByEnname(ename+"xzbzr");
                }
                List<SysRoleActivity> roleActivities = new ArrayList<>();
                roleActivities.add(sysRoleActivity);
                role1.setRoleActivityList(roleActivities);
                roleDao.insertRoleActivity(role1);
            }else if(office.getName().equals("市场部")){
                String enname = UserUtils.getRoleActivity(com.getId()) + StringUtils.getPinYinHeadChar("市场研发管理部主任", 1);
                SysRoleActivity sysRoleActivity = new SysRoleActivity();
                SysRoleActivity sysRoleActivity2 = new SysRoleActivity();
                if ("1".equals(type)){
                    sysRoleActivity.setName("市场研发管理部负责人");
                    sysRoleActivity.setEnname(enname);
                    sysRoleActivity.setSort("8000");
                    sysRoleActivity.setType("3");
                    sysRoleActivity.setDelType("1");
                    sysRoleActivity.setCompanyId(com.getId());
                    sysRoleActivityService.save(sysRoleActivity);
                    String enname2 = UserUtils.getRoleActivity(com.getId()) + StringUtils.getPinYinHeadChar("调整审批人", 1);
                    sysRoleActivity2.setName("调整审批人");
                    sysRoleActivity2.setEnname(enname2);
                    sysRoleActivity2.setSort("8100");
                    sysRoleActivity2.setType("3");
                    sysRoleActivity2.setDelType("1");
                    sysRoleActivity2.setCompanyId(com.getId());
                    sysRoleActivityService.save(sysRoleActivity2);
                }else {
                    sysRoleActivity = sysRoleActivityService.getByEnname(ename+"scyfglbzr");
                    sysRoleActivity2 = sysRoleActivityService.getByEnname(ename+"dzspr");
                }

                List<SysRoleActivity> roleActivities = new ArrayList<>();
                roleActivities.add(sysRoleActivity);
                roleActivities.add(sysRoleActivity2);
                role1.setRoleActivityList(roleActivities);
                roleDao.insertRoleActivity(role1);
            }
            roleList.add(role1);
        }
        SysRoleActivity roleActivity = new SysRoleActivity();
        if ("1".equals(type)) {
            roleActivity.setName("业务部负责人");
            roleActivity.setEnname(UserUtils.getRoleActivity(com.getId()) + StringUtils.getPinYinHeadChar("部门主任", 1));
            roleActivity.setSort("6500");
            roleActivity.setType("2");
            roleActivity.setCompanyId(com.getId());
            roleActivity.setDelType("1");
            sysRoleActivityService.save(roleActivity);
        }else if("2".equals(type)) {
            roleActivity = sysRoleActivityService.getByEnname(ename+"bmzr");
        }
        for (Role role1 : roleList) {
            List<SysRoleActivity> roleActivities = new ArrayList<>();
            roleActivities.add(roleActivity);
            role1.setRoleActivityList(roleActivities);
            roleDao.insertRoleActivity(role1);
        }
        return roleList;
    }

    @Transactional(readOnly = false)
    public List<Role> createOfficeManagerRoleNoActivity(Office com,Office comOffice,List<Office> offices,String type) {
        List<Role> roleList = new ArrayList<>();
        for (Office office:offices) {
            //人力部门负责人
            Role role1 = new Role();
            role1.setName("部门负责人");
            role1.setSysData(Global.YES);
            role1.setDataScope("4");
            role1.setNumber("2");
            role1.setIsNewRecord(false);
            role1.setCompany(com);
            role1.setOffice(office);
            if ("2".equals(type)) {
                role1.setBranchCompany(comOffice);
            }
            role1.setRoleType("security-role");
            role1.setSysData("1");
            role1.setUseable("1");
            role1.setEnname("bmzr");
            String defalutMenu1 = "1,27,29,e3b80fae59ab4b4099d8349ecaf8c4dc,61dd3aa279684f92bb1c5d4019a32503,b0c5737cdc4f452684ce93589427c97d,af6b0e5d61fc44ecbebb449703a3df72,18c14562978e4b129fe3afb7dca8b868,6123fcb3d810492789e82a63b0d4310a,0aa3712414d049a6a24e8bcddeae509a,95a6a82dc5fc4d07b46e5df57a0606a3,62,b6fa4abb624349918fe531274fc15d23,89,76bdb735faa34667a6cbf13ebe937533,90,f5b2028cfad9470085c7c846de33193a,28abe4b0cb3b4c8480f84c07a5dfaf06,c7fa36c7142f481397c2cc12e2bc828a,e2efcf80fc3747329fe02d489dbfda95,e3b5b7451e164089aba0b9bf844b0398,438c28c3cbe44ed7bff951b2a3dc77cd,59e50eebe56743a78eecbc6eff1ef94c,cc1027689aca41a89302217127253034,200fe6cc80174065b382c961a4379d79,4c13ca96376341dfa25980e86964c1a9,638be8b3b5284e5088003917685bc19b,811600aadcb54b098b7dda4bf95ffc51,ce8dc7c97d93481dbd76cabb6e905b55,aa25b3904abb4bd7b779e5be37b4382e,aae6f4c7fd4c4a25afa5285210edfded,84ff9d5c165c4528af9361b6127cf5af,31b349a3624b47f28eb86d614f9f95b5,040b42e395de41d6805a958eb1897863,34874260e6fc4200a76b33c3cfa4d80d,5b6c6ea5ed2d45478e8144351d853e21,7c032c32512847f882a87416359ef0a3,6d30ea8c710a42b89da2f2506059b0ef,2ca7bf4ce6804a1c9e5b331d2df27c72,7717cbb6d71f405d890e9b648cbd2088,bddb0400764c42b08e427059c7752057,23d6c56239804053910e5c5aec481330,7f8b2018113545bcbdbfc87e7a236bec,547e121ffb714d16ae142b6935abb78f,af24101b962444f29d166b638b2fea30,6ea6519ec4d548128aa42f3ce004def3,4379bf54c2a543869abf5031b97b8efe,5e6da3f5368b4f06b7268b31a1157e9f,2bbf0e75f21d42938954391d0776fbde,545b39dacf564a5eb9d1658f80a0b8b0,b4ddedf9b451494a97a1ff9476e3897d,1b145ca23d3e4d81994a84d30a262d18,10a8c7c1d34e46dd8ef461cdad928afc,5ae4911d14bb48b39885b1fe522e999d,7f09def2533b45fdb3c733f1e72ca5fb,9ef29dca19d342b795efcbfe32793602,be72d72ae7034c509c164467b2abddda,ab879705f2354813acf8724404520e28,82487b87b97b475b98892b80cf83c96a,0f4a461b97be48eeab43bd052f630057,72e1a28b7251438a89b0a4f6bd522b89,e1f9a246057347d78a1966d6dc0168e3,5d3645d53ce14c9ab498fd9f050ba264,d361e6890cc84b049fc8268c2a8c4952,5cc0002cbcc14d3485ab8ae6d57368f2,fa7bb243643c44b18e989e3419eeb171,62768b6c65134f1e927b1807bc72115a,7b69b0e5d2d344e6aaa2b8726c23561a,434bbb1f9f8140b1beeb57d0c4db660d,7920c4c8da294d87bcb94e7756bf35ae,0752b85660694033bbdb013f30cde17f,bd6258cfbea64896b0401517e440d6a2,9aa3539682514546866d2998d741ac6c,eb3ad93ce6a44921b0490c842dd47d8c,9ce19042d497473c92d199a5e5a7daf0,1b3eb6ba64ea4589a8e2d42efa5d3f89,6b2dd5ff2a754a53a80798341a7d35d8,e53d754160f74d28b6d091829fc289f8,92d762bca040407bb4ce4b06a2cfa195,aa8a2b00622147259c85290f4d95b46d,0be3e487d874487fbfb5c49b52440a48,ce60b25f7da04af8b7cefccf2dbc7d36,6464ac9cc3fc47d88598f6c4e74caa19,04b312df403d4855b013743206ddb1de,17356c44ae894782a1015ec2ddaeaeda,a06eaefba7084e219f0c6ce4010b82e7,95a54e4db7e74f0586cd85cb4e2bd6f7,1a93047d6b0f4e3ca70fcf836b2517c4,93263a29941648979bb8de774978a880,8eba834a28864375aa3137590d26f37e,3c52b0a890524c6fb66cefc32114c60e,8b20519b794c42d28fdecbd0ccbba5c1,77189ceba655488681964a609c28256d,39e776aaf3644d24bb78ecf43704b71b,0dd5fa1192e34d5fac5e78ca38969126,1a6f82a80a054bc7ba4b516187231a54,5993ce4ec80a41fb9d4ea0713b74f4a4,3b42071536124a9c9dbf2ec319004b29,a966beefe58e4f95b24e89f4172cb7ce,2e1e75dbe28841529576dd1cd48d2437,cdd8829e5976438aaa66ce608f0175f5,71015695cf1445ecacce90669c67d733,31e9dd26b7314d8a9bf97dcf6dbdb22d,e5ecaa091bb0464f8553d4e7aa60647d,39055be3a23945868dbf4b5930043a88,07ba7c9aed104657aac3be87aa346d3b,5a7a9bfff0f44a2089a7d277f2c04557,be53cc152339400ba705c1d1dceff9ab,ed591ccba0564abd9979899049dc7b36,9a4425b817e94dbe844d7fd3e96a9033,bf4e54bfe9984441a8efa4dd8a4a23a6,5a1bcf2a2e0943f88bf655ca011fb288,a12c8904ff0147a99010eaa5aa0ea704,e7c6f2581c6e41bf8fe88a3d7fadc986,89709767beea48d0b0fb047316eb37d1,04d6a49b35514f91be43e9576f8d32fc,bd50e783ec714e5d86aafcca54e87c33,4acb0c7d002b47e29be7ebe9f40490f6,67b5ddef866b4982aa5afef7aad69924,7e090632f713440bba11ed6fc09eb291,f308051e11ca47eebb12709725b52eb4,dbea976ef1d248cabf13b88d311c4533,e0132620d4044e38a92f75b8335005ed,1b598563135e42d4bfe729cf32649f14,c4de9219f4f14684ae21d0904b7b78df,520983ab7dd64104a47e273941236b2d,133ba69bd04046b89eadaf793b9063db,11f24ca662044292a8ca95930ccc3581,0f30b4d621294d4fab359611cf82ab6f,b3c93fcd54b544e5b2c8e70ee7eb93ce,79ebe6595ce44208853ba553186b8348,8d639afccbec43f2be8d3ed88702893b,4e3483f3bbbb48f59b0e92a7d0ccc1c0,b160fccc15fa4ba09c74018d82d16eb7,52f248eadea74cf2b7780bf991d95e4e,d4cb9017aa1f4f8ca34ae8d31c986210,eff19819b1284424a9ad7879991ee080,79551897f86b4338bd09b41c5953a7db,f11bd4ef5ddd4c82865fad3f1f9d80a9,76ee95093a8748b29d55df1cb8cdebd0,081af742f6384a20b492ca99704f51ca,c26c844bbb2f4746904948f3ecfeb1ae,ba9bbc5581a043d8b9c204b8ddb6aa01,cde1afb048a743bb9caedb5770aabca1,0b2b37a28bb0457bbc4bf33c11b4c6e1,9f94d338d6c94b95814b2c50f9e463d1,7d1745ce2c444db9bf4b4e100c476fe8,e711e93aadd54eef8feeb0fa07f054d2,0e914774210646f784d02fb22f8df7db,ca0ce5e9eb09477c8c99b7b94e7869ce,760d9072f9bc480fa36fdb876930467c,5742a628f13045cda0317c3d4d839bc5,b9309305d94f4ee8b66eebbcae9f2db4,a42216d3eedd48f0ba3d3bef2470605f,9cdcda30da704c21b19ea0616de3ea86,f05ecbf7e4b04b268b38cb6e5c8124bf,788d2c1496874eeeae3b7aabbab95f76,8b112af90e054671adc30373c6dafd21,3bdb29667807493292a0ddc9bc0ccdb7,ffd6ebecc869410dab6410a07f3362ae,196868a4610b45588062aed357fc7c58,4cdd2d11be4c4c098b27c10d7cb3fa9e,fda40125207d426dbf86270002ff0c4a,7af028c6048c44cc91415a58d25d286e,ff7d2c8e96e840e1a5a690e4e0f96bad,9f806cf868dc48d398836ebd60bd2ec6,50ff5cdc6c8645b3bb61b8327d204f01,bafb57ea8d84430cbc0c11e5c67643f1,e2c91860b96c46619a74215627560235,f5b68a43bbae44c8a1fe821dfcaca26e,70b771772a0b4f1cb1e592f9a01b08a5,7992a39d64b2483789ae274e2df3c01d,f8d711312da14871a77b58f41a6f6242,673b27b0ab9148b7ae03933c18433971,624886a950bd4d18900a877738696432,685d8d9267f745f39bcc6fc2cc553d31,7d40ad05a1a54165a28ec897ddbcef01,4bcdb5199e8442078999bb8611ea6398,3b0c0ffea0cf45db816993baf64b3fb3,ad2d6d9581034e04addcb5b034a88fb3,8440c4fc5056426996e3e488b2261d28,e367c4223d794572a05fd256759d6a53,1b2dc5784c3c41988d5634e566b4a404,11be1538d7f54d7786004503b2e684a7,b462cb8c2ec7485d938416831a9771cc,f747656a36204e9588aba97064332436,78d0b2337b484c6298c9b3afc01fdb85,8a63d1f8d4bc4eb4a4db437491c9983a,2247f64a15e54053abb2e13152003c6b,d78ef9c5997d4e94804b8126cc387ea1,6ce3cd5a86ef43219dbda2b78e974c93,414b5fd5ebee45d98eca809a524dbe93,4b9ea9db5c834b698a78c7052c87b432,8590fa7adc144f49ae3d2575b7280c08,6148e4b152b9494fb9e7035e9b58e624,be1cff2a29f64800a5c4f91009711988,49b85b35bca5461993eac91ffd0b1fe4,5abae35db6de475e904425af8dc8ecbb,e3bc44b82c3543758d5a6c02b41dd526,89323021918e444bae3537521046d19f,141dd3257f85455d86f3828cc0217b5a,7bc2ad90009a4de2a0fa2947ccc09886,e51874f4b6ac455db76e30582659368b,519dee074e4c48ae9edd58d7c1e330da,060ab186c22f4f42a83f1622bf47ca52,8873bbcf4b42445f8117108769358a5c,46fa5286441a444095b917165c31304c,fa9fdd56222a4ce7ba2b37922afe0eaa,6885932aa4b443babc2b69fdd495889c,be71b947b33a48d8aee36ad0c60d10b8,b52457719429466687a2f228d20571ac,0bf937d745494d998cb8bd2cf3a63f4e,c17feb3860184dd59931f35814ad68b0,4cc68cbd880c48948a1f18fe18ec938c,c37b0691a05c4d659dcd01571f1907da,c1334fe3e8484079bfd62e479307b5b9,ebb58027b6c647b7965eeb31b63ec0c0,1821c5f9392e4859a829c78ebda44df6,dca21ab79bd14004b94c55e36604c30d,8d96cc3457644378b9b643243464f735,631437054079401fbe2c3c9a3f3bc375,c2f78ab4372243da8775f4d71a16fef5,03792664a45d4f63ac3efb899ce1b61e,fec04c46683a41e5a344f4d057342829,320fa70de6bc4890a0135bbfe8735b49,5a03493dce9f4323a0da7bf4fd5e78ab,b9bd46a1da1846d099566fd5c6db00cb,647021c7f798472889fd82f6c4a27ca2,313e905b78d746cb9e574b9634c3a113,623fd328c4cd4ee8b21eb03425b1f100,4676b4d1130146629fd98e2d2eb9a5b4,34d380813a36481ea2534c46d18b5927,894d87e55ac640b99e966a8786a2392b,477ba74d182a4ec0b86926dc67a0f419,362ca5fdca174fa3bdd6325a947320db,1ac3ad8dfa184fe4b6815950d3d23fdd,fdebee18c71f4a9db60096e75ad34d83,91432be4178a4781917fba0abd51df61,480c71baa65145f4bd037332d68e5e4f,536e3a3702b0453d92939846dc1f33b7,0333aedf1be640c78d2ee02473b7fd97,cbb3a6f1772b4f72a8af1375ef16e914,367ad9c43a6246dfac364ca59874883b,bac18430f6db466195a3a7116cdfb958,c4ec39be11be4b0eae321ee0559be8e6,8f7ffae7e47d462aa4142db7f06633d5,82e3b2423d86425195929fc3810ce7a4,72973e0415004dfea220babc479be082,41d0f568bb1e4fed8ad965a79d9bccef,5854ef1d24784ca0a046514b19286113,04afc3dd1eec4d13bc06cd01e0a5e838,db76e3333da846d5ae55b3b9dd735f26,7aa0ca0bf1514fa6a3a14dcde44cf0b9,9c668726c9b6449a9045d3f78408a2a1,7caf148d68a6431bbe56bcc6472d8720,babd36d1d3484fb99fe4e7ca78560ccf,7507865343e24cff80d55a29ec57f0e1,63f8b7d9b1f34851802e0408bf997ea5,a3369d5dd6b74af7b35edf1f10350881,9b8b8c3f86854416b82d57583896c1ff,881815c772564a8dbd5eb6de65ecbe7a,4b1bcf46753a48258d1d514440140b7f,1ca4c6a49e794954a235a82ee8981ecd,ab381d9129bb4b979fe3774e262524f8,a07d2c98831a4d79bc6c5c4288a24720,d8debd28039d494aa7fe18bc8e81af27,d0402b1f00f94acca96276c6ea8c8397,ebcdebd9639d4ec2b7072ad32bf48f50,636b3211100245919c09177ccea562e0,280c4440e18a487396a81b92cd3efa13,5163cf6740334a7d83375385c3ae2b67,a6083e4fab7a43498bfb3257658f9278,3b5c8b5cfb124ebd8f4a798d2a47d832,9fda332474534f96955b5bce8548947f,b5f5e62aba9842d8b48a28111ca6a372,c0cecff1207c4692a634d7de449d8d6d,e4c6428f1d264544b0eec52214e7b6ae,cbc8c5490a7f46b996fd4c3a9711f2d7,cfbdd82f4ce741a88109353c5fc898e4,6bfb965bc1b240a9ae2bd7a1e884ea51,eb36d63a993148289950af9098727759,f7357c3954d2408facaa6771a758ebcc,c05853b94ab8405eb26e0a8bd481b662,da1e6d3ebfa140b3a645342fd3bb5d95,988f3f6b607d442b8df006c980ddc2cd,b6af3c01968141baa3e99ca4b8d83847,75f669faee5d4023a425f2461e5fc5e9,165eb05677aa45fc9a53dfd8bfb7a45d,409891afd7cc4e12ba6b002581fef4f3,02ee0a87956f4bfab91b88282e502a3d,48601418028145ff84a450c877ea35aa,8de73f8d59eb45aa8c61138ab93c4f86,5dd82a46f48945b4bbb61f58b9d1c856,97993db997bd4ff88acfe1340ca86845,282aee84915b43179a09101b3ee6ad0e,8750e994985f43f383321bf0d6ca7932";

            role1.setMenuIds(defalutMenu1);
            String[] defalutMenus1 = defalutMenu1.split(",");
            ArrayList<String> menuIdList1 = new ArrayList<String>();
            for (int i = 0; i < defalutMenus1.length; i++) {
                menuIdList1.add(defalutMenus1[i]);
            }
            role1.setMenuIdList(menuIdList1);
           /*if (!UserUtils.getUser().isAdmin() && role1.getSysData().equals(Global.YES) && !UserUtils.getSelectRole().get(0).getEnname().contains("gly")) {
               return null;
           }*/
            if (Global.isDemoMode()) {
                return null;
            }
           /*if (!"true".equals(checkName(role1.getOldName(), role1.getName(),role1.getCompany().getId()))) {
               return null;
           }*/
           /*if (!"true".equals(checkEnname(role1.getOldEnname(), role1.getEnname()))) {
               return null;
           }*/
            systemService.saveRole(role1, true,1);
            roleList.add(role1);
        }
        //roleDao.insertRoleOffice(role1);
        return roleList;
    }
    @Transactional(readOnly = false)
    public List<Role> createCompanyLeaderRole( Office com,Office comOffice,List<Office> offices,String type) {
        List<Role> roleList = new ArrayList<>();
        for (Office office:offices) {
            //分管领导
            Role role1 = new Role();
            role1.setName("分管领导");
            role1.setNumber("1");
            role1.setSysData(Global.YES);
            role1.setDataScope("4");
            role1.setIsNewRecord(false);
            role1.setCompany(com);
            role1.setOffice(office);
            if ("2".equals(type)) {
                role1.setBranchCompany(comOffice);
            }
            role1.setRoleType("security-role");
            role1.setSysData("1");
            role1.setUseable("1");
            role1.setEnname("fgld");
            String defalutMenu1 = "1,27,29,e3b80fae59ab4b4099d8349ecaf8c4dc,61dd3aa279684f92bb1c5d4019a32503,b0c5737cdc4f452684ce93589427c97d,af6b0e5d61fc44ecbebb449703a3df72,18c14562978e4b129fe3afb7dca8b868,6123fcb3d810492789e82a63b0d4310a,0aa3712414d049a6a24e8bcddeae509a,95a6a82dc5fc4d07b46e5df57a0606a3,62,b6fa4abb624349918fe531274fc15d23,89,76bdb735faa34667a6cbf13ebe937533,90,f5b2028cfad9470085c7c846de33193a,28abe4b0cb3b4c8480f84c07a5dfaf06,c7fa36c7142f481397c2cc12e2bc828a,e2efcf80fc3747329fe02d489dbfda95,e3b5b7451e164089aba0b9bf844b0398,438c28c3cbe44ed7bff951b2a3dc77cd,59e50eebe56743a78eecbc6eff1ef94c,cc1027689aca41a89302217127253034,200fe6cc80174065b382c961a4379d79,4c13ca96376341dfa25980e86964c1a9,638be8b3b5284e5088003917685bc19b,811600aadcb54b098b7dda4bf95ffc51,ce8dc7c97d93481dbd76cabb6e905b55,aa25b3904abb4bd7b779e5be37b4382e,aae6f4c7fd4c4a25afa5285210edfded,84ff9d5c165c4528af9361b6127cf5af,31b349a3624b47f28eb86d614f9f95b5,040b42e395de41d6805a958eb1897863,34874260e6fc4200a76b33c3cfa4d80d,5b6c6ea5ed2d45478e8144351d853e21,7c032c32512847f882a87416359ef0a3,6d30ea8c710a42b89da2f2506059b0ef,2ca7bf4ce6804a1c9e5b331d2df27c72,7717cbb6d71f405d890e9b648cbd2088,bddb0400764c42b08e427059c7752057,23d6c56239804053910e5c5aec481330,7f8b2018113545bcbdbfc87e7a236bec,547e121ffb714d16ae142b6935abb78f,af24101b962444f29d166b638b2fea30,6ea6519ec4d548128aa42f3ce004def3,4379bf54c2a543869abf5031b97b8efe,5e6da3f5368b4f06b7268b31a1157e9f,2bbf0e75f21d42938954391d0776fbde,545b39dacf564a5eb9d1658f80a0b8b0,b4ddedf9b451494a97a1ff9476e3897d,1b145ca23d3e4d81994a84d30a262d18,10a8c7c1d34e46dd8ef461cdad928afc,5ae4911d14bb48b39885b1fe522e999d,7f09def2533b45fdb3c733f1e72ca5fb,9ef29dca19d342b795efcbfe32793602,be72d72ae7034c509c164467b2abddda,ab879705f2354813acf8724404520e28,82487b87b97b475b98892b80cf83c96a,0f4a461b97be48eeab43bd052f630057,72e1a28b7251438a89b0a4f6bd522b89,e1f9a246057347d78a1966d6dc0168e3,5d3645d53ce14c9ab498fd9f050ba264,d361e6890cc84b049fc8268c2a8c4952,5cc0002cbcc14d3485ab8ae6d57368f2,fa7bb243643c44b18e989e3419eeb171,62768b6c65134f1e927b1807bc72115a,7b69b0e5d2d344e6aaa2b8726c23561a,434bbb1f9f8140b1beeb57d0c4db660d,7920c4c8da294d87bcb94e7756bf35ae,0752b85660694033bbdb013f30cde17f,bd6258cfbea64896b0401517e440d6a2,9aa3539682514546866d2998d741ac6c,eb3ad93ce6a44921b0490c842dd47d8c,9ce19042d497473c92d199a5e5a7daf0,1b3eb6ba64ea4589a8e2d42efa5d3f89,6b2dd5ff2a754a53a80798341a7d35d8,e53d754160f74d28b6d091829fc289f8,92d762bca040407bb4ce4b06a2cfa195,aa8a2b00622147259c85290f4d95b46d,0be3e487d874487fbfb5c49b52440a48,ce60b25f7da04af8b7cefccf2dbc7d36,6464ac9cc3fc47d88598f6c4e74caa19,04b312df403d4855b013743206ddb1de,17356c44ae894782a1015ec2ddaeaeda,a06eaefba7084e219f0c6ce4010b82e7,95a54e4db7e74f0586cd85cb4e2bd6f7,1a93047d6b0f4e3ca70fcf836b2517c4,93263a29941648979bb8de774978a880,8eba834a28864375aa3137590d26f37e,3c52b0a890524c6fb66cefc32114c60e,8b20519b794c42d28fdecbd0ccbba5c1,77189ceba655488681964a609c28256d,39e776aaf3644d24bb78ecf43704b71b,0dd5fa1192e34d5fac5e78ca38969126,1a6f82a80a054bc7ba4b516187231a54,5993ce4ec80a41fb9d4ea0713b74f4a4,3b42071536124a9c9dbf2ec319004b29,a966beefe58e4f95b24e89f4172cb7ce,2e1e75dbe28841529576dd1cd48d2437,cdd8829e5976438aaa66ce608f0175f5,71015695cf1445ecacce90669c67d733,31e9dd26b7314d8a9bf97dcf6dbdb22d,e5ecaa091bb0464f8553d4e7aa60647d,39055be3a23945868dbf4b5930043a88,07ba7c9aed104657aac3be87aa346d3b,5a7a9bfff0f44a2089a7d277f2c04557,be53cc152339400ba705c1d1dceff9ab,ed591ccba0564abd9979899049dc7b36,9a4425b817e94dbe844d7fd3e96a9033,bf4e54bfe9984441a8efa4dd8a4a23a6,5a1bcf2a2e0943f88bf655ca011fb288,a12c8904ff0147a99010eaa5aa0ea704,e7c6f2581c6e41bf8fe88a3d7fadc986,89709767beea48d0b0fb047316eb37d1,04d6a49b35514f91be43e9576f8d32fc,bd50e783ec714e5d86aafcca54e87c33,4acb0c7d002b47e29be7ebe9f40490f6,67b5ddef866b4982aa5afef7aad69924,7e090632f713440bba11ed6fc09eb291,f308051e11ca47eebb12709725b52eb4,dbea976ef1d248cabf13b88d311c4533,e0132620d4044e38a92f75b8335005ed,1b598563135e42d4bfe729cf32649f14,c4de9219f4f14684ae21d0904b7b78df,520983ab7dd64104a47e273941236b2d,133ba69bd04046b89eadaf793b9063db,11f24ca662044292a8ca95930ccc3581,0f30b4d621294d4fab359611cf82ab6f,b3c93fcd54b544e5b2c8e70ee7eb93ce,79ebe6595ce44208853ba553186b8348,8d639afccbec43f2be8d3ed88702893b,4e3483f3bbbb48f59b0e92a7d0ccc1c0,b160fccc15fa4ba09c74018d82d16eb7,52f248eadea74cf2b7780bf991d95e4e,d4cb9017aa1f4f8ca34ae8d31c986210,eff19819b1284424a9ad7879991ee080,79551897f86b4338bd09b41c5953a7db,f11bd4ef5ddd4c82865fad3f1f9d80a9,76ee95093a8748b29d55df1cb8cdebd0,081af742f6384a20b492ca99704f51ca,c26c844bbb2f4746904948f3ecfeb1ae,ba9bbc5581a043d8b9c204b8ddb6aa01,cde1afb048a743bb9caedb5770aabca1,0b2b37a28bb0457bbc4bf33c11b4c6e1,9f94d338d6c94b95814b2c50f9e463d1,7d1745ce2c444db9bf4b4e100c476fe8,e711e93aadd54eef8feeb0fa07f054d2,0e914774210646f784d02fb22f8df7db,ca0ce5e9eb09477c8c99b7b94e7869ce,760d9072f9bc480fa36fdb876930467c,5742a628f13045cda0317c3d4d839bc5,b9309305d94f4ee8b66eebbcae9f2db4,a42216d3eedd48f0ba3d3bef2470605f,9cdcda30da704c21b19ea0616de3ea86,f05ecbf7e4b04b268b38cb6e5c8124bf,788d2c1496874eeeae3b7aabbab95f76,8b112af90e054671adc30373c6dafd21,3bdb29667807493292a0ddc9bc0ccdb7,ffd6ebecc869410dab6410a07f3362ae,196868a4610b45588062aed357fc7c58,4cdd2d11be4c4c098b27c10d7cb3fa9e,fda40125207d426dbf86270002ff0c4a,7af028c6048c44cc91415a58d25d286e,ff7d2c8e96e840e1a5a690e4e0f96bad,9f806cf868dc48d398836ebd60bd2ec6,50ff5cdc6c8645b3bb61b8327d204f01,bafb57ea8d84430cbc0c11e5c67643f1,e2c91860b96c46619a74215627560235,f5b68a43bbae44c8a1fe821dfcaca26e,70b771772a0b4f1cb1e592f9a01b08a5,7992a39d64b2483789ae274e2df3c01d,f8d711312da14871a77b58f41a6f6242,673b27b0ab9148b7ae03933c18433971,624886a950bd4d18900a877738696432,685d8d9267f745f39bcc6fc2cc553d31,7d40ad05a1a54165a28ec897ddbcef01,4bcdb5199e8442078999bb8611ea6398,3b0c0ffea0cf45db816993baf64b3fb3,ad2d6d9581034e04addcb5b034a88fb3,8440c4fc5056426996e3e488b2261d28,e367c4223d794572a05fd256759d6a53,1b2dc5784c3c41988d5634e566b4a404,11be1538d7f54d7786004503b2e684a7,b462cb8c2ec7485d938416831a9771cc,f747656a36204e9588aba97064332436,78d0b2337b484c6298c9b3afc01fdb85,8a63d1f8d4bc4eb4a4db437491c9983a,2247f64a15e54053abb2e13152003c6b,d78ef9c5997d4e94804b8126cc387ea1,6ce3cd5a86ef43219dbda2b78e974c93,414b5fd5ebee45d98eca809a524dbe93,4b9ea9db5c834b698a78c7052c87b432,8590fa7adc144f49ae3d2575b7280c08,6148e4b152b9494fb9e7035e9b58e624,be1cff2a29f64800a5c4f91009711988,49b85b35bca5461993eac91ffd0b1fe4,5abae35db6de475e904425af8dc8ecbb,e3bc44b82c3543758d5a6c02b41dd526,89323021918e444bae3537521046d19f,141dd3257f85455d86f3828cc0217b5a,7bc2ad90009a4de2a0fa2947ccc09886,e51874f4b6ac455db76e30582659368b,519dee074e4c48ae9edd58d7c1e330da,060ab186c22f4f42a83f1622bf47ca52,8873bbcf4b42445f8117108769358a5c,46fa5286441a444095b917165c31304c,fa9fdd56222a4ce7ba2b37922afe0eaa,6885932aa4b443babc2b69fdd495889c,be71b947b33a48d8aee36ad0c60d10b8,b52457719429466687a2f228d20571ac,0bf937d745494d998cb8bd2cf3a63f4e,c17feb3860184dd59931f35814ad68b0,4cc68cbd880c48948a1f18fe18ec938c,c37b0691a05c4d659dcd01571f1907da,c1334fe3e8484079bfd62e479307b5b9,ebb58027b6c647b7965eeb31b63ec0c0,1821c5f9392e4859a829c78ebda44df6,dca21ab79bd14004b94c55e36604c30d,8d96cc3457644378b9b643243464f735,631437054079401fbe2c3c9a3f3bc375,c2f78ab4372243da8775f4d71a16fef5,03792664a45d4f63ac3efb899ce1b61e,fec04c46683a41e5a344f4d057342829,320fa70de6bc4890a0135bbfe8735b49,5a03493dce9f4323a0da7bf4fd5e78ab,b9bd46a1da1846d099566fd5c6db00cb,647021c7f798472889fd82f6c4a27ca2,313e905b78d746cb9e574b9634c3a113,623fd328c4cd4ee8b21eb03425b1f100,4676b4d1130146629fd98e2d2eb9a5b4,34d380813a36481ea2534c46d18b5927,894d87e55ac640b99e966a8786a2392b,477ba74d182a4ec0b86926dc67a0f419,362ca5fdca174fa3bdd6325a947320db,1ac3ad8dfa184fe4b6815950d3d23fdd,fdebee18c71f4a9db60096e75ad34d83,91432be4178a4781917fba0abd51df61,480c71baa65145f4bd037332d68e5e4f,536e3a3702b0453d92939846dc1f33b7,0333aedf1be640c78d2ee02473b7fd97,cbb3a6f1772b4f72a8af1375ef16e914,367ad9c43a6246dfac364ca59874883b,bac18430f6db466195a3a7116cdfb958,c4ec39be11be4b0eae321ee0559be8e6,8f7ffae7e47d462aa4142db7f06633d5,82e3b2423d86425195929fc3810ce7a4,72973e0415004dfea220babc479be082,41d0f568bb1e4fed8ad965a79d9bccef,5854ef1d24784ca0a046514b19286113,04afc3dd1eec4d13bc06cd01e0a5e838,db76e3333da846d5ae55b3b9dd735f26,7aa0ca0bf1514fa6a3a14dcde44cf0b9,9c668726c9b6449a9045d3f78408a2a1,7caf148d68a6431bbe56bcc6472d8720,babd36d1d3484fb99fe4e7ca78560ccf,7507865343e24cff80d55a29ec57f0e1,63f8b7d9b1f34851802e0408bf997ea5,a3369d5dd6b74af7b35edf1f10350881,9b8b8c3f86854416b82d57583896c1ff,881815c772564a8dbd5eb6de65ecbe7a,4b1bcf46753a48258d1d514440140b7f,1ca4c6a49e794954a235a82ee8981ecd,ab381d9129bb4b979fe3774e262524f8,a07d2c98831a4d79bc6c5c4288a24720,d8debd28039d494aa7fe18bc8e81af27,d0402b1f00f94acca96276c6ea8c8397,ebcdebd9639d4ec2b7072ad32bf48f50,636b3211100245919c09177ccea562e0,280c4440e18a487396a81b92cd3efa13,5163cf6740334a7d83375385c3ae2b67,a6083e4fab7a43498bfb3257658f9278,3b5c8b5cfb124ebd8f4a798d2a47d832,9fda332474534f96955b5bce8548947f,b5f5e62aba9842d8b48a28111ca6a372,c0cecff1207c4692a634d7de449d8d6d,e4c6428f1d264544b0eec52214e7b6ae,cbc8c5490a7f46b996fd4c3a9711f2d7,cfbdd82f4ce741a88109353c5fc898e4,6bfb965bc1b240a9ae2bd7a1e884ea51,eb36d63a993148289950af9098727759,f7357c3954d2408facaa6771a758ebcc,c05853b94ab8405eb26e0a8bd481b662,da1e6d3ebfa140b3a645342fd3bb5d95,988f3f6b607d442b8df006c980ddc2cd,b6af3c01968141baa3e99ca4b8d83847,75f669faee5d4023a425f2461e5fc5e9,165eb05677aa45fc9a53dfd8bfb7a45d,409891afd7cc4e12ba6b002581fef4f3,02ee0a87956f4bfab91b88282e502a3d,48601418028145ff84a450c877ea35aa,8de73f8d59eb45aa8c61138ab93c4f86,5dd82a46f48945b4bbb61f58b9d1c856,97993db997bd4ff88acfe1340ca86845,282aee84915b43179a09101b3ee6ad0e,8750e994985f43f383321bf0d6ca7932";

            role1.setMenuIds(defalutMenu1);
            String[] defalutMenus1 = defalutMenu1.split(",");
            ArrayList<String> menuIdList1 = new ArrayList<String>();
            for (int i = 0; i < defalutMenus1.length; i++) {
                menuIdList1.add(defalutMenus1[i]);
            }
            role1.setMenuIdList(menuIdList1);
           /*if (!UserUtils.getUser().isAdmin() && role1.getSysData().equals(Global.YES) && !UserUtils.getSelectRole().get(0).getEnname().contains("gly")) {
               return null;
           }*/
            if (Global.isDemoMode()) {
                return null;
            }
           /*if (!"true".equals(checkName(role1.getOldName(), role1.getName(),role1.getCompany().getId()))) {
               return null;
           }*/
           /*if (!"true".equals(checkEnname(role1.getOldEnname(), role1.getEnname()))) {
               return null;
           }*/
            systemService.saveRole(role1, true,1);
            roleList.add(role1);
        }
        SysRoleActivity roleActivity = new SysRoleActivity();
        if ("1".equals(type)) {
            roleActivity.setName("业务分管领导");
            roleActivity.setEnname(UserUtils.getRoleActivity(com.getId()) + StringUtils.getPinYinHeadChar("分管领导", 1));
            roleActivity.setSort("7500");
            roleActivity.setType("2");
            roleActivity.setCompanyId(com.getId());
            roleActivity.setDelType("1");
            sysRoleActivityService.save(roleActivity);

            //roleDao.insertRoleOffice(role1);
        }else {
            String enname = UserUtils.getRoleActivityEnname(com.getId(),UserUtils.getUser());
            roleActivity = sysRoleActivityService.getByEnname(enname+"fgld");
        }
        for (Role role1 : roleList) {
            List<SysRoleActivity> roleActivities = new ArrayList<>();
            roleActivities.add(roleActivity);
            role1.setRoleActivityList(roleActivities);
            roleDao.insertRoleActivity(role1);
        }
        return roleList;
    }

    @Transactional(readOnly = false)
    public Role createCompanyLeaderRoleNoActivity( Office com,Office comOffice,Office office,String type) {
        //分管领导
        Role role1 = new Role();
        role1.setName("分管领导");
        role1.setSysData(Global.YES);
        role1.setDataScope("4");
        role1.setIsNewRecord(false);
        role1.setCompany(com);
        role1.setNumber("1");
        role1.setOffice(office);
        if ("2".equals(type)) {
            role1.setBranchCompany(comOffice);
        }
        role1.setRoleType("security-role");
        role1.setSysData("1");
        role1.setUseable("1");
        role1.setEnname("fgld");
        String defalutMenu1 = "1,27,29,e3b80fae59ab4b4099d8349ecaf8c4dc,61dd3aa279684f92bb1c5d4019a32503,b0c5737cdc4f452684ce93589427c97d,af6b0e5d61fc44ecbebb449703a3df72,18c14562978e4b129fe3afb7dca8b868,6123fcb3d810492789e82a63b0d4310a,0aa3712414d049a6a24e8bcddeae509a,95a6a82dc5fc4d07b46e5df57a0606a3,62,b6fa4abb624349918fe531274fc15d23,89,76bdb735faa34667a6cbf13ebe937533,90,f5b2028cfad9470085c7c846de33193a,28abe4b0cb3b4c8480f84c07a5dfaf06,c7fa36c7142f481397c2cc12e2bc828a,e2efcf80fc3747329fe02d489dbfda95,e3b5b7451e164089aba0b9bf844b0398,438c28c3cbe44ed7bff951b2a3dc77cd,59e50eebe56743a78eecbc6eff1ef94c,cc1027689aca41a89302217127253034,200fe6cc80174065b382c961a4379d79,4c13ca96376341dfa25980e86964c1a9,638be8b3b5284e5088003917685bc19b,811600aadcb54b098b7dda4bf95ffc51,ce8dc7c97d93481dbd76cabb6e905b55,aa25b3904abb4bd7b779e5be37b4382e,aae6f4c7fd4c4a25afa5285210edfded,84ff9d5c165c4528af9361b6127cf5af,31b349a3624b47f28eb86d614f9f95b5,040b42e395de41d6805a958eb1897863,34874260e6fc4200a76b33c3cfa4d80d,5b6c6ea5ed2d45478e8144351d853e21,7c032c32512847f882a87416359ef0a3,6d30ea8c710a42b89da2f2506059b0ef,2ca7bf4ce6804a1c9e5b331d2df27c72,7717cbb6d71f405d890e9b648cbd2088,bddb0400764c42b08e427059c7752057,23d6c56239804053910e5c5aec481330,7f8b2018113545bcbdbfc87e7a236bec,547e121ffb714d16ae142b6935abb78f,af24101b962444f29d166b638b2fea30,6ea6519ec4d548128aa42f3ce004def3,4379bf54c2a543869abf5031b97b8efe,5e6da3f5368b4f06b7268b31a1157e9f,2bbf0e75f21d42938954391d0776fbde,545b39dacf564a5eb9d1658f80a0b8b0,b4ddedf9b451494a97a1ff9476e3897d,1b145ca23d3e4d81994a84d30a262d18,10a8c7c1d34e46dd8ef461cdad928afc,5ae4911d14bb48b39885b1fe522e999d,7f09def2533b45fdb3c733f1e72ca5fb,9ef29dca19d342b795efcbfe32793602,be72d72ae7034c509c164467b2abddda,ab879705f2354813acf8724404520e28,82487b87b97b475b98892b80cf83c96a,0f4a461b97be48eeab43bd052f630057,72e1a28b7251438a89b0a4f6bd522b89,e1f9a246057347d78a1966d6dc0168e3,5d3645d53ce14c9ab498fd9f050ba264,d361e6890cc84b049fc8268c2a8c4952,5cc0002cbcc14d3485ab8ae6d57368f2,fa7bb243643c44b18e989e3419eeb171,62768b6c65134f1e927b1807bc72115a,7b69b0e5d2d344e6aaa2b8726c23561a,434bbb1f9f8140b1beeb57d0c4db660d,7920c4c8da294d87bcb94e7756bf35ae,0752b85660694033bbdb013f30cde17f,bd6258cfbea64896b0401517e440d6a2,9aa3539682514546866d2998d741ac6c,eb3ad93ce6a44921b0490c842dd47d8c,9ce19042d497473c92d199a5e5a7daf0,1b3eb6ba64ea4589a8e2d42efa5d3f89,6b2dd5ff2a754a53a80798341a7d35d8,e53d754160f74d28b6d091829fc289f8,92d762bca040407bb4ce4b06a2cfa195,aa8a2b00622147259c85290f4d95b46d,0be3e487d874487fbfb5c49b52440a48,ce60b25f7da04af8b7cefccf2dbc7d36,6464ac9cc3fc47d88598f6c4e74caa19,04b312df403d4855b013743206ddb1de,17356c44ae894782a1015ec2ddaeaeda,a06eaefba7084e219f0c6ce4010b82e7,95a54e4db7e74f0586cd85cb4e2bd6f7,1a93047d6b0f4e3ca70fcf836b2517c4,93263a29941648979bb8de774978a880,8eba834a28864375aa3137590d26f37e,3c52b0a890524c6fb66cefc32114c60e,8b20519b794c42d28fdecbd0ccbba5c1,77189ceba655488681964a609c28256d,39e776aaf3644d24bb78ecf43704b71b,0dd5fa1192e34d5fac5e78ca38969126,1a6f82a80a054bc7ba4b516187231a54,5993ce4ec80a41fb9d4ea0713b74f4a4,3b42071536124a9c9dbf2ec319004b29,a966beefe58e4f95b24e89f4172cb7ce,2e1e75dbe28841529576dd1cd48d2437,cdd8829e5976438aaa66ce608f0175f5,71015695cf1445ecacce90669c67d733,31e9dd26b7314d8a9bf97dcf6dbdb22d,e5ecaa091bb0464f8553d4e7aa60647d,39055be3a23945868dbf4b5930043a88,07ba7c9aed104657aac3be87aa346d3b,5a7a9bfff0f44a2089a7d277f2c04557,be53cc152339400ba705c1d1dceff9ab,ed591ccba0564abd9979899049dc7b36,9a4425b817e94dbe844d7fd3e96a9033,bf4e54bfe9984441a8efa4dd8a4a23a6,5a1bcf2a2e0943f88bf655ca011fb288,a12c8904ff0147a99010eaa5aa0ea704,e7c6f2581c6e41bf8fe88a3d7fadc986,89709767beea48d0b0fb047316eb37d1,04d6a49b35514f91be43e9576f8d32fc,bd50e783ec714e5d86aafcca54e87c33,4acb0c7d002b47e29be7ebe9f40490f6,67b5ddef866b4982aa5afef7aad69924,7e090632f713440bba11ed6fc09eb291,f308051e11ca47eebb12709725b52eb4,dbea976ef1d248cabf13b88d311c4533,e0132620d4044e38a92f75b8335005ed,1b598563135e42d4bfe729cf32649f14,c4de9219f4f14684ae21d0904b7b78df,520983ab7dd64104a47e273941236b2d,133ba69bd04046b89eadaf793b9063db,11f24ca662044292a8ca95930ccc3581,0f30b4d621294d4fab359611cf82ab6f,b3c93fcd54b544e5b2c8e70ee7eb93ce,79ebe6595ce44208853ba553186b8348,8d639afccbec43f2be8d3ed88702893b,4e3483f3bbbb48f59b0e92a7d0ccc1c0,b160fccc15fa4ba09c74018d82d16eb7,52f248eadea74cf2b7780bf991d95e4e,d4cb9017aa1f4f8ca34ae8d31c986210,eff19819b1284424a9ad7879991ee080,79551897f86b4338bd09b41c5953a7db,f11bd4ef5ddd4c82865fad3f1f9d80a9,76ee95093a8748b29d55df1cb8cdebd0,081af742f6384a20b492ca99704f51ca,c26c844bbb2f4746904948f3ecfeb1ae,ba9bbc5581a043d8b9c204b8ddb6aa01,cde1afb048a743bb9caedb5770aabca1,0b2b37a28bb0457bbc4bf33c11b4c6e1,9f94d338d6c94b95814b2c50f9e463d1,7d1745ce2c444db9bf4b4e100c476fe8,e711e93aadd54eef8feeb0fa07f054d2,0e914774210646f784d02fb22f8df7db,ca0ce5e9eb09477c8c99b7b94e7869ce,760d9072f9bc480fa36fdb876930467c,5742a628f13045cda0317c3d4d839bc5,b9309305d94f4ee8b66eebbcae9f2db4,a42216d3eedd48f0ba3d3bef2470605f,9cdcda30da704c21b19ea0616de3ea86,f05ecbf7e4b04b268b38cb6e5c8124bf,788d2c1496874eeeae3b7aabbab95f76,8b112af90e054671adc30373c6dafd21,3bdb29667807493292a0ddc9bc0ccdb7,ffd6ebecc869410dab6410a07f3362ae,196868a4610b45588062aed357fc7c58,4cdd2d11be4c4c098b27c10d7cb3fa9e,fda40125207d426dbf86270002ff0c4a,7af028c6048c44cc91415a58d25d286e,ff7d2c8e96e840e1a5a690e4e0f96bad,9f806cf868dc48d398836ebd60bd2ec6,50ff5cdc6c8645b3bb61b8327d204f01,bafb57ea8d84430cbc0c11e5c67643f1,e2c91860b96c46619a74215627560235,f5b68a43bbae44c8a1fe821dfcaca26e,70b771772a0b4f1cb1e592f9a01b08a5,7992a39d64b2483789ae274e2df3c01d,f8d711312da14871a77b58f41a6f6242,673b27b0ab9148b7ae03933c18433971,624886a950bd4d18900a877738696432,685d8d9267f745f39bcc6fc2cc553d31,7d40ad05a1a54165a28ec897ddbcef01,4bcdb5199e8442078999bb8611ea6398,3b0c0ffea0cf45db816993baf64b3fb3,ad2d6d9581034e04addcb5b034a88fb3,8440c4fc5056426996e3e488b2261d28,e367c4223d794572a05fd256759d6a53,1b2dc5784c3c41988d5634e566b4a404,11be1538d7f54d7786004503b2e684a7,b462cb8c2ec7485d938416831a9771cc,f747656a36204e9588aba97064332436,78d0b2337b484c6298c9b3afc01fdb85,8a63d1f8d4bc4eb4a4db437491c9983a,2247f64a15e54053abb2e13152003c6b,d78ef9c5997d4e94804b8126cc387ea1,6ce3cd5a86ef43219dbda2b78e974c93,414b5fd5ebee45d98eca809a524dbe93,4b9ea9db5c834b698a78c7052c87b432,8590fa7adc144f49ae3d2575b7280c08,6148e4b152b9494fb9e7035e9b58e624,be1cff2a29f64800a5c4f91009711988,49b85b35bca5461993eac91ffd0b1fe4,5abae35db6de475e904425af8dc8ecbb,e3bc44b82c3543758d5a6c02b41dd526,89323021918e444bae3537521046d19f,141dd3257f85455d86f3828cc0217b5a,7bc2ad90009a4de2a0fa2947ccc09886,e51874f4b6ac455db76e30582659368b,519dee074e4c48ae9edd58d7c1e330da,060ab186c22f4f42a83f1622bf47ca52,8873bbcf4b42445f8117108769358a5c,46fa5286441a444095b917165c31304c,fa9fdd56222a4ce7ba2b37922afe0eaa,6885932aa4b443babc2b69fdd495889c,be71b947b33a48d8aee36ad0c60d10b8,b52457719429466687a2f228d20571ac,0bf937d745494d998cb8bd2cf3a63f4e,c17feb3860184dd59931f35814ad68b0,4cc68cbd880c48948a1f18fe18ec938c,c37b0691a05c4d659dcd01571f1907da,c1334fe3e8484079bfd62e479307b5b9,ebb58027b6c647b7965eeb31b63ec0c0,1821c5f9392e4859a829c78ebda44df6,dca21ab79bd14004b94c55e36604c30d,8d96cc3457644378b9b643243464f735,631437054079401fbe2c3c9a3f3bc375,c2f78ab4372243da8775f4d71a16fef5,03792664a45d4f63ac3efb899ce1b61e,fec04c46683a41e5a344f4d057342829,320fa70de6bc4890a0135bbfe8735b49,5a03493dce9f4323a0da7bf4fd5e78ab,b9bd46a1da1846d099566fd5c6db00cb,647021c7f798472889fd82f6c4a27ca2,313e905b78d746cb9e574b9634c3a113,623fd328c4cd4ee8b21eb03425b1f100,4676b4d1130146629fd98e2d2eb9a5b4,34d380813a36481ea2534c46d18b5927,894d87e55ac640b99e966a8786a2392b,477ba74d182a4ec0b86926dc67a0f419,362ca5fdca174fa3bdd6325a947320db,1ac3ad8dfa184fe4b6815950d3d23fdd,fdebee18c71f4a9db60096e75ad34d83,91432be4178a4781917fba0abd51df61,480c71baa65145f4bd037332d68e5e4f,536e3a3702b0453d92939846dc1f33b7,0333aedf1be640c78d2ee02473b7fd97,cbb3a6f1772b4f72a8af1375ef16e914,367ad9c43a6246dfac364ca59874883b,bac18430f6db466195a3a7116cdfb958,c4ec39be11be4b0eae321ee0559be8e6,8f7ffae7e47d462aa4142db7f06633d5,82e3b2423d86425195929fc3810ce7a4,72973e0415004dfea220babc479be082,41d0f568bb1e4fed8ad965a79d9bccef,5854ef1d24784ca0a046514b19286113,04afc3dd1eec4d13bc06cd01e0a5e838,db76e3333da846d5ae55b3b9dd735f26,7aa0ca0bf1514fa6a3a14dcde44cf0b9,9c668726c9b6449a9045d3f78408a2a1,7caf148d68a6431bbe56bcc6472d8720,babd36d1d3484fb99fe4e7ca78560ccf,7507865343e24cff80d55a29ec57f0e1,63f8b7d9b1f34851802e0408bf997ea5,a3369d5dd6b74af7b35edf1f10350881,9b8b8c3f86854416b82d57583896c1ff,881815c772564a8dbd5eb6de65ecbe7a,4b1bcf46753a48258d1d514440140b7f,1ca4c6a49e794954a235a82ee8981ecd,ab381d9129bb4b979fe3774e262524f8,a07d2c98831a4d79bc6c5c4288a24720,d8debd28039d494aa7fe18bc8e81af27,d0402b1f00f94acca96276c6ea8c8397,ebcdebd9639d4ec2b7072ad32bf48f50,636b3211100245919c09177ccea562e0,280c4440e18a487396a81b92cd3efa13,5163cf6740334a7d83375385c3ae2b67,a6083e4fab7a43498bfb3257658f9278,3b5c8b5cfb124ebd8f4a798d2a47d832,9fda332474534f96955b5bce8548947f,b5f5e62aba9842d8b48a28111ca6a372,c0cecff1207c4692a634d7de449d8d6d,e4c6428f1d264544b0eec52214e7b6ae,cbc8c5490a7f46b996fd4c3a9711f2d7,cfbdd82f4ce741a88109353c5fc898e4,6bfb965bc1b240a9ae2bd7a1e884ea51,eb36d63a993148289950af9098727759,f7357c3954d2408facaa6771a758ebcc,c05853b94ab8405eb26e0a8bd481b662,da1e6d3ebfa140b3a645342fd3bb5d95,988f3f6b607d442b8df006c980ddc2cd,b6af3c01968141baa3e99ca4b8d83847,75f669faee5d4023a425f2461e5fc5e9,165eb05677aa45fc9a53dfd8bfb7a45d,409891afd7cc4e12ba6b002581fef4f3,02ee0a87956f4bfab91b88282e502a3d,48601418028145ff84a450c877ea35aa,8de73f8d59eb45aa8c61138ab93c4f86,5dd82a46f48945b4bbb61f58b9d1c856,97993db997bd4ff88acfe1340ca86845,282aee84915b43179a09101b3ee6ad0e,8750e994985f43f383321bf0d6ca7932";

        role1.setMenuIds(defalutMenu1);
        String[] defalutMenus1 = defalutMenu1.split(",");
        ArrayList<String> menuIdList1 = new ArrayList<String>();
        for (int i = 0; i < defalutMenus1.length; i++) {
            menuIdList1.add(defalutMenus1[i]);
        }
        role1.setMenuIdList(menuIdList1);
           /*if (!UserUtils.getUser().isAdmin() && role1.getSysData().equals(Global.YES) && !UserUtils.getSelectRole().get(0).getEnname().contains("gly")) {
               return null;
           }*/
        if (Global.isDemoMode()) {
            return null;
        }
           /*if (!"true".equals(checkName(role1.getOldName(), role1.getName(),role1.getCompany().getId()))) {
               return null;
           }*/
           /*if (!"true".equals(checkEnname(role1.getOldEnname(), role1.getEnname()))) {
               return null;
           }*/
        systemService.saveRole(role1, true,1);
        return role1;
    }

    @Transactional(readOnly = false)
    public List<Role> createDeptRole(Office com,Office comOffice, List<Office> offices,String type) {
        List<Role> roleList = new ArrayList<>();
        for (Office office:offices) {
            //公司新用户
            Role role1 = new Role();
            role1.setName("员工");
            role1.setNumber("3");
            role1.setSysData(Global.YES);
            role1.setDataScope("8");
            role1.setIsNewRecord(false);
            role1.setCompany(com);
            role1.setOffice(office);
            if ("2".equals(type)) {
                role1.setBranchCompany(comOffice);
            }
            role1.setRoleType("user");
            role1.setSysData("1");
            role1.setUseable("1");
            role1.setEnname("yg");
            String defalutMenu1 = "1,27,29,e3b80fae59ab4b4099d8349ecaf8c4dc,61dd3aa279684f92bb1c5d4019a32503,b0c5737cdc4f452684ce93589427c97d,af6b0e5d61fc44ecbebb449703a3df72,18c14562978e4b129fe3afb7dca8b868,6123fcb3d810492789e82a63b0d4310a,0aa3712414d049a6a24e8bcddeae509a,95a6a82dc5fc4d07b46e5df57a0606a3,62,b6fa4abb624349918fe531274fc15d23,89,76bdb735faa34667a6cbf13ebe937533,90,f5b2028cfad9470085c7c846de33193a,28abe4b0cb3b4c8480f84c07a5dfaf06,c7fa36c7142f481397c2cc12e2bc828a,e2efcf80fc3747329fe02d489dbfda95,e3b5b7451e164089aba0b9bf844b0398,438c28c3cbe44ed7bff951b2a3dc77cd,59e50eebe56743a78eecbc6eff1ef94c,cc1027689aca41a89302217127253034,200fe6cc80174065b382c961a4379d79,4c13ca96376341dfa25980e86964c1a9,638be8b3b5284e5088003917685bc19b,811600aadcb54b098b7dda4bf95ffc51,ce8dc7c97d93481dbd76cabb6e905b55,aa25b3904abb4bd7b779e5be37b4382e,aae6f4c7fd4c4a25afa5285210edfded,84ff9d5c165c4528af9361b6127cf5af,31b349a3624b47f28eb86d614f9f95b5,040b42e395de41d6805a958eb1897863,34874260e6fc4200a76b33c3cfa4d80d,5b6c6ea5ed2d45478e8144351d853e21,7c032c32512847f882a87416359ef0a3,6d30ea8c710a42b89da2f2506059b0ef,2ca7bf4ce6804a1c9e5b331d2df27c72,7717cbb6d71f405d890e9b648cbd2088,bddb0400764c42b08e427059c7752057,23d6c56239804053910e5c5aec481330,7f8b2018113545bcbdbfc87e7a236bec,547e121ffb714d16ae142b6935abb78f,af24101b962444f29d166b638b2fea30,6ea6519ec4d548128aa42f3ce004def3,4379bf54c2a543869abf5031b97b8efe,5e6da3f5368b4f06b7268b31a1157e9f,2bbf0e75f21d42938954391d0776fbde,545b39dacf564a5eb9d1658f80a0b8b0,b4ddedf9b451494a97a1ff9476e3897d,1b145ca23d3e4d81994a84d30a262d18,10a8c7c1d34e46dd8ef461cdad928afc,5ae4911d14bb48b39885b1fe522e999d,7f09def2533b45fdb3c733f1e72ca5fb,9ef29dca19d342b795efcbfe32793602,be72d72ae7034c509c164467b2abddda,ab879705f2354813acf8724404520e28,82487b87b97b475b98892b80cf83c96a,0f4a461b97be48eeab43bd052f630057,72e1a28b7251438a89b0a4f6bd522b89,e1f9a246057347d78a1966d6dc0168e3,5d3645d53ce14c9ab498fd9f050ba264,d361e6890cc84b049fc8268c2a8c4952,5cc0002cbcc14d3485ab8ae6d57368f2,fa7bb243643c44b18e989e3419eeb171,62768b6c65134f1e927b1807bc72115a,7b69b0e5d2d344e6aaa2b8726c23561a,434bbb1f9f8140b1beeb57d0c4db660d,7920c4c8da294d87bcb94e7756bf35ae,0752b85660694033bbdb013f30cde17f,bd6258cfbea64896b0401517e440d6a2,9aa3539682514546866d2998d741ac6c,eb3ad93ce6a44921b0490c842dd47d8c,9ce19042d497473c92d199a5e5a7daf0,1b3eb6ba64ea4589a8e2d42efa5d3f89,6b2dd5ff2a754a53a80798341a7d35d8,e53d754160f74d28b6d091829fc289f8,92d762bca040407bb4ce4b06a2cfa195,aa8a2b00622147259c85290f4d95b46d,0be3e487d874487fbfb5c49b52440a48,ce60b25f7da04af8b7cefccf2dbc7d36,6464ac9cc3fc47d88598f6c4e74caa19,04b312df403d4855b013743206ddb1de,17356c44ae894782a1015ec2ddaeaeda,a06eaefba7084e219f0c6ce4010b82e7,95a54e4db7e74f0586cd85cb4e2bd6f7,1a93047d6b0f4e3ca70fcf836b2517c4,93263a29941648979bb8de774978a880,8eba834a28864375aa3137590d26f37e,3c52b0a890524c6fb66cefc32114c60e,8b20519b794c42d28fdecbd0ccbba5c1,77189ceba655488681964a609c28256d,39e776aaf3644d24bb78ecf43704b71b,0dd5fa1192e34d5fac5e78ca38969126,1a6f82a80a054bc7ba4b516187231a54,5993ce4ec80a41fb9d4ea0713b74f4a4,3b42071536124a9c9dbf2ec319004b29,a966beefe58e4f95b24e89f4172cb7ce,2e1e75dbe28841529576dd1cd48d2437,cdd8829e5976438aaa66ce608f0175f5,71015695cf1445ecacce90669c67d733,31e9dd26b7314d8a9bf97dcf6dbdb22d,e5ecaa091bb0464f8553d4e7aa60647d,39055be3a23945868dbf4b5930043a88,07ba7c9aed104657aac3be87aa346d3b,5a7a9bfff0f44a2089a7d277f2c04557,be53cc152339400ba705c1d1dceff9ab,ed591ccba0564abd9979899049dc7b36,9a4425b817e94dbe844d7fd3e96a9033,bf4e54bfe9984441a8efa4dd8a4a23a6,5a1bcf2a2e0943f88bf655ca011fb288,a12c8904ff0147a99010eaa5aa0ea704,e7c6f2581c6e41bf8fe88a3d7fadc986,89709767beea48d0b0fb047316eb37d1,04d6a49b35514f91be43e9576f8d32fc,bd50e783ec714e5d86aafcca54e87c33,4acb0c7d002b47e29be7ebe9f40490f6,67b5ddef866b4982aa5afef7aad69924,7e090632f713440bba11ed6fc09eb291,f308051e11ca47eebb12709725b52eb4,dbea976ef1d248cabf13b88d311c4533,e0132620d4044e38a92f75b8335005ed,1b598563135e42d4bfe729cf32649f14,c4de9219f4f14684ae21d0904b7b78df,520983ab7dd64104a47e273941236b2d,133ba69bd04046b89eadaf793b9063db,11f24ca662044292a8ca95930ccc3581,0f30b4d621294d4fab359611cf82ab6f,b3c93fcd54b544e5b2c8e70ee7eb93ce,79ebe6595ce44208853ba553186b8348,8d639afccbec43f2be8d3ed88702893b,4e3483f3bbbb48f59b0e92a7d0ccc1c0,b160fccc15fa4ba09c74018d82d16eb7,52f248eadea74cf2b7780bf991d95e4e,d4cb9017aa1f4f8ca34ae8d31c986210,eff19819b1284424a9ad7879991ee080,79551897f86b4338bd09b41c5953a7db,f11bd4ef5ddd4c82865fad3f1f9d80a9,76ee95093a8748b29d55df1cb8cdebd0,081af742f6384a20b492ca99704f51ca,c26c844bbb2f4746904948f3ecfeb1ae,ba9bbc5581a043d8b9c204b8ddb6aa01,cde1afb048a743bb9caedb5770aabca1,0b2b37a28bb0457bbc4bf33c11b4c6e1,9f94d338d6c94b95814b2c50f9e463d1,7d1745ce2c444db9bf4b4e100c476fe8,e711e93aadd54eef8feeb0fa07f054d2,0e914774210646f784d02fb22f8df7db,ca0ce5e9eb09477c8c99b7b94e7869ce,760d9072f9bc480fa36fdb876930467c,5742a628f13045cda0317c3d4d839bc5,b9309305d94f4ee8b66eebbcae9f2db4,a42216d3eedd48f0ba3d3bef2470605f,9cdcda30da704c21b19ea0616de3ea86,f05ecbf7e4b04b268b38cb6e5c8124bf,788d2c1496874eeeae3b7aabbab95f76,8b112af90e054671adc30373c6dafd21,3bdb29667807493292a0ddc9bc0ccdb7,ffd6ebecc869410dab6410a07f3362ae,196868a4610b45588062aed357fc7c58,4cdd2d11be4c4c098b27c10d7cb3fa9e,fda40125207d426dbf86270002ff0c4a,7af028c6048c44cc91415a58d25d286e,ff7d2c8e96e840e1a5a690e4e0f96bad,9f806cf868dc48d398836ebd60bd2ec6,50ff5cdc6c8645b3bb61b8327d204f01,bafb57ea8d84430cbc0c11e5c67643f1,e2c91860b96c46619a74215627560235,f5b68a43bbae44c8a1fe821dfcaca26e,70b771772a0b4f1cb1e592f9a01b08a5,7992a39d64b2483789ae274e2df3c01d,f8d711312da14871a77b58f41a6f6242,673b27b0ab9148b7ae03933c18433971,624886a950bd4d18900a877738696432,685d8d9267f745f39bcc6fc2cc553d31,7d40ad05a1a54165a28ec897ddbcef01,4bcdb5199e8442078999bb8611ea6398,3b0c0ffea0cf45db816993baf64b3fb3,ad2d6d9581034e04addcb5b034a88fb3,8440c4fc5056426996e3e488b2261d28,e367c4223d794572a05fd256759d6a53,1b2dc5784c3c41988d5634e566b4a404,11be1538d7f54d7786004503b2e684a7,b462cb8c2ec7485d938416831a9771cc,f747656a36204e9588aba97064332436,78d0b2337b484c6298c9b3afc01fdb85,8a63d1f8d4bc4eb4a4db437491c9983a,2247f64a15e54053abb2e13152003c6b,d78ef9c5997d4e94804b8126cc387ea1,6ce3cd5a86ef43219dbda2b78e974c93,414b5fd5ebee45d98eca809a524dbe93,4b9ea9db5c834b698a78c7052c87b432,8590fa7adc144f49ae3d2575b7280c08,6148e4b152b9494fb9e7035e9b58e624,be1cff2a29f64800a5c4f91009711988,49b85b35bca5461993eac91ffd0b1fe4,5abae35db6de475e904425af8dc8ecbb,e3bc44b82c3543758d5a6c02b41dd526,89323021918e444bae3537521046d19f,141dd3257f85455d86f3828cc0217b5a,7bc2ad90009a4de2a0fa2947ccc09886,e51874f4b6ac455db76e30582659368b,519dee074e4c48ae9edd58d7c1e330da,060ab186c22f4f42a83f1622bf47ca52,8873bbcf4b42445f8117108769358a5c,46fa5286441a444095b917165c31304c,fa9fdd56222a4ce7ba2b37922afe0eaa,6885932aa4b443babc2b69fdd495889c,be71b947b33a48d8aee36ad0c60d10b8,b52457719429466687a2f228d20571ac,0bf937d745494d998cb8bd2cf3a63f4e,c17feb3860184dd59931f35814ad68b0,4cc68cbd880c48948a1f18fe18ec938c,c37b0691a05c4d659dcd01571f1907da,c1334fe3e8484079bfd62e479307b5b9,ebb58027b6c647b7965eeb31b63ec0c0,1821c5f9392e4859a829c78ebda44df6,dca21ab79bd14004b94c55e36604c30d,8d96cc3457644378b9b643243464f735,631437054079401fbe2c3c9a3f3bc375,c2f78ab4372243da8775f4d71a16fef5,03792664a45d4f63ac3efb899ce1b61e,fec04c46683a41e5a344f4d057342829,320fa70de6bc4890a0135bbfe8735b49,5a03493dce9f4323a0da7bf4fd5e78ab,b9bd46a1da1846d099566fd5c6db00cb,647021c7f798472889fd82f6c4a27ca2,313e905b78d746cb9e574b9634c3a113,623fd328c4cd4ee8b21eb03425b1f100,4676b4d1130146629fd98e2d2eb9a5b4,34d380813a36481ea2534c46d18b5927,894d87e55ac640b99e966a8786a2392b,477ba74d182a4ec0b86926dc67a0f419,362ca5fdca174fa3bdd6325a947320db,1ac3ad8dfa184fe4b6815950d3d23fdd,fdebee18c71f4a9db60096e75ad34d83,91432be4178a4781917fba0abd51df61,480c71baa65145f4bd037332d68e5e4f,536e3a3702b0453d92939846dc1f33b7,0333aedf1be640c78d2ee02473b7fd97,cbb3a6f1772b4f72a8af1375ef16e914,367ad9c43a6246dfac364ca59874883b,bac18430f6db466195a3a7116cdfb958,c4ec39be11be4b0eae321ee0559be8e6,8f7ffae7e47d462aa4142db7f06633d5,82e3b2423d86425195929fc3810ce7a4,72973e0415004dfea220babc479be082,41d0f568bb1e4fed8ad965a79d9bccef,5854ef1d24784ca0a046514b19286113,04afc3dd1eec4d13bc06cd01e0a5e838,db76e3333da846d5ae55b3b9dd735f26,7aa0ca0bf1514fa6a3a14dcde44cf0b9,9c668726c9b6449a9045d3f78408a2a1,7caf148d68a6431bbe56bcc6472d8720,babd36d1d3484fb99fe4e7ca78560ccf,7507865343e24cff80d55a29ec57f0e1,63f8b7d9b1f34851802e0408bf997ea5,a3369d5dd6b74af7b35edf1f10350881,9b8b8c3f86854416b82d57583896c1ff,881815c772564a8dbd5eb6de65ecbe7a,4b1bcf46753a48258d1d514440140b7f,1ca4c6a49e794954a235a82ee8981ecd,ab381d9129bb4b979fe3774e262524f8,a07d2c98831a4d79bc6c5c4288a24720,d8debd28039d494aa7fe18bc8e81af27,d0402b1f00f94acca96276c6ea8c8397,ebcdebd9639d4ec2b7072ad32bf48f50,636b3211100245919c09177ccea562e0,280c4440e18a487396a81b92cd3efa13,5163cf6740334a7d83375385c3ae2b67,a6083e4fab7a43498bfb3257658f9278,3b5c8b5cfb124ebd8f4a798d2a47d832,9fda332474534f96955b5bce8548947f,b5f5e62aba9842d8b48a28111ca6a372,c0cecff1207c4692a634d7de449d8d6d,e4c6428f1d264544b0eec52214e7b6ae,cbc8c5490a7f46b996fd4c3a9711f2d7,cfbdd82f4ce741a88109353c5fc898e4,6bfb965bc1b240a9ae2bd7a1e884ea51,eb36d63a993148289950af9098727759,f7357c3954d2408facaa6771a758ebcc,c05853b94ab8405eb26e0a8bd481b662,da1e6d3ebfa140b3a645342fd3bb5d95,988f3f6b607d442b8df006c980ddc2cd,b6af3c01968141baa3e99ca4b8d83847,75f669faee5d4023a425f2461e5fc5e9,165eb05677aa45fc9a53dfd8bfb7a45d,409891afd7cc4e12ba6b002581fef4f3,02ee0a87956f4bfab91b88282e502a3d,48601418028145ff84a450c877ea35aa,8de73f8d59eb45aa8c61138ab93c4f86,5dd82a46f48945b4bbb61f58b9d1c856,97993db997bd4ff88acfe1340ca86845,282aee84915b43179a09101b3ee6ad0e,8750e994985f43f383321bf0d6ca7932";


            role1.setMenuIds(defalutMenu1);
            String[] defalutMenus1 = defalutMenu1.split(",");
            ArrayList<String> menuIdList1 = new ArrayList<String>();
            for (int i = 0; i < defalutMenus1.length; i++) {
                menuIdList1.add(defalutMenus1[i]);
            }
            role1.setMenuIdList(menuIdList1);
            /*if (!UserUtils.getUser().isAdmin() && role1.getSysData().equals(Global.YES) && !UserUtils.getSelectRole().get(0).getEnname().contains("gly")) {
                return null;
            }*/
            if (Global.isDemoMode()) {
                return null;
            }
           /* if (!"true".equals(checkName(role1.getOldName(), role1.getName(),role1.getCompany().getId()))) {
                return null;
            }*/
            /*if (!"true".equals(checkEnname(role1.getOldEnname(), role1.getEnname()))) {
                return null;
            }*/
            systemService.saveRole(role1, true,1);
            roleList.add(role1);
        }
        return roleList;
    }

    @Transactional(readOnly = false)
    public Role createFinanceRole( Office com,Office office, Office deptFinance,String type) {
        //公司新用户
        Role role1 = new Role();
        role1.setName("财务主管");
        role1.setNumber("1");
        role1.setSysData(Global.YES);
        if (type.equals("1")){
            role1.setDataScope("2");
        }else {
            role1.setDataScope("5");
            role1.setBranchCompany(office);
        }
        role1.setIsNewRecord(false);
        role1.setCompany(com);
        role1.setRoleType("assignment");
        role1.setSysData("1");
        role1.setUseable("1");
        role1.setEnname("zghj");

        String defalutMenu1 = "1,27,29,e3b80fae59ab4b4099d8349ecaf8c4dc,61dd3aa279684f92bb1c5d4019a32503,b0c5737cdc4f452684ce93589427c97d,af6b0e5d61fc44ecbebb449703a3df72,18c14562978e4b129fe3afb7dca8b868,6123fcb3d810492789e82a63b0d4310a,0aa3712414d049a6a24e8bcddeae509a,95a6a82dc5fc4d07b46e5df57a0606a3,62,b6fa4abb624349918fe531274fc15d23,89,76bdb735faa34667a6cbf13ebe937533,90,f5b2028cfad9470085c7c846de33193a,28abe4b0cb3b4c8480f84c07a5dfaf06,c7fa36c7142f481397c2cc12e2bc828a,e2efcf80fc3747329fe02d489dbfda95,e3b5b7451e164089aba0b9bf844b0398,438c28c3cbe44ed7bff951b2a3dc77cd,59e50eebe56743a78eecbc6eff1ef94c,cc1027689aca41a89302217127253034,200fe6cc80174065b382c961a4379d79,4c13ca96376341dfa25980e86964c1a9,638be8b3b5284e5088003917685bc19b,811600aadcb54b098b7dda4bf95ffc51,ce8dc7c97d93481dbd76cabb6e905b55,aa25b3904abb4bd7b779e5be37b4382e,aae6f4c7fd4c4a25afa5285210edfded,84ff9d5c165c4528af9361b6127cf5af,31b349a3624b47f28eb86d614f9f95b5,040b42e395de41d6805a958eb1897863,34874260e6fc4200a76b33c3cfa4d80d,5b6c6ea5ed2d45478e8144351d853e21,7c032c32512847f882a87416359ef0a3,6d30ea8c710a42b89da2f2506059b0ef,2ca7bf4ce6804a1c9e5b331d2df27c72,7717cbb6d71f405d890e9b648cbd2088,bddb0400764c42b08e427059c7752057,23d6c56239804053910e5c5aec481330,7f8b2018113545bcbdbfc87e7a236bec,547e121ffb714d16ae142b6935abb78f,af24101b962444f29d166b638b2fea30,6ea6519ec4d548128aa42f3ce004def3,4379bf54c2a543869abf5031b97b8efe,5e6da3f5368b4f06b7268b31a1157e9f,2bbf0e75f21d42938954391d0776fbde,545b39dacf564a5eb9d1658f80a0b8b0,b4ddedf9b451494a97a1ff9476e3897d,1b145ca23d3e4d81994a84d30a262d18,10a8c7c1d34e46dd8ef461cdad928afc,5ae4911d14bb48b39885b1fe522e999d,7f09def2533b45fdb3c733f1e72ca5fb,9ef29dca19d342b795efcbfe32793602,be72d72ae7034c509c164467b2abddda,ab879705f2354813acf8724404520e28,82487b87b97b475b98892b80cf83c96a,0f4a461b97be48eeab43bd052f630057,72e1a28b7251438a89b0a4f6bd522b89,e1f9a246057347d78a1966d6dc0168e3,5d3645d53ce14c9ab498fd9f050ba264,d361e6890cc84b049fc8268c2a8c4952,5cc0002cbcc14d3485ab8ae6d57368f2,fa7bb243643c44b18e989e3419eeb171,62768b6c65134f1e927b1807bc72115a,7b69b0e5d2d344e6aaa2b8726c23561a,434bbb1f9f8140b1beeb57d0c4db660d,7920c4c8da294d87bcb94e7756bf35ae,0752b85660694033bbdb013f30cde17f,bd6258cfbea64896b0401517e440d6a2,9aa3539682514546866d2998d741ac6c,eb3ad93ce6a44921b0490c842dd47d8c,9ce19042d497473c92d199a5e5a7daf0,1b3eb6ba64ea4589a8e2d42efa5d3f89,6b2dd5ff2a754a53a80798341a7d35d8,e53d754160f74d28b6d091829fc289f8,92d762bca040407bb4ce4b06a2cfa195,aa8a2b00622147259c85290f4d95b46d,0be3e487d874487fbfb5c49b52440a48,ce60b25f7da04af8b7cefccf2dbc7d36,6464ac9cc3fc47d88598f6c4e74caa19,04b312df403d4855b013743206ddb1de,17356c44ae894782a1015ec2ddaeaeda,a06eaefba7084e219f0c6ce4010b82e7,95a54e4db7e74f0586cd85cb4e2bd6f7,1a93047d6b0f4e3ca70fcf836b2517c4,93263a29941648979bb8de774978a880,8eba834a28864375aa3137590d26f37e,3c52b0a890524c6fb66cefc32114c60e,8b20519b794c42d28fdecbd0ccbba5c1,77189ceba655488681964a609c28256d,39e776aaf3644d24bb78ecf43704b71b,0dd5fa1192e34d5fac5e78ca38969126,1a6f82a80a054bc7ba4b516187231a54,5993ce4ec80a41fb9d4ea0713b74f4a4,3b42071536124a9c9dbf2ec319004b29,a966beefe58e4f95b24e89f4172cb7ce,2e1e75dbe28841529576dd1cd48d2437,cdd8829e5976438aaa66ce608f0175f5,71015695cf1445ecacce90669c67d733,31e9dd26b7314d8a9bf97dcf6dbdb22d,e5ecaa091bb0464f8553d4e7aa60647d,39055be3a23945868dbf4b5930043a88,07ba7c9aed104657aac3be87aa346d3b,5a7a9bfff0f44a2089a7d277f2c04557,be53cc152339400ba705c1d1dceff9ab,ed591ccba0564abd9979899049dc7b36,9a4425b817e94dbe844d7fd3e96a9033,bf4e54bfe9984441a8efa4dd8a4a23a6,5a1bcf2a2e0943f88bf655ca011fb288,a12c8904ff0147a99010eaa5aa0ea704,e7c6f2581c6e41bf8fe88a3d7fadc986,89709767beea48d0b0fb047316eb37d1,04d6a49b35514f91be43e9576f8d32fc,bd50e783ec714e5d86aafcca54e87c33,4acb0c7d002b47e29be7ebe9f40490f6,67b5ddef866b4982aa5afef7aad69924,7e090632f713440bba11ed6fc09eb291,f308051e11ca47eebb12709725b52eb4,dbea976ef1d248cabf13b88d311c4533,e0132620d4044e38a92f75b8335005ed,1b598563135e42d4bfe729cf32649f14,c4de9219f4f14684ae21d0904b7b78df,520983ab7dd64104a47e273941236b2d,133ba69bd04046b89eadaf793b9063db,11f24ca662044292a8ca95930ccc3581,0f30b4d621294d4fab359611cf82ab6f,b3c93fcd54b544e5b2c8e70ee7eb93ce,79ebe6595ce44208853ba553186b8348,8d639afccbec43f2be8d3ed88702893b,4e3483f3bbbb48f59b0e92a7d0ccc1c0,b160fccc15fa4ba09c74018d82d16eb7,52f248eadea74cf2b7780bf991d95e4e,d4cb9017aa1f4f8ca34ae8d31c986210,eff19819b1284424a9ad7879991ee080,79551897f86b4338bd09b41c5953a7db,f11bd4ef5ddd4c82865fad3f1f9d80a9,76ee95093a8748b29d55df1cb8cdebd0,081af742f6384a20b492ca99704f51ca,c26c844bbb2f4746904948f3ecfeb1ae,ba9bbc5581a043d8b9c204b8ddb6aa01,cde1afb048a743bb9caedb5770aabca1,0b2b37a28bb0457bbc4bf33c11b4c6e1,9f94d338d6c94b95814b2c50f9e463d1,7d1745ce2c444db9bf4b4e100c476fe8,e711e93aadd54eef8feeb0fa07f054d2,0e914774210646f784d02fb22f8df7db,ca0ce5e9eb09477c8c99b7b94e7869ce,760d9072f9bc480fa36fdb876930467c,5742a628f13045cda0317c3d4d839bc5,b9309305d94f4ee8b66eebbcae9f2db4,a42216d3eedd48f0ba3d3bef2470605f,9cdcda30da704c21b19ea0616de3ea86,f05ecbf7e4b04b268b38cb6e5c8124bf,788d2c1496874eeeae3b7aabbab95f76,8b112af90e054671adc30373c6dafd21,3bdb29667807493292a0ddc9bc0ccdb7,ffd6ebecc869410dab6410a07f3362ae,196868a4610b45588062aed357fc7c58,4cdd2d11be4c4c098b27c10d7cb3fa9e,fda40125207d426dbf86270002ff0c4a,7af028c6048c44cc91415a58d25d286e,ff7d2c8e96e840e1a5a690e4e0f96bad,9f806cf868dc48d398836ebd60bd2ec6,50ff5cdc6c8645b3bb61b8327d204f01,bafb57ea8d84430cbc0c11e5c67643f1,e2c91860b96c46619a74215627560235,f5b68a43bbae44c8a1fe821dfcaca26e,70b771772a0b4f1cb1e592f9a01b08a5,7992a39d64b2483789ae274e2df3c01d,f8d711312da14871a77b58f41a6f6242,673b27b0ab9148b7ae03933c18433971,624886a950bd4d18900a877738696432,685d8d9267f745f39bcc6fc2cc553d31,7d40ad05a1a54165a28ec897ddbcef01,4bcdb5199e8442078999bb8611ea6398,3b0c0ffea0cf45db816993baf64b3fb3,ad2d6d9581034e04addcb5b034a88fb3,8440c4fc5056426996e3e488b2261d28,e367c4223d794572a05fd256759d6a53,1b2dc5784c3c41988d5634e566b4a404,11be1538d7f54d7786004503b2e684a7,b462cb8c2ec7485d938416831a9771cc,f747656a36204e9588aba97064332436,78d0b2337b484c6298c9b3afc01fdb85,8a63d1f8d4bc4eb4a4db437491c9983a,2247f64a15e54053abb2e13152003c6b,d78ef9c5997d4e94804b8126cc387ea1,6ce3cd5a86ef43219dbda2b78e974c93,414b5fd5ebee45d98eca809a524dbe93,4b9ea9db5c834b698a78c7052c87b432,8590fa7adc144f49ae3d2575b7280c08,6148e4b152b9494fb9e7035e9b58e624,be1cff2a29f64800a5c4f91009711988,49b85b35bca5461993eac91ffd0b1fe4,5abae35db6de475e904425af8dc8ecbb,e3bc44b82c3543758d5a6c02b41dd526,89323021918e444bae3537521046d19f,141dd3257f85455d86f3828cc0217b5a,7bc2ad90009a4de2a0fa2947ccc09886,e51874f4b6ac455db76e30582659368b,519dee074e4c48ae9edd58d7c1e330da,060ab186c22f4f42a83f1622bf47ca52,8873bbcf4b42445f8117108769358a5c,46fa5286441a444095b917165c31304c,fa9fdd56222a4ce7ba2b37922afe0eaa,6885932aa4b443babc2b69fdd495889c,be71b947b33a48d8aee36ad0c60d10b8,b52457719429466687a2f228d20571ac,0bf937d745494d998cb8bd2cf3a63f4e,c17feb3860184dd59931f35814ad68b0,4cc68cbd880c48948a1f18fe18ec938c,c37b0691a05c4d659dcd01571f1907da,c1334fe3e8484079bfd62e479307b5b9,ebb58027b6c647b7965eeb31b63ec0c0,1821c5f9392e4859a829c78ebda44df6,dca21ab79bd14004b94c55e36604c30d,8d96cc3457644378b9b643243464f735,631437054079401fbe2c3c9a3f3bc375,c2f78ab4372243da8775f4d71a16fef5,03792664a45d4f63ac3efb899ce1b61e,fec04c46683a41e5a344f4d057342829,320fa70de6bc4890a0135bbfe8735b49,5a03493dce9f4323a0da7bf4fd5e78ab,b9bd46a1da1846d099566fd5c6db00cb,647021c7f798472889fd82f6c4a27ca2,313e905b78d746cb9e574b9634c3a113,623fd328c4cd4ee8b21eb03425b1f100,4676b4d1130146629fd98e2d2eb9a5b4,34d380813a36481ea2534c46d18b5927,894d87e55ac640b99e966a8786a2392b,477ba74d182a4ec0b86926dc67a0f419,362ca5fdca174fa3bdd6325a947320db,1ac3ad8dfa184fe4b6815950d3d23fdd,fdebee18c71f4a9db60096e75ad34d83,91432be4178a4781917fba0abd51df61,480c71baa65145f4bd037332d68e5e4f,536e3a3702b0453d92939846dc1f33b7,0333aedf1be640c78d2ee02473b7fd97,cbb3a6f1772b4f72a8af1375ef16e914,367ad9c43a6246dfac364ca59874883b,bac18430f6db466195a3a7116cdfb958,c4ec39be11be4b0eae321ee0559be8e6,8f7ffae7e47d462aa4142db7f06633d5,82e3b2423d86425195929fc3810ce7a4,72973e0415004dfea220babc479be082,41d0f568bb1e4fed8ad965a79d9bccef,5854ef1d24784ca0a046514b19286113,04afc3dd1eec4d13bc06cd01e0a5e838,db76e3333da846d5ae55b3b9dd735f26,7aa0ca0bf1514fa6a3a14dcde44cf0b9,9c668726c9b6449a9045d3f78408a2a1,7caf148d68a6431bbe56bcc6472d8720,babd36d1d3484fb99fe4e7ca78560ccf,7507865343e24cff80d55a29ec57f0e1,63f8b7d9b1f34851802e0408bf997ea5,a3369d5dd6b74af7b35edf1f10350881,9b8b8c3f86854416b82d57583896c1ff,881815c772564a8dbd5eb6de65ecbe7a,4b1bcf46753a48258d1d514440140b7f,1ca4c6a49e794954a235a82ee8981ecd,ab381d9129bb4b979fe3774e262524f8,a07d2c98831a4d79bc6c5c4288a24720,d8debd28039d494aa7fe18bc8e81af27,d0402b1f00f94acca96276c6ea8c8397,ebcdebd9639d4ec2b7072ad32bf48f50,636b3211100245919c09177ccea562e0,280c4440e18a487396a81b92cd3efa13,5163cf6740334a7d83375385c3ae2b67,a6083e4fab7a43498bfb3257658f9278,3b5c8b5cfb124ebd8f4a798d2a47d832,9fda332474534f96955b5bce8548947f,b5f5e62aba9842d8b48a28111ca6a372,c0cecff1207c4692a634d7de449d8d6d,e4c6428f1d264544b0eec52214e7b6ae,cbc8c5490a7f46b996fd4c3a9711f2d7,cfbdd82f4ce741a88109353c5fc898e4,6bfb965bc1b240a9ae2bd7a1e884ea51,eb36d63a993148289950af9098727759,f7357c3954d2408facaa6771a758ebcc,c05853b94ab8405eb26e0a8bd481b662,da1e6d3ebfa140b3a645342fd3bb5d95,988f3f6b607d442b8df006c980ddc2cd,b6af3c01968141baa3e99ca4b8d83847,75f669faee5d4023a425f2461e5fc5e9,165eb05677aa45fc9a53dfd8bfb7a45d,409891afd7cc4e12ba6b002581fef4f3,02ee0a87956f4bfab91b88282e502a3d,48601418028145ff84a450c877ea35aa,8de73f8d59eb45aa8c61138ab93c4f86,5dd82a46f48945b4bbb61f58b9d1c856,97993db997bd4ff88acfe1340ca86845,282aee84915b43179a09101b3ee6ad0e,8750e994985f43f383321bf0d6ca7932";

        role1.setMenuIds(defalutMenu1);
        String[] defalutMenus1 = defalutMenu1.split(",");
        ArrayList<String> menuIdList1 = new ArrayList<String>();
        for (int i = 0; i < defalutMenus1.length; i++) {
            menuIdList1.add(defalutMenus1[i]);
        }
        role1.setMenuIdList(menuIdList1);
        if (Global.isDemoMode()) {
            return null;
        }
        role1.setOffice(deptFinance);
        systemService.saveRole(role1,true,1);
        SysRoleActivity sysRoleActivity = new SysRoleActivity();
        SysRoleActivity roleActivity = new SysRoleActivity();
        if ("1".equals(type)) {
            String name = StringUtils.getPinYinHeadChar(office.getName(), 2) + System.currentTimeMillis();
            String enname = name + StringUtils.getPinYinHeadChar("财务主管", 1);

            sysRoleActivity.setName("财务主管");
            sysRoleActivity.setEnname(enname);
            sysRoleActivity.setSort("7000");
            sysRoleActivity.setType("3");
            sysRoleActivity.setCompanyId(com.getId());
            sysRoleActivity.setDelType("1");
            sysRoleActivityService.save(sysRoleActivity);
            roleActivity.setName("发票管理员");
            roleActivity.setEnname(name + StringUtils.getPinYinHeadChar("发票管理员", 1));
            roleActivity.setSort("8000");
            roleActivity.setType("3");
            roleActivity.setCompanyId(com.getId());
            roleActivity.setDelType("1");
            sysRoleActivityService.save(roleActivity);
        }else {
            String enname = UserUtils.getRoleActivityEnname(com.getId(),UserUtils.getUser());
            sysRoleActivity = sysRoleActivityService.getByEnname(enname+"cwzg");
            roleActivity = sysRoleActivityService.getByEnname(enname+"fpgly");
        }
        if (sysRoleActivity!=null) {
            List<SysRoleActivity> roleActivities = new ArrayList<>();
            roleActivities.add(sysRoleActivity);
            roleActivities.add(roleActivity);
            role1.setRoleActivityList(roleActivities);
            roleDao.insertRoleActivity(role1);
        }
        return role1;
    }

    @Transactional(readOnly = false)
    public Role createFinanceRoleZgkj( Office com,Office office,Office deptFinance,String type) {
        //公司新用户
        Role role1 = new Role();
        role1.setNumber("1");
        role1.setName("主管会计");
        role1.setSysData(Global.YES);
        if (type.equals("1")){
            role1.setDataScope("2");
        }else {
            role1.setBranchCompany(office);
            role1.setDataScope("5");
        }
        role1.setOffice(deptFinance);
        role1.setIsNewRecord(false);
        role1.setCompany(com);
        role1.setRoleType("assignment");
        role1.setSysData("1");
        role1.setUseable("1");
        role1.setEnname("zghj");

        String defalutMenu1 = "1,27,29,e3b80fae59ab4b4099d8349ecaf8c4dc,61dd3aa279684f92bb1c5d4019a32503,b0c5737cdc4f452684ce93589427c97d,af6b0e5d61fc44ecbebb449703a3df72,18c14562978e4b129fe3afb7dca8b868,6123fcb3d810492789e82a63b0d4310a,0aa3712414d049a6a24e8bcddeae509a,95a6a82dc5fc4d07b46e5df57a0606a3,62,b6fa4abb624349918fe531274fc15d23,89,76bdb735faa34667a6cbf13ebe937533,90,f5b2028cfad9470085c7c846de33193a,28abe4b0cb3b4c8480f84c07a5dfaf06,c7fa36c7142f481397c2cc12e2bc828a,e2efcf80fc3747329fe02d489dbfda95,e3b5b7451e164089aba0b9bf844b0398,438c28c3cbe44ed7bff951b2a3dc77cd,59e50eebe56743a78eecbc6eff1ef94c,cc1027689aca41a89302217127253034,200fe6cc80174065b382c961a4379d79,4c13ca96376341dfa25980e86964c1a9,638be8b3b5284e5088003917685bc19b,811600aadcb54b098b7dda4bf95ffc51,ce8dc7c97d93481dbd76cabb6e905b55,aa25b3904abb4bd7b779e5be37b4382e,aae6f4c7fd4c4a25afa5285210edfded,84ff9d5c165c4528af9361b6127cf5af,31b349a3624b47f28eb86d614f9f95b5,040b42e395de41d6805a958eb1897863,34874260e6fc4200a76b33c3cfa4d80d,5b6c6ea5ed2d45478e8144351d853e21,7c032c32512847f882a87416359ef0a3,6d30ea8c710a42b89da2f2506059b0ef,2ca7bf4ce6804a1c9e5b331d2df27c72,7717cbb6d71f405d890e9b648cbd2088,bddb0400764c42b08e427059c7752057,23d6c56239804053910e5c5aec481330,7f8b2018113545bcbdbfc87e7a236bec,547e121ffb714d16ae142b6935abb78f,af24101b962444f29d166b638b2fea30,6ea6519ec4d548128aa42f3ce004def3,4379bf54c2a543869abf5031b97b8efe,5e6da3f5368b4f06b7268b31a1157e9f,2bbf0e75f21d42938954391d0776fbde,545b39dacf564a5eb9d1658f80a0b8b0,b4ddedf9b451494a97a1ff9476e3897d,1b145ca23d3e4d81994a84d30a262d18,10a8c7c1d34e46dd8ef461cdad928afc,5ae4911d14bb48b39885b1fe522e999d,7f09def2533b45fdb3c733f1e72ca5fb,9ef29dca19d342b795efcbfe32793602,be72d72ae7034c509c164467b2abddda,ab879705f2354813acf8724404520e28,82487b87b97b475b98892b80cf83c96a,0f4a461b97be48eeab43bd052f630057,72e1a28b7251438a89b0a4f6bd522b89,e1f9a246057347d78a1966d6dc0168e3,5d3645d53ce14c9ab498fd9f050ba264,d361e6890cc84b049fc8268c2a8c4952,5cc0002cbcc14d3485ab8ae6d57368f2,fa7bb243643c44b18e989e3419eeb171,62768b6c65134f1e927b1807bc72115a,7b69b0e5d2d344e6aaa2b8726c23561a,434bbb1f9f8140b1beeb57d0c4db660d,7920c4c8da294d87bcb94e7756bf35ae,0752b85660694033bbdb013f30cde17f,bd6258cfbea64896b0401517e440d6a2,9aa3539682514546866d2998d741ac6c,eb3ad93ce6a44921b0490c842dd47d8c,9ce19042d497473c92d199a5e5a7daf0,1b3eb6ba64ea4589a8e2d42efa5d3f89,6b2dd5ff2a754a53a80798341a7d35d8,e53d754160f74d28b6d091829fc289f8,92d762bca040407bb4ce4b06a2cfa195,aa8a2b00622147259c85290f4d95b46d,0be3e487d874487fbfb5c49b52440a48,ce60b25f7da04af8b7cefccf2dbc7d36,6464ac9cc3fc47d88598f6c4e74caa19,04b312df403d4855b013743206ddb1de,17356c44ae894782a1015ec2ddaeaeda,a06eaefba7084e219f0c6ce4010b82e7,95a54e4db7e74f0586cd85cb4e2bd6f7,1a93047d6b0f4e3ca70fcf836b2517c4,93263a29941648979bb8de774978a880,8eba834a28864375aa3137590d26f37e,3c52b0a890524c6fb66cefc32114c60e,8b20519b794c42d28fdecbd0ccbba5c1,77189ceba655488681964a609c28256d,39e776aaf3644d24bb78ecf43704b71b,0dd5fa1192e34d5fac5e78ca38969126,1a6f82a80a054bc7ba4b516187231a54,5993ce4ec80a41fb9d4ea0713b74f4a4,3b42071536124a9c9dbf2ec319004b29,a966beefe58e4f95b24e89f4172cb7ce,2e1e75dbe28841529576dd1cd48d2437,cdd8829e5976438aaa66ce608f0175f5,71015695cf1445ecacce90669c67d733,31e9dd26b7314d8a9bf97dcf6dbdb22d,e5ecaa091bb0464f8553d4e7aa60647d,39055be3a23945868dbf4b5930043a88,07ba7c9aed104657aac3be87aa346d3b,5a7a9bfff0f44a2089a7d277f2c04557,be53cc152339400ba705c1d1dceff9ab,ed591ccba0564abd9979899049dc7b36,9a4425b817e94dbe844d7fd3e96a9033,bf4e54bfe9984441a8efa4dd8a4a23a6,5a1bcf2a2e0943f88bf655ca011fb288,a12c8904ff0147a99010eaa5aa0ea704,e7c6f2581c6e41bf8fe88a3d7fadc986,89709767beea48d0b0fb047316eb37d1,04d6a49b35514f91be43e9576f8d32fc,bd50e783ec714e5d86aafcca54e87c33,4acb0c7d002b47e29be7ebe9f40490f6,67b5ddef866b4982aa5afef7aad69924,7e090632f713440bba11ed6fc09eb291,f308051e11ca47eebb12709725b52eb4,dbea976ef1d248cabf13b88d311c4533,e0132620d4044e38a92f75b8335005ed,1b598563135e42d4bfe729cf32649f14,c4de9219f4f14684ae21d0904b7b78df,520983ab7dd64104a47e273941236b2d,133ba69bd04046b89eadaf793b9063db,11f24ca662044292a8ca95930ccc3581,0f30b4d621294d4fab359611cf82ab6f,b3c93fcd54b544e5b2c8e70ee7eb93ce,79ebe6595ce44208853ba553186b8348,8d639afccbec43f2be8d3ed88702893b,4e3483f3bbbb48f59b0e92a7d0ccc1c0,b160fccc15fa4ba09c74018d82d16eb7,52f248eadea74cf2b7780bf991d95e4e,d4cb9017aa1f4f8ca34ae8d31c986210,eff19819b1284424a9ad7879991ee080,79551897f86b4338bd09b41c5953a7db,f11bd4ef5ddd4c82865fad3f1f9d80a9,76ee95093a8748b29d55df1cb8cdebd0,081af742f6384a20b492ca99704f51ca,c26c844bbb2f4746904948f3ecfeb1ae,ba9bbc5581a043d8b9c204b8ddb6aa01,cde1afb048a743bb9caedb5770aabca1,0b2b37a28bb0457bbc4bf33c11b4c6e1,9f94d338d6c94b95814b2c50f9e463d1,7d1745ce2c444db9bf4b4e100c476fe8,e711e93aadd54eef8feeb0fa07f054d2,0e914774210646f784d02fb22f8df7db,ca0ce5e9eb09477c8c99b7b94e7869ce,760d9072f9bc480fa36fdb876930467c,5742a628f13045cda0317c3d4d839bc5,b9309305d94f4ee8b66eebbcae9f2db4,a42216d3eedd48f0ba3d3bef2470605f,9cdcda30da704c21b19ea0616de3ea86,f05ecbf7e4b04b268b38cb6e5c8124bf,788d2c1496874eeeae3b7aabbab95f76,8b112af90e054671adc30373c6dafd21,3bdb29667807493292a0ddc9bc0ccdb7,ffd6ebecc869410dab6410a07f3362ae,196868a4610b45588062aed357fc7c58,4cdd2d11be4c4c098b27c10d7cb3fa9e,fda40125207d426dbf86270002ff0c4a,7af028c6048c44cc91415a58d25d286e,ff7d2c8e96e840e1a5a690e4e0f96bad,9f806cf868dc48d398836ebd60bd2ec6,50ff5cdc6c8645b3bb61b8327d204f01,bafb57ea8d84430cbc0c11e5c67643f1,e2c91860b96c46619a74215627560235,f5b68a43bbae44c8a1fe821dfcaca26e,70b771772a0b4f1cb1e592f9a01b08a5,7992a39d64b2483789ae274e2df3c01d,f8d711312da14871a77b58f41a6f6242,673b27b0ab9148b7ae03933c18433971,624886a950bd4d18900a877738696432,685d8d9267f745f39bcc6fc2cc553d31,7d40ad05a1a54165a28ec897ddbcef01,4bcdb5199e8442078999bb8611ea6398,3b0c0ffea0cf45db816993baf64b3fb3,ad2d6d9581034e04addcb5b034a88fb3,8440c4fc5056426996e3e488b2261d28,e367c4223d794572a05fd256759d6a53,1b2dc5784c3c41988d5634e566b4a404,11be1538d7f54d7786004503b2e684a7,b462cb8c2ec7485d938416831a9771cc,f747656a36204e9588aba97064332436,78d0b2337b484c6298c9b3afc01fdb85,8a63d1f8d4bc4eb4a4db437491c9983a,2247f64a15e54053abb2e13152003c6b,d78ef9c5997d4e94804b8126cc387ea1,6ce3cd5a86ef43219dbda2b78e974c93,414b5fd5ebee45d98eca809a524dbe93,4b9ea9db5c834b698a78c7052c87b432,8590fa7adc144f49ae3d2575b7280c08,6148e4b152b9494fb9e7035e9b58e624,be1cff2a29f64800a5c4f91009711988,49b85b35bca5461993eac91ffd0b1fe4,5abae35db6de475e904425af8dc8ecbb,e3bc44b82c3543758d5a6c02b41dd526,89323021918e444bae3537521046d19f,141dd3257f85455d86f3828cc0217b5a,7bc2ad90009a4de2a0fa2947ccc09886,e51874f4b6ac455db76e30582659368b,519dee074e4c48ae9edd58d7c1e330da,060ab186c22f4f42a83f1622bf47ca52,8873bbcf4b42445f8117108769358a5c,46fa5286441a444095b917165c31304c,fa9fdd56222a4ce7ba2b37922afe0eaa,6885932aa4b443babc2b69fdd495889c,be71b947b33a48d8aee36ad0c60d10b8,b52457719429466687a2f228d20571ac,0bf937d745494d998cb8bd2cf3a63f4e,c17feb3860184dd59931f35814ad68b0,4cc68cbd880c48948a1f18fe18ec938c,c37b0691a05c4d659dcd01571f1907da,c1334fe3e8484079bfd62e479307b5b9,ebb58027b6c647b7965eeb31b63ec0c0,1821c5f9392e4859a829c78ebda44df6,dca21ab79bd14004b94c55e36604c30d,8d96cc3457644378b9b643243464f735,631437054079401fbe2c3c9a3f3bc375,c2f78ab4372243da8775f4d71a16fef5,03792664a45d4f63ac3efb899ce1b61e,fec04c46683a41e5a344f4d057342829,320fa70de6bc4890a0135bbfe8735b49,5a03493dce9f4323a0da7bf4fd5e78ab,b9bd46a1da1846d099566fd5c6db00cb,647021c7f798472889fd82f6c4a27ca2,313e905b78d746cb9e574b9634c3a113,623fd328c4cd4ee8b21eb03425b1f100,4676b4d1130146629fd98e2d2eb9a5b4,34d380813a36481ea2534c46d18b5927,894d87e55ac640b99e966a8786a2392b,477ba74d182a4ec0b86926dc67a0f419,362ca5fdca174fa3bdd6325a947320db,1ac3ad8dfa184fe4b6815950d3d23fdd,fdebee18c71f4a9db60096e75ad34d83,91432be4178a4781917fba0abd51df61,480c71baa65145f4bd037332d68e5e4f,536e3a3702b0453d92939846dc1f33b7,0333aedf1be640c78d2ee02473b7fd97,cbb3a6f1772b4f72a8af1375ef16e914,367ad9c43a6246dfac364ca59874883b,bac18430f6db466195a3a7116cdfb958,c4ec39be11be4b0eae321ee0559be8e6,8f7ffae7e47d462aa4142db7f06633d5,82e3b2423d86425195929fc3810ce7a4,72973e0415004dfea220babc479be082,41d0f568bb1e4fed8ad965a79d9bccef,5854ef1d24784ca0a046514b19286113,04afc3dd1eec4d13bc06cd01e0a5e838,db76e3333da846d5ae55b3b9dd735f26,7aa0ca0bf1514fa6a3a14dcde44cf0b9,9c668726c9b6449a9045d3f78408a2a1,7caf148d68a6431bbe56bcc6472d8720,babd36d1d3484fb99fe4e7ca78560ccf,7507865343e24cff80d55a29ec57f0e1,63f8b7d9b1f34851802e0408bf997ea5,a3369d5dd6b74af7b35edf1f10350881,9b8b8c3f86854416b82d57583896c1ff,881815c772564a8dbd5eb6de65ecbe7a,4b1bcf46753a48258d1d514440140b7f,1ca4c6a49e794954a235a82ee8981ecd,ab381d9129bb4b979fe3774e262524f8,a07d2c98831a4d79bc6c5c4288a24720,d8debd28039d494aa7fe18bc8e81af27,d0402b1f00f94acca96276c6ea8c8397,ebcdebd9639d4ec2b7072ad32bf48f50,636b3211100245919c09177ccea562e0,280c4440e18a487396a81b92cd3efa13,5163cf6740334a7d83375385c3ae2b67,a6083e4fab7a43498bfb3257658f9278,3b5c8b5cfb124ebd8f4a798d2a47d832,9fda332474534f96955b5bce8548947f,b5f5e62aba9842d8b48a28111ca6a372,c0cecff1207c4692a634d7de449d8d6d,e4c6428f1d264544b0eec52214e7b6ae,cbc8c5490a7f46b996fd4c3a9711f2d7,cfbdd82f4ce741a88109353c5fc898e4,6bfb965bc1b240a9ae2bd7a1e884ea51,eb36d63a993148289950af9098727759,f7357c3954d2408facaa6771a758ebcc,c05853b94ab8405eb26e0a8bd481b662,da1e6d3ebfa140b3a645342fd3bb5d95,988f3f6b607d442b8df006c980ddc2cd,b6af3c01968141baa3e99ca4b8d83847,75f669faee5d4023a425f2461e5fc5e9,165eb05677aa45fc9a53dfd8bfb7a45d,409891afd7cc4e12ba6b002581fef4f3,02ee0a87956f4bfab91b88282e502a3d,48601418028145ff84a450c877ea35aa,8de73f8d59eb45aa8c61138ab93c4f86,5dd82a46f48945b4bbb61f58b9d1c856,97993db997bd4ff88acfe1340ca86845,282aee84915b43179a09101b3ee6ad0e,8750e994985f43f383321bf0d6ca7932";

        role1.setMenuIds(defalutMenu1);
        String[] defalutMenus1 = defalutMenu1.split(",");
        ArrayList<String> menuIdList1 = new ArrayList<String>();
        for (int i = 0; i < defalutMenus1.length; i++) {
            menuIdList1.add(defalutMenus1[i]);
        }
        role1.setMenuIdList(menuIdList1);
        if (Global.isDemoMode()) {
            return null;
        }
        systemService.saveRole(role1,true,1);
        SysRoleActivity sysRoleActivity = new SysRoleActivity();
        if ("1".equals(type)) {
            String enname = UserUtils.getRoleActivity(com.getId()) + StringUtils.getPinYinHeadChar("主管会计", 1);

            sysRoleActivity.setName("主管会计");
            sysRoleActivity.setEnname(enname);
            sysRoleActivity.setSort("7100");
            sysRoleActivity.setType("3");
            sysRoleActivity.setCompanyId(com.getId());
            sysRoleActivity.setDelType("1");
            sysRoleActivityService.save(sysRoleActivity);

        }else {
            String enname = UserUtils.getRoleActivityEnname(com.getId(),UserUtils.getUser());
            sysRoleActivity = sysRoleActivityService.getByEnname(enname+"zghj");
        }
        if (sysRoleActivity!=null) {
            List<SysRoleActivity> roleActivities = new ArrayList<>();
            roleActivities.add(sysRoleActivity);
            role1.setRoleActivityList(roleActivities);
            roleDao.insertRoleActivity(role1);
        }
        return role1;
    }

    @Transactional(readOnly = false)
    public Role createFinanceRoleCN( Office com,Office office,Office deptFinance,String type) {
        //公司新用户
        Role role1 = new Role();
        role1.setNumber("4");
        role1.setName("出纳");
        role1.setSysData(Global.YES);
        if (type.equals("1")){
            role1.setDataScope("2");
        }else {
            role1.setDataScope("5");
            role1.setBranchCompany(office);
        }
        role1.setOffice(deptFinance);
        role1.setIsNewRecord(false);
        role1.setCompany(com);
        role1.setRoleType("assignment");
        role1.setSysData("1");
        role1.setUseable("1");
        role1.setEnname("cn");

        String defalutMenu1 = "1,27,29,e3b80fae59ab4b4099d8349ecaf8c4dc,61dd3aa279684f92bb1c5d4019a32503,b0c5737cdc4f452684ce93589427c97d,af6b0e5d61fc44ecbebb449703a3df72,18c14562978e4b129fe3afb7dca8b868,6123fcb3d810492789e82a63b0d4310a,0aa3712414d049a6a24e8bcddeae509a,95a6a82dc5fc4d07b46e5df57a0606a3,62,b6fa4abb624349918fe531274fc15d23,89,76bdb735faa34667a6cbf13ebe937533,90,f5b2028cfad9470085c7c846de33193a,28abe4b0cb3b4c8480f84c07a5dfaf06,c7fa36c7142f481397c2cc12e2bc828a,e2efcf80fc3747329fe02d489dbfda95,e3b5b7451e164089aba0b9bf844b0398,438c28c3cbe44ed7bff951b2a3dc77cd,59e50eebe56743a78eecbc6eff1ef94c,cc1027689aca41a89302217127253034,200fe6cc80174065b382c961a4379d79,4c13ca96376341dfa25980e86964c1a9,638be8b3b5284e5088003917685bc19b,811600aadcb54b098b7dda4bf95ffc51,ce8dc7c97d93481dbd76cabb6e905b55,aa25b3904abb4bd7b779e5be37b4382e,aae6f4c7fd4c4a25afa5285210edfded,84ff9d5c165c4528af9361b6127cf5af,31b349a3624b47f28eb86d614f9f95b5,040b42e395de41d6805a958eb1897863,34874260e6fc4200a76b33c3cfa4d80d,5b6c6ea5ed2d45478e8144351d853e21,7c032c32512847f882a87416359ef0a3,6d30ea8c710a42b89da2f2506059b0ef,2ca7bf4ce6804a1c9e5b331d2df27c72,7717cbb6d71f405d890e9b648cbd2088,bddb0400764c42b08e427059c7752057,23d6c56239804053910e5c5aec481330,7f8b2018113545bcbdbfc87e7a236bec,547e121ffb714d16ae142b6935abb78f,af24101b962444f29d166b638b2fea30,6ea6519ec4d548128aa42f3ce004def3,4379bf54c2a543869abf5031b97b8efe,5e6da3f5368b4f06b7268b31a1157e9f,2bbf0e75f21d42938954391d0776fbde,545b39dacf564a5eb9d1658f80a0b8b0,b4ddedf9b451494a97a1ff9476e3897d,1b145ca23d3e4d81994a84d30a262d18,10a8c7c1d34e46dd8ef461cdad928afc,5ae4911d14bb48b39885b1fe522e999d,7f09def2533b45fdb3c733f1e72ca5fb,9ef29dca19d342b795efcbfe32793602,be72d72ae7034c509c164467b2abddda,ab879705f2354813acf8724404520e28,82487b87b97b475b98892b80cf83c96a,0f4a461b97be48eeab43bd052f630057,72e1a28b7251438a89b0a4f6bd522b89,e1f9a246057347d78a1966d6dc0168e3,5d3645d53ce14c9ab498fd9f050ba264,d361e6890cc84b049fc8268c2a8c4952,5cc0002cbcc14d3485ab8ae6d57368f2,fa7bb243643c44b18e989e3419eeb171,62768b6c65134f1e927b1807bc72115a,7b69b0e5d2d344e6aaa2b8726c23561a,434bbb1f9f8140b1beeb57d0c4db660d,7920c4c8da294d87bcb94e7756bf35ae,0752b85660694033bbdb013f30cde17f,bd6258cfbea64896b0401517e440d6a2,9aa3539682514546866d2998d741ac6c,eb3ad93ce6a44921b0490c842dd47d8c,9ce19042d497473c92d199a5e5a7daf0,1b3eb6ba64ea4589a8e2d42efa5d3f89,6b2dd5ff2a754a53a80798341a7d35d8,e53d754160f74d28b6d091829fc289f8,92d762bca040407bb4ce4b06a2cfa195,aa8a2b00622147259c85290f4d95b46d,0be3e487d874487fbfb5c49b52440a48,ce60b25f7da04af8b7cefccf2dbc7d36,6464ac9cc3fc47d88598f6c4e74caa19,04b312df403d4855b013743206ddb1de,17356c44ae894782a1015ec2ddaeaeda,a06eaefba7084e219f0c6ce4010b82e7,95a54e4db7e74f0586cd85cb4e2bd6f7,1a93047d6b0f4e3ca70fcf836b2517c4,93263a29941648979bb8de774978a880,8eba834a28864375aa3137590d26f37e,3c52b0a890524c6fb66cefc32114c60e,8b20519b794c42d28fdecbd0ccbba5c1,77189ceba655488681964a609c28256d,39e776aaf3644d24bb78ecf43704b71b,0dd5fa1192e34d5fac5e78ca38969126,1a6f82a80a054bc7ba4b516187231a54,5993ce4ec80a41fb9d4ea0713b74f4a4,3b42071536124a9c9dbf2ec319004b29,a966beefe58e4f95b24e89f4172cb7ce,2e1e75dbe28841529576dd1cd48d2437,cdd8829e5976438aaa66ce608f0175f5,71015695cf1445ecacce90669c67d733,31e9dd26b7314d8a9bf97dcf6dbdb22d,e5ecaa091bb0464f8553d4e7aa60647d,39055be3a23945868dbf4b5930043a88,07ba7c9aed104657aac3be87aa346d3b,5a7a9bfff0f44a2089a7d277f2c04557,be53cc152339400ba705c1d1dceff9ab,ed591ccba0564abd9979899049dc7b36,9a4425b817e94dbe844d7fd3e96a9033,bf4e54bfe9984441a8efa4dd8a4a23a6,5a1bcf2a2e0943f88bf655ca011fb288,a12c8904ff0147a99010eaa5aa0ea704,e7c6f2581c6e41bf8fe88a3d7fadc986,89709767beea48d0b0fb047316eb37d1,04d6a49b35514f91be43e9576f8d32fc,bd50e783ec714e5d86aafcca54e87c33,4acb0c7d002b47e29be7ebe9f40490f6,67b5ddef866b4982aa5afef7aad69924,7e090632f713440bba11ed6fc09eb291,f308051e11ca47eebb12709725b52eb4,dbea976ef1d248cabf13b88d311c4533,e0132620d4044e38a92f75b8335005ed,1b598563135e42d4bfe729cf32649f14,c4de9219f4f14684ae21d0904b7b78df,520983ab7dd64104a47e273941236b2d,133ba69bd04046b89eadaf793b9063db,11f24ca662044292a8ca95930ccc3581,0f30b4d621294d4fab359611cf82ab6f,b3c93fcd54b544e5b2c8e70ee7eb93ce,79ebe6595ce44208853ba553186b8348,8d639afccbec43f2be8d3ed88702893b,4e3483f3bbbb48f59b0e92a7d0ccc1c0,b160fccc15fa4ba09c74018d82d16eb7,52f248eadea74cf2b7780bf991d95e4e,d4cb9017aa1f4f8ca34ae8d31c986210,eff19819b1284424a9ad7879991ee080,79551897f86b4338bd09b41c5953a7db,f11bd4ef5ddd4c82865fad3f1f9d80a9,76ee95093a8748b29d55df1cb8cdebd0,081af742f6384a20b492ca99704f51ca,c26c844bbb2f4746904948f3ecfeb1ae,ba9bbc5581a043d8b9c204b8ddb6aa01,cde1afb048a743bb9caedb5770aabca1,0b2b37a28bb0457bbc4bf33c11b4c6e1,9f94d338d6c94b95814b2c50f9e463d1,7d1745ce2c444db9bf4b4e100c476fe8,e711e93aadd54eef8feeb0fa07f054d2,0e914774210646f784d02fb22f8df7db,ca0ce5e9eb09477c8c99b7b94e7869ce,760d9072f9bc480fa36fdb876930467c,5742a628f13045cda0317c3d4d839bc5,b9309305d94f4ee8b66eebbcae9f2db4,a42216d3eedd48f0ba3d3bef2470605f,9cdcda30da704c21b19ea0616de3ea86,f05ecbf7e4b04b268b38cb6e5c8124bf,788d2c1496874eeeae3b7aabbab95f76,8b112af90e054671adc30373c6dafd21,3bdb29667807493292a0ddc9bc0ccdb7,ffd6ebecc869410dab6410a07f3362ae,196868a4610b45588062aed357fc7c58,4cdd2d11be4c4c098b27c10d7cb3fa9e,fda40125207d426dbf86270002ff0c4a,7af028c6048c44cc91415a58d25d286e,ff7d2c8e96e840e1a5a690e4e0f96bad,9f806cf868dc48d398836ebd60bd2ec6,50ff5cdc6c8645b3bb61b8327d204f01,bafb57ea8d84430cbc0c11e5c67643f1,e2c91860b96c46619a74215627560235,f5b68a43bbae44c8a1fe821dfcaca26e,70b771772a0b4f1cb1e592f9a01b08a5,7992a39d64b2483789ae274e2df3c01d,f8d711312da14871a77b58f41a6f6242,673b27b0ab9148b7ae03933c18433971,624886a950bd4d18900a877738696432,685d8d9267f745f39bcc6fc2cc553d31,7d40ad05a1a54165a28ec897ddbcef01,4bcdb5199e8442078999bb8611ea6398,3b0c0ffea0cf45db816993baf64b3fb3,ad2d6d9581034e04addcb5b034a88fb3,8440c4fc5056426996e3e488b2261d28,e367c4223d794572a05fd256759d6a53,1b2dc5784c3c41988d5634e566b4a404,11be1538d7f54d7786004503b2e684a7,b462cb8c2ec7485d938416831a9771cc,f747656a36204e9588aba97064332436,78d0b2337b484c6298c9b3afc01fdb85,8a63d1f8d4bc4eb4a4db437491c9983a,2247f64a15e54053abb2e13152003c6b,d78ef9c5997d4e94804b8126cc387ea1,6ce3cd5a86ef43219dbda2b78e974c93,414b5fd5ebee45d98eca809a524dbe93,4b9ea9db5c834b698a78c7052c87b432,8590fa7adc144f49ae3d2575b7280c08,6148e4b152b9494fb9e7035e9b58e624,be1cff2a29f64800a5c4f91009711988,49b85b35bca5461993eac91ffd0b1fe4,5abae35db6de475e904425af8dc8ecbb,e3bc44b82c3543758d5a6c02b41dd526,89323021918e444bae3537521046d19f,141dd3257f85455d86f3828cc0217b5a,7bc2ad90009a4de2a0fa2947ccc09886,e51874f4b6ac455db76e30582659368b,519dee074e4c48ae9edd58d7c1e330da,060ab186c22f4f42a83f1622bf47ca52,8873bbcf4b42445f8117108769358a5c,46fa5286441a444095b917165c31304c,fa9fdd56222a4ce7ba2b37922afe0eaa,6885932aa4b443babc2b69fdd495889c,be71b947b33a48d8aee36ad0c60d10b8,b52457719429466687a2f228d20571ac,0bf937d745494d998cb8bd2cf3a63f4e,c17feb3860184dd59931f35814ad68b0,4cc68cbd880c48948a1f18fe18ec938c,c37b0691a05c4d659dcd01571f1907da,c1334fe3e8484079bfd62e479307b5b9,ebb58027b6c647b7965eeb31b63ec0c0,1821c5f9392e4859a829c78ebda44df6,dca21ab79bd14004b94c55e36604c30d,8d96cc3457644378b9b643243464f735,631437054079401fbe2c3c9a3f3bc375,c2f78ab4372243da8775f4d71a16fef5,03792664a45d4f63ac3efb899ce1b61e,fec04c46683a41e5a344f4d057342829,320fa70de6bc4890a0135bbfe8735b49,5a03493dce9f4323a0da7bf4fd5e78ab,b9bd46a1da1846d099566fd5c6db00cb,647021c7f798472889fd82f6c4a27ca2,313e905b78d746cb9e574b9634c3a113,623fd328c4cd4ee8b21eb03425b1f100,4676b4d1130146629fd98e2d2eb9a5b4,34d380813a36481ea2534c46d18b5927,894d87e55ac640b99e966a8786a2392b,477ba74d182a4ec0b86926dc67a0f419,362ca5fdca174fa3bdd6325a947320db,1ac3ad8dfa184fe4b6815950d3d23fdd,fdebee18c71f4a9db60096e75ad34d83,91432be4178a4781917fba0abd51df61,480c71baa65145f4bd037332d68e5e4f,536e3a3702b0453d92939846dc1f33b7,0333aedf1be640c78d2ee02473b7fd97,cbb3a6f1772b4f72a8af1375ef16e914,367ad9c43a6246dfac364ca59874883b,bac18430f6db466195a3a7116cdfb958,c4ec39be11be4b0eae321ee0559be8e6,8f7ffae7e47d462aa4142db7f06633d5,82e3b2423d86425195929fc3810ce7a4,72973e0415004dfea220babc479be082,41d0f568bb1e4fed8ad965a79d9bccef,5854ef1d24784ca0a046514b19286113,04afc3dd1eec4d13bc06cd01e0a5e838,db76e3333da846d5ae55b3b9dd735f26,7aa0ca0bf1514fa6a3a14dcde44cf0b9,9c668726c9b6449a9045d3f78408a2a1,7caf148d68a6431bbe56bcc6472d8720,babd36d1d3484fb99fe4e7ca78560ccf,7507865343e24cff80d55a29ec57f0e1,63f8b7d9b1f34851802e0408bf997ea5,a3369d5dd6b74af7b35edf1f10350881,9b8b8c3f86854416b82d57583896c1ff,881815c772564a8dbd5eb6de65ecbe7a,4b1bcf46753a48258d1d514440140b7f,1ca4c6a49e794954a235a82ee8981ecd,ab381d9129bb4b979fe3774e262524f8,a07d2c98831a4d79bc6c5c4288a24720,d8debd28039d494aa7fe18bc8e81af27,d0402b1f00f94acca96276c6ea8c8397,ebcdebd9639d4ec2b7072ad32bf48f50,636b3211100245919c09177ccea562e0,280c4440e18a487396a81b92cd3efa13,5163cf6740334a7d83375385c3ae2b67,a6083e4fab7a43498bfb3257658f9278,3b5c8b5cfb124ebd8f4a798d2a47d832,9fda332474534f96955b5bce8548947f,b5f5e62aba9842d8b48a28111ca6a372,c0cecff1207c4692a634d7de449d8d6d,e4c6428f1d264544b0eec52214e7b6ae,cbc8c5490a7f46b996fd4c3a9711f2d7,cfbdd82f4ce741a88109353c5fc898e4,6bfb965bc1b240a9ae2bd7a1e884ea51,eb36d63a993148289950af9098727759,f7357c3954d2408facaa6771a758ebcc,c05853b94ab8405eb26e0a8bd481b662,da1e6d3ebfa140b3a645342fd3bb5d95,988f3f6b607d442b8df006c980ddc2cd,b6af3c01968141baa3e99ca4b8d83847,75f669faee5d4023a425f2461e5fc5e9,165eb05677aa45fc9a53dfd8bfb7a45d,409891afd7cc4e12ba6b002581fef4f3,02ee0a87956f4bfab91b88282e502a3d,48601418028145ff84a450c877ea35aa,8de73f8d59eb45aa8c61138ab93c4f86,5dd82a46f48945b4bbb61f58b9d1c856,97993db997bd4ff88acfe1340ca86845,282aee84915b43179a09101b3ee6ad0e,8750e994985f43f383321bf0d6ca7932";

        role1.setMenuIds(defalutMenu1);
        String[] defalutMenus1 = defalutMenu1.split(",");
        ArrayList<String> menuIdList1 = new ArrayList<String>();
        for (int i = 0; i < defalutMenus1.length; i++) {
            menuIdList1.add(defalutMenus1[i]);
        }
        role1.setMenuIdList(menuIdList1);
        if (Global.isDemoMode()) {
            return null;
        }
        systemService.saveRole(role1,true,1);
        SysRoleActivity sysRoleActivity = new SysRoleActivity();
        if ("1".equals(type)) {
            String enname = UserUtils.getRoleActivity(com.getId()) + StringUtils.getPinYinHeadChar("出纳", 1);
            sysRoleActivity.setName("出纳");
            sysRoleActivity.setEnname(enname);
            sysRoleActivity.setSort("7200");
            sysRoleActivity.setType("3");
            sysRoleActivity.setDelType("1");
            sysRoleActivity.setCompanyId(com.getId());
            sysRoleActivityService.save(sysRoleActivity);

        }else {
            String enname = UserUtils.getRoleActivityEnname(com.getId(),UserUtils.getUser());
            sysRoleActivity = sysRoleActivityService.getByEnname(enname+"cn");
        }
        if (sysRoleActivity!=null) {
            List<SysRoleActivity> roleActivities = new ArrayList<>();
            roleActivities.add(sysRoleActivity);
            role1.setRoleActivityList(roleActivities);
            roleDao.insertRoleActivity(role1);
        }
        return role1;
    }
    @Transactional(readOnly = false)
    public Role createTBZYRole( Office com,Office office,Office deptSCB,String type) {
        //公司新用户
        Role role1 = new Role();
        role1.setNumber("4");
        role1.setName("投标专员");
        role1.setSysData(Global.YES);
        if (type.equals("1")){
            role1.setDataScope("2");
        }else {
            role1.setDataScope("5");
            role1.setBranchCompany(office);
        }
        role1.setOffice(deptSCB);
        role1.setIsNewRecord(false);
        role1.setCompany(com);
        role1.setRoleType("assignment");
        role1.setSysData("1");
        role1.setUseable("1");
        role1.setEnname("tbzy");

        String defalutMenu1 = "1,27,29,e3b80fae59ab4b4099d8349ecaf8c4dc,61dd3aa279684f92bb1c5d4019a32503,b0c5737cdc4f452684ce93589427c97d,af6b0e5d61fc44ecbebb449703a3df72,18c14562978e4b129fe3afb7dca8b868,6123fcb3d810492789e82a63b0d4310a,0aa3712414d049a6a24e8bcddeae509a,95a6a82dc5fc4d07b46e5df57a0606a3,62,b6fa4abb624349918fe531274fc15d23,89,76bdb735faa34667a6cbf13ebe937533,90,f5b2028cfad9470085c7c846de33193a,28abe4b0cb3b4c8480f84c07a5dfaf06,c7fa36c7142f481397c2cc12e2bc828a,e2efcf80fc3747329fe02d489dbfda95,e3b5b7451e164089aba0b9bf844b0398,438c28c3cbe44ed7bff951b2a3dc77cd,59e50eebe56743a78eecbc6eff1ef94c,cc1027689aca41a89302217127253034,200fe6cc80174065b382c961a4379d79,4c13ca96376341dfa25980e86964c1a9,638be8b3b5284e5088003917685bc19b,811600aadcb54b098b7dda4bf95ffc51,ce8dc7c97d93481dbd76cabb6e905b55,aa25b3904abb4bd7b779e5be37b4382e,aae6f4c7fd4c4a25afa5285210edfded,84ff9d5c165c4528af9361b6127cf5af,31b349a3624b47f28eb86d614f9f95b5,040b42e395de41d6805a958eb1897863,34874260e6fc4200a76b33c3cfa4d80d,5b6c6ea5ed2d45478e8144351d853e21,7c032c32512847f882a87416359ef0a3,6d30ea8c710a42b89da2f2506059b0ef,2ca7bf4ce6804a1c9e5b331d2df27c72,7717cbb6d71f405d890e9b648cbd2088,bddb0400764c42b08e427059c7752057,23d6c56239804053910e5c5aec481330,7f8b2018113545bcbdbfc87e7a236bec,547e121ffb714d16ae142b6935abb78f,af24101b962444f29d166b638b2fea30,6ea6519ec4d548128aa42f3ce004def3,4379bf54c2a543869abf5031b97b8efe,5e6da3f5368b4f06b7268b31a1157e9f,2bbf0e75f21d42938954391d0776fbde,545b39dacf564a5eb9d1658f80a0b8b0,b4ddedf9b451494a97a1ff9476e3897d,1b145ca23d3e4d81994a84d30a262d18,10a8c7c1d34e46dd8ef461cdad928afc,5ae4911d14bb48b39885b1fe522e999d,7f09def2533b45fdb3c733f1e72ca5fb,9ef29dca19d342b795efcbfe32793602,be72d72ae7034c509c164467b2abddda,ab879705f2354813acf8724404520e28,82487b87b97b475b98892b80cf83c96a,0f4a461b97be48eeab43bd052f630057,72e1a28b7251438a89b0a4f6bd522b89,e1f9a246057347d78a1966d6dc0168e3,5d3645d53ce14c9ab498fd9f050ba264,d361e6890cc84b049fc8268c2a8c4952,5cc0002cbcc14d3485ab8ae6d57368f2,fa7bb243643c44b18e989e3419eeb171,62768b6c65134f1e927b1807bc72115a,7b69b0e5d2d344e6aaa2b8726c23561a,434bbb1f9f8140b1beeb57d0c4db660d,7920c4c8da294d87bcb94e7756bf35ae,0752b85660694033bbdb013f30cde17f,bd6258cfbea64896b0401517e440d6a2,9aa3539682514546866d2998d741ac6c,eb3ad93ce6a44921b0490c842dd47d8c,9ce19042d497473c92d199a5e5a7daf0,1b3eb6ba64ea4589a8e2d42efa5d3f89,6b2dd5ff2a754a53a80798341a7d35d8,e53d754160f74d28b6d091829fc289f8,92d762bca040407bb4ce4b06a2cfa195,aa8a2b00622147259c85290f4d95b46d,0be3e487d874487fbfb5c49b52440a48,ce60b25f7da04af8b7cefccf2dbc7d36,6464ac9cc3fc47d88598f6c4e74caa19,04b312df403d4855b013743206ddb1de,17356c44ae894782a1015ec2ddaeaeda,a06eaefba7084e219f0c6ce4010b82e7,95a54e4db7e74f0586cd85cb4e2bd6f7,1a93047d6b0f4e3ca70fcf836b2517c4,93263a29941648979bb8de774978a880,8eba834a28864375aa3137590d26f37e,3c52b0a890524c6fb66cefc32114c60e,8b20519b794c42d28fdecbd0ccbba5c1,77189ceba655488681964a609c28256d,39e776aaf3644d24bb78ecf43704b71b,0dd5fa1192e34d5fac5e78ca38969126,1a6f82a80a054bc7ba4b516187231a54,5993ce4ec80a41fb9d4ea0713b74f4a4,3b42071536124a9c9dbf2ec319004b29,a966beefe58e4f95b24e89f4172cb7ce,2e1e75dbe28841529576dd1cd48d2437,cdd8829e5976438aaa66ce608f0175f5,71015695cf1445ecacce90669c67d733,31e9dd26b7314d8a9bf97dcf6dbdb22d,e5ecaa091bb0464f8553d4e7aa60647d,39055be3a23945868dbf4b5930043a88,07ba7c9aed104657aac3be87aa346d3b,5a7a9bfff0f44a2089a7d277f2c04557,be53cc152339400ba705c1d1dceff9ab,ed591ccba0564abd9979899049dc7b36,9a4425b817e94dbe844d7fd3e96a9033,bf4e54bfe9984441a8efa4dd8a4a23a6,5a1bcf2a2e0943f88bf655ca011fb288,a12c8904ff0147a99010eaa5aa0ea704,e7c6f2581c6e41bf8fe88a3d7fadc986,89709767beea48d0b0fb047316eb37d1,04d6a49b35514f91be43e9576f8d32fc,bd50e783ec714e5d86aafcca54e87c33,4acb0c7d002b47e29be7ebe9f40490f6,67b5ddef866b4982aa5afef7aad69924,7e090632f713440bba11ed6fc09eb291,f308051e11ca47eebb12709725b52eb4,dbea976ef1d248cabf13b88d311c4533,e0132620d4044e38a92f75b8335005ed,1b598563135e42d4bfe729cf32649f14,c4de9219f4f14684ae21d0904b7b78df,520983ab7dd64104a47e273941236b2d,133ba69bd04046b89eadaf793b9063db,11f24ca662044292a8ca95930ccc3581,0f30b4d621294d4fab359611cf82ab6f,b3c93fcd54b544e5b2c8e70ee7eb93ce,79ebe6595ce44208853ba553186b8348,8d639afccbec43f2be8d3ed88702893b,4e3483f3bbbb48f59b0e92a7d0ccc1c0,b160fccc15fa4ba09c74018d82d16eb7,52f248eadea74cf2b7780bf991d95e4e,d4cb9017aa1f4f8ca34ae8d31c986210,eff19819b1284424a9ad7879991ee080,79551897f86b4338bd09b41c5953a7db,f11bd4ef5ddd4c82865fad3f1f9d80a9,76ee95093a8748b29d55df1cb8cdebd0,081af742f6384a20b492ca99704f51ca,c26c844bbb2f4746904948f3ecfeb1ae,ba9bbc5581a043d8b9c204b8ddb6aa01,cde1afb048a743bb9caedb5770aabca1,0b2b37a28bb0457bbc4bf33c11b4c6e1,9f94d338d6c94b95814b2c50f9e463d1,7d1745ce2c444db9bf4b4e100c476fe8,e711e93aadd54eef8feeb0fa07f054d2,0e914774210646f784d02fb22f8df7db,ca0ce5e9eb09477c8c99b7b94e7869ce,760d9072f9bc480fa36fdb876930467c,5742a628f13045cda0317c3d4d839bc5,b9309305d94f4ee8b66eebbcae9f2db4,a42216d3eedd48f0ba3d3bef2470605f,9cdcda30da704c21b19ea0616de3ea86,f05ecbf7e4b04b268b38cb6e5c8124bf,788d2c1496874eeeae3b7aabbab95f76,8b112af90e054671adc30373c6dafd21,3bdb29667807493292a0ddc9bc0ccdb7,ffd6ebecc869410dab6410a07f3362ae,196868a4610b45588062aed357fc7c58,4cdd2d11be4c4c098b27c10d7cb3fa9e,fda40125207d426dbf86270002ff0c4a,7af028c6048c44cc91415a58d25d286e,ff7d2c8e96e840e1a5a690e4e0f96bad,9f806cf868dc48d398836ebd60bd2ec6,50ff5cdc6c8645b3bb61b8327d204f01,bafb57ea8d84430cbc0c11e5c67643f1,e2c91860b96c46619a74215627560235,f5b68a43bbae44c8a1fe821dfcaca26e,70b771772a0b4f1cb1e592f9a01b08a5,7992a39d64b2483789ae274e2df3c01d,f8d711312da14871a77b58f41a6f6242,673b27b0ab9148b7ae03933c18433971,624886a950bd4d18900a877738696432,685d8d9267f745f39bcc6fc2cc553d31,7d40ad05a1a54165a28ec897ddbcef01,4bcdb5199e8442078999bb8611ea6398,3b0c0ffea0cf45db816993baf64b3fb3,ad2d6d9581034e04addcb5b034a88fb3,8440c4fc5056426996e3e488b2261d28,e367c4223d794572a05fd256759d6a53,1b2dc5784c3c41988d5634e566b4a404,11be1538d7f54d7786004503b2e684a7,b462cb8c2ec7485d938416831a9771cc,f747656a36204e9588aba97064332436,78d0b2337b484c6298c9b3afc01fdb85,8a63d1f8d4bc4eb4a4db437491c9983a,2247f64a15e54053abb2e13152003c6b,d78ef9c5997d4e94804b8126cc387ea1,6ce3cd5a86ef43219dbda2b78e974c93,414b5fd5ebee45d98eca809a524dbe93,4b9ea9db5c834b698a78c7052c87b432,8590fa7adc144f49ae3d2575b7280c08,6148e4b152b9494fb9e7035e9b58e624,be1cff2a29f64800a5c4f91009711988,49b85b35bca5461993eac91ffd0b1fe4,5abae35db6de475e904425af8dc8ecbb,e3bc44b82c3543758d5a6c02b41dd526,89323021918e444bae3537521046d19f,141dd3257f85455d86f3828cc0217b5a,7bc2ad90009a4de2a0fa2947ccc09886,e51874f4b6ac455db76e30582659368b,519dee074e4c48ae9edd58d7c1e330da,060ab186c22f4f42a83f1622bf47ca52,8873bbcf4b42445f8117108769358a5c,46fa5286441a444095b917165c31304c,fa9fdd56222a4ce7ba2b37922afe0eaa,6885932aa4b443babc2b69fdd495889c,be71b947b33a48d8aee36ad0c60d10b8,b52457719429466687a2f228d20571ac,0bf937d745494d998cb8bd2cf3a63f4e,c17feb3860184dd59931f35814ad68b0,4cc68cbd880c48948a1f18fe18ec938c,c37b0691a05c4d659dcd01571f1907da,c1334fe3e8484079bfd62e479307b5b9,ebb58027b6c647b7965eeb31b63ec0c0,1821c5f9392e4859a829c78ebda44df6,dca21ab79bd14004b94c55e36604c30d,8d96cc3457644378b9b643243464f735,631437054079401fbe2c3c9a3f3bc375,c2f78ab4372243da8775f4d71a16fef5,03792664a45d4f63ac3efb899ce1b61e,fec04c46683a41e5a344f4d057342829,320fa70de6bc4890a0135bbfe8735b49,5a03493dce9f4323a0da7bf4fd5e78ab,b9bd46a1da1846d099566fd5c6db00cb,647021c7f798472889fd82f6c4a27ca2,313e905b78d746cb9e574b9634c3a113,623fd328c4cd4ee8b21eb03425b1f100,4676b4d1130146629fd98e2d2eb9a5b4,34d380813a36481ea2534c46d18b5927,894d87e55ac640b99e966a8786a2392b,477ba74d182a4ec0b86926dc67a0f419,362ca5fdca174fa3bdd6325a947320db,1ac3ad8dfa184fe4b6815950d3d23fdd,fdebee18c71f4a9db60096e75ad34d83,91432be4178a4781917fba0abd51df61,480c71baa65145f4bd037332d68e5e4f,536e3a3702b0453d92939846dc1f33b7,0333aedf1be640c78d2ee02473b7fd97,cbb3a6f1772b4f72a8af1375ef16e914,367ad9c43a6246dfac364ca59874883b,bac18430f6db466195a3a7116cdfb958,c4ec39be11be4b0eae321ee0559be8e6,8f7ffae7e47d462aa4142db7f06633d5,82e3b2423d86425195929fc3810ce7a4,72973e0415004dfea220babc479be082,41d0f568bb1e4fed8ad965a79d9bccef,5854ef1d24784ca0a046514b19286113,04afc3dd1eec4d13bc06cd01e0a5e838,db76e3333da846d5ae55b3b9dd735f26,7aa0ca0bf1514fa6a3a14dcde44cf0b9,9c668726c9b6449a9045d3f78408a2a1,7caf148d68a6431bbe56bcc6472d8720,babd36d1d3484fb99fe4e7ca78560ccf,7507865343e24cff80d55a29ec57f0e1,63f8b7d9b1f34851802e0408bf997ea5,a3369d5dd6b74af7b35edf1f10350881,9b8b8c3f86854416b82d57583896c1ff,881815c772564a8dbd5eb6de65ecbe7a,4b1bcf46753a48258d1d514440140b7f,1ca4c6a49e794954a235a82ee8981ecd,ab381d9129bb4b979fe3774e262524f8,a07d2c98831a4d79bc6c5c4288a24720,d8debd28039d494aa7fe18bc8e81af27,d0402b1f00f94acca96276c6ea8c8397,ebcdebd9639d4ec2b7072ad32bf48f50,636b3211100245919c09177ccea562e0,280c4440e18a487396a81b92cd3efa13,5163cf6740334a7d83375385c3ae2b67,a6083e4fab7a43498bfb3257658f9278,3b5c8b5cfb124ebd8f4a798d2a47d832,9fda332474534f96955b5bce8548947f,b5f5e62aba9842d8b48a28111ca6a372,c0cecff1207c4692a634d7de449d8d6d,e4c6428f1d264544b0eec52214e7b6ae,cbc8c5490a7f46b996fd4c3a9711f2d7,cfbdd82f4ce741a88109353c5fc898e4,6bfb965bc1b240a9ae2bd7a1e884ea51,eb36d63a993148289950af9098727759,f7357c3954d2408facaa6771a758ebcc,c05853b94ab8405eb26e0a8bd481b662,da1e6d3ebfa140b3a645342fd3bb5d95,988f3f6b607d442b8df006c980ddc2cd,b6af3c01968141baa3e99ca4b8d83847,75f669faee5d4023a425f2461e5fc5e9,165eb05677aa45fc9a53dfd8bfb7a45d,409891afd7cc4e12ba6b002581fef4f3,02ee0a87956f4bfab91b88282e502a3d,48601418028145ff84a450c877ea35aa,8de73f8d59eb45aa8c61138ab93c4f86,5dd82a46f48945b4bbb61f58b9d1c856,97993db997bd4ff88acfe1340ca86845,282aee84915b43179a09101b3ee6ad0e,8750e994985f43f383321bf0d6ca7932";

        role1.setMenuIds(defalutMenu1);
        String[] defalutMenus1 = defalutMenu1.split(",");
        ArrayList<String> menuIdList1 = new ArrayList<String>();
        for (int i = 0; i < defalutMenus1.length; i++) {
            menuIdList1.add(defalutMenus1[i]);
        }
        role1.setMenuIdList(menuIdList1);
        if (Global.isDemoMode()) {
            return null;
        }
        systemService.saveRole(role1,true,1);
        SysRoleActivity sysRoleActivity = new SysRoleActivity();
        if ("1".equals(type)) {
            String enname = UserUtils.getRoleActivity(com.getId()) + StringUtils.getPinYinHeadChar("投标专员", 1);
            sysRoleActivity.setName("投标专员");
            sysRoleActivity.setEnname(enname);
            sysRoleActivity.setSort("7300");
            sysRoleActivity.setType("3");
            sysRoleActivity.setDelType("1");
            sysRoleActivity.setCompanyId(com.getId());
            sysRoleActivityService.save(sysRoleActivity);

        }else {
            String enname = UserUtils.getRoleActivityEnname(com.getId(),UserUtils.getUser());
            sysRoleActivity = sysRoleActivityService.getByEnname(enname+"tbzy");
        }
        if (sysRoleActivity!=null) {
            List<SysRoleActivity> roleActivities = new ArrayList<>();
            roleActivities.add(sysRoleActivity);
            role1.setRoleActivityList(roleActivities);
            roleDao.insertRoleActivity(role1);
        }
        return role1;
    }
    @Transactional(readOnly = false)
    public Role createCompanyRLZYRole( Office com,Office office,Office deptOffice,String type) {
        //公司新用户
        Role role1 = new Role();
        role1.setNumber("4");
        role1.setName("人力专员");
        role1.setSysData(Global.YES);
        if (type.equals("1")){
            role1.setDataScope("2");
        }else {
            role1.setDataScope("5");
            role1.setBranchCompany(office);
        }
        role1.setOffice(deptOffice);
        role1.setIsNewRecord(false);
        role1.setCompany(com);
        role1.setRoleType("assignment");
        role1.setSysData("1");
        role1.setUseable("1");
        role1.setEnname("rlzy");

        String defalutMenu1 = "1,27,29,e3b80fae59ab4b4099d8349ecaf8c4dc,61dd3aa279684f92bb1c5d4019a32503,b0c5737cdc4f452684ce93589427c97d,af6b0e5d61fc44ecbebb449703a3df72,18c14562978e4b129fe3afb7dca8b868,6123fcb3d810492789e82a63b0d4310a,0aa3712414d049a6a24e8bcddeae509a,95a6a82dc5fc4d07b46e5df57a0606a3,62,b6fa4abb624349918fe531274fc15d23,89,76bdb735faa34667a6cbf13ebe937533,90,f5b2028cfad9470085c7c846de33193a,28abe4b0cb3b4c8480f84c07a5dfaf06,c7fa36c7142f481397c2cc12e2bc828a,e2efcf80fc3747329fe02d489dbfda95,e3b5b7451e164089aba0b9bf844b0398,438c28c3cbe44ed7bff951b2a3dc77cd,59e50eebe56743a78eecbc6eff1ef94c,cc1027689aca41a89302217127253034,200fe6cc80174065b382c961a4379d79,4c13ca96376341dfa25980e86964c1a9,638be8b3b5284e5088003917685bc19b,811600aadcb54b098b7dda4bf95ffc51,ce8dc7c97d93481dbd76cabb6e905b55,aa25b3904abb4bd7b779e5be37b4382e,aae6f4c7fd4c4a25afa5285210edfded,84ff9d5c165c4528af9361b6127cf5af,31b349a3624b47f28eb86d614f9f95b5,040b42e395de41d6805a958eb1897863,34874260e6fc4200a76b33c3cfa4d80d,5b6c6ea5ed2d45478e8144351d853e21,7c032c32512847f882a87416359ef0a3,6d30ea8c710a42b89da2f2506059b0ef,2ca7bf4ce6804a1c9e5b331d2df27c72,7717cbb6d71f405d890e9b648cbd2088,bddb0400764c42b08e427059c7752057,23d6c56239804053910e5c5aec481330,7f8b2018113545bcbdbfc87e7a236bec,547e121ffb714d16ae142b6935abb78f,af24101b962444f29d166b638b2fea30,6ea6519ec4d548128aa42f3ce004def3,4379bf54c2a543869abf5031b97b8efe,5e6da3f5368b4f06b7268b31a1157e9f,2bbf0e75f21d42938954391d0776fbde,545b39dacf564a5eb9d1658f80a0b8b0,b4ddedf9b451494a97a1ff9476e3897d,1b145ca23d3e4d81994a84d30a262d18,10a8c7c1d34e46dd8ef461cdad928afc,5ae4911d14bb48b39885b1fe522e999d,7f09def2533b45fdb3c733f1e72ca5fb,9ef29dca19d342b795efcbfe32793602,be72d72ae7034c509c164467b2abddda,ab879705f2354813acf8724404520e28,82487b87b97b475b98892b80cf83c96a,0f4a461b97be48eeab43bd052f630057,72e1a28b7251438a89b0a4f6bd522b89,e1f9a246057347d78a1966d6dc0168e3,5d3645d53ce14c9ab498fd9f050ba264,d361e6890cc84b049fc8268c2a8c4952,5cc0002cbcc14d3485ab8ae6d57368f2,fa7bb243643c44b18e989e3419eeb171,62768b6c65134f1e927b1807bc72115a,7b69b0e5d2d344e6aaa2b8726c23561a,434bbb1f9f8140b1beeb57d0c4db660d,7920c4c8da294d87bcb94e7756bf35ae,0752b85660694033bbdb013f30cde17f,bd6258cfbea64896b0401517e440d6a2,9aa3539682514546866d2998d741ac6c,eb3ad93ce6a44921b0490c842dd47d8c,9ce19042d497473c92d199a5e5a7daf0,1b3eb6ba64ea4589a8e2d42efa5d3f89,6b2dd5ff2a754a53a80798341a7d35d8,e53d754160f74d28b6d091829fc289f8,92d762bca040407bb4ce4b06a2cfa195,aa8a2b00622147259c85290f4d95b46d,0be3e487d874487fbfb5c49b52440a48,ce60b25f7da04af8b7cefccf2dbc7d36,6464ac9cc3fc47d88598f6c4e74caa19,04b312df403d4855b013743206ddb1de,17356c44ae894782a1015ec2ddaeaeda,a06eaefba7084e219f0c6ce4010b82e7,95a54e4db7e74f0586cd85cb4e2bd6f7,1a93047d6b0f4e3ca70fcf836b2517c4,93263a29941648979bb8de774978a880,8eba834a28864375aa3137590d26f37e,3c52b0a890524c6fb66cefc32114c60e,8b20519b794c42d28fdecbd0ccbba5c1,77189ceba655488681964a609c28256d,39e776aaf3644d24bb78ecf43704b71b,0dd5fa1192e34d5fac5e78ca38969126,1a6f82a80a054bc7ba4b516187231a54,5993ce4ec80a41fb9d4ea0713b74f4a4,3b42071536124a9c9dbf2ec319004b29,a966beefe58e4f95b24e89f4172cb7ce,2e1e75dbe28841529576dd1cd48d2437,cdd8829e5976438aaa66ce608f0175f5,71015695cf1445ecacce90669c67d733,31e9dd26b7314d8a9bf97dcf6dbdb22d,e5ecaa091bb0464f8553d4e7aa60647d,39055be3a23945868dbf4b5930043a88,07ba7c9aed104657aac3be87aa346d3b,5a7a9bfff0f44a2089a7d277f2c04557,be53cc152339400ba705c1d1dceff9ab,ed591ccba0564abd9979899049dc7b36,9a4425b817e94dbe844d7fd3e96a9033,bf4e54bfe9984441a8efa4dd8a4a23a6,5a1bcf2a2e0943f88bf655ca011fb288,a12c8904ff0147a99010eaa5aa0ea704,e7c6f2581c6e41bf8fe88a3d7fadc986,89709767beea48d0b0fb047316eb37d1,04d6a49b35514f91be43e9576f8d32fc,bd50e783ec714e5d86aafcca54e87c33,4acb0c7d002b47e29be7ebe9f40490f6,67b5ddef866b4982aa5afef7aad69924,7e090632f713440bba11ed6fc09eb291,f308051e11ca47eebb12709725b52eb4,dbea976ef1d248cabf13b88d311c4533,e0132620d4044e38a92f75b8335005ed,1b598563135e42d4bfe729cf32649f14,c4de9219f4f14684ae21d0904b7b78df,520983ab7dd64104a47e273941236b2d,133ba69bd04046b89eadaf793b9063db,11f24ca662044292a8ca95930ccc3581,0f30b4d621294d4fab359611cf82ab6f,b3c93fcd54b544e5b2c8e70ee7eb93ce,79ebe6595ce44208853ba553186b8348,8d639afccbec43f2be8d3ed88702893b,4e3483f3bbbb48f59b0e92a7d0ccc1c0,b160fccc15fa4ba09c74018d82d16eb7,52f248eadea74cf2b7780bf991d95e4e,d4cb9017aa1f4f8ca34ae8d31c986210,eff19819b1284424a9ad7879991ee080,79551897f86b4338bd09b41c5953a7db,f11bd4ef5ddd4c82865fad3f1f9d80a9,76ee95093a8748b29d55df1cb8cdebd0,081af742f6384a20b492ca99704f51ca,c26c844bbb2f4746904948f3ecfeb1ae,ba9bbc5581a043d8b9c204b8ddb6aa01,cde1afb048a743bb9caedb5770aabca1,0b2b37a28bb0457bbc4bf33c11b4c6e1,9f94d338d6c94b95814b2c50f9e463d1,7d1745ce2c444db9bf4b4e100c476fe8,e711e93aadd54eef8feeb0fa07f054d2,0e914774210646f784d02fb22f8df7db,ca0ce5e9eb09477c8c99b7b94e7869ce,760d9072f9bc480fa36fdb876930467c,5742a628f13045cda0317c3d4d839bc5,b9309305d94f4ee8b66eebbcae9f2db4,a42216d3eedd48f0ba3d3bef2470605f,9cdcda30da704c21b19ea0616de3ea86,f05ecbf7e4b04b268b38cb6e5c8124bf,788d2c1496874eeeae3b7aabbab95f76,8b112af90e054671adc30373c6dafd21,3bdb29667807493292a0ddc9bc0ccdb7,ffd6ebecc869410dab6410a07f3362ae,196868a4610b45588062aed357fc7c58,4cdd2d11be4c4c098b27c10d7cb3fa9e,fda40125207d426dbf86270002ff0c4a,7af028c6048c44cc91415a58d25d286e,ff7d2c8e96e840e1a5a690e4e0f96bad,9f806cf868dc48d398836ebd60bd2ec6,50ff5cdc6c8645b3bb61b8327d204f01,bafb57ea8d84430cbc0c11e5c67643f1,e2c91860b96c46619a74215627560235,f5b68a43bbae44c8a1fe821dfcaca26e,70b771772a0b4f1cb1e592f9a01b08a5,7992a39d64b2483789ae274e2df3c01d,f8d711312da14871a77b58f41a6f6242,673b27b0ab9148b7ae03933c18433971,624886a950bd4d18900a877738696432,685d8d9267f745f39bcc6fc2cc553d31,7d40ad05a1a54165a28ec897ddbcef01,4bcdb5199e8442078999bb8611ea6398,3b0c0ffea0cf45db816993baf64b3fb3,ad2d6d9581034e04addcb5b034a88fb3,8440c4fc5056426996e3e488b2261d28,e367c4223d794572a05fd256759d6a53,1b2dc5784c3c41988d5634e566b4a404,11be1538d7f54d7786004503b2e684a7,b462cb8c2ec7485d938416831a9771cc,f747656a36204e9588aba97064332436,78d0b2337b484c6298c9b3afc01fdb85,8a63d1f8d4bc4eb4a4db437491c9983a,2247f64a15e54053abb2e13152003c6b,d78ef9c5997d4e94804b8126cc387ea1,6ce3cd5a86ef43219dbda2b78e974c93,414b5fd5ebee45d98eca809a524dbe93,4b9ea9db5c834b698a78c7052c87b432,8590fa7adc144f49ae3d2575b7280c08,6148e4b152b9494fb9e7035e9b58e624,be1cff2a29f64800a5c4f91009711988,49b85b35bca5461993eac91ffd0b1fe4,5abae35db6de475e904425af8dc8ecbb,e3bc44b82c3543758d5a6c02b41dd526,89323021918e444bae3537521046d19f,141dd3257f85455d86f3828cc0217b5a,7bc2ad90009a4de2a0fa2947ccc09886,e51874f4b6ac455db76e30582659368b,519dee074e4c48ae9edd58d7c1e330da,060ab186c22f4f42a83f1622bf47ca52,8873bbcf4b42445f8117108769358a5c,46fa5286441a444095b917165c31304c,fa9fdd56222a4ce7ba2b37922afe0eaa,6885932aa4b443babc2b69fdd495889c,be71b947b33a48d8aee36ad0c60d10b8,b52457719429466687a2f228d20571ac,0bf937d745494d998cb8bd2cf3a63f4e,c17feb3860184dd59931f35814ad68b0,4cc68cbd880c48948a1f18fe18ec938c,c37b0691a05c4d659dcd01571f1907da,c1334fe3e8484079bfd62e479307b5b9,ebb58027b6c647b7965eeb31b63ec0c0,1821c5f9392e4859a829c78ebda44df6,dca21ab79bd14004b94c55e36604c30d,8d96cc3457644378b9b643243464f735,631437054079401fbe2c3c9a3f3bc375,c2f78ab4372243da8775f4d71a16fef5,03792664a45d4f63ac3efb899ce1b61e,fec04c46683a41e5a344f4d057342829,320fa70de6bc4890a0135bbfe8735b49,5a03493dce9f4323a0da7bf4fd5e78ab,b9bd46a1da1846d099566fd5c6db00cb,647021c7f798472889fd82f6c4a27ca2,313e905b78d746cb9e574b9634c3a113,623fd328c4cd4ee8b21eb03425b1f100,4676b4d1130146629fd98e2d2eb9a5b4,34d380813a36481ea2534c46d18b5927,894d87e55ac640b99e966a8786a2392b,477ba74d182a4ec0b86926dc67a0f419,362ca5fdca174fa3bdd6325a947320db,1ac3ad8dfa184fe4b6815950d3d23fdd,fdebee18c71f4a9db60096e75ad34d83,91432be4178a4781917fba0abd51df61,480c71baa65145f4bd037332d68e5e4f,536e3a3702b0453d92939846dc1f33b7,0333aedf1be640c78d2ee02473b7fd97,cbb3a6f1772b4f72a8af1375ef16e914,367ad9c43a6246dfac364ca59874883b,bac18430f6db466195a3a7116cdfb958,c4ec39be11be4b0eae321ee0559be8e6,8f7ffae7e47d462aa4142db7f06633d5,82e3b2423d86425195929fc3810ce7a4,72973e0415004dfea220babc479be082,41d0f568bb1e4fed8ad965a79d9bccef,5854ef1d24784ca0a046514b19286113,04afc3dd1eec4d13bc06cd01e0a5e838,db76e3333da846d5ae55b3b9dd735f26,7aa0ca0bf1514fa6a3a14dcde44cf0b9,9c668726c9b6449a9045d3f78408a2a1,7caf148d68a6431bbe56bcc6472d8720,babd36d1d3484fb99fe4e7ca78560ccf,7507865343e24cff80d55a29ec57f0e1,63f8b7d9b1f34851802e0408bf997ea5,a3369d5dd6b74af7b35edf1f10350881,9b8b8c3f86854416b82d57583896c1ff,881815c772564a8dbd5eb6de65ecbe7a,4b1bcf46753a48258d1d514440140b7f,1ca4c6a49e794954a235a82ee8981ecd,ab381d9129bb4b979fe3774e262524f8,a07d2c98831a4d79bc6c5c4288a24720,d8debd28039d494aa7fe18bc8e81af27,d0402b1f00f94acca96276c6ea8c8397,ebcdebd9639d4ec2b7072ad32bf48f50,636b3211100245919c09177ccea562e0,280c4440e18a487396a81b92cd3efa13,5163cf6740334a7d83375385c3ae2b67,a6083e4fab7a43498bfb3257658f9278,3b5c8b5cfb124ebd8f4a798d2a47d832,9fda332474534f96955b5bce8548947f,b5f5e62aba9842d8b48a28111ca6a372,c0cecff1207c4692a634d7de449d8d6d,e4c6428f1d264544b0eec52214e7b6ae,cbc8c5490a7f46b996fd4c3a9711f2d7,cfbdd82f4ce741a88109353c5fc898e4,6bfb965bc1b240a9ae2bd7a1e884ea51,eb36d63a993148289950af9098727759,f7357c3954d2408facaa6771a758ebcc,c05853b94ab8405eb26e0a8bd481b662,da1e6d3ebfa140b3a645342fd3bb5d95,988f3f6b607d442b8df006c980ddc2cd,b6af3c01968141baa3e99ca4b8d83847,75f669faee5d4023a425f2461e5fc5e9,165eb05677aa45fc9a53dfd8bfb7a45d,409891afd7cc4e12ba6b002581fef4f3,02ee0a87956f4bfab91b88282e502a3d,48601418028145ff84a450c877ea35aa,8de73f8d59eb45aa8c61138ab93c4f86,5dd82a46f48945b4bbb61f58b9d1c856,97993db997bd4ff88acfe1340ca86845,282aee84915b43179a09101b3ee6ad0e,8750e994985f43f383321bf0d6ca7932";

        role1.setMenuIds(defalutMenu1);
        String[] defalutMenus1 = defalutMenu1.split(",");
        ArrayList<String> menuIdList1 = new ArrayList<String>();
        for (int i = 0; i < defalutMenus1.length; i++) {
            menuIdList1.add(defalutMenus1[i]);
        }
        role1.setMenuIdList(menuIdList1);
        if (Global.isDemoMode()) {
            return null;
        }
        systemService.saveRole(role1,true,1);
        SysRoleActivity sysRoleActivity = new SysRoleActivity();
        if ("1".equals(type)) {
            String enname = UserUtils.getRoleActivity(com.getId()) + StringUtils.getPinYinHeadChar("人力专员", 1);
            sysRoleActivity.setName("人力专员");
            sysRoleActivity.setEnname(enname);
            sysRoleActivity.setSort("7300");
            sysRoleActivity.setType("3");
            sysRoleActivity.setDelType("1");
            sysRoleActivity.setCompanyId(com.getId());
            sysRoleActivityService.save(sysRoleActivity);

        }else {
            String enname = UserUtils.getRoleActivityEnname(com.getId(),UserUtils.getUser());
            sysRoleActivity = sysRoleActivityService.getByEnname(enname+"rlzy");
        }
        if (sysRoleActivity!=null) {
            List<SysRoleActivity> roleActivities = new ArrayList<>();
            roleActivities.add(sysRoleActivity);
            role1.setRoleActivityList(roleActivities);
            roleDao.insertRoleActivity(role1);
        }
        return role1;
    }
    @Transactional(readOnly = false)
    public Role createCompanyALTBZYRole( Office com,Office office,Office deptOffice,String type) {
        //公司新用户
        Role role1 = new Role();
        role1.setNumber("4");
        role1.setName("案例投标专员");
        role1.setSysData(Global.YES);
        if (type.equals("1")){
            role1.setDataScope("2");
        }else {
            role1.setDataScope("5");
            role1.setBranchCompany(office);
        }
        role1.setOffice(deptOffice);
        role1.setIsNewRecord(false);
        role1.setCompany(com);
        role1.setRoleType("assignment");
        role1.setSysData("1");
        role1.setUseable("1");
        role1.setEnname("altbzy");

        String defalutMenu1 = "1,27,29,e3b80fae59ab4b4099d8349ecaf8c4dc,61dd3aa279684f92bb1c5d4019a32503,b0c5737cdc4f452684ce93589427c97d,af6b0e5d61fc44ecbebb449703a3df72,18c14562978e4b129fe3afb7dca8b868,6123fcb3d810492789e82a63b0d4310a,0aa3712414d049a6a24e8bcddeae509a,95a6a82dc5fc4d07b46e5df57a0606a3,62,b6fa4abb624349918fe531274fc15d23,89,76bdb735faa34667a6cbf13ebe937533,90,f5b2028cfad9470085c7c846de33193a,28abe4b0cb3b4c8480f84c07a5dfaf06,c7fa36c7142f481397c2cc12e2bc828a,e2efcf80fc3747329fe02d489dbfda95,e3b5b7451e164089aba0b9bf844b0398,438c28c3cbe44ed7bff951b2a3dc77cd,59e50eebe56743a78eecbc6eff1ef94c,cc1027689aca41a89302217127253034,200fe6cc80174065b382c961a4379d79,4c13ca96376341dfa25980e86964c1a9,638be8b3b5284e5088003917685bc19b,811600aadcb54b098b7dda4bf95ffc51,ce8dc7c97d93481dbd76cabb6e905b55,aa25b3904abb4bd7b779e5be37b4382e,aae6f4c7fd4c4a25afa5285210edfded,84ff9d5c165c4528af9361b6127cf5af,31b349a3624b47f28eb86d614f9f95b5,040b42e395de41d6805a958eb1897863,34874260e6fc4200a76b33c3cfa4d80d,5b6c6ea5ed2d45478e8144351d853e21,7c032c32512847f882a87416359ef0a3,6d30ea8c710a42b89da2f2506059b0ef,2ca7bf4ce6804a1c9e5b331d2df27c72,7717cbb6d71f405d890e9b648cbd2088,bddb0400764c42b08e427059c7752057,23d6c56239804053910e5c5aec481330,7f8b2018113545bcbdbfc87e7a236bec,547e121ffb714d16ae142b6935abb78f,af24101b962444f29d166b638b2fea30,6ea6519ec4d548128aa42f3ce004def3,4379bf54c2a543869abf5031b97b8efe,5e6da3f5368b4f06b7268b31a1157e9f,2bbf0e75f21d42938954391d0776fbde,545b39dacf564a5eb9d1658f80a0b8b0,b4ddedf9b451494a97a1ff9476e3897d,1b145ca23d3e4d81994a84d30a262d18,10a8c7c1d34e46dd8ef461cdad928afc,5ae4911d14bb48b39885b1fe522e999d,7f09def2533b45fdb3c733f1e72ca5fb,9ef29dca19d342b795efcbfe32793602,be72d72ae7034c509c164467b2abddda,ab879705f2354813acf8724404520e28,82487b87b97b475b98892b80cf83c96a,0f4a461b97be48eeab43bd052f630057,72e1a28b7251438a89b0a4f6bd522b89,e1f9a246057347d78a1966d6dc0168e3,5d3645d53ce14c9ab498fd9f050ba264,d361e6890cc84b049fc8268c2a8c4952,5cc0002cbcc14d3485ab8ae6d57368f2,fa7bb243643c44b18e989e3419eeb171,62768b6c65134f1e927b1807bc72115a,7b69b0e5d2d344e6aaa2b8726c23561a,434bbb1f9f8140b1beeb57d0c4db660d,7920c4c8da294d87bcb94e7756bf35ae,0752b85660694033bbdb013f30cde17f,bd6258cfbea64896b0401517e440d6a2,9aa3539682514546866d2998d741ac6c,eb3ad93ce6a44921b0490c842dd47d8c,9ce19042d497473c92d199a5e5a7daf0,1b3eb6ba64ea4589a8e2d42efa5d3f89,6b2dd5ff2a754a53a80798341a7d35d8,e53d754160f74d28b6d091829fc289f8,92d762bca040407bb4ce4b06a2cfa195,aa8a2b00622147259c85290f4d95b46d,0be3e487d874487fbfb5c49b52440a48,ce60b25f7da04af8b7cefccf2dbc7d36,6464ac9cc3fc47d88598f6c4e74caa19,04b312df403d4855b013743206ddb1de,17356c44ae894782a1015ec2ddaeaeda,a06eaefba7084e219f0c6ce4010b82e7,95a54e4db7e74f0586cd85cb4e2bd6f7,1a93047d6b0f4e3ca70fcf836b2517c4,93263a29941648979bb8de774978a880,8eba834a28864375aa3137590d26f37e,3c52b0a890524c6fb66cefc32114c60e,8b20519b794c42d28fdecbd0ccbba5c1,77189ceba655488681964a609c28256d,39e776aaf3644d24bb78ecf43704b71b,0dd5fa1192e34d5fac5e78ca38969126,1a6f82a80a054bc7ba4b516187231a54,5993ce4ec80a41fb9d4ea0713b74f4a4,3b42071536124a9c9dbf2ec319004b29,a966beefe58e4f95b24e89f4172cb7ce,2e1e75dbe28841529576dd1cd48d2437,cdd8829e5976438aaa66ce608f0175f5,71015695cf1445ecacce90669c67d733,31e9dd26b7314d8a9bf97dcf6dbdb22d,e5ecaa091bb0464f8553d4e7aa60647d,39055be3a23945868dbf4b5930043a88,07ba7c9aed104657aac3be87aa346d3b,5a7a9bfff0f44a2089a7d277f2c04557,be53cc152339400ba705c1d1dceff9ab,ed591ccba0564abd9979899049dc7b36,9a4425b817e94dbe844d7fd3e96a9033,bf4e54bfe9984441a8efa4dd8a4a23a6,5a1bcf2a2e0943f88bf655ca011fb288,a12c8904ff0147a99010eaa5aa0ea704,e7c6f2581c6e41bf8fe88a3d7fadc986,89709767beea48d0b0fb047316eb37d1,04d6a49b35514f91be43e9576f8d32fc,bd50e783ec714e5d86aafcca54e87c33,4acb0c7d002b47e29be7ebe9f40490f6,67b5ddef866b4982aa5afef7aad69924,7e090632f713440bba11ed6fc09eb291,f308051e11ca47eebb12709725b52eb4,dbea976ef1d248cabf13b88d311c4533,e0132620d4044e38a92f75b8335005ed,1b598563135e42d4bfe729cf32649f14,c4de9219f4f14684ae21d0904b7b78df,520983ab7dd64104a47e273941236b2d,133ba69bd04046b89eadaf793b9063db,11f24ca662044292a8ca95930ccc3581,0f30b4d621294d4fab359611cf82ab6f,b3c93fcd54b544e5b2c8e70ee7eb93ce,79ebe6595ce44208853ba553186b8348,8d639afccbec43f2be8d3ed88702893b,4e3483f3bbbb48f59b0e92a7d0ccc1c0,b160fccc15fa4ba09c74018d82d16eb7,52f248eadea74cf2b7780bf991d95e4e,d4cb9017aa1f4f8ca34ae8d31c986210,eff19819b1284424a9ad7879991ee080,79551897f86b4338bd09b41c5953a7db,f11bd4ef5ddd4c82865fad3f1f9d80a9,76ee95093a8748b29d55df1cb8cdebd0,081af742f6384a20b492ca99704f51ca,c26c844bbb2f4746904948f3ecfeb1ae,ba9bbc5581a043d8b9c204b8ddb6aa01,cde1afb048a743bb9caedb5770aabca1,0b2b37a28bb0457bbc4bf33c11b4c6e1,9f94d338d6c94b95814b2c50f9e463d1,7d1745ce2c444db9bf4b4e100c476fe8,e711e93aadd54eef8feeb0fa07f054d2,0e914774210646f784d02fb22f8df7db,ca0ce5e9eb09477c8c99b7b94e7869ce,760d9072f9bc480fa36fdb876930467c,5742a628f13045cda0317c3d4d839bc5,b9309305d94f4ee8b66eebbcae9f2db4,a42216d3eedd48f0ba3d3bef2470605f,9cdcda30da704c21b19ea0616de3ea86,f05ecbf7e4b04b268b38cb6e5c8124bf,788d2c1496874eeeae3b7aabbab95f76,8b112af90e054671adc30373c6dafd21,3bdb29667807493292a0ddc9bc0ccdb7,ffd6ebecc869410dab6410a07f3362ae,196868a4610b45588062aed357fc7c58,4cdd2d11be4c4c098b27c10d7cb3fa9e,fda40125207d426dbf86270002ff0c4a,7af028c6048c44cc91415a58d25d286e,ff7d2c8e96e840e1a5a690e4e0f96bad,9f806cf868dc48d398836ebd60bd2ec6,50ff5cdc6c8645b3bb61b8327d204f01,bafb57ea8d84430cbc0c11e5c67643f1,e2c91860b96c46619a74215627560235,f5b68a43bbae44c8a1fe821dfcaca26e,70b771772a0b4f1cb1e592f9a01b08a5,7992a39d64b2483789ae274e2df3c01d,f8d711312da14871a77b58f41a6f6242,673b27b0ab9148b7ae03933c18433971,624886a950bd4d18900a877738696432,685d8d9267f745f39bcc6fc2cc553d31,7d40ad05a1a54165a28ec897ddbcef01,4bcdb5199e8442078999bb8611ea6398,3b0c0ffea0cf45db816993baf64b3fb3,ad2d6d9581034e04addcb5b034a88fb3,8440c4fc5056426996e3e488b2261d28,e367c4223d794572a05fd256759d6a53,1b2dc5784c3c41988d5634e566b4a404,11be1538d7f54d7786004503b2e684a7,b462cb8c2ec7485d938416831a9771cc,f747656a36204e9588aba97064332436,78d0b2337b484c6298c9b3afc01fdb85,8a63d1f8d4bc4eb4a4db437491c9983a,2247f64a15e54053abb2e13152003c6b,d78ef9c5997d4e94804b8126cc387ea1,6ce3cd5a86ef43219dbda2b78e974c93,414b5fd5ebee45d98eca809a524dbe93,4b9ea9db5c834b698a78c7052c87b432,8590fa7adc144f49ae3d2575b7280c08,6148e4b152b9494fb9e7035e9b58e624,be1cff2a29f64800a5c4f91009711988,49b85b35bca5461993eac91ffd0b1fe4,5abae35db6de475e904425af8dc8ecbb,e3bc44b82c3543758d5a6c02b41dd526,89323021918e444bae3537521046d19f,141dd3257f85455d86f3828cc0217b5a,7bc2ad90009a4de2a0fa2947ccc09886,e51874f4b6ac455db76e30582659368b,519dee074e4c48ae9edd58d7c1e330da,060ab186c22f4f42a83f1622bf47ca52,8873bbcf4b42445f8117108769358a5c,46fa5286441a444095b917165c31304c,fa9fdd56222a4ce7ba2b37922afe0eaa,6885932aa4b443babc2b69fdd495889c,be71b947b33a48d8aee36ad0c60d10b8,b52457719429466687a2f228d20571ac,0bf937d745494d998cb8bd2cf3a63f4e,c17feb3860184dd59931f35814ad68b0,4cc68cbd880c48948a1f18fe18ec938c,c37b0691a05c4d659dcd01571f1907da,c1334fe3e8484079bfd62e479307b5b9,ebb58027b6c647b7965eeb31b63ec0c0,1821c5f9392e4859a829c78ebda44df6,dca21ab79bd14004b94c55e36604c30d,8d96cc3457644378b9b643243464f735,631437054079401fbe2c3c9a3f3bc375,c2f78ab4372243da8775f4d71a16fef5,03792664a45d4f63ac3efb899ce1b61e,fec04c46683a41e5a344f4d057342829,320fa70de6bc4890a0135bbfe8735b49,5a03493dce9f4323a0da7bf4fd5e78ab,b9bd46a1da1846d099566fd5c6db00cb,647021c7f798472889fd82f6c4a27ca2,313e905b78d746cb9e574b9634c3a113,623fd328c4cd4ee8b21eb03425b1f100,4676b4d1130146629fd98e2d2eb9a5b4,34d380813a36481ea2534c46d18b5927,894d87e55ac640b99e966a8786a2392b,477ba74d182a4ec0b86926dc67a0f419,362ca5fdca174fa3bdd6325a947320db,1ac3ad8dfa184fe4b6815950d3d23fdd,fdebee18c71f4a9db60096e75ad34d83,91432be4178a4781917fba0abd51df61,480c71baa65145f4bd037332d68e5e4f,536e3a3702b0453d92939846dc1f33b7,0333aedf1be640c78d2ee02473b7fd97,cbb3a6f1772b4f72a8af1375ef16e914,367ad9c43a6246dfac364ca59874883b,bac18430f6db466195a3a7116cdfb958,c4ec39be11be4b0eae321ee0559be8e6,8f7ffae7e47d462aa4142db7f06633d5,82e3b2423d86425195929fc3810ce7a4,72973e0415004dfea220babc479be082,41d0f568bb1e4fed8ad965a79d9bccef,5854ef1d24784ca0a046514b19286113,04afc3dd1eec4d13bc06cd01e0a5e838,db76e3333da846d5ae55b3b9dd735f26,7aa0ca0bf1514fa6a3a14dcde44cf0b9,9c668726c9b6449a9045d3f78408a2a1,7caf148d68a6431bbe56bcc6472d8720,babd36d1d3484fb99fe4e7ca78560ccf,7507865343e24cff80d55a29ec57f0e1,63f8b7d9b1f34851802e0408bf997ea5,a3369d5dd6b74af7b35edf1f10350881,9b8b8c3f86854416b82d57583896c1ff,881815c772564a8dbd5eb6de65ecbe7a,4b1bcf46753a48258d1d514440140b7f,1ca4c6a49e794954a235a82ee8981ecd,ab381d9129bb4b979fe3774e262524f8,a07d2c98831a4d79bc6c5c4288a24720,d8debd28039d494aa7fe18bc8e81af27,d0402b1f00f94acca96276c6ea8c8397,ebcdebd9639d4ec2b7072ad32bf48f50,636b3211100245919c09177ccea562e0,280c4440e18a487396a81b92cd3efa13,5163cf6740334a7d83375385c3ae2b67,a6083e4fab7a43498bfb3257658f9278,3b5c8b5cfb124ebd8f4a798d2a47d832,9fda332474534f96955b5bce8548947f,b5f5e62aba9842d8b48a28111ca6a372,c0cecff1207c4692a634d7de449d8d6d,e4c6428f1d264544b0eec52214e7b6ae,cbc8c5490a7f46b996fd4c3a9711f2d7,cfbdd82f4ce741a88109353c5fc898e4,6bfb965bc1b240a9ae2bd7a1e884ea51,eb36d63a993148289950af9098727759,f7357c3954d2408facaa6771a758ebcc,c05853b94ab8405eb26e0a8bd481b662,da1e6d3ebfa140b3a645342fd3bb5d95,988f3f6b607d442b8df006c980ddc2cd,b6af3c01968141baa3e99ca4b8d83847,75f669faee5d4023a425f2461e5fc5e9,165eb05677aa45fc9a53dfd8bfb7a45d,409891afd7cc4e12ba6b002581fef4f3,02ee0a87956f4bfab91b88282e502a3d,48601418028145ff84a450c877ea35aa,8de73f8d59eb45aa8c61138ab93c4f86,5dd82a46f48945b4bbb61f58b9d1c856,97993db997bd4ff88acfe1340ca86845,282aee84915b43179a09101b3ee6ad0e,8750e994985f43f383321bf0d6ca7932";

        role1.setMenuIds(defalutMenu1);
        String[] defalutMenus1 = defalutMenu1.split(",");
        ArrayList<String> menuIdList1 = new ArrayList<String>();
        for (int i = 0; i < defalutMenus1.length; i++) {
            menuIdList1.add(defalutMenus1[i]);
        }
        role1.setMenuIdList(menuIdList1);
        if (Global.isDemoMode()) {
            return null;
        }
        systemService.saveRole(role1,true,1);
        SysRoleActivity sysRoleActivity = new SysRoleActivity();
        if ("1".equals(type)) {
            String enname = UserUtils.getRoleActivity(com.getId()) + StringUtils.getPinYinHeadChar("案例投标专员", 1);
            sysRoleActivity.setName("案例投标专员");
            sysRoleActivity.setEnname(enname);
            sysRoleActivity.setSort("7300");
            sysRoleActivity.setType("3");
            sysRoleActivity.setDelType("1");
            sysRoleActivity.setCompanyId(com.getId());
            sysRoleActivityService.save(sysRoleActivity);

        }else {
            String enname = UserUtils.getRoleActivityEnname(com.getId(),UserUtils.getUser());
            sysRoleActivity = sysRoleActivityService.getByEnname(enname+"altbzy");
        }
        if (sysRoleActivity!=null) {
            List<SysRoleActivity> roleActivities = new ArrayList<>();
            roleActivities.add(sysRoleActivity);
            role1.setRoleActivityList(roleActivities);
            roleDao.insertRoleActivity(role1);
        }
        return role1;
    }
    @Transactional(readOnly = false)
    public Role createGZRRole( Office com,Office office,Office deptBGS,String type) {
        //公司新用户
        Role role1 = new Role();
        role1.setNumber("4");
        role1.setName("盖章人");
        role1.setSysData(Global.YES);
        if (type.equals("1")){
            role1.setDataScope("2");
        }else {
            role1.setDataScope("5");
            role1.setBranchCompany(office);
        }
        role1.setOffice(deptBGS);
        role1.setIsNewRecord(false);
        role1.setCompany(com);
        role1.setRoleType("assignment");
        role1.setSysData("1");
        role1.setUseable("1");
        role1.setEnname("cn");

        String defalutMenu1 = "1,27,29,e3b80fae59ab4b4099d8349ecaf8c4dc,61dd3aa279684f92bb1c5d4019a32503,b0c5737cdc4f452684ce93589427c97d,af6b0e5d61fc44ecbebb449703a3df72,18c14562978e4b129fe3afb7dca8b868,6123fcb3d810492789e82a63b0d4310a,0aa3712414d049a6a24e8bcddeae509a,95a6a82dc5fc4d07b46e5df57a0606a3,62,b6fa4abb624349918fe531274fc15d23,89,76bdb735faa34667a6cbf13ebe937533,90,f5b2028cfad9470085c7c846de33193a,28abe4b0cb3b4c8480f84c07a5dfaf06,c7fa36c7142f481397c2cc12e2bc828a,e2efcf80fc3747329fe02d489dbfda95,e3b5b7451e164089aba0b9bf844b0398,438c28c3cbe44ed7bff951b2a3dc77cd,59e50eebe56743a78eecbc6eff1ef94c,cc1027689aca41a89302217127253034,200fe6cc80174065b382c961a4379d79,4c13ca96376341dfa25980e86964c1a9,638be8b3b5284e5088003917685bc19b,811600aadcb54b098b7dda4bf95ffc51,ce8dc7c97d93481dbd76cabb6e905b55,aa25b3904abb4bd7b779e5be37b4382e,aae6f4c7fd4c4a25afa5285210edfded,84ff9d5c165c4528af9361b6127cf5af,31b349a3624b47f28eb86d614f9f95b5,040b42e395de41d6805a958eb1897863,34874260e6fc4200a76b33c3cfa4d80d,5b6c6ea5ed2d45478e8144351d853e21,7c032c32512847f882a87416359ef0a3,6d30ea8c710a42b89da2f2506059b0ef,2ca7bf4ce6804a1c9e5b331d2df27c72,7717cbb6d71f405d890e9b648cbd2088,bddb0400764c42b08e427059c7752057,23d6c56239804053910e5c5aec481330,7f8b2018113545bcbdbfc87e7a236bec,547e121ffb714d16ae142b6935abb78f,af24101b962444f29d166b638b2fea30,6ea6519ec4d548128aa42f3ce004def3,4379bf54c2a543869abf5031b97b8efe,5e6da3f5368b4f06b7268b31a1157e9f,2bbf0e75f21d42938954391d0776fbde,545b39dacf564a5eb9d1658f80a0b8b0,b4ddedf9b451494a97a1ff9476e3897d,1b145ca23d3e4d81994a84d30a262d18,10a8c7c1d34e46dd8ef461cdad928afc,5ae4911d14bb48b39885b1fe522e999d,7f09def2533b45fdb3c733f1e72ca5fb,9ef29dca19d342b795efcbfe32793602,be72d72ae7034c509c164467b2abddda,ab879705f2354813acf8724404520e28,82487b87b97b475b98892b80cf83c96a,0f4a461b97be48eeab43bd052f630057,72e1a28b7251438a89b0a4f6bd522b89,e1f9a246057347d78a1966d6dc0168e3,5d3645d53ce14c9ab498fd9f050ba264,d361e6890cc84b049fc8268c2a8c4952,5cc0002cbcc14d3485ab8ae6d57368f2,fa7bb243643c44b18e989e3419eeb171,62768b6c65134f1e927b1807bc72115a,7b69b0e5d2d344e6aaa2b8726c23561a,434bbb1f9f8140b1beeb57d0c4db660d,7920c4c8da294d87bcb94e7756bf35ae,0752b85660694033bbdb013f30cde17f,bd6258cfbea64896b0401517e440d6a2,9aa3539682514546866d2998d741ac6c,eb3ad93ce6a44921b0490c842dd47d8c,9ce19042d497473c92d199a5e5a7daf0,1b3eb6ba64ea4589a8e2d42efa5d3f89,6b2dd5ff2a754a53a80798341a7d35d8,e53d754160f74d28b6d091829fc289f8,92d762bca040407bb4ce4b06a2cfa195,aa8a2b00622147259c85290f4d95b46d,0be3e487d874487fbfb5c49b52440a48,ce60b25f7da04af8b7cefccf2dbc7d36,6464ac9cc3fc47d88598f6c4e74caa19,04b312df403d4855b013743206ddb1de,17356c44ae894782a1015ec2ddaeaeda,a06eaefba7084e219f0c6ce4010b82e7,95a54e4db7e74f0586cd85cb4e2bd6f7,1a93047d6b0f4e3ca70fcf836b2517c4,93263a29941648979bb8de774978a880,8eba834a28864375aa3137590d26f37e,3c52b0a890524c6fb66cefc32114c60e,8b20519b794c42d28fdecbd0ccbba5c1,77189ceba655488681964a609c28256d,39e776aaf3644d24bb78ecf43704b71b,0dd5fa1192e34d5fac5e78ca38969126,1a6f82a80a054bc7ba4b516187231a54,5993ce4ec80a41fb9d4ea0713b74f4a4,3b42071536124a9c9dbf2ec319004b29,a966beefe58e4f95b24e89f4172cb7ce,2e1e75dbe28841529576dd1cd48d2437,cdd8829e5976438aaa66ce608f0175f5,71015695cf1445ecacce90669c67d733,31e9dd26b7314d8a9bf97dcf6dbdb22d,e5ecaa091bb0464f8553d4e7aa60647d,39055be3a23945868dbf4b5930043a88,07ba7c9aed104657aac3be87aa346d3b,5a7a9bfff0f44a2089a7d277f2c04557,be53cc152339400ba705c1d1dceff9ab,ed591ccba0564abd9979899049dc7b36,9a4425b817e94dbe844d7fd3e96a9033,bf4e54bfe9984441a8efa4dd8a4a23a6,5a1bcf2a2e0943f88bf655ca011fb288,a12c8904ff0147a99010eaa5aa0ea704,e7c6f2581c6e41bf8fe88a3d7fadc986,89709767beea48d0b0fb047316eb37d1,04d6a49b35514f91be43e9576f8d32fc,bd50e783ec714e5d86aafcca54e87c33,4acb0c7d002b47e29be7ebe9f40490f6,67b5ddef866b4982aa5afef7aad69924,7e090632f713440bba11ed6fc09eb291,f308051e11ca47eebb12709725b52eb4,dbea976ef1d248cabf13b88d311c4533,e0132620d4044e38a92f75b8335005ed,1b598563135e42d4bfe729cf32649f14,c4de9219f4f14684ae21d0904b7b78df,520983ab7dd64104a47e273941236b2d,133ba69bd04046b89eadaf793b9063db,11f24ca662044292a8ca95930ccc3581,0f30b4d621294d4fab359611cf82ab6f,b3c93fcd54b544e5b2c8e70ee7eb93ce,79ebe6595ce44208853ba553186b8348,8d639afccbec43f2be8d3ed88702893b,4e3483f3bbbb48f59b0e92a7d0ccc1c0,b160fccc15fa4ba09c74018d82d16eb7,52f248eadea74cf2b7780bf991d95e4e,d4cb9017aa1f4f8ca34ae8d31c986210,eff19819b1284424a9ad7879991ee080,79551897f86b4338bd09b41c5953a7db,f11bd4ef5ddd4c82865fad3f1f9d80a9,76ee95093a8748b29d55df1cb8cdebd0,081af742f6384a20b492ca99704f51ca,c26c844bbb2f4746904948f3ecfeb1ae,ba9bbc5581a043d8b9c204b8ddb6aa01,cde1afb048a743bb9caedb5770aabca1,0b2b37a28bb0457bbc4bf33c11b4c6e1,9f94d338d6c94b95814b2c50f9e463d1,7d1745ce2c444db9bf4b4e100c476fe8,e711e93aadd54eef8feeb0fa07f054d2,0e914774210646f784d02fb22f8df7db,ca0ce5e9eb09477c8c99b7b94e7869ce,760d9072f9bc480fa36fdb876930467c,5742a628f13045cda0317c3d4d839bc5,b9309305d94f4ee8b66eebbcae9f2db4,a42216d3eedd48f0ba3d3bef2470605f,9cdcda30da704c21b19ea0616de3ea86,f05ecbf7e4b04b268b38cb6e5c8124bf,788d2c1496874eeeae3b7aabbab95f76,8b112af90e054671adc30373c6dafd21,3bdb29667807493292a0ddc9bc0ccdb7,ffd6ebecc869410dab6410a07f3362ae,196868a4610b45588062aed357fc7c58,4cdd2d11be4c4c098b27c10d7cb3fa9e,fda40125207d426dbf86270002ff0c4a,7af028c6048c44cc91415a58d25d286e,ff7d2c8e96e840e1a5a690e4e0f96bad,9f806cf868dc48d398836ebd60bd2ec6,50ff5cdc6c8645b3bb61b8327d204f01,bafb57ea8d84430cbc0c11e5c67643f1,e2c91860b96c46619a74215627560235,f5b68a43bbae44c8a1fe821dfcaca26e,70b771772a0b4f1cb1e592f9a01b08a5,7992a39d64b2483789ae274e2df3c01d,f8d711312da14871a77b58f41a6f6242,673b27b0ab9148b7ae03933c18433971,624886a950bd4d18900a877738696432,685d8d9267f745f39bcc6fc2cc553d31,7d40ad05a1a54165a28ec897ddbcef01,4bcdb5199e8442078999bb8611ea6398,3b0c0ffea0cf45db816993baf64b3fb3,ad2d6d9581034e04addcb5b034a88fb3,8440c4fc5056426996e3e488b2261d28,e367c4223d794572a05fd256759d6a53,1b2dc5784c3c41988d5634e566b4a404,11be1538d7f54d7786004503b2e684a7,b462cb8c2ec7485d938416831a9771cc,f747656a36204e9588aba97064332436,78d0b2337b484c6298c9b3afc01fdb85,8a63d1f8d4bc4eb4a4db437491c9983a,2247f64a15e54053abb2e13152003c6b,d78ef9c5997d4e94804b8126cc387ea1,6ce3cd5a86ef43219dbda2b78e974c93,414b5fd5ebee45d98eca809a524dbe93,4b9ea9db5c834b698a78c7052c87b432,8590fa7adc144f49ae3d2575b7280c08,6148e4b152b9494fb9e7035e9b58e624,be1cff2a29f64800a5c4f91009711988,49b85b35bca5461993eac91ffd0b1fe4,5abae35db6de475e904425af8dc8ecbb,e3bc44b82c3543758d5a6c02b41dd526,89323021918e444bae3537521046d19f,141dd3257f85455d86f3828cc0217b5a,7bc2ad90009a4de2a0fa2947ccc09886,e51874f4b6ac455db76e30582659368b,519dee074e4c48ae9edd58d7c1e330da,060ab186c22f4f42a83f1622bf47ca52,8873bbcf4b42445f8117108769358a5c,46fa5286441a444095b917165c31304c,fa9fdd56222a4ce7ba2b37922afe0eaa,6885932aa4b443babc2b69fdd495889c,be71b947b33a48d8aee36ad0c60d10b8,b52457719429466687a2f228d20571ac,0bf937d745494d998cb8bd2cf3a63f4e,c17feb3860184dd59931f35814ad68b0,4cc68cbd880c48948a1f18fe18ec938c,c37b0691a05c4d659dcd01571f1907da,c1334fe3e8484079bfd62e479307b5b9,ebb58027b6c647b7965eeb31b63ec0c0,1821c5f9392e4859a829c78ebda44df6,dca21ab79bd14004b94c55e36604c30d,8d96cc3457644378b9b643243464f735,631437054079401fbe2c3c9a3f3bc375,c2f78ab4372243da8775f4d71a16fef5,03792664a45d4f63ac3efb899ce1b61e,fec04c46683a41e5a344f4d057342829,320fa70de6bc4890a0135bbfe8735b49,5a03493dce9f4323a0da7bf4fd5e78ab,b9bd46a1da1846d099566fd5c6db00cb,647021c7f798472889fd82f6c4a27ca2,313e905b78d746cb9e574b9634c3a113,623fd328c4cd4ee8b21eb03425b1f100,4676b4d1130146629fd98e2d2eb9a5b4,34d380813a36481ea2534c46d18b5927,894d87e55ac640b99e966a8786a2392b,477ba74d182a4ec0b86926dc67a0f419,362ca5fdca174fa3bdd6325a947320db,1ac3ad8dfa184fe4b6815950d3d23fdd,fdebee18c71f4a9db60096e75ad34d83,91432be4178a4781917fba0abd51df61,480c71baa65145f4bd037332d68e5e4f,536e3a3702b0453d92939846dc1f33b7,0333aedf1be640c78d2ee02473b7fd97,cbb3a6f1772b4f72a8af1375ef16e914,367ad9c43a6246dfac364ca59874883b,bac18430f6db466195a3a7116cdfb958,c4ec39be11be4b0eae321ee0559be8e6,8f7ffae7e47d462aa4142db7f06633d5,82e3b2423d86425195929fc3810ce7a4,72973e0415004dfea220babc479be082,41d0f568bb1e4fed8ad965a79d9bccef,5854ef1d24784ca0a046514b19286113,04afc3dd1eec4d13bc06cd01e0a5e838,db76e3333da846d5ae55b3b9dd735f26,7aa0ca0bf1514fa6a3a14dcde44cf0b9,9c668726c9b6449a9045d3f78408a2a1,7caf148d68a6431bbe56bcc6472d8720,babd36d1d3484fb99fe4e7ca78560ccf,7507865343e24cff80d55a29ec57f0e1,63f8b7d9b1f34851802e0408bf997ea5,a3369d5dd6b74af7b35edf1f10350881,9b8b8c3f86854416b82d57583896c1ff,881815c772564a8dbd5eb6de65ecbe7a,4b1bcf46753a48258d1d514440140b7f,1ca4c6a49e794954a235a82ee8981ecd,ab381d9129bb4b979fe3774e262524f8,a07d2c98831a4d79bc6c5c4288a24720,d8debd28039d494aa7fe18bc8e81af27,d0402b1f00f94acca96276c6ea8c8397,ebcdebd9639d4ec2b7072ad32bf48f50,636b3211100245919c09177ccea562e0,280c4440e18a487396a81b92cd3efa13,5163cf6740334a7d83375385c3ae2b67,a6083e4fab7a43498bfb3257658f9278,3b5c8b5cfb124ebd8f4a798d2a47d832,9fda332474534f96955b5bce8548947f,b5f5e62aba9842d8b48a28111ca6a372,c0cecff1207c4692a634d7de449d8d6d,e4c6428f1d264544b0eec52214e7b6ae,cbc8c5490a7f46b996fd4c3a9711f2d7,cfbdd82f4ce741a88109353c5fc898e4,6bfb965bc1b240a9ae2bd7a1e884ea51,eb36d63a993148289950af9098727759,f7357c3954d2408facaa6771a758ebcc,c05853b94ab8405eb26e0a8bd481b662,da1e6d3ebfa140b3a645342fd3bb5d95,988f3f6b607d442b8df006c980ddc2cd,b6af3c01968141baa3e99ca4b8d83847,75f669faee5d4023a425f2461e5fc5e9,165eb05677aa45fc9a53dfd8bfb7a45d,409891afd7cc4e12ba6b002581fef4f3,02ee0a87956f4bfab91b88282e502a3d,48601418028145ff84a450c877ea35aa,8de73f8d59eb45aa8c61138ab93c4f86,5dd82a46f48945b4bbb61f58b9d1c856,97993db997bd4ff88acfe1340ca86845,282aee84915b43179a09101b3ee6ad0e,8750e994985f43f383321bf0d6ca7932";

        role1.setMenuIds(defalutMenu1);
        String[] defalutMenus1 = defalutMenu1.split(",");
        ArrayList<String> menuIdList1 = new ArrayList<String>();
        for (int i = 0; i < defalutMenus1.length; i++) {
            menuIdList1.add(defalutMenus1[i]);
        }
        role1.setMenuIdList(menuIdList1);
        if (Global.isDemoMode()) {
            return null;
        }
        systemService.saveRole(role1,true,1);
        SysRoleActivity sysRoleActivity = new SysRoleActivity();
        if ("1".equals(type)) {
            String enname = UserUtils.getRoleActivity(com.getId()) + StringUtils.getPinYinHeadChar("盖章人", 1);
            sysRoleActivity.setName("盖章人");
            sysRoleActivity.setEnname(enname);
            sysRoleActivity.setSort("7400");
            sysRoleActivity.setType("3");
            sysRoleActivity.setDelType("1");
            sysRoleActivity.setCompanyId(com.getId());
            sysRoleActivityService.save(sysRoleActivity);

        }else {
            String enname = UserUtils.getRoleActivityEnname(com.getId(),UserUtils.getUser());
            sysRoleActivity = sysRoleActivityService.getByEnname(enname+"gzr");
        }
        List<SysRoleActivity> roleActivities = new ArrayList<>();
        roleActivities.add(sysRoleActivity);
        role1.setRoleActivityList(roleActivities);
        roleDao.insertRoleActivity(role1);
        return role1;
    }

    @Transactional(readOnly = false)
    public Role createCompanyRole(Office office) {

        //自动生成管理员岗位
        Role role = new Role();
        role.setNumber("0");
        role.setName("公司管理员");
        //	role.setSysData(Global.YES);
        role.setDataScope("2");
        role.setIsNewRecord(false);

        role.setOffice(office);
        role.setRoleType("security-role");
        role.setCompany(office);
        role.setSysData("1");
        role.setUseable("1");
        role.setEnname("gly");
        //role.setEnname(StringUtils.getPinYinHeadChar(company.getName(), 2)+System.currentTimeMillis() + "gly");
        String defalutMenu = "1,27,29,e3b80fae59ab4b4099d8349ecaf8c4dc,61dd3aa279684f92bb1c5d4019a32503,b0c5737cdc4f452684ce93589427c97d,af6b0e5d61fc44ecbebb449703a3df72,18c14562978e4b129fe3afb7dca8b868,6123fcb3d810492789e82a63b0d4310a,0aa3712414d049a6a24e8bcddeae509a,9fe46041bfd54291a8c8410c596b57bf,af0a174b4f424bc09a8cc0db83a64105,95a6a82dc5fc4d07b46e5df57a0606a3,9fdf5971a9e64fac8bbffec2825a5f97,843fdb78b8854bd48e503287f795a7d0,8d369ea660e04241a6c88d75c49a68ce,a90ba79ffbcb40dfa705fe4992e29d22,624c57fb83a54ff686c1e686bebde4c5,78bd8a7958a84617a06bcf72837eae0b,900f410b044542eeba2382d18f5d2704,6fa1a6734aeb4237a22b61b10f0affb8,62,b6fa4abb624349918fe531274fc15d23,89,76bdb735faa34667a6cbf13ebe937533,90,f5b2028cfad9470085c7c846de33193a,28abe4b0cb3b4c8480f84c07a5dfaf06,c7fa36c7142f481397c2cc12e2bc828a,e2efcf80fc3747329fe02d489dbfda95,e3b5b7451e164089aba0b9bf844b0398,438c28c3cbe44ed7bff951b2a3dc77cd,59e50eebe56743a78eecbc6eff1ef94c,bffd7de1c574450bb48484d1be66b197,cc1027689aca41a89302217127253034,200fe6cc80174065b382c961a4379d79,33fbe92e979b4260afb86c2db8f8fa2c,7d940fcac02248ef8074311e268a3a31,892f39ada6be4026a412eae188d7a5b6,4c13ca96376341dfa25980e86964c1a9,638be8b3b5284e5088003917685bc19b,811600aadcb54b098b7dda4bf95ffc51,ce8dc7c97d93481dbd76cabb6e905b55,aa25b3904abb4bd7b779e5be37b4382e,aae6f4c7fd4c4a25afa5285210edfded,84ff9d5c165c4528af9361b6127cf5af,31b349a3624b47f28eb86d614f9f95b5,040b42e395de41d6805a958eb1897863,34874260e6fc4200a76b33c3cfa4d80d,5b6c6ea5ed2d45478e8144351d853e21,7c032c32512847f882a87416359ef0a3,6d30ea8c710a42b89da2f2506059b0ef,6ac94964592a4dcf937a6027a4df5f98,a0970d1e021d4d6882e307dfc2a8e945,8aa598e4291a4d9591213c24c6427fa2,2ca7bf4ce6804a1c9e5b331d2df27c72,7717cbb6d71f405d890e9b648cbd2088,2fc21a3364f545358510b2dafd1c4095,3c61d9d5a71e4b7faa97db881b2da6bd,4477d2be9bf143c299c1b53f908e8f1c,bddb0400764c42b08e427059c7752057,23d6c56239804053910e5c5aec481330,7f8b2018113545bcbdbfc87e7a236bec,547e121ffb714d16ae142b6935abb78f,af24101b962444f29d166b638b2fea30,6ea6519ec4d548128aa42f3ce004def3,4379bf54c2a543869abf5031b97b8efe,5e6da3f5368b4f06b7268b31a1157e9f,2bbf0e75f21d42938954391d0776fbde,545b39dacf564a5eb9d1658f80a0b8b0,b4ddedf9b451494a97a1ff9476e3897d,1b145ca23d3e4d81994a84d30a262d18,10a8c7c1d34e46dd8ef461cdad928afc,5ae4911d14bb48b39885b1fe522e999d,7f09def2533b45fdb3c733f1e72ca5fb,9ef29dca19d342b795efcbfe32793602,be72d72ae7034c509c164467b2abddda,ab879705f2354813acf8724404520e28,82487b87b97b475b98892b80cf83c96a,0f4a461b97be48eeab43bd052f630057,72e1a28b7251438a89b0a4f6bd522b89,e1f9a246057347d78a1966d6dc0168e3,5d3645d53ce14c9ab498fd9f050ba264,d361e6890cc84b049fc8268c2a8c4952,5cc0002cbcc14d3485ab8ae6d57368f2,fa7bb243643c44b18e989e3419eeb171,62768b6c65134f1e927b1807bc72115a,7b69b0e5d2d344e6aaa2b8726c23561a,434bbb1f9f8140b1beeb57d0c4db660d,7920c4c8da294d87bcb94e7756bf35ae,0752b85660694033bbdb013f30cde17f,bd6258cfbea64896b0401517e440d6a2,9aa3539682514546866d2998d741ac6c,eb3ad93ce6a44921b0490c842dd47d8c,9ce19042d497473c92d199a5e5a7daf0,1b3eb6ba64ea4589a8e2d42efa5d3f89,6b2dd5ff2a754a53a80798341a7d35d8,e53d754160f74d28b6d091829fc289f8,92d762bca040407bb4ce4b06a2cfa195,aa8a2b00622147259c85290f4d95b46d,0be3e487d874487fbfb5c49b52440a48,ce60b25f7da04af8b7cefccf2dbc7d36,6464ac9cc3fc47d88598f6c4e74caa19,04b312df403d4855b013743206ddb1de,17356c44ae894782a1015ec2ddaeaeda,a06eaefba7084e219f0c6ce4010b82e7,95a54e4db7e74f0586cd85cb4e2bd6f7,1a93047d6b0f4e3ca70fcf836b2517c4,93263a29941648979bb8de774978a880,8eba834a28864375aa3137590d26f37e,3c52b0a890524c6fb66cefc32114c60e,8b20519b794c42d28fdecbd0ccbba5c1,2d74c531c4d14056973fd177652e7dc4,1a9e2d47fcfc4a5cb6ea0b2bb4bf24f9,4124a6a03e03480eba58e00a97667815,77189ceba655488681964a609c28256d,39e776aaf3644d24bb78ecf43704b71b,0dd5fa1192e34d5fac5e78ca38969126,1a6f82a80a054bc7ba4b516187231a54,5993ce4ec80a41fb9d4ea0713b74f4a4,3b42071536124a9c9dbf2ec319004b29,a966beefe58e4f95b24e89f4172cb7ce,2e1e75dbe28841529576dd1cd48d2437,cdd8829e5976438aaa66ce608f0175f5,b9312c6148b54697ad2510d33f081df4,71015695cf1445ecacce90669c67d733,31e9dd26b7314d8a9bf97dcf6dbdb22d,e5ecaa091bb0464f8553d4e7aa60647d,39055be3a23945868dbf4b5930043a88,90a22ce7f0254ecba4f90a6fb7c15763,a6a7913b5b714ae49f5026a7dacfa4cb,b76d1d2fbfa34db7852eba3845748130,07ba7c9aed104657aac3be87aa346d3b,5a7a9bfff0f44a2089a7d277f2c04557,be53cc152339400ba705c1d1dceff9ab,ef56135adf7445e59ed8f41947e7a72c,ba14a56dcce149a48379b32df5f5bd8f,ab4bd14c2a394c26a201df1d753bf02d,ed591ccba0564abd9979899049dc7b36,9a4425b817e94dbe844d7fd3e96a9033,bf4e54bfe9984441a8efa4dd8a4a23a6,5a1bcf2a2e0943f88bf655ca011fb288,a12c8904ff0147a99010eaa5aa0ea704,e7c6f2581c6e41bf8fe88a3d7fadc986,89709767beea48d0b0fb047316eb37d1,04d6a49b35514f91be43e9576f8d32fc,bd50e783ec714e5d86aafcca54e87c33,4acb0c7d002b47e29be7ebe9f40490f6,67b5ddef866b4982aa5afef7aad69924,7e090632f713440bba11ed6fc09eb291,f308051e11ca47eebb12709725b52eb4,dbea976ef1d248cabf13b88d311c4533,e0132620d4044e38a92f75b8335005ed,1b598563135e42d4bfe729cf32649f14,c4de9219f4f14684ae21d0904b7b78df,520983ab7dd64104a47e273941236b2d,133ba69bd04046b89eadaf793b9063db,11f24ca662044292a8ca95930ccc3581,0f30b4d621294d4fab359611cf82ab6f,b3c93fcd54b544e5b2c8e70ee7eb93ce,79ebe6595ce44208853ba553186b8348,8d639afccbec43f2be8d3ed88702893b,4e3483f3bbbb48f59b0e92a7d0ccc1c0,b160fccc15fa4ba09c74018d82d16eb7,52f248eadea74cf2b7780bf991d95e4e,d4cb9017aa1f4f8ca34ae8d31c986210,eff19819b1284424a9ad7879991ee080,79551897f86b4338bd09b41c5953a7db,f11bd4ef5ddd4c82865fad3f1f9d80a9,76ee95093a8748b29d55df1cb8cdebd0,081af742f6384a20b492ca99704f51ca,c26c844bbb2f4746904948f3ecfeb1ae,ba9bbc5581a043d8b9c204b8ddb6aa01,cde1afb048a743bb9caedb5770aabca1,0b2b37a28bb0457bbc4bf33c11b4c6e1,9f94d338d6c94b95814b2c50f9e463d1,7d1745ce2c444db9bf4b4e100c476fe8,e711e93aadd54eef8feeb0fa07f054d2,0e914774210646f784d02fb22f8df7db,ca0ce5e9eb09477c8c99b7b94e7869ce,760d9072f9bc480fa36fdb876930467c,5742a628f13045cda0317c3d4d839bc5,b9309305d94f4ee8b66eebbcae9f2db4,a42216d3eedd48f0ba3d3bef2470605f,9cdcda30da704c21b19ea0616de3ea86,f05ecbf7e4b04b268b38cb6e5c8124bf,788d2c1496874eeeae3b7aabbab95f76,8b112af90e054671adc30373c6dafd21,3bdb29667807493292a0ddc9bc0ccdb7,ffd6ebecc869410dab6410a07f3362ae,196868a4610b45588062aed357fc7c58,4cdd2d11be4c4c098b27c10d7cb3fa9e,fda40125207d426dbf86270002ff0c4a,7af028c6048c44cc91415a58d25d286e,ff7d2c8e96e840e1a5a690e4e0f96bad,5abae35db6de475e904425af8dc8ecbb,e3bc44b82c3543758d5a6c02b41dd526,89323021918e444bae3537521046d19f,141dd3257f85455d86f3828cc0217b5a,7bc2ad90009a4de2a0fa2947ccc09886,e51874f4b6ac455db76e30582659368b,519dee074e4c48ae9edd58d7c1e330da,a4816b8b30e341d296669f627f23ade3,93a9760729eb4ef187d60fbad59e3d31,e980504789844b2f96b417bf439fa964,f5c6bbf5b71d44749af30b795ef8c222,05e1cca3da5340f59db876bb238287f3,060ab186c22f4f42a83f1622bf47ca52,8873bbcf4b42445f8117108769358a5c,46fa5286441a444095b917165c31304c,fa9fdd56222a4ce7ba2b37922afe0eaa,6885932aa4b443babc2b69fdd495889c,be71b947b33a48d8aee36ad0c60d10b8,b52457719429466687a2f228d20571ac,723592e24a3d48bdb2ef109b8ee46837,0bf937d745494d998cb8bd2cf3a63f4e,c17feb3860184dd59931f35814ad68b0,4cc68cbd880c48948a1f18fe18ec938c,c37b0691a05c4d659dcd01571f1907da,c1334fe3e8484079bfd62e479307b5b9,ebb58027b6c647b7965eeb31b63ec0c0,720fc5746f7b4e5984bbc8d1aaeca215,cc4be4c10bd949408e4c8c29c9b842ce,88d963bd3d404bd0895c136f5012a1de,8dc4d97f0ac64509ad6cc5ba298e0a5b,73c96c78c81e48399d16a67923e1f651,084471d4393044d4802e3e27f71988f3,bd53b341c57e4b63b7444a1082339c2d,d6658a5ee06c4036ab9bdc8b553255a6,2c5ea7d85c164bc3b4545d9c46334488,9b3c62a0af4c4cac9deb415baaf04fa5,22d943bb9e614ca69148743ff28e2362,f635b5ed489b401986c95cd8e5430bbd,2b014748b6714cb988b1c4142d3c0204,cc4716f1ba934c18848061b690a137ed,d7d80fefd53943efafc88f3768a6aff2,b692b96453204e299753df8f0751f4ec,1e66a862627f40e0b9470dea5220308a,1a098f10396f462a95d20d712722973a,67fdc15ef4f149ec8dfda74f0e1612fc,474566ddb2be4de785e8a920530cc320,47f42a430d3f4dc1872f8cfcc628b527,be5027b456b84bbaa5fa016254122089,b3b39abe13e54a3c8727cdb6387e1a1b,7b4205ea12c34b28aea4806f2e26104a,342f70da525349e6b86a120cbddf9e5a,a9a79b1171e3410291ae7d0ec68f1c8b,d8008fa60f6d4c279cac795cc2944ded,78a51382773b4a7fbac46beee2846edf,a01c30e0521d4082a7e2edd25c3fa476,c525320a19c64f4c97afdc50d9f90348,4897d1f2afe14aadaa342a615e50e69a,f43e7af70a4344489cd7a2b000af0a66,a6e1e26f26734705a6b599f4772de663,4b1985e929f54406ae345b9035c8526e,aee0c1fdaab84ce2889424f8323c243b,165eb05677aa45fc9a53dfd8bfb7a45d,409891afd7cc4e12ba6b002581fef4f3,02ee0a87956f4bfab91b88282e502a3d,48601418028145ff84a450c877ea35aa,8de73f8d59eb45aa8c61138ab93c4f86,5dd82a46f48945b4bbb61f58b9d1c856,97993db997bd4ff88acfe1340ca86845,282aee84915b43179a09101b3ee6ad0e,8750e994985f43f383321bf0d6ca7932,3,4,5,6,68f9151151174868ab436e11e03bf548,79f0ffa47dbe43ffa8824d97612d344f,ec7cf7a144a440cab217aabd4ffb7788,20,21,22,2a0f940fbe304a05a6b4040ddf6df279,79fca849d3da4a82a4ade3f6b9f45126,f34887a78fa245c1977603ca7dc98e11,b9a776f5d7194406bb466388e3af9d08,17,18,19,a4c3dcee6cbc4fc6a0bf617d8619edf3,8930e4aad1ba4a1c958d303968d8c442,4da8be6fe3e14a8aae57583402e6934f,b7d20f0244084f41a041fcb7ef85db8f,fb46733b041949d0a35cbe7951f71b89,d5d57e587eec4398bb13b8861afd92b9,b76e8723777d4f979861a4431b4282e6,7,8,9,3c1c639c76f14f6f9903b0143371ea09,6509eed6eb634030a46723a18814035c,f93f9a3a2226461dace3b8992cf055ba,de7c50d276454f80881c41a096ecf55c,14,15,16,c6e0080e06014abd9240f870aadf3200,5239527958e94d418997b584b85d8b80,10,11,12,6d3a6777693f47c98e9b3051cacbcfdb,9bc1aa1053144a608b73f6fbd841d1c6,c0053ff4df384b129307dafe502e723d,7e33cac762cc46d784cedf970344fa47,46ba10003e70488ab46ae7faa81d5334,8c3d779f0bec4f47b7b5232a44acc0a9,6d3fa8801e6b406ebc846a9d6457573c,f4ecf5d2654c46769057362ebfb21afe,402c4c50ebdc476795b75231de699571,4e37cb62b4a34f52a304ddc9d9147905,648ad6abfff24af396fcec1d8d05a856,f133ae5367914b2892459d03012720b3,db8dea379c8e4333bda690d4d096b331,26a3901d047241aeb4a413ae1c05ee3a,a05ddf1532e049179cb68e2ce2322f1e,4f56225f24294a90853d70556d763f6f,ab08526072e74d4888a8a081db9548b4,67713a8a2024414fb34ab22f1dce088c,a6c17ad4f46e452fb16824a65aacd2ee,52595fbde8cb41b3920e286cb7821fba,b32a8d8f1e6a42fcb0300049cd985a3f,2c67ca46306a41eb93d29e26c3ea0cea,0f354016b42b4b73bc65b1c88a412662,7ae1473d3bfa47e980545f2a27d6aec0,fdc01f6b4f98465ca133d78f431461e2,5b99fb57fd5e42d48f1a11b22ba49c90,eb081fd3c96a4787bfa0ff42be64cfc0,01dde345bc3e4e6ea1c90169ccb66c26,7f23f2ca985d44f7a96918f499715426,23202589fe45431799af1fe05cba3a24,0ad156f7d5c44aee9028e032ea2b2b23,105dc6beaa6646f9971ab14f8c957521,f6efef2068b24205a911c97122403cbf,75c1cf011e714a6095d1486a4fcc62d1,c0d648b4ff514fa3ad6ddf620d6c7e2d,e909925ac4284251b38d6e05ed52fea8,000128bbf191457a9b4bcc337443ec06,8e1206a37db2464d8d08221928da6d16";

        role.setMenuIds(defalutMenu);
        String[] defalutMenus = defalutMenu.split(",");
        ArrayList<String> menuIdList = new ArrayList<String>();
        for (int i = 0; i < defalutMenus.length; i++) {
            menuIdList.add(defalutMenus[i]);
        }
        role.setMenuIdList(menuIdList);
        if (!UserUtils.getUser().isAdmin() && role.getSysData().equals(Global.YES)) {
            return null;
        }
        if (Global.isDemoMode()) {
            return null;
        }
        /*if (!"true".equals(checkName(role.getOldName(), role.getName(),role.getCompany().getId()))) {
            return null;
        }*/
        /*logger.info(role.getOldEnname() + "--------------------" + role.getEnname());
        if (!"true".equals(checkEnname(role.getOldEnname(), role.getEnname()))) {
            return null;
        }*/
        role.setOffice(office);
        systemService.saveRole(role,true,1);
        //roleDao.insertRoleOffice(role);
        return role;
    }
    //添加子公司岗位
    @Transactional(readOnly = false)
    public Role createCompanyRoleOR(Office office) {

        //自动生成管理员岗位
        Role role = new Role();
        role.setNumber("0");
        role.setName("子公司管理员");
        role.setDataScope("2");
        role.setIsNewRecord(false);
        role.setBranchCompany(office);
        role.setOffice(office);
        role.setRoleType("security-role");
        role.setCompany(office);
        role.setSysData("1");
        role.setUseable("1");
        role.setEnname("zgly");
        // role.setEnname(StringUtils.getPinYinHeadChar(company.getName(), 2)+System.currentTimeMillis() + "gly");

        String defalutMenu = "1,27,29,e3b80fae59ab4b4099d8349ecaf8c4dc,61dd3aa279684f92bb1c5d4019a32503,b0c5737cdc4f452684ce93589427c97d,af6b0e5d61fc44ecbebb449703a3df72,18c14562978e4b129fe3afb7dca8b868,6123fcb3d810492789e82a63b0d4310a,0aa3712414d049a6a24e8bcddeae509a,9fe46041bfd54291a8c8410c596b57bf,af0a174b4f424bc09a8cc0db83a64105,95a6a82dc5fc4d07b46e5df57a0606a3,9fdf5971a9e64fac8bbffec2825a5f97,843fdb78b8854bd48e503287f795a7d0,8d369ea660e04241a6c88d75c49a68ce,a90ba79ffbcb40dfa705fe4992e29d22,624c57fb83a54ff686c1e686bebde4c5,78bd8a7958a84617a06bcf72837eae0b,900f410b044542eeba2382d18f5d2704,6fa1a6734aeb4237a22b61b10f0affb8,62,b6fa4abb624349918fe531274fc15d23,89,76bdb735faa34667a6cbf13ebe937533,90,f5b2028cfad9470085c7c846de33193a,28abe4b0cb3b4c8480f84c07a5dfaf06,c7fa36c7142f481397c2cc12e2bc828a,e2efcf80fc3747329fe02d489dbfda95,e3b5b7451e164089aba0b9bf844b0398,438c28c3cbe44ed7bff951b2a3dc77cd,59e50eebe56743a78eecbc6eff1ef94c,bffd7de1c574450bb48484d1be66b197,cc1027689aca41a89302217127253034,200fe6cc80174065b382c961a4379d79,33fbe92e979b4260afb86c2db8f8fa2c,7d940fcac02248ef8074311e268a3a31,892f39ada6be4026a412eae188d7a5b6,4c13ca96376341dfa25980e86964c1a9,638be8b3b5284e5088003917685bc19b,811600aadcb54b098b7dda4bf95ffc51,ce8dc7c97d93481dbd76cabb6e905b55,aa25b3904abb4bd7b779e5be37b4382e,aae6f4c7fd4c4a25afa5285210edfded,84ff9d5c165c4528af9361b6127cf5af,31b349a3624b47f28eb86d614f9f95b5,040b42e395de41d6805a958eb1897863,34874260e6fc4200a76b33c3cfa4d80d,5b6c6ea5ed2d45478e8144351d853e21,7c032c32512847f882a87416359ef0a3,6d30ea8c710a42b89da2f2506059b0ef,6ac94964592a4dcf937a6027a4df5f98,a0970d1e021d4d6882e307dfc2a8e945,8aa598e4291a4d9591213c24c6427fa2,2ca7bf4ce6804a1c9e5b331d2df27c72,7717cbb6d71f405d890e9b648cbd2088,2fc21a3364f545358510b2dafd1c4095,3c61d9d5a71e4b7faa97db881b2da6bd,4477d2be9bf143c299c1b53f908e8f1c,bddb0400764c42b08e427059c7752057,23d6c56239804053910e5c5aec481330,7f8b2018113545bcbdbfc87e7a236bec,547e121ffb714d16ae142b6935abb78f,af24101b962444f29d166b638b2fea30,6ea6519ec4d548128aa42f3ce004def3,4379bf54c2a543869abf5031b97b8efe,5e6da3f5368b4f06b7268b31a1157e9f,2bbf0e75f21d42938954391d0776fbde,545b39dacf564a5eb9d1658f80a0b8b0,b4ddedf9b451494a97a1ff9476e3897d,1b145ca23d3e4d81994a84d30a262d18,10a8c7c1d34e46dd8ef461cdad928afc,5ae4911d14bb48b39885b1fe522e999d,7f09def2533b45fdb3c733f1e72ca5fb,9ef29dca19d342b795efcbfe32793602,be72d72ae7034c509c164467b2abddda,ab879705f2354813acf8724404520e28,82487b87b97b475b98892b80cf83c96a,0f4a461b97be48eeab43bd052f630057,72e1a28b7251438a89b0a4f6bd522b89,e1f9a246057347d78a1966d6dc0168e3,5d3645d53ce14c9ab498fd9f050ba264,d361e6890cc84b049fc8268c2a8c4952,5cc0002cbcc14d3485ab8ae6d57368f2,fa7bb243643c44b18e989e3419eeb171,62768b6c65134f1e927b1807bc72115a,7b69b0e5d2d344e6aaa2b8726c23561a,434bbb1f9f8140b1beeb57d0c4db660d,7920c4c8da294d87bcb94e7756bf35ae,0752b85660694033bbdb013f30cde17f,bd6258cfbea64896b0401517e440d6a2,9aa3539682514546866d2998d741ac6c,eb3ad93ce6a44921b0490c842dd47d8c,9ce19042d497473c92d199a5e5a7daf0,1b3eb6ba64ea4589a8e2d42efa5d3f89,6b2dd5ff2a754a53a80798341a7d35d8,e53d754160f74d28b6d091829fc289f8,92d762bca040407bb4ce4b06a2cfa195,aa8a2b00622147259c85290f4d95b46d,0be3e487d874487fbfb5c49b52440a48,ce60b25f7da04af8b7cefccf2dbc7d36,6464ac9cc3fc47d88598f6c4e74caa19,04b312df403d4855b013743206ddb1de,17356c44ae894782a1015ec2ddaeaeda,a06eaefba7084e219f0c6ce4010b82e7,95a54e4db7e74f0586cd85cb4e2bd6f7,1a93047d6b0f4e3ca70fcf836b2517c4,93263a29941648979bb8de774978a880,8eba834a28864375aa3137590d26f37e,3c52b0a890524c6fb66cefc32114c60e,8b20519b794c42d28fdecbd0ccbba5c1,2d74c531c4d14056973fd177652e7dc4,1a9e2d47fcfc4a5cb6ea0b2bb4bf24f9,4124a6a03e03480eba58e00a97667815,77189ceba655488681964a609c28256d,39e776aaf3644d24bb78ecf43704b71b,0dd5fa1192e34d5fac5e78ca38969126,1a6f82a80a054bc7ba4b516187231a54,5993ce4ec80a41fb9d4ea0713b74f4a4,3b42071536124a9c9dbf2ec319004b29,a966beefe58e4f95b24e89f4172cb7ce,2e1e75dbe28841529576dd1cd48d2437,cdd8829e5976438aaa66ce608f0175f5,b9312c6148b54697ad2510d33f081df4,71015695cf1445ecacce90669c67d733,31e9dd26b7314d8a9bf97dcf6dbdb22d,e5ecaa091bb0464f8553d4e7aa60647d,39055be3a23945868dbf4b5930043a88,90a22ce7f0254ecba4f90a6fb7c15763,a6a7913b5b714ae49f5026a7dacfa4cb,b76d1d2fbfa34db7852eba3845748130,07ba7c9aed104657aac3be87aa346d3b,5a7a9bfff0f44a2089a7d277f2c04557,be53cc152339400ba705c1d1dceff9ab,ef56135adf7445e59ed8f41947e7a72c,ba14a56dcce149a48379b32df5f5bd8f,ab4bd14c2a394c26a201df1d753bf02d,ed591ccba0564abd9979899049dc7b36,9a4425b817e94dbe844d7fd3e96a9033,bf4e54bfe9984441a8efa4dd8a4a23a6,5a1bcf2a2e0943f88bf655ca011fb288,a12c8904ff0147a99010eaa5aa0ea704,e7c6f2581c6e41bf8fe88a3d7fadc986,89709767beea48d0b0fb047316eb37d1,04d6a49b35514f91be43e9576f8d32fc,bd50e783ec714e5d86aafcca54e87c33,4acb0c7d002b47e29be7ebe9f40490f6,67b5ddef866b4982aa5afef7aad69924,7e090632f713440bba11ed6fc09eb291,f308051e11ca47eebb12709725b52eb4,dbea976ef1d248cabf13b88d311c4533,e0132620d4044e38a92f75b8335005ed,1b598563135e42d4bfe729cf32649f14,c4de9219f4f14684ae21d0904b7b78df,520983ab7dd64104a47e273941236b2d,133ba69bd04046b89eadaf793b9063db,11f24ca662044292a8ca95930ccc3581,0f30b4d621294d4fab359611cf82ab6f,b3c93fcd54b544e5b2c8e70ee7eb93ce,79ebe6595ce44208853ba553186b8348,8d639afccbec43f2be8d3ed88702893b,4e3483f3bbbb48f59b0e92a7d0ccc1c0,b160fccc15fa4ba09c74018d82d16eb7,52f248eadea74cf2b7780bf991d95e4e,d4cb9017aa1f4f8ca34ae8d31c986210,eff19819b1284424a9ad7879991ee080,79551897f86b4338bd09b41c5953a7db,f11bd4ef5ddd4c82865fad3f1f9d80a9,76ee95093a8748b29d55df1cb8cdebd0,081af742f6384a20b492ca99704f51ca,c26c844bbb2f4746904948f3ecfeb1ae,ba9bbc5581a043d8b9c204b8ddb6aa01,cde1afb048a743bb9caedb5770aabca1,0b2b37a28bb0457bbc4bf33c11b4c6e1,9f94d338d6c94b95814b2c50f9e463d1,7d1745ce2c444db9bf4b4e100c476fe8,e711e93aadd54eef8feeb0fa07f054d2,0e914774210646f784d02fb22f8df7db,ca0ce5e9eb09477c8c99b7b94e7869ce,760d9072f9bc480fa36fdb876930467c,5742a628f13045cda0317c3d4d839bc5,b9309305d94f4ee8b66eebbcae9f2db4,a42216d3eedd48f0ba3d3bef2470605f,9cdcda30da704c21b19ea0616de3ea86,f05ecbf7e4b04b268b38cb6e5c8124bf,788d2c1496874eeeae3b7aabbab95f76,8b112af90e054671adc30373c6dafd21,3bdb29667807493292a0ddc9bc0ccdb7,ffd6ebecc869410dab6410a07f3362ae,196868a4610b45588062aed357fc7c58,4cdd2d11be4c4c098b27c10d7cb3fa9e,fda40125207d426dbf86270002ff0c4a,7af028c6048c44cc91415a58d25d286e,ff7d2c8e96e840e1a5a690e4e0f96bad,5abae35db6de475e904425af8dc8ecbb,e3bc44b82c3543758d5a6c02b41dd526,89323021918e444bae3537521046d19f,141dd3257f85455d86f3828cc0217b5a,7bc2ad90009a4de2a0fa2947ccc09886,e51874f4b6ac455db76e30582659368b,519dee074e4c48ae9edd58d7c1e330da,a4816b8b30e341d296669f627f23ade3,93a9760729eb4ef187d60fbad59e3d31,e980504789844b2f96b417bf439fa964,f5c6bbf5b71d44749af30b795ef8c222,05e1cca3da5340f59db876bb238287f3,060ab186c22f4f42a83f1622bf47ca52,8873bbcf4b42445f8117108769358a5c,46fa5286441a444095b917165c31304c,fa9fdd56222a4ce7ba2b37922afe0eaa,6885932aa4b443babc2b69fdd495889c,be71b947b33a48d8aee36ad0c60d10b8,b52457719429466687a2f228d20571ac,723592e24a3d48bdb2ef109b8ee46837,0bf937d745494d998cb8bd2cf3a63f4e,c17feb3860184dd59931f35814ad68b0,4cc68cbd880c48948a1f18fe18ec938c,c37b0691a05c4d659dcd01571f1907da,c1334fe3e8484079bfd62e479307b5b9,ebb58027b6c647b7965eeb31b63ec0c0,720fc5746f7b4e5984bbc8d1aaeca215,cc4be4c10bd949408e4c8c29c9b842ce,88d963bd3d404bd0895c136f5012a1de,8dc4d97f0ac64509ad6cc5ba298e0a5b,73c96c78c81e48399d16a67923e1f651,084471d4393044d4802e3e27f71988f3,bd53b341c57e4b63b7444a1082339c2d,d6658a5ee06c4036ab9bdc8b553255a6,2c5ea7d85c164bc3b4545d9c46334488,9b3c62a0af4c4cac9deb415baaf04fa5,22d943bb9e614ca69148743ff28e2362,f635b5ed489b401986c95cd8e5430bbd,2b014748b6714cb988b1c4142d3c0204,cc4716f1ba934c18848061b690a137ed,d7d80fefd53943efafc88f3768a6aff2,b692b96453204e299753df8f0751f4ec,1e66a862627f40e0b9470dea5220308a,1a098f10396f462a95d20d712722973a,67fdc15ef4f149ec8dfda74f0e1612fc,474566ddb2be4de785e8a920530cc320,47f42a430d3f4dc1872f8cfcc628b527,be5027b456b84bbaa5fa016254122089,b3b39abe13e54a3c8727cdb6387e1a1b,7b4205ea12c34b28aea4806f2e26104a,342f70da525349e6b86a120cbddf9e5a,a9a79b1171e3410291ae7d0ec68f1c8b,d8008fa60f6d4c279cac795cc2944ded,78a51382773b4a7fbac46beee2846edf,a01c30e0521d4082a7e2edd25c3fa476,c525320a19c64f4c97afdc50d9f90348,4897d1f2afe14aadaa342a615e50e69a,f43e7af70a4344489cd7a2b000af0a66,a6e1e26f26734705a6b599f4772de663,4b1985e929f54406ae345b9035c8526e,aee0c1fdaab84ce2889424f8323c243b,165eb05677aa45fc9a53dfd8bfb7a45d,409891afd7cc4e12ba6b002581fef4f3,02ee0a87956f4bfab91b88282e502a3d,48601418028145ff84a450c877ea35aa,8de73f8d59eb45aa8c61138ab93c4f86,5dd82a46f48945b4bbb61f58b9d1c856,97993db997bd4ff88acfe1340ca86845,282aee84915b43179a09101b3ee6ad0e,8750e994985f43f383321bf0d6ca7932,3,4,5,6,68f9151151174868ab436e11e03bf548,79f0ffa47dbe43ffa8824d97612d344f,ec7cf7a144a440cab217aabd4ffb7788,20,21,22,2a0f940fbe304a05a6b4040ddf6df279,79fca849d3da4a82a4ade3f6b9f45126,f34887a78fa245c1977603ca7dc98e11,b9a776f5d7194406bb466388e3af9d08,17,18,19,a4c3dcee6cbc4fc6a0bf617d8619edf3,8930e4aad1ba4a1c958d303968d8c442,4da8be6fe3e14a8aae57583402e6934f,b7d20f0244084f41a041fcb7ef85db8f,fb46733b041949d0a35cbe7951f71b89,d5d57e587eec4398bb13b8861afd92b9,b76e8723777d4f979861a4431b4282e6,7,8,9,3c1c639c76f14f6f9903b0143371ea09,6509eed6eb634030a46723a18814035c,f93f9a3a2226461dace3b8992cf055ba,de7c50d276454f80881c41a096ecf55c,14,15,16,c6e0080e06014abd9240f870aadf3200,5239527958e94d418997b584b85d8b80,10,11,12,6d3a6777693f47c98e9b3051cacbcfdb,9bc1aa1053144a608b73f6fbd841d1c6,c0053ff4df384b129307dafe502e723d,7e33cac762cc46d784cedf970344fa47,46ba10003e70488ab46ae7faa81d5334,8c3d779f0bec4f47b7b5232a44acc0a9,6d3fa8801e6b406ebc846a9d6457573c,f4ecf5d2654c46769057362ebfb21afe,402c4c50ebdc476795b75231de699571,4e37cb62b4a34f52a304ddc9d9147905,648ad6abfff24af396fcec1d8d05a856,f133ae5367914b2892459d03012720b3,db8dea379c8e4333bda690d4d096b331,26a3901d047241aeb4a413ae1c05ee3a,a05ddf1532e049179cb68e2ce2322f1e,4f56225f24294a90853d70556d763f6f,ab08526072e74d4888a8a081db9548b4,67713a8a2024414fb34ab22f1dce088c,a6c17ad4f46e452fb16824a65aacd2ee,52595fbde8cb41b3920e286cb7821fba,b32a8d8f1e6a42fcb0300049cd985a3f,2c67ca46306a41eb93d29e26c3ea0cea,0f354016b42b4b73bc65b1c88a412662,7ae1473d3bfa47e980545f2a27d6aec0,fdc01f6b4f98465ca133d78f431461e2,5b99fb57fd5e42d48f1a11b22ba49c90,eb081fd3c96a4787bfa0ff42be64cfc0,01dde345bc3e4e6ea1c90169ccb66c26,7f23f2ca985d44f7a96918f499715426,23202589fe45431799af1fe05cba3a24,0ad156f7d5c44aee9028e032ea2b2b23,105dc6beaa6646f9971ab14f8c957521,f6efef2068b24205a911c97122403cbf,75c1cf011e714a6095d1486a4fcc62d1,c0d648b4ff514fa3ad6ddf620d6c7e2d,e909925ac4284251b38d6e05ed52fea8,000128bbf191457a9b4bcc337443ec06,8e1206a37db2464d8d08221928da6d16";
        role.setMenuIds(defalutMenu);
        String[] defalutMenus = defalutMenu.split(",");
        ArrayList<String> menuIdList = new ArrayList<String>();
        for (int i = 0; i < defalutMenus.length; i++) {
            menuIdList.add(defalutMenus[i]);
        }
        role.setMenuIdList(menuIdList);
        /*if (role.getSysData().equals(Global.YES)) {
            if(!UserUtils.getUser().isAdmin()&&!UserUtils.getSelectRole().get(0).getEnname().contains("gly")){
                return null;
            }
        }*/
        if (Global.isDemoMode()) {
            return null;
        }
        /*if (!"true".equals(checkName(role.getOldName(), role.getName(),role.getCompany().getId()))) {
            return null;
        }*/
       /* logger.info(role.getOldEnname() + "--------------------" + role.getEnname());
        if (!"true".equals(checkEnname(role.getOldEnname(), role.getEnname()))) {
            return null;
        }*/
        role.setOffice(office);
        systemService.saveRole(role,true,1);
        //roleDao.insertRoleOffice(role);
        return role;
    }
    //添加分公司岗位
    @Transactional(readOnly = false)
    public Role createCompanyRoleAND(Office com, Office office) {

        //自动生成管理员岗位
        Role role = new Role();
        role.setNumber("0");
        role.setName("分公司管理员");
        //	role.setSysData(Global.YES);
        role.setDataScope("5");
        role.setIsNewRecord(false);

        role.setOffice(office);
        role.setRoleType("security-role");
        role.setCompany(com);
        role.setSysData("1");
        role.setUseable("1");
        role.setEnname("fgly");
        role.setBranchCompany(office);
        //role.setEnname(StringUtils.getPinYinHeadChar(company.getName(), 2)+System.currentTimeMillis() + "gly");
        String defalutMenu ="1,27,29,e3b80fae59ab4b4099d8349ecaf8c4dc,61dd3aa279684f92bb1c5d4019a32503,b0c5737cdc4f452684ce93589427c97d,af6b0e5d61fc44ecbebb449703a3df72,18c14562978e4b129fe3afb7dca8b868,6123fcb3d810492789e82a63b0d4310a,0aa3712414d049a6a24e8bcddeae509a,95a6a82dc5fc4d07b46e5df57a0606a3,62,b6fa4abb624349918fe531274fc15d23,89,76bdb735faa34667a6cbf13ebe937533,90,f5b2028cfad9470085c7c846de33193a,28abe4b0cb3b4c8480f84c07a5dfaf06,c7fa36c7142f481397c2cc12e2bc828a,e2efcf80fc3747329fe02d489dbfda95,e3b5b7451e164089aba0b9bf844b0398,438c28c3cbe44ed7bff951b2a3dc77cd,59e50eebe56743a78eecbc6eff1ef94c,cc1027689aca41a89302217127253034,200fe6cc80174065b382c961a4379d79,4c13ca96376341dfa25980e86964c1a9,638be8b3b5284e5088003917685bc19b,811600aadcb54b098b7dda4bf95ffc51,ce8dc7c97d93481dbd76cabb6e905b55,aa25b3904abb4bd7b779e5be37b4382e,aae6f4c7fd4c4a25afa5285210edfded,84ff9d5c165c4528af9361b6127cf5af,31b349a3624b47f28eb86d614f9f95b5,040b42e395de41d6805a958eb1897863,34874260e6fc4200a76b33c3cfa4d80d,5b6c6ea5ed2d45478e8144351d853e21,7c032c32512847f882a87416359ef0a3,6d30ea8c710a42b89da2f2506059b0ef,2ca7bf4ce6804a1c9e5b331d2df27c72,7717cbb6d71f405d890e9b648cbd2088,bddb0400764c42b08e427059c7752057,23d6c56239804053910e5c5aec481330,7f8b2018113545bcbdbfc87e7a236bec,547e121ffb714d16ae142b6935abb78f,af24101b962444f29d166b638b2fea30,6ea6519ec4d548128aa42f3ce004def3,4379bf54c2a543869abf5031b97b8efe,5e6da3f5368b4f06b7268b31a1157e9f,2bbf0e75f21d42938954391d0776fbde,545b39dacf564a5eb9d1658f80a0b8b0,b4ddedf9b451494a97a1ff9476e3897d,1b145ca23d3e4d81994a84d30a262d18,10a8c7c1d34e46dd8ef461cdad928afc,5ae4911d14bb48b39885b1fe522e999d,7f09def2533b45fdb3c733f1e72ca5fb,9ef29dca19d342b795efcbfe32793602,be72d72ae7034c509c164467b2abddda,ab879705f2354813acf8724404520e28,82487b87b97b475b98892b80cf83c96a,0f4a461b97be48eeab43bd052f630057,72e1a28b7251438a89b0a4f6bd522b89,e1f9a246057347d78a1966d6dc0168e3,5d3645d53ce14c9ab498fd9f050ba264,d361e6890cc84b049fc8268c2a8c4952,5cc0002cbcc14d3485ab8ae6d57368f2,fa7bb243643c44b18e989e3419eeb171,62768b6c65134f1e927b1807bc72115a,7b69b0e5d2d344e6aaa2b8726c23561a,434bbb1f9f8140b1beeb57d0c4db660d,7920c4c8da294d87bcb94e7756bf35ae,0752b85660694033bbdb013f30cde17f,bd6258cfbea64896b0401517e440d6a2,9aa3539682514546866d2998d741ac6c,eb3ad93ce6a44921b0490c842dd47d8c,9ce19042d497473c92d199a5e5a7daf0,1b3eb6ba64ea4589a8e2d42efa5d3f89,6b2dd5ff2a754a53a80798341a7d35d8,e53d754160f74d28b6d091829fc289f8,92d762bca040407bb4ce4b06a2cfa195,aa8a2b00622147259c85290f4d95b46d,0be3e487d874487fbfb5c49b52440a48,ce60b25f7da04af8b7cefccf2dbc7d36,6464ac9cc3fc47d88598f6c4e74caa19,04b312df403d4855b013743206ddb1de,17356c44ae894782a1015ec2ddaeaeda,a06eaefba7084e219f0c6ce4010b82e7,95a54e4db7e74f0586cd85cb4e2bd6f7,1a93047d6b0f4e3ca70fcf836b2517c4,93263a29941648979bb8de774978a880,8eba834a28864375aa3137590d26f37e,3c52b0a890524c6fb66cefc32114c60e,8b20519b794c42d28fdecbd0ccbba5c1,77189ceba655488681964a609c28256d,39e776aaf3644d24bb78ecf43704b71b,0dd5fa1192e34d5fac5e78ca38969126,1a6f82a80a054bc7ba4b516187231a54,5993ce4ec80a41fb9d4ea0713b74f4a4,3b42071536124a9c9dbf2ec319004b29,a966beefe58e4f95b24e89f4172cb7ce,2e1e75dbe28841529576dd1cd48d2437,cdd8829e5976438aaa66ce608f0175f5,71015695cf1445ecacce90669c67d733,31e9dd26b7314d8a9bf97dcf6dbdb22d,e5ecaa091bb0464f8553d4e7aa60647d,39055be3a23945868dbf4b5930043a88,07ba7c9aed104657aac3be87aa346d3b,5a7a9bfff0f44a2089a7d277f2c04557,be53cc152339400ba705c1d1dceff9ab,ed591ccba0564abd9979899049dc7b36,9a4425b817e94dbe844d7fd3e96a9033,bf4e54bfe9984441a8efa4dd8a4a23a6,5a1bcf2a2e0943f88bf655ca011fb288,a12c8904ff0147a99010eaa5aa0ea704,e7c6f2581c6e41bf8fe88a3d7fadc986,89709767beea48d0b0fb047316eb37d1,04d6a49b35514f91be43e9576f8d32fc,bd50e783ec714e5d86aafcca54e87c33,4acb0c7d002b47e29be7ebe9f40490f6,67b5ddef866b4982aa5afef7aad69924,7e090632f713440bba11ed6fc09eb291,f308051e11ca47eebb12709725b52eb4,dbea976ef1d248cabf13b88d311c4533,e0132620d4044e38a92f75b8335005ed,1b598563135e42d4bfe729cf32649f14,c4de9219f4f14684ae21d0904b7b78df,520983ab7dd64104a47e273941236b2d,133ba69bd04046b89eadaf793b9063db,11f24ca662044292a8ca95930ccc3581,0f30b4d621294d4fab359611cf82ab6f,b3c93fcd54b544e5b2c8e70ee7eb93ce,79ebe6595ce44208853ba553186b8348,8d639afccbec43f2be8d3ed88702893b,4e3483f3bbbb48f59b0e92a7d0ccc1c0,b160fccc15fa4ba09c74018d82d16eb7,52f248eadea74cf2b7780bf991d95e4e,d4cb9017aa1f4f8ca34ae8d31c986210,eff19819b1284424a9ad7879991ee080,79551897f86b4338bd09b41c5953a7db,f11bd4ef5ddd4c82865fad3f1f9d80a9,76ee95093a8748b29d55df1cb8cdebd0,081af742f6384a20b492ca99704f51ca,c26c844bbb2f4746904948f3ecfeb1ae,ba9bbc5581a043d8b9c204b8ddb6aa01,cde1afb048a743bb9caedb5770aabca1,0b2b37a28bb0457bbc4bf33c11b4c6e1,9f94d338d6c94b95814b2c50f9e463d1,7d1745ce2c444db9bf4b4e100c476fe8,e711e93aadd54eef8feeb0fa07f054d2,0e914774210646f784d02fb22f8df7db,ca0ce5e9eb09477c8c99b7b94e7869ce,760d9072f9bc480fa36fdb876930467c,5742a628f13045cda0317c3d4d839bc5,b9309305d94f4ee8b66eebbcae9f2db4,a42216d3eedd48f0ba3d3bef2470605f,9cdcda30da704c21b19ea0616de3ea86,f05ecbf7e4b04b268b38cb6e5c8124bf,788d2c1496874eeeae3b7aabbab95f76,8b112af90e054671adc30373c6dafd21,3bdb29667807493292a0ddc9bc0ccdb7,ffd6ebecc869410dab6410a07f3362ae,196868a4610b45588062aed357fc7c58,4cdd2d11be4c4c098b27c10d7cb3fa9e,fda40125207d426dbf86270002ff0c4a,7af028c6048c44cc91415a58d25d286e,ff7d2c8e96e840e1a5a690e4e0f96bad,9f806cf868dc48d398836ebd60bd2ec6,50ff5cdc6c8645b3bb61b8327d204f01,bafb57ea8d84430cbc0c11e5c67643f1,e2c91860b96c46619a74215627560235,f5b68a43bbae44c8a1fe821dfcaca26e,70b771772a0b4f1cb1e592f9a01b08a5,7992a39d64b2483789ae274e2df3c01d,f8d711312da14871a77b58f41a6f6242,673b27b0ab9148b7ae03933c18433971,624886a950bd4d18900a877738696432,685d8d9267f745f39bcc6fc2cc553d31,7d40ad05a1a54165a28ec897ddbcef01,4bcdb5199e8442078999bb8611ea6398,3b0c0ffea0cf45db816993baf64b3fb3,ad2d6d9581034e04addcb5b034a88fb3,8440c4fc5056426996e3e488b2261d28,e367c4223d794572a05fd256759d6a53,1b2dc5784c3c41988d5634e566b4a404,11be1538d7f54d7786004503b2e684a7,b462cb8c2ec7485d938416831a9771cc,f747656a36204e9588aba97064332436,78d0b2337b484c6298c9b3afc01fdb85,8a63d1f8d4bc4eb4a4db437491c9983a,2247f64a15e54053abb2e13152003c6b,d78ef9c5997d4e94804b8126cc387ea1,6ce3cd5a86ef43219dbda2b78e974c93,414b5fd5ebee45d98eca809a524dbe93,4b9ea9db5c834b698a78c7052c87b432,8590fa7adc144f49ae3d2575b7280c08,6148e4b152b9494fb9e7035e9b58e624,be1cff2a29f64800a5c4f91009711988,49b85b35bca5461993eac91ffd0b1fe4,5abae35db6de475e904425af8dc8ecbb,e3bc44b82c3543758d5a6c02b41dd526,89323021918e444bae3537521046d19f,141dd3257f85455d86f3828cc0217b5a,7bc2ad90009a4de2a0fa2947ccc09886,e51874f4b6ac455db76e30582659368b,519dee074e4c48ae9edd58d7c1e330da,060ab186c22f4f42a83f1622bf47ca52,8873bbcf4b42445f8117108769358a5c,46fa5286441a444095b917165c31304c,fa9fdd56222a4ce7ba2b37922afe0eaa,6885932aa4b443babc2b69fdd495889c,be71b947b33a48d8aee36ad0c60d10b8,b52457719429466687a2f228d20571ac,0bf937d745494d998cb8bd2cf3a63f4e,c17feb3860184dd59931f35814ad68b0,4cc68cbd880c48948a1f18fe18ec938c,c37b0691a05c4d659dcd01571f1907da,c1334fe3e8484079bfd62e479307b5b9,ebb58027b6c647b7965eeb31b63ec0c0,1821c5f9392e4859a829c78ebda44df6,dca21ab79bd14004b94c55e36604c30d,8d96cc3457644378b9b643243464f735,631437054079401fbe2c3c9a3f3bc375,c2f78ab4372243da8775f4d71a16fef5,03792664a45d4f63ac3efb899ce1b61e,fec04c46683a41e5a344f4d057342829,320fa70de6bc4890a0135bbfe8735b49,5a03493dce9f4323a0da7bf4fd5e78ab,b9bd46a1da1846d099566fd5c6db00cb,647021c7f798472889fd82f6c4a27ca2,313e905b78d746cb9e574b9634c3a113,623fd328c4cd4ee8b21eb03425b1f100,4676b4d1130146629fd98e2d2eb9a5b4,34d380813a36481ea2534c46d18b5927,894d87e55ac640b99e966a8786a2392b,477ba74d182a4ec0b86926dc67a0f419,362ca5fdca174fa3bdd6325a947320db,1ac3ad8dfa184fe4b6815950d3d23fdd,fdebee18c71f4a9db60096e75ad34d83,91432be4178a4781917fba0abd51df61,480c71baa65145f4bd037332d68e5e4f,536e3a3702b0453d92939846dc1f33b7,0333aedf1be640c78d2ee02473b7fd97,cbb3a6f1772b4f72a8af1375ef16e914,367ad9c43a6246dfac364ca59874883b,bac18430f6db466195a3a7116cdfb958,c4ec39be11be4b0eae321ee0559be8e6,8f7ffae7e47d462aa4142db7f06633d5,82e3b2423d86425195929fc3810ce7a4,72973e0415004dfea220babc479be082,41d0f568bb1e4fed8ad965a79d9bccef,5854ef1d24784ca0a046514b19286113,04afc3dd1eec4d13bc06cd01e0a5e838,db76e3333da846d5ae55b3b9dd735f26,7aa0ca0bf1514fa6a3a14dcde44cf0b9,9c668726c9b6449a9045d3f78408a2a1,7caf148d68a6431bbe56bcc6472d8720,babd36d1d3484fb99fe4e7ca78560ccf,7507865343e24cff80d55a29ec57f0e1,63f8b7d9b1f34851802e0408bf997ea5,a3369d5dd6b74af7b35edf1f10350881,9b8b8c3f86854416b82d57583896c1ff,881815c772564a8dbd5eb6de65ecbe7a,4b1bcf46753a48258d1d514440140b7f,1ca4c6a49e794954a235a82ee8981ecd,ab381d9129bb4b979fe3774e262524f8,a07d2c98831a4d79bc6c5c4288a24720,d8debd28039d494aa7fe18bc8e81af27,d0402b1f00f94acca96276c6ea8c8397,ebcdebd9639d4ec2b7072ad32bf48f50,636b3211100245919c09177ccea562e0,280c4440e18a487396a81b92cd3efa13,5163cf6740334a7d83375385c3ae2b67,a6083e4fab7a43498bfb3257658f9278,3b5c8b5cfb124ebd8f4a798d2a47d832,9fda332474534f96955b5bce8548947f,b5f5e62aba9842d8b48a28111ca6a372,c0cecff1207c4692a634d7de449d8d6d,e4c6428f1d264544b0eec52214e7b6ae,cbc8c5490a7f46b996fd4c3a9711f2d7,cfbdd82f4ce741a88109353c5fc898e4,6bfb965bc1b240a9ae2bd7a1e884ea51,eb36d63a993148289950af9098727759,f7357c3954d2408facaa6771a758ebcc,c05853b94ab8405eb26e0a8bd481b662,da1e6d3ebfa140b3a645342fd3bb5d95,988f3f6b607d442b8df006c980ddc2cd,b6af3c01968141baa3e99ca4b8d83847,75f669faee5d4023a425f2461e5fc5e9,165eb05677aa45fc9a53dfd8bfb7a45d,409891afd7cc4e12ba6b002581fef4f3,02ee0a87956f4bfab91b88282e502a3d,48601418028145ff84a450c877ea35aa,8de73f8d59eb45aa8c61138ab93c4f86,5dd82a46f48945b4bbb61f58b9d1c856,97993db997bd4ff88acfe1340ca86845,282aee84915b43179a09101b3ee6ad0e,8750e994985f43f383321bf0d6ca7932";
        role.setMenuIds(defalutMenu);
        String[] defalutMenus = defalutMenu.split(",");
        ArrayList<String> menuIdList = new ArrayList<String>();
        for (int i = 0; i < defalutMenus.length; i++) {
            menuIdList.add(defalutMenus[i]);
        }
        role.setMenuIdList(menuIdList);
        /*if (role.getSysData().equals(Global.YES)) {
            if(!UserUtils.getUser().isAdmin()&&!UserUtils.getSelectRole().get(0).getEnname().contains("gly")){
                return null;
            }
        }*/
        if (Global.isDemoMode()) {
            return null;
        }
        /*if (!"true".equals(checkName(role.getOldName(), role.getName(),role.getCompany().getId()))) {
            return null;
        }*/
        /*logger.info(role.getOldEnname() + "--------------------" + role.getEnname());
        if (!"true".equals(checkEnname(role.getOldEnname(), role.getEnname()))) {
            return null;
        } */
        systemService.saveRole(role,true,1);
        return role;
    }


    /**
     * 验证岗位名是否有效
     *
     * @param oldName
     * @param name
     * @return
     */
    /*public String checkName(String oldName, String name,String comId) {
        if (name != null && name.equals(oldName)) {
            return "true";
        } else if (name != null && systemService.getRoleByName(name,comId) == null) {
            return "true";
        }
        return "false";
    }*/

   /* *//**
     * 验证岗位英文名是否有效
     *
     * @param oldEnname
     * @param enname
     * @return
     *//*
    public String checkEnname(String oldEnname, String enname) {
        if (enname != null && enname.equals(oldEnname)) {
            return "true";
        } else if (enname != null && systemService.getRoleByEnname(enname) == null) {
            return "true";
        }
        return "false";
    }*/

    /**
     * 验证手机号是否唯一
     */
    public String findUniqueBymobile(String mobile, String comId) {
        List<User> list = companyDao.findUniqueBymobile(mobile);
        if (list.size() > 0) {
            List<String> mobiles = companyDao.selectCompanyMobile(comId);
            int num = 0;
            for (User user : list) {
                if (mobiles.contains(user.getMobile())) {
                    num = num + 1;
                    break;
                }
            }
            if (num > 0) {
                return "3";
            } else if (list.size() > 0) {
                return "1";
            } else {
                return "2";
            }
        } else {
            return "2";
        }
    }

    /**
     * 验证手机号是否唯一
     */
    public boolean checkNumber(String number) {
        Integer count = companyDao.checkNumber(number);
        if (count!=null && count>0){
            return false;
        }else {
            return true;
        }
    }

    /**
     * 审批企业信息
     *
     * @param company
     * @return
     */
    @Transactional(readOnly = false)
    public String applyCompany(Company company, HttpServletRequest request) throws Exception {
        //创建公司1
        Office comOffice = this.createOffice(company);
        //根据公司创建管理员岗位
        Role comRole = this.createCompanyRole(comOffice);
        if (comRole == null) {
            return "2";
        }
        //根据公司创建人力部门
        Office deptOffice = this.createOfficeDept(comOffice,1);
        //根据公司创建财务部门
        Office deptFinance = this.createOfficeFinance(comOffice,1);
        //根据公司创建招标部门
        //Office deptZb = this.createOfficeDeptZB(comOffice);
        //根据公司创建市场部
        Office deptSCB = this.createOfficeSCB(comOffice,1);
        //根据公司创建办公室
        Office deptBGS = this.createOfficeBGS(comOffice,1);
        //根据公司创建质量监管办
        Office deptZLJG = this.createOfficeZLJG(comOffice,1);
        //根据公司创建业务部
        Office deptYWB = this.createOfficeYWB(comOffice,1);
        //根据公司和财务部门创建财务的岗位
        List<Office> offices = new ArrayList<>();
        offices.add(deptOffice);
        offices.add(deptFinance);
        //ffices.add(deptZb);
        offices.add(deptSCB);
        offices.add(deptBGS);
        offices.add(deptZLJG);
        offices.add(deptYWB);
        //根据公司和人力部门创建新员工的岗位
        this.createDeptRole(comOffice,comOffice, offices,"1");
        //根据公司和财务部门创建财务的岗位
        this.createFinanceRole( comOffice,comOffice,deptFinance,"1");
        //根据公司和财务部门创建主管会计的岗位
        this.createFinanceRoleZgkj( comOffice,comOffice,deptFinance,"1");
        //根据公司和财务部门创建出纳的岗位
        this.createFinanceRoleCN( comOffice,comOffice,deptFinance,"1");
        //根据公司和财务部门创建出纳的岗位
        this.createGZRRole(comOffice,comOffice,deptBGS,"1");
        //根据部门创建部门负责人岗位
        this.createOfficeManagerRole(comOffice,comOffice,offices,"1");
        //根据公司和财务部门创建投标专员
        this.createTBZYRole( comOffice,comOffice,deptSCB,"1");
        //根据公司创建分管领导岗位
        this.createCompanyLeaderRole(comOffice,comOffice,offices,"1");
        //根据公司创建归档管理员岗位
        this.createCompanyGDManagerRole(comOffice,comOffice,deptBGS,"1");
        //根据公司创建董事长岗位
        this.createCompanyManagerRole( comOffice,comOffice,"1");
        //根据公司创建文件管理员岗位
       /* this.createCompanyFileManagerRole(company, comOffice);*/
        //根据公司创建总经理岗位
        //this.createCompanyManagerRole(company, comOffice);
        //根据公司创建总经理岗位
        this.createCompanyChairmanRole( comOffice,comOffice,"1");
        //根据公司创建文件总工岗位
        this.createCompanyZongGongManagerRole(comOffice, comOffice,deptZLJG,"1");
        //根据公司创建文件总工岗位
        this.createCompanyBZSHBManagerRole(comOffice, comOffice,deptZLJG,"1");
        //根据公司创建行政主管
        this.createCompanyHZZGRole(comOffice, comOffice,deptBGS,"1");
        //根据公司创建投标专员
        this.createCompanyRLZYRole(comOffice, comOffice,deptOffice,"1");
        //根据公司创建投标专员
        this.createCompanyALTBZYRole(comOffice, comOffice,deptZLJG,"1");
        //根据公司创建人事岗位
        //this.createCompanyHRRole(comRole,company, comOffice, comOffice,deptOffice,"1");
        Useroffice useroffice = new Useroffice();
        useroffice.setUserId(company.getTuser().getId());
        useroffice.setCompanyId(comOffice.getId());
        useroffice.setOfficeId(deptOffice.getId());
//		userofficeService.save(useroffice);
        comRole.setOffice(comOffice);
        User user = systemService.assignUserToRole(comRole, systemService.getUser(company.getTuser().getId()),0);
        company.getTuser().setCompany(comOffice);
        company.getTuser().setOffice(deptOffice);
        company.setCompanyId(comOffice.getId());
        this.save(company);//保存
        logger.info("company=" + company);
       /* Group group = new Group();
        group.groupname(company.getName() + "官方群").desc(company.getName() + "官方群")._public(true).maxusers(1000).approval(true).owner(company.getTuser().getId());
        Object obj = easemobChatGroup.createChatGroup(group);
        JSONObject json = JSONObject.fromObject(obj);
        String groupid = null;
        if (json != null && !json.get("data").toString().equals("")) {
            JSONObject data = JSONObject.fromObject(json.get("data").toString());
            if (data != null && !data.get("groupid").toString().equals(""))
                groupid = data.get("groupid").toString();
        }*/
       /* EasemobInfo easemobImuser = new EasemobInfo();
        easemobImuser.setUserId(company.getTuser().getId());
        easemobImuser.setGroupId(groupid);
        easemobImuser.setJoinStatus("1");
        easemobImuser.setGroupName(company.getName() + "官方群");
        easemobInfoService.save(easemobImuser);*/
        LayGroup layGroup = new LayGroup();
        layGroup.setGroupname(company.getName() + "官方群");
        layGroupService.insertLayGroup(layGroup,company.getTuser(),comOffice);
        //生成公司二维码，使用公司名
        String realPath = Global.getUserfilesBaseDir() + Global.USERFILES_BASE_URL
                + comOffice.getId() + "/qrcode/";
        FileUtils.createDirectory(realPath);
        String name = comOffice.getId() + ".png"; //encoderImgId此处二维码的图片名
        String filePath = realPath + name;  //存放路径
        String content = comOffice.getId() + "," + comOffice.getName() + "," + comOffice.getMaster() + "," + UserUtils.get(comOffice.getMaster()).getName();
        logger.info("encoderQRCode.content=" + content);
        TwoDimensionCode.encoderQRCode(content, filePath, "png");//执行生成二维码
        OSSClientUtil ossUtil = new OSSClientUtil();
        deptOffice.setCompanyQrcode(request.getContextPath() + Global.USERFILES_BASE_URL
                + comOffice.getId() + "/qrcode/" + name);
        comOffice.setCompanyQrcode(request.getContextPath() + Global.USERFILES_BASE_URL
                + comOffice.getId() + "/qrcode/" + name);
        deptOffice.setGroupId(layGroup.getId());
        comOffice.setGroupId(layGroup.getId());
        deptOffice.setGroupName(company.getName() + "官方群");
        deptOffice.setLogo(comOffice.getLogo());
        comOffice.setLogo(comOffice.getLogo());
        comOffice.setGroupName(company.getName() + "官方群");
        if (deptOffice.getCompanyQrcode() != null && !deptOffice.getCompanyQrcode().equals("")) {
            InputStream inputStream = null;
            HttpURLConnection httpURLConnection;
            URL url = new URL(request.getScheme() + "://" + request.getServerName() + ":" + request.getServerPort() + deptOffice.getCompanyQrcode());
            httpURLConnection = (HttpURLConnection) url.openConnection();
            // 设置网络连接超时时间
            httpURLConnection.setConnectTimeout(3000);
            // 设置应用程序要从网络连接读取数据
            httpURLConnection.setDoInput(true);

            httpURLConnection.setRequestMethod("GET");
            int responseCode = httpURLConnection.getResponseCode();
            if (responseCode == 200) {
                // 从服务器返回一个输入流
                inputStream = httpURLConnection.getInputStream();

            }
            String[] arr = deptOffice.getCompanyQrcode().split("/");
            logger.info("arr[arr.length-1]=" + arr[arr.length - 1]);
            String qrcodeName = System.currentTimeMillis() + arr[arr.length - 1];
            ossUtil.uploadFile2OSS(inputStream, Global.getRqcode(), qrcodeName);
            deptOffice.setCompanyQrcode(Global.getAliyunUrl() + "/" + Global.getRqcode() + qrcodeName);
            comOffice.setCompanyQrcode(Global.getAliyunUrl() + "/" + Global.getRqcode() + qrcodeName);
            officeService.save(deptOffice);
            officeService.save(comOffice);
            useroffice.setStatus("3");
            useroffice.setDelFlag("0");
            useroffice.setRemarks("管理员");
            useroffice.setMaster(UserUtils.getUser().getId());
            useroffice.setUserId(company.getTuser().getId());
            userofficeService.save(useroffice);
            //boolean b = systemService.outUserInRole(new Role("1"), systemService.getUser(company.getTuser().getId()));
            systemService.saveCurrentCompanyById(company.getTuser().getId(), comOffice.getId(), deptOffice.getId());
            SerialNumTpl entity = new SerialNumTpl();
            entity.setCompany(new Office("0"));
            List<SerialNumTpl> numTplList = serialNumTplDao.findList(entity);
            if(numTplList!=null&&numTplList.size()>0){
                for (SerialNumTpl serialNumTpl : numTplList) {
                    serialNumTpl.setCompany(new Office(company.getCompanyId()));
                    serialNumTpl.setSerialNum(0);
                    serialNumTpl.setId("");
                    serialNumTpl.setCreateBy(company.getCreateBy());
                    serialNumTpl.setId(IdGen.uuid());;
                    serialNumTplDao.insert(serialNumTpl);
                }
            }
            //反馈审核结果给申请人
            sendNotifyToUser(company);
            //将申请通知设置为已读
            WorkProjectNotify notify = new WorkProjectNotify();
            notify.setNotifyId(company.getId());
            workProjectNotifyService.readByNotifyId(notify);

            return "1";
        } else {
            return null;
        }
    }
    //反馈审批结果给申请人
    private void sendNotifyToUser(Company company) {
        WorkProjectNotify notify = new WorkProjectNotify();
        notify.setType("19");
        notify.setRemarks("待通知");
        notify.setUser(company.getCreateBy());
        notify.setNotifyId(company.getCompanyId());
        notify.setStatus("0");
        if (company.getUSEABLE().equals("1")) {
            //通过1
            notify.setTitle("您的企业申请已通过");
            notify.setContent("[通过]");
        } else {
            //拒绝2
            notify.setTitle("您的企业申请未通过");
            notify.setContent("[拒绝]");
        }
        //给申请人所在的所有公司发送通知
        Useroffice useroffice = new Useroffice();
        useroffice.setUserId(company.getCreateBy().getId());
        useroffice.setStatus("3");
        List<Useroffice> userofficeList = userofficeService.findList(useroffice);
        if(userofficeList!=null && userofficeList.size()>0){
            for(Useroffice u : userofficeList){
                notify.setId("");
                notify.setCompanyId(u.getCompanyId());
                workProjectNotifyService.save(notify);
            }
        }

    }

    /**
     * 添加子公司公司部门岗位
     *
     * @param comOffice
     * @return
     */
    @Transactional(readOnly = false)
    public String applyCompanyOR(Office comOffice, Company company,HttpServletRequest request) throws Exception {
        //根据公司创建管理员岗位
        Role comRole = this.createCompanyRoleOR(comOffice);
        if (comRole == null) {
            return "2";
        }
        //根据公司创建人力部门
        Office deptOffice = this.createOfficeDept(comOffice,1);
        //根据公司创建财务部门
        Office deptFinance = this.createOfficeFinance(comOffice,1);
        //根据公司创建招标部门
        //Office deptZb = this.createOfficeDeptZB(comOffice);
        //根据公司创建市场部
        Office deptSCB = this.createOfficeSCB(comOffice,1);
        //根据公司创建办公室
        Office deptBGS = this.createOfficeBGS(comOffice,1);
        //根据公司创建质量监管办
        Office deptZLJG = this.createOfficeZLJG(comOffice,1);
        //根据公司业务部
        Office deptYWB = this.createOfficeYWB(comOffice,1);
        //根据公司和财务部门创建财务的岗位
        List<Office> offices = new ArrayList<>();
        offices.add(deptOffice);
        offices.add(deptFinance);
        //offices.add(deptZb);
        offices.add(deptSCB);
        offices.add(deptBGS);
        offices.add(deptZLJG);
        offices.add(deptYWB);
        //根据公司和人力部门创建新员工的岗位
        this.createDeptRole( comOffice,comOffice, offices,"1");
        this.createFinanceRole( comOffice,comOffice, deptFinance,"1");
        //根据部门创建部门负责人岗位
        this.createOfficeManagerRole( comOffice,comOffice,offices,"1");
        //根据公司创建分管领导岗位
        this.createCompanyLeaderRole( comOffice,comOffice,offices,"1");
        //根据公司创建归档管理员岗位
        this.createCompanyGDManagerRole( comOffice,comOffice,deptBGS,"1");
        //根据公司和财务部门创建主管会计的岗位
        this.createFinanceRoleZgkj( comOffice,comOffice,deptFinance,"1");
        //根据公司和财务部门创建出纳的岗位
        this.createFinanceRoleCN( comOffice,comOffice,deptFinance,"1");
        //根据公司创建归档管理员岗位
        this.createCompanyManagerRole( comOffice,comOffice,"1");
        //根据公司和财务部门创建投标专员
        this.createTBZYRole( comOffice,comOffice,deptSCB,"1");
        //根据公司和财务部门创建盖章人的岗位
        this.createGZRRole(comOffice,comOffice,deptBGS,"1");
        //根据公司创建文件管理员岗位
       /* this.createCompanyFileManagerRole(company, comOffice);*/
        //根据公司创建总经理岗位
        //this.createCompanyManagerRole(company, comOffice);
        //根据公司创建总经理岗位
        this.createCompanyChairmanRole(comOffice,comOffice,"1");
        //根据公司创建文件总工岗位
        this.createCompanyZongGongManagerRole( comOffice, comOffice,deptZLJG,"1");
        //根据公司创建文件总工岗位
        this.createCompanyBZSHBManagerRole( comOffice, comOffice,deptZLJG,"1");
        //根据公司创建行政主管
        this.createCompanyHZZGRole(comOffice, comOffice,deptBGS,"1");
        //根据公司创建投标专员
        this.createCompanyRLZYRole(comOffice, comOffice,deptOffice,"1");
        //根据公司创建投标专员
        this.createCompanyALTBZYRole(comOffice, comOffice,deptZLJG,"1");
        //根据公司创建人事岗位
        //this.createCompanyHRRole(comRole,company, comOffice,comOffice, deptOffice,"1");
        Useroffice useroffice = new Useroffice();
        useroffice.setUserId(company.getTuser().getId());
        useroffice.setCompanyId(comOffice.getId());
        useroffice.setOfficeId(deptOffice.getId());
//		userofficeService.save(useroffice);
        comRole.setOffice(comOffice);
        User user = systemService.assignUserToRole(comRole, systemService.getUser(company.getTuser().getId()),0);
        company.getTuser().setCompany(comOffice);
        company.getTuser().setOffice(deptOffice);
        company.setCompanyId(comOffice.getId());
        company.setLegalPerson(user.getName());
        super.save(company);//保存不通知
        LayGroup layGroup = new LayGroup();
        layGroup.setGroupname(company.getName() + "官方群");
        layGroupService.insertLayGroup(layGroup,company.getTuser(),comOffice);
        //生成公司二维码，使用公司名
        String realPath = Global.getUserfilesBaseDir() + Global.USERFILES_BASE_URL
                + comOffice.getId() + "/qrcode/";
        FileUtils.createDirectory(realPath);
        String name = comOffice.getId() + ".png"; //encoderImgId此处二维码的图片名
        String filePath = realPath + name;  //存放路径
        String content = comOffice.getId() + "," + comOffice.getName() + "," + comOffice.getMaster() + "," + UserUtils.get(comOffice.getMaster()).getName();
        logger.info("encoderQRCode.content=" + content);
        TwoDimensionCode.encoderQRCode(content, filePath, "png");//执行生成二维码
        OSSClientUtil ossUtil = new OSSClientUtil();
        deptOffice.setCompanyQrcode(request.getContextPath() + Global.USERFILES_BASE_URL
                + comOffice.getId() + "/qrcode/" + name);
        comOffice.setCompanyQrcode(request.getContextPath() + Global.USERFILES_BASE_URL
                + comOffice.getId() + "/qrcode/" + name);
        deptOffice.setLogo(comOffice.getLogo());
        deptOffice.setGroupId(layGroup.getId());
        comOffice.setGroupId(layGroup.getId());
        deptOffice.setGroupName(company.getName() + "官方群");
        comOffice.setLogo(comOffice.getLogo());
        comOffice.setGroupName(company.getName() + "官方群");
        if (deptOffice.getCompanyQrcode() != null && !deptOffice.getCompanyQrcode().equals("")) {
            InputStream inputStream = null;
            HttpURLConnection httpURLConnection;
            URL url = new URL(request.getScheme() + "://" + request.getServerName() + ":" + request.getServerPort() + deptOffice.getCompanyQrcode());
            httpURLConnection = (HttpURLConnection) url.openConnection();
            // 设置网络连接超时时间
            httpURLConnection.setConnectTimeout(3000);
            // 设置应用程序要从网络连接读取数据
            httpURLConnection.setDoInput(true);

            httpURLConnection.setRequestMethod("GET");
            int responseCode = httpURLConnection.getResponseCode();
            if (responseCode == 200) {
                // 从服务器返回一个输入流
                inputStream = httpURLConnection.getInputStream();

            }
            String[] arr = deptOffice.getCompanyQrcode().split("/");
            logger.info("arr[arr.length-1]=" + arr[arr.length - 1]);
            String qrcodeName = System.currentTimeMillis() + arr[arr.length - 1];
            ossUtil.uploadFile2OSS(inputStream, Global.getRqcode(), qrcodeName);
            deptOffice.setCompanyQrcode(Global.getAliyunUrl() + "/" + Global.getRqcode() + qrcodeName);
            comOffice.setCompanyQrcode(Global.getAliyunUrl() + "/" + Global.getRqcode() + qrcodeName);
            officeService.save(deptOffice);
            officeService.save(comOffice);
            useroffice.setStatus("3");
            useroffice.setDelFlag("0");
            useroffice.setRemarks("管理员");
            useroffice.setMaster(UserUtils.getUser().getId());
            useroffice.setUserId(company.getTuser().getId());
            userofficeService.save(useroffice);
            //boolean b = systemService.outUserInRole(new Role("1"), systemService.getUser(company.getTuser().getId()));
            if(user.getRemove().equals("remove")){
                systemService.saveCurrentCompanyById(company.getTuser().getId(), comOffice.getId(), deptOffice.getId());
            }
            return "1";
        } else {
            return null;
        }
    }

    /**
     * 添加分公司部门岗位
     *
     * @param com
     * @return
     */
    @Transactional(readOnly = false)
    public String applyCompanyAnd(Office com,Office comOffice,Company company, HttpServletRequest request) throws Exception {
        //根据公司创建管理员岗位
        Role comRole = this.createCompanyRoleAND( com,comOffice);
        if (comRole == null) {
            return "2";
        }
        //根据公司创建人力部门
        Office deptOffice = this.createOfficeDeptAND(comOffice,2);
        //根据公司创建财务部门
        Office deptFinance = this.createOfficeFinance(comOffice,2);
        //根据公司创建招标部门
        //Office deptZb = this.createOfficeDeptZB(comOffice);
        //根据公司创建市场部
        Office deptSCB = this.createOfficeSCB(comOffice,2);
        //根据公司创建办公室
        Office deptBGS = this.createOfficeBGS(comOffice,2);
        //根据公司创建质量监管办
        Office deptZLJG = this.createOfficeZLJG(comOffice,2);
        //根据公司业务部
        Office deptYWB = this.createOfficeYWB(comOffice,2);
        comRole.setOffice(comOffice);
        //roleDao.insertRoleOffice(comRole);
        //根据公司和财务部门创建财务的岗位
        List<Office> offices = new ArrayList<>();
        offices.add(deptOffice);
        offices.add(deptFinance);
        //offices.add(deptZb);
        offices.add(deptSCB);
        offices.add(deptBGS);
        offices.add(deptZLJG);
        offices.add(deptYWB);
        //根据公司和人力部门创建新员工的岗位
        this.createDeptRole(com,comOffice, offices,"2");
        //根据公司和财务部门创建财务的岗位
        this.createFinanceRole( com,comOffice, deptFinance,"2");
        //根据部门创建部门负责人岗位
        this.createOfficeManagerRole( com,comOffice,offices,"2");
        //根据公司创建分管领导岗位
        this.createCompanyLeaderRole( com,comOffice,offices,"2");
        //根据公司创建归档管理员岗位
        this.createCompanyGDManagerRole( com,comOffice,deptBGS,"2");
        //根据公司和财务部门创建主管会计的岗位
        this.createFinanceRoleZgkj( com,comOffice,deptFinance,"2");
        //根据公司和财务部门创建出纳的岗位
        this.createFinanceRoleCN( com,comOffice,deptFinance,"2");
        //根据公司和财务部门创建投标专员
        this.createTBZYRole( com,comOffice,deptSCB,"2");
        //根据公司和财务部门创建盖章人的岗位
        this.createGZRRole(com,comOffice,deptBGS,"2");
        //根据公司创建归档管理员岗位
        this.createCompanyManagerRole( com,comOffice,"2");
        //根据公司创建文件管理员岗位
       /* this.createCompanyFileManagerRole(company, comOffice);*/
        //根据公司创建总经理岗位
        //this.createCompanyManagerRole(company, comOffice);
        //根据公司创建总经理岗位
        this.createCompanyChairmanRole(com,comOffice,"2");
        //根据公司创建文件总工岗位
        this.createCompanyZongGongManagerRole(com, comOffice,deptZLJG,"2");
        //根据公司创建文件总工岗位
        this.createCompanyBZSHBManagerRole(com, comOffice,deptZLJG,"2");
        //根据公司创建行政主管
        this.createCompanyHZZGRole(com, comOffice,deptBGS,"2");
        //根据公司创建投标专员
        this.createCompanyRLZYRole(com, comOffice,deptOffice,"2");
        //根据公司创建投标专员
        this.createCompanyALTBZYRole(comOffice, comOffice,deptZLJG,"1");
        //根据公司创建人事岗位
        //this.createCompanyHRRole(comRole,company, com, comOffice,deptOffice,"2");
        comRole.setOffice(comOffice);

        // //userofficeService.save(useroffice);
       // User user = systemService.assignUserToRole(comRole, systemService.getUser(company.getTuser().getId()));
            /*logger.info("company=" + company);
            //生成公司二维码，使用公司名
            String realPath = Global.getUserfilesBaseDir() + Global.USERFILES_BASE_URL
                    + comOffice.getId() + "/qrcode/";
            FileUtils.createDirectory(realPath);
            String name = comOffice.getId() + ".png"; //encoderImgId此处二维码的图片名
            String filePath = realPath + name;  //存放路径
            String content = comOffice.getId() + "," + comOffice.getName() + "," + comOffice.getMaster() + "," + UserUtils.get(comOffice.getMaster()).getName();
            logger.info("encoderQRCode.content=" + content);
            TwoDimensionCode.encoderQRCode(content, filePath, "png");//执行生成二维码
            OSSClientUtil ossUtil = new OSSClientUtil();
            deptOffice.setCompanyQrcode(request.getContextPath() + Global.USERFILES_BASE_URL
                    + comOffice.getId() + "/qrcode/" + name);
            comOffice.setCompanyQrcode(request.getContextPath() + Global.USERFILES_BASE_URL
                    + comOffice.getId() + "/qrcode/" + name);
            deptOffice.setLogo(comOffice.getLogo());
            if (deptOffice.getCompanyQrcode() != null && !deptOffice.getCompanyQrcode().equals("")) {
                InputStream inputStream = null;
                HttpURLConnection httpURLConnection;
                URL url = new URL(request.getScheme() + "://" + request.getServerName() + ":" + request.getServerPort() + deptOffice.getCompanyQrcode());
                httpURLConnection = (HttpURLConnection) url.openConnection();
                // 设置网络连接超时时间
                httpURLConnection.setConnectTimeout(3000);
                // 设置应用程序要从网络连接读取数据
                httpURLConnection.setDoInput(true);

                httpURLConnection.setRequestMethod("GET");
                int responseCode = httpURLConnection.getResponseCode();
                if (responseCode == 200) {
                    // 从服务器返回一个输入流
                    inputStream = httpURLConnection.getInputStream();

                }
                String[] arr = deptOffice.getCompanyQrcode().split("/");
                String qrcodeName = System.currentTimeMillis() + arr[arr.length - 1];
                ossUtil.uploadFile2OSS(inputStream, Global.getRqcode(), qrcodeName);
                deptOffice.setCompanyQrcode(Global.getAliyunUrl() + "/" + Global.getRqcode() + qrcodeName);
                comOffice.setCompanyQrcode(Global.getAliyunUrl() + "/" + Global.getRqcode() + qrcodeName);
                officeService.save(deptOffice);
                officeService.save(comOffice);
                Useroffice useroffice = new Useroffice();
                useroffice.setUserId(company.getTuser().getId());
                useroffice.setCompanyId(com.getId());
                List<Useroffice> useroffices = userofficeService.findList(useroffice);
                if (useroffices!=null && useroffices.size()!=0){
                    useroffice = useroffices.get(0);
                }
                useroffice.setOfficeId(deptOffice.getId());
                useroffice.setStatus("3");
                useroffice.setDelFlag("0");
                useroffice.setRemarks("管理员");
                useroffice.setMaster(UserUtils.getUser().getId());
                useroffice.setUserId(company.getTuser().getId());
                userofficeService.save(useroffice);
                //boolean b = systemService.outUserInRole(new Role("1"), systemService.getUser(company.getTuser().getId()));
                if(user.getRemove().equals("remove")){
                    systemService.saveCurrentCompanyById(company.getTuser().getId(), com.getId(), deptOffice.getId());
                }
                return "1";
            } else {
                return null;
            }*/

        return "1";
    }

    /**
     * 添加部门岗位
     *
     * @param office
     * @return
     */
    @Transactional(readOnly = false)
    public Boolean applyOffice(Office office) throws Exception {
        //根据公司和人力部门创建新员工的岗位
        List<Office> offices = new ArrayList<>();
        offices.add(office);
        String[] parentIds = office.getParentIds().split(",");
        Office branchOffice = new Office();
        Office com = UserUtils.getSelectCompany();
        for (int i = parentIds.length - 1; i > 0; i--) {
            Office o = officeService.get(parentIds[i]);
            if (o.getType().equals("3")) {
                branchOffice = o;
                break;
            }
        }
        if (branchOffice!=null && StringUtils.isNotBlank(branchOffice.getId())){
            this.createDeptRole(com,branchOffice, offices,"2");
            //根据部门创建部门负责人岗位
            this.createOfficeManagerRoleNoActivity(com,branchOffice,offices,"2");
            //根据部门创建部门负责人岗位
            this.createCompanyLeaderRoleNoActivity(com,branchOffice,office,"2");
            office.setBranchOffice(branchOffice.getId());
            officeService.saveBranchOffice(office);
        }else {
            this.createDeptRole(com,com, offices,"1");
            //根据部门创建部门负责人岗位
            this.createOfficeManagerRoleNoActivity(com,com,offices,"1");
            //根据部门创建部门负责人岗位
            this.createCompanyLeaderRoleNoActivity(com,com,office,"1");
        }

        return true;

    }


     /*@Transactional(readOnly = false)
    public int approveAll(String ids, HttpServletRequest request) {
       try {
            String idArray[] = ids.split(",");
            int successNum = 0;
            int failureNum = 0;

            for (String id : idArray) {
                Company company = this.get(id);
                if ("1".equals(company.getUSEABLE())) {
                    failureNum++;
                    //addMessage(redirectAttributes, "审批企业失败，已审批通过，正在使用中");
                } else if ("2".equals(company.getUSEABLE())) {
                    failureNum++;
                    //addMessage(redirectAttributes, "审批企业失败，已审批驳回");
                } else {
                    company.setUSEABLE("1");
                    Office comOffice = this.createOffice(company);
                    Role comRole = this.createCompanyRole(company, comOffice);
                    Office deptOffice = this.createOfficeDept(comOffice);
                    this.createDeptRole(comRole,company, comOffice, deptOffice);
                    if (UserUtils.getSelectRole().get(0).getId().equals("1")) {
                        systemService.outUserInRole(new Role("1"), systemService.getUser(company.getTuser().getId()));
                    }
                    User user = systemService.assignUserToRole(comRole, systemService.getUser(company.getTuser().getId()));
                    company.getTuser().setCompany(comOffice);
                    company.getTuser().setOffice(deptOffice);
                    company.setCompanyId(comOffice.getId());
                    this.save(company);//保存

                    Group group = new Group();
                    group.groupname(company.getName() + "官方群").desc(company.getName() + "官方群")._public(true).maxusers(1000).approval(true).owner(company.getTuser().getId());
                    Object obj = easemobChatGroup.createChatGroup(group);
                    JSONObject json = JSONObject.fromObject(obj);

                    String groupid = null;
                    if (json != null && !json.get("data").toString().equals("")) {
                        JSONObject data = JSONObject.fromObject(json.get("data").toString());
                        if (data != null && !data.get("groupid").toString().equals(""))
                            groupid = data.get("groupid").toString();
                    }

                    EasemobInfo easemobImuser = new EasemobInfo();
                    easemobImuser.setUserId(company.getTuser().getId());
                    easemobImuser.setGroupId(groupid);
                    easemobImuser.setJoinStatus("1");
                    easemobImuser.setGroupName(company.getName() + "官方群");
                    easemobInfoService.save(easemobImuser);
                    //生成公司二维码，使用公司名
                    String realPath = Global.getUserfilesBaseDir() + Global.USERFILES_BASE_URL
                            + comOffice.getId() + "/qrcode/";
                    FileUtils.createDirectory(realPath);
                    String name = comOffice.getId() + ".png"; //encoderImgId此处二维码的图片名
                    String filePath = realPath + name;  //存放路径
                    String content = comOffice.getId() + "," + comOffice.getName() + "," + comOffice.getMaster() + "," + UserUtils.get(comOffice.getMaster()).getName();
                    logger.info("encoderQRCode.content=" + content);
                    TwoDimensionCode.encoderQRCode(content, filePath, "png");//执行生成二维码
                    OSSClientUtil ossUtil = new OSSClientUtil();
                    deptOffice.setCompanyQrcode(request.getContextPath() + Global.USERFILES_BASE_URL
                            + comOffice.getId() + "/qrcode/" + name);
                    comOffice.setCompanyQrcode(request.getContextPath() + Global.USERFILES_BASE_URL
                            + comOffice.getId() + "/qrcode/" + name);
                    deptOffice.setGroupId(groupid);
                    comOffice.setGroupId(groupid);
                    deptOffice.setGroupName(company.getName() + "官方群");
                    deptOffice.setLogo(comOffice.getLogo());
                    comOffice.setGroupName(company.getName() + "官方群");
                    if (deptOffice.getCompanyQrcode() != null && !deptOffice.getCompanyQrcode().equals("")) {
                        InputStream inputStream = null;
                        HttpURLConnection httpURLConnection;
                        try {
                            URL url = new URL(request.getScheme() + "://" + request.getServerName() + ":" + request.getServerPort() + deptOffice.getCompanyQrcode());
                            httpURLConnection = (HttpURLConnection) url.openConnection();
                            // 设置网络连接超时时间
                            httpURLConnection.setConnectTimeout(3000);
                            // 设置应用程序要从网络连接读取数据
                            httpURLConnection.setDoInput(true);

                            httpURLConnection.setRequestMethod("GET");
                            int responseCode = httpURLConnection.getResponseCode();
                            if (responseCode == 200) {
                                // 从服务器返回一个输入流
                                inputStream = httpURLConnection.getInputStream();

                            }
                            String[] arr = deptOffice.getCompanyQrcode().split("/");
                            logger.info("arr[arr.length-1]=" + arr[arr.length - 1]);
                            String qrcodeName = System.currentTimeMillis() + arr[arr.length - 1];
                            ossUtil.uploadFile2OSS(inputStream, Global.getRqcode(), qrcodeName);
                            deptOffice.setCompanyQrcode(Global.getAliyunUrl() + "/" + Global.getRqcode() + qrcodeName);
                            comOffice.setCompanyQrcode(Global.getAliyunUrl() + "/" + Global.getRqcode() + qrcodeName);
                        } catch (MalformedURLException e) {
                            // TODO Auto-generated catch block
                            e.printStackTrace();
                        } catch (IOException e) {
                            // TODO Auto-generated catch block
                            e.printStackTrace();
                        } finally {
                            inputStream.close();
                        }
                    }
                    officeService.save(deptOffice);
                    officeService.save(comOffice);
                    successNum++;
                }
            }
            return successNum;
        } catch (Exception e) {
            return -1;
        }
    }*/

    /**
     * 查询当前用户在当前公司所在的岗位是不是管理员
     */
    public List<String> getUserRoleManager(Company company) {

        try {
            List<String> roleList = null;
            roleList = companyDao.getUserRoleManager(company);
            return roleList;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * 判断管理员用户所在公司有几个管理员
     */
    public List<Role> getUserRoleComManager(Company company) {
        try {
            List<Role> roleComList = null;
            roleComList = companyDao.getUserRoleComManager(company);
            return roleComList;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    @Transactional(readOnly = false)
    public void deleteWorkAppShow(String companyId, String id) {
        companyDao.deleteWorkAppShow(companyId, id);
    }



    /**
     *根据公司id查询公司下所有的用户
     */
    public List<Useroffice> getUserCompanyId(Company company){
        List<Useroffice> userOfficeList = companyDao.updateUserCompanyId(company);
        return userOfficeList;
    }
    //通过公司名称获取公司信息
    public Company getByName(String name){
        Company company = companyDao.findByName(name);
        return company;
    }
    ;
}